*-- Author :    Hartwig Albrecht   03/03/95
*-- CFOPEN implemented by Ivan Belyaev

      SUBROUTINE IFOPEN (LUN, CNAME, COPT, LREC, IOS, IPNT)

************************************************************************
*
* Description: open file for sequential or direct access.
*              Does NOT work on IBM (expects DDNAME instead of file name)
*
* Called from: anywhere
*
* External references: CFOPEN
*
* Argument in/out description
*   LUN    in     logical unit number
*   CNAME  in     data set name
*                 ' ' for a scratch file
*   COPT   in     combination of (any order; lower or upper case):
*
*                 'l'  open with C, C++ (CFOPEN)
*                 'r'  open for reading                        
*                 'r+' open for read/write                     
*                 'w'  create or truncate for writing          
*                 'w+' open for write/read, create or truncate 
*                 'a'  append                                  
*                 'a+' open for append/read                    
*
*                      open with Fortran ('l' not given):
*                 'b'  binary (unformatted)
*                 'f'  formatted; 'f' and 'b' are mutually exlusive
*                 'n'  new: file must not yet exist
*                 'o'  old: file must exist
*                 'on' file may or may not exist
*                 'ao' append to an existing file
*                 'a'  append (if file exists) or create new file
*                      (if not)
*
*                      open with GPACK ('g' is given):
*                 'g'  open with GPACK
*                 'm'  use Shared Memory
*
*   LREC   in     record length for direct access
*                 0 for sequential access
*                 Shared Memory size for GPACK Shared Memory
*   IOS    out    io status (IOS = 0 --> ok)
*   IPNT   out    pointer from CFOPEN
*

      IMPLICIT NONE

#include "arte/Arte.fi"
#include "arte/ArteFlag.fi"

      INTEGER I, IPNT, LREC, LUN, IOS, LENOCC, CHIND1, IN_OUT_FLAG
      CHARACTER *(*) CNAME, COPT
      CHARACTER*256  CNEWNAME
      CHARACTER CSTA*7, CFOR*11, CACC*10, CFULL*256, CC*2
      LOGICAL  C_FILE 

      CHARACTER*256 F1NAME 
      CHARACTER*256 F2NAME 
 
#include "arte/Arte.ff"

      CNEWNAME = CNAME

      IF (CNEWNAME .EQ. ' ' .OR. CNEWNAME(1:1) .EQ. '/')  THEN
        CFULL = CNEWNAME
      ELSE
        CFULL = SRCNAME
        CALL CHAPPC (CFULL, '/')
        CALL CHAPPC (CFULL, CNEWNAME)
      END IF

      IN_OUT_FLAG = 1		! 0 = IN;  1 = OUT
      IF (CHIND1 (COPT, 'I') .NE. 0) IN_OUT_FLAG = 0
      
      IF (CHIND1 (COPT, 'G') .NE. 0) THEN
        CALL GARTOPEN (LUN, CFULL, 0, IN_OUT_FLAG, LREC, IOS)
        IF (IOS .NE. 0)  GO TO 40
        GO TO 90
      ENDIF

      IF (CHIND1 (COPT, 'M') .NE. 0) THEN
        CALL GARTOPEN (LUN, CFULL, 1, IN_OUT_FLAG, LREC, IOS)
        IF (IOS .NE. 0)  GO TO 40
        GO TO 90
      ENDIF
      
      CC = 'r+'
      CSTA = 'UNKNOWN'
      CFOR = ' '
      CACC = 'SEQUENTIAL'
      C_FILE = .FALSE. 


      IF (CHIND1 (COPT, 'B') .NE. 0)  CFOR = 'UNFORMATTED'
      IF (CHIND1 (COPT, 'F') .NE. 0)  THEN
        IF (CFOR .NE. ' ')  GO TO 80
        CFOR = 'FORMATTED'
      END IF

      IF (CHIND1 (COPT, 'N') .NE. 0)  THEN
        IF (CNEWNAME .EQ. ' ')  GO TO 70
        CSTA = 'NEW'
        CC = 'w'
      END IF

      IF (CHIND1 (COPT, 'O') .NE. 0)  THEN
        IF (CNEWNAME .EQ. ' ')  GO TO 70
        IF (CSTA(1:1) .NE. 'N')  CSTA = 'OLD'
      END IF

      IF (CHIND1 (COPT, 'A') .NE. 0)  THEN
        IF (LREC .NE. 0)  GO TO 60
        IF (CNEWNAME .EQ. ' ')  GO TO 70
        CACC = 'APPEND'
        CC = 'a'
        IF (CHIND1 (COPT, 'R') .NE. 0 .OR.
     +      CHIND1 (COPT, 'W') .NE. 0)  GO TO 50
        IF (CHIND1 (COPT, '+') .NE. 0)  CC = 'a+'
      END IF

      IF (CHIND1 (COPT, 'L') .NE. 0)  C_FILE = .TRUE. 

      IF (CHIND1 (COPT, 'R') .NE. 0)  THEN
        IF (CHIND1 (COPT, 'W') .NE. 0)  GO TO 50
        C_FILE = .TRUE.
        CC = 'r'
        IF (CHIND1 (COPT, '+') .NE. 0)  CC = 'r+'
      END IF

      IF (CHIND1 (COPT, 'W') .NE. 0)  THEN
        C_FILE = .TRUE.
        CC = 'w'
        IF (CHIND1 (COPT, '+') .NE. 0)  CC = 'w+'
      END IF

      
C     *     Stage (if nesessary)
      F1NAME = CNEWNAME 
      CALL UTSTAGE( LUN , F1NAME , F2NAME )
      CNEWNAME   = F2NAME
C     

      IF( C_FILE )THEN
C*     see manual for FZFILE and CFOPEN routines and CFIO package

        CALL CFOPEN(IPNT,0,900,CC,0,CNEWNAME,IOS)
        IF( IPNT .LE. 0  .OR. IOS .NE. 0 )THEN
          C78='IFOPEN:IPNT,CC,IOS='
          CALL CH78AI( IPNT )
          CALL CH78CC( CC         ) 
          CALL CH78CI( IOS        ) 
          CALL CH78BC( 'for file' ) 
          CALL CH78BC( CNEWNAME       ) 
          CALL ME78EC (7634544) 
          IF( IOS .EQ. 0 )IOS = -999
        ENDIF
        GO TO 90
      ENDIF 

      IPNT = 0
      IF (LREC .EQ. 0)  THEN
        IF (CNEWNAME .EQ. ' ')  THEN

C         sequential, scratch

          IF (CFOR .EQ. ' ')  THEN
            OPEN (LUN, STATUS = 'SCRATCH', IOSTAT = IOS)
          ELSE
            OPEN (LUN, STATUS = 'SCRATCH', FORM = CFOR, IOSTAT = IOS)
          END IF
        ELSE

C         sequential, named

          IF (CFOR .EQ. ' ')  THEN
            OPEN (LUN, STATUS = CSTA, FILE = CNEWNAME, ACCESS = CACC,
     +        IOSTAT = IOS)
          ELSE
            OPEN (LUN, STATUS = CSTA, FILE = CNEWNAME, ACCESS = CACC,
     +        FORM = CFOR, IOSTAT = IOS)
          END IF
        END IF
      ELSE
        IF (CNEWNAME .EQ. ' ')  THEN

C         direct access, scratch

          IF (CFOR .EQ. ' ')  THEN
            OPEN (LUN, STATUS = 'SCRATCH', ACCESS = 'DIRECT',
     +        RECL = LREC, IOSTAT = IOS)
          ELSE
            OPEN (LUN, STATUS = 'SCRATCH', ACCESS = 'DIRECT',
     +        RECL = LREC, FORM = CFOR, IOSTAT = IOS)
          END IF
        ELSE

C         direct access, named

          IF (CFOR .EQ. ' ')  THEN
            OPEN (LUN, STATUS = CSTA, FILE = CNEWNAME,
     +        ACCESS = 'DIRECT', RECL = LREC, IOSTAT = IOS)
          ELSE
            OPEN (LUN, STATUS = CSTA, FILE = CNEWNAME,
     +        ACCESS = 'DIRECT', RECL = LREC, FORM = CFOR, IOSTAT = IOS)
          END IF
        END IF
      END IF

      IF (IOS .EQ. 0)  GO TO 90
      C78 = 'OPEN: iostat='
      CALL CH78AI (IOS)
      CALL CH78BC ('for file')
      CALL CH78BC (CNEWNAME)
      CALL ME78EC (88)
      GO TO 90
      
 40   C78 = 'MMOPEN: gp_open returns rc ='
      CALL CH78BI (IOS)
      CALL ME78EC (92)
      CALL IOCLOS (LUN)
      GO TO 90

 50   CALL MEERRC (1345, 'OPEN: options A, R, W are mutually exclusive')
      GO TO 90

 60   CALL MEERRC (298,
     +  'OPEN: option A only for sequential data sets (LREC = 0)')
      GO TO 90


 70   CALL MEERRC (926,
     +  'OPEN: options N, O, A require a data set name')
      GO TO 90

 80   CALL MEERRC (825,
     +  'OPEN: options B and F are mutually exclusive')

 90   END

************************************************************************
* End of IFOPEN
************************************************************************
