#ifndef _ARTEFUNC_H_
#define _ARTEFUNC_H_

#include <functional>
#include <arte/arte.hh>

//
// compare ARTE pointers
//
template<class T>
class lesscomp
{
public:
  int operator()( ArtePointer<T> e1, ArtePointer<T> e2 ) const {
    return e1.index() < e2.index();
  }
};


#if 0
//
// compare members
//
template<class T,class Result>
class comparisonMember:
  public binary_function<ArtePointer<T>, ArtePointer<T>,int >
{
public:
/*   typedef T argument_type; */
/*   typedef Result result_type; */
  comparisonMember( Result T::* m ): member( m ) {}
  int operator()( ArtePointer<T> el1, ArtePointer<T> el2 ) {
    return (*el1).*member < (*el2).*member;
  }
private:
  Result T::* member;
};


template<class T,class Result>
comparisonMember<T, Result> compareMember( Result T::* m )
{ return comparisonMember<T,Result>( m ); }
#endif


//
// compare entry (replaces compare member)
//
template<class T,class Result>
class comparisonEntry:
  public binary_function<ArtePointer<T>, ArtePointer<T>,int >
{
public:
/*   typedef T argument_type; */
/*   typedef Result result_type; */
  comparisonEntry( Result T::* m ): member( m ) {}
  int operator()( ArtePointer<T> el1, ArtePointer<T> el2 ) {
    return (*el1).*member < (*el2).*member;
  }
private:
  Result T::* member;
};


template<class T,class Result>
comparisonEntry<T, Result> compareEntry( Result T::* m )
{ return comparisonEntry<T,Result>( m ); }


//
// compare call
//
template<class T,class Result>
class comparisonCall:
  public binary_function<T*,T*,int >
{
public:
  typedef T* argument_type;
  typedef int result_type;
  comparisonCall( Result (T::* m)() const ): member( m ) {}
  int operator()( const T* el1, const T* el2 ) const {
    return ((*el1).*member)() < ((*el2).*member)();
  }
private:
  Result (T::* member)() const;
};


template<class T,class Result>
comparisonCall<T, Result> compareCall( Result (T::* m)() const )
{ return comparisonCall<T,Result>( m ); }



//
// get ArteTable entry
// 
template<class T,class Result>
class getterEntry: public unary_function<ArtePointer<T>,Result>
{
public:
  typedef ArtePointer<T> argument_type;
  typedef Result result_type;
  getterEntry( Result T::* m ): member( m ) {}
  Result operator()( ArtePointer<T> el ) const {
    return (*el).*member;
  }
private:
  Result T::* member;
};


template<class T,class Result>
getterEntry<T, Result> getEntry( Result T::* m )
{ return getterEntry<T,Result>( m ); }


//
// set ArteTable entry
// 
template<class T,class EntryType>
class setterEntry: public binary_function<ArtePointer<T>,EntryType,void>
{
public:
  typedef ArtePointer<T> first_argument_type;
  typedef EntryType      second_argument_type;
  typedef void           result_type;
  setterEntry( EntryType T::* m ): member( m ) {}
  void operator()( ArtePointer<T> el, EntryType val ) const {
    (*el).*member = val;
  }
private:
  EntryType T::* member;
};


template<class T,class EntryType>
setterEntry<T, EntryType> setEntry( EntryType T::* m )
{ return setterEntry<T,EntryType>( m ); }


//
// get members
// 
template<class T,class Result>
class getterMember: public unary_function<T,Result>
{
public:
  typedef T argument_type;
  typedef Result result_type;
  getterMember( Result T::* m ): member( m ) {}
  Result operator()( T el ) const {
    return el.*member;
  }
private:
  Result T::* member;
};


template<class T,class Result>
getterMember<T, Result> getMember( Result T::* m )
{ return getterMember<T,Result>( m ); }


//
// get members
// 
template<class T,class Result>
class callerMember: public unary_function<T*,Result>
{
public:
  typedef T argument_type;
  typedef Result result_type;

  callerMember( Result (T::* m)() const ): member( m ) {}
  Result operator()( T* el ) const {
    return ((*el).*member)();
  }
private:
  Result (T::* member)() const;
};


template<class T,class Result>
callerMember<T, Result> callMember( Result (T::* m)() const )
{ return callerMember<T,Result>( m ); }


template<class In1, class In2, class Out,
  class BinaryFunction1, class BinaryFunction2>
int findmatch( In1 beg1, In1 end1, In2 beg2, In2 end2,
	       Out out,
	       BinaryFunction1 ahead, BinaryFunction2 match )
{
  int matchnum = 0;
  In1 iter1 = beg1;
  In2 iter2 = beg2;
  
  int ahe;         // cache for ahead result
  In1 start;
  
  if( iter1!=end1 ) {
    for(;;) {
      while( iter2!=end2 && (ahe=ahead( *iter1, *iter2 ))>0 ) ++iter2;
      if( iter2==end2 ) break;
      start = iter1;
      do {
	if( ahe==0 && match( *iter1, *iter2 ) ) {
	  *out = make_pair( *iter1, *iter2 );
	  ++matchnum;
	}
	++iter1;
      } while( iter1!=end1 && ( ahe = ahead( *iter1, *iter2 )) <= 0 );
      //if( iter1==end1 ) break;
      iter1=start;
      ++iter2;
    }
  }
  return matchnum;
}
#endif
