//                              -*- Mode: C++ -*-
//: CmdCarePid04.C
//
//
// Author: roberto spighi  
// ECAL group 
// HERA-B experiment, Desy
// e-mail: spighi@bo.infn.it
//
// Created 1/9/97
// Last Update ........
//

#include "care/CmdCarePid04par.hh"
#include "care/ESPID04.hh"
#include "arte/ArteMessage.hh"

/////////////// The function seen by kuip
#ifdef USEKUIP
extern "C" {
  void carepid04par_(){
    CmdCarePid04par pid04Cmd;
    pid04Cmd.readAndExe();
    //    pid04Cmd.dump();
  }
}; // end of extern
#endif
//////////////////////////////////////////////////////////

/*
CLASS
  CmdCarePid04par
  Base class to deal with the CAREPID04PAR kuip command

class CmdCarePid04par {
public:
  //-- Constructors
  CmdCarePid04par();
  //-- Destructor
  // Destructor: currently void
  virtual ~CmdCarePid04par();
  
  //-- Behavioural operations
  //   Reset stored data to default values
  virtual void defValues();
  //--
  //   Execute stored command
  virtual void execute();
  //--
  //   Read a command from external
  virtual void read();
  //--
  //   Dump internal data
  virtual void dump();

private:
  ///////////////////// Data Part /////////////////////////////////
  //--
  int  cal,particle;
  float like04[9];
};

*/

CmdCarePid04par::CmdCarePid04par(){
  defValues();
}

CmdCarePid04par::~CmdCarePid04par(){}

void CmdCarePid04par::defValues(){

  cal = 1;
  particle = 1;
  like04[0] = 169.56;
  like04[1] = 1.4967;
  like04[2] = 0.1375;
  like04[3] = 0.63451;
  like04[4] = 1.5409;
  like04[5] = 0.94375;
  like04[6] = 0.19489;
  like04[7] = 16.131;
  like04[8] = -7.3106;
}

//--
//   Execute stored command
void CmdCarePid04par::execute(){
  ESPID04::setparlike04(cal,particle,like04);
}

# ifdef USEKUIP
//--
//   Read acommand from external
void CmdCarePid04par::read(){
  cal = readInt();
  particle = readInt();
  like04[0] = readFloat();
  like04[1] = readFloat();
  like04[2] = readFloat();
  like04[3] = readFloat();
  like04[4] = readFloat();
  like04[5] = readFloat();
  like04[6] = readFloat();
  like04[7] = readFloat();
  like04[8] = readFloat();
}
# endif
                          
//--
//   Printout Information for the current MinimumEnergy data stored
void CmdCarePid04par::dump(){
  // ECHitCellArray::dumpCutOff();
  arteMsg<<" Cal:"<<cal<<" Particle:"<<particle;
  arteMsg.allp();
  for(int i=0; i<9; i++){
    arteMsg<<"Par: "<<i<<" value:"<<like04[i];
    arteMsg.allp();
  }
}







