//                              -*- Mode: C++ -*-
//: CmdCareShowDcal.C
//
//
// Author: Mauro Villa   
// ECAL group 
// HERA-B experiment, Desy
// e-mail: villa@bo.infn.it
//
// Created 12/6/97
// Last Update ........
//

#include "care/CmdCareShowDcal.hh"
#include "arte/ArteMessage.hh"
#include "arte/arte.hh"
#include "care/ECDcalIO.hh"
#include <iomanip.h>
/////////////// The function seen by kuip

#ifdef USEKUIP
extern "C" {
  void swdcal_(){
    CmdCareShowDcal showdcal;
    showdcal.readAndExe();
  }
}
#endif

/////////////// CmdCareShowDcal class member function definition

CmdCareShowDcal::CmdCareShowDcal() : start(0), end(0){}
CmdCareShowDcal::~CmdCareShowDcal(){}

void CmdCareShowDcal::defValues(){
  start = 0;
  end = 0;
}


//--
//   Execute stored command
void CmdCareShowDcal::execute(){
  // Check if the bank exists and is not empty
  if( ArtePointer<DCAL>::getLink()()==0 ){
    arteMsg<<" The DCAL bank is not booked !";
    arteMsg.allp();
    return;
  }
  if( ArtePointer<DCAL>::nrow()==0 ){
    arteMsg<<" The DCAL bank is empty !";
    arteMsg.allp();
    return;
  }
  // find the real start and stop
  int shortcounter = 1;
  int rstart = start;
  int rend = end;
  if( rstart==0){
    rstart = 1;
    rend = ArtePointer<DCAL>::nrow()*2;
  }
  // Define the data types you can have in the buffer:
  unsigned short head1=0, head2=0, siz=0, fed=0, idmk=0, bx=0, value=0;

  // cheching values
  unsigned short firstbx =0x8000;
  unsigned int i;
  unsigned int ndat = 8;
  bool show;
  bool sparse;
  char occur[256];
  for(i=0; i<256; i++)
        occur[i] = 0;


  ECDcalIO dcal;
  dcal.start(); 
  // read the bank header
  dcal.getBankHeader(head1, head2);	
  // Always show the header
  arteMsg<<shortcounter<<": Bank headers:  0x"<<(hex)<<head1
         <<" 0x"<<(hex)<<head2<<(dec);
  arteMsg.allp();
  shortcounter +=2;
  
  unshort fed_id = dcal.nextFED(); // read the Front End Detector IDentifier
  while( fed_id>0) {

    if( shortcounter>=rstart && shortcounter<=rend ) show = true;
    else show = false;
    // read the header      
    unsigned int nch = dcal.getFedHeader(siz, fed, idmk, bx);
    sparse = dcal.sparse(); // test if sparsified

    if( firstbx==0x8000){
      firstbx = bx;
    }   
    if( !( (idmk>>14)!=0 || bx!=firstbx  || 
	   fed<0x10 || (fed>0xF2 && fed<0xEC00) || fed>0xECFF ) )
        occur[ fed & 0xFF ]++;
    if( show ) {
      arteMsg<<shortcounter<<": Fed header:  size "<<siz
             <<"  FED id "<<(hex)<<fed
             <<"  RID id "<<idmk<<(dec)<<"  BX "<<bx;
      if( siz!=72 && !sparse ) arteMsg<<" wrong size!";
      if( (idmk>>14)!=0 || bx!=firstbx ) arteMsg<<" wrong header!";
      if( fed<0x10 || (fed>0xF2 && fed<0xEC00) || fed>0xECFF) 
        arteMsg<<" wrong FED id!";
      if( occur[ fed & 0xFF ]>1) arteMsg<<" FED read "
                                        <<(int) occur[ fed&0xFF]
                                        <<" times ";
      arteMsg.allp();
    }
    shortcounter +=4;

    if( show ) {
      int ofs = 0;
      ndat = (sparse ? 4 : 8);
      for(i=0; i<nch; i++){
	dcal.getNextShort(ofs,value);
	shortcounter++;
	if( (i%ndat)==0 ) arteMsg<<shortcounter<<": ";
	if( !sparse ){
	  arteMsg<<(hex)<<setw(8)<<value<<(dec);
	} else {
	  arteMsg<<" ch"<<(dec)<<setw(3)<<ofs<<(hex)<<setw(8)<<value<<(dec);
	}
        if( value>4095 ) arteMsg<<"*";
        else arteMsg<<" ";
	if( (i%ndat)==(ndat-1) ) arteMsg.allp();
      }
      if( (i%ndat)!=0 ) arteMsg.allp();
    } else {
      shortcounter+=nch;
    }
    fed_id = dcal.nextFED(); // go and take the next FED
  }
}


#ifdef USEKUIP
//--
//   Read a command from external
void CmdCareShowDcal::read(){
  start = readInt();
  end = readInt();
  if( end<=start ) end = start;
}
#endif

void CmdCareShowDcal::dump(){
  int rstart = start;
  int rend = end;
  if( rstart==0){
    rstart = 1;
    rend = ArtePointer<DCAL>::nrow();
  }
  arteMsg<<" Showing DCAL bank from  "<<rstart
         <<" to "<<rend;
  arteMsg.allp();
}
