//                              -*- Mode: C++ -*-
//:ECDBaseIO.C
//
// Author: Mauro Villa   
// ECAL group 
// HERA-B experiment, Desy
// e-mail: villa@bo.infn.it
//
// Created 1/09/98
// Last Update ........
//
//


#define ASCIIDBASE
#undef LEDADBASE


/* standard includes */
#include <iostream.h>
#include <fstream.h>

#include "care/ECDBaseIO.hh"


#ifdef LEDADBASE
extern "C" 
{
  int EcalLedaInitialization(void);
  int openTheLedaDB(char*);
  int closeTheLedaDB(void);
  int setLedaStamp(int,int);
  int StoreEcalEcbo(struct ECBO*, int);
  int StoreEcalEchs(struct ECHS*, int);
  int StoreEcalEchf(struct ECHF*, int);
  int FetchEcalEcboOnLeda(void);
  int FetchEcalEchsOnLeda(void);
  int FetchEcalEchfOnLeda(void);
  int FetchEcalEcbo(struct ECBO*);
  int FetchEcalEchs(struct ECHS*);
  int FetchEcalEchf(struct ECHF*);
}
#endif
#ifdef ASCIIDBASE
  int StoreEcalEcbo(DBpointer_t, struct ECBO*, int);
  int StoreEcalEchs(DBpointer_t, struct ECHS*, int);
  int StoreEcalEchf(DBpointer_t, struct ECHF*, int);
  int FetchEcalEcbo(DBpointer_t, struct ECBO*, int);
  int FetchEcalEchs(DBpointer_t, struct ECHS*, int);
  int FetchEcalEchf(DBpointer_t, struct ECHF*, int);
#endif


/**
class ECDBaseIO.

*/

ECDBaseIO* ECDBaseIO::ecdbaseio = 0;

ECDBaseIO::ECDBaseIO(char * dbnam) : 
  databasename(0), dbstatus(closed), dbpointer(0) 
{
  if(dbnam!=0)
    {
      lasttimetag.major = 0;
      lasttimetag.minor = 0;
      setDBname(dbnam);
    }
}

ECDBaseIO* ECDBaseIO::initialize(char * dbname) {
  if(ecdbaseio==0){
    ecdbaseio = new ECDBaseIO(dbname);
#   ifdef LEDADBASE
    EcalLedaInitialization();
#   endif
  }
  return ecdbaseio;
}

/// destructor
ECDBaseIO::~ECDBaseIO(){
  if( dbstatus==opened){
    close();
  }
  delete [] databasename;
  databasename = 0;
}

/// set the DB name
char * ECDBaseIO::setDBname(char *dbnam){
  if( dbstatus==opened){
    cerr<<" Shouldn't you close your DB before changing it?"<<endl;
    close();
  }
  if( databasename!=0){
    delete [] databasename;
  }
  int len = strlen(dbnam);
  databasename = new char[len];
  strcpy(databasename, dbnam);  
  return databasename;
}

//--
/// Open the database
int ECDBaseIO::open(int mode)
{
  if(dbstatus!=closed)
    {
      cerr<<" Shouldn't you close your DB before opening it?"<<endl;
      close();
    }
# ifdef ASCIIDBASE
  dbpointer = new fstream(databasename, mode );
  if( !dbpointer || !dbpointer->is_open()){
     cerr<<" Error opening the DB"<<endl;
  }
# endif
#ifdef LEDADBASE
  openTheLedaDB(databasename);
#endif

  dbstatus = opened;
  return 0;
}

int ECDBaseIO::close(){
 if(dbstatus!=opened){
    cerr<<" Shouldn't you open your DB before closing it?"<<endl;
  }
  //
# ifdef LEDADBASE
  closeTheLedaDB();
# endif
# ifdef ASCIIDBASE
  dbpointer->close();
  delete dbpointer;
# endif
  dbstatus = closed;
  //
  return 0;
}

//--
/// Store the data in memory onto the database
int ECDBaseIO::store(TimeTag_t timetag){
  if(dbstatus!=opened){
   cerr<<" Shouldn't you open your DB before writing on it?"<<endl;
   open();
  }
  // define the table indeces
# ifdef LEDADBASE
  setLedaStamp(timetag.major,timetag.minor);
  lasttimetag = timetag;
  // call C functions for LEDA to store the STL banks
  StoreEcalEcbo(ptrEcbo, entriesEcbo);
  StoreEcalEchs(ptrEchs, entriesEchs);
  StoreEcalEchf(ptrEchf, entriesEchf);
# endif
# ifdef ASCIIDBASE
  (*dbpointer)<<timetag.major<<" "<<timetag.minor<<endl;
  (*dbpointer)<<entriesEcbo<<" "<<entriesEchs<<" "<<entriesEchf<<endl;
  // call C functions for LEDA to store the STL banks
  StoreEcalEcbo(dbpointer, ptrEcbo, entriesEcbo);
  StoreEcalEchs(dbpointer, ptrEchs, entriesEchs);
  StoreEcalEchf(dbpointer, ptrEchf, entriesEchf);
# endif
  return 0;
}

//--


/// Load DB Tables as LEDA Tables
int ECDBaseIO::fetchTables(TimeTag_t timetag){
  if(dbstatus!=opened){
   cerr<<" Shouldn't you open your DB before reading from it?"<<endl;
   open();
  }
# ifdef LEDADBASE
  // define the table indeces
  setLedaStamp(timetag.major,timetag.minor);
  lasttimetag = timetag;
  entriesEcbo = FetchEcalEcboOnLeda();
  entriesEchs = FetchEcalEchsOnLeda();
  entriesEchf = FetchEcalEchfOnLeda();
# endif
# ifdef ASCIIDBASE
  (*dbpointer)>>timetag.major>>timetag.minor;
  (*dbpointer)>>entriesEcbo>>entriesEchs>>entriesEchf;
# endif

  return 0;
}
//--



/// Read the data from the database
int ECDBaseIO::read(){
  if(dbstatus!=opened){
   cerr<<" Shouldn't you open your DB before reading from it?"<<endl;
   open();
  }
# ifdef LEDADBASE
  FetchEcalEcbo(ptrEcbo);
  FetchEcalEchs(ptrEchs);
  FetchEcalEchf(ptrEchf);
# endif
# ifdef ASCIIDBASE
  FetchEcalEcbo(dbpointer, ptrEcbo,entriesEcbo);
  FetchEcalEchs(dbpointer, ptrEchs,entriesEchs);
  FetchEcalEchf(dbpointer, ptrEchf,entriesEchf);
# endif

  return 0;
}

#ifdef ASCIIDBASE
//-- Store ecal data
int StoreEcalEcbo(DBpointer_t dbpointer, struct ECBO* ecbo, int n){

  (*dbpointer)<<"##Start_bank_ECBO"<<endl;
  for(int i=0; i<n; i++, ecbo++){
    (*dbpointer)<<ecbo->fdid<<" "<<ecbo->stat<<" "
		<<ecbo->numb<<" "<<ecbo->link<<endl;
    for(int j=0; j<32; j++)
      (*dbpointer)<<ecbo->chan[j]<<" ";
    (*dbpointer)<<endl;
  }
  (*dbpointer)<<"##end_bank_ECBO "<<endl;
  return 0;
}

//-- Store ecal data ECBO
int FetchEcalEcbo(DBpointer_t dbpointer, struct ECBO* ecbo, int n){

  char string[50];
  (*dbpointer)>>string;
  for(int i=0; i<n; i++, ecbo++){
    (*dbpointer)>>ecbo->fdid>>ecbo->stat
		>>ecbo->numb>>ecbo->link;
    for(int j=0; j<32; j++)
      (*dbpointer)>>ecbo->chan[j];
  }
  (*dbpointer)>>string;
  return 0;
}


//-- Store ecal data ECHS
int StoreEcalEchs(DBpointer_t dbpointer, struct ECHS* echs, int n){

  (*dbpointer)<<"##Start_bank_ECHS "<<endl;
  for(int i=0; i<n; i++, echs++){
    (*dbpointer)<<echs->peds<<endl;
    for(int j=0; j<6; j++)
      (*dbpointer)<<echs->pcal[j]<<" ";
    (*dbpointer)<<endl;
    for(int k=0; k<6; k++)
      (*dbpointer)<<echs->ecal[k]<<" ";
    (*dbpointer)<<endl;
    (*dbpointer)<<echs->thr1<<" "<<echs->thr2<<" "
		<<echs->mip<<" "<<echs->vmip<<" "
		<<echs->chst<<endl;
    (*dbpointer)<<echs->fdid<<" "<<echs->chan<<" "
		<<(echs->ecbo).index()<<endl;
  }
  (*dbpointer)<<"##end_bank_ECHS "<<endl;
  return 0;
}

//-- Read ECHS data
int FetchEcalEchs(DBpointer_t dbpointer, struct ECHS* echs, int n){

  int iecbo =0;
  char string[50];
  (*dbpointer)>>string;
  for(int i=0; i<n; i++, echs++){
    (*dbpointer)>>echs->peds;
    for(int j=0; j<6; j++)
      (*dbpointer)>>echs->pcal[j];
    for(int k=0; k<6; k++)
      (*dbpointer)>>echs->ecal[k];
    (*dbpointer)>>echs->thr1>>echs->thr2
		>>echs->mip>>echs->vmip
		>>echs->chst;
    (*dbpointer)>>echs->fdid>>echs->chan
		>>iecbo;
    echs->ecbo = iecbo;
  }
  (*dbpointer)>>string;
  return 0;
}


//-- Store ecal data ECHF
int StoreEcalEchf(DBpointer_t dbpointer, struct ECHF* echf, int n){

  (*dbpointer)<<"##Start_bank_ECHF "<<endl;
  for(int i=0; i<n; i++, echf++){
    (*dbpointer)<<echf->stat<<" "<<echf->pedf<<" "
		<<echf->wid<<" "<<echf->q<<endl;
  }
  (*dbpointer)<<"##end_bank_ECHF "<<endl;
  return 0;
}

//-- Read ECHF data
int FetchEcalEchf(DBpointer_t dbpointer, struct ECHF* echf, int n){

  char string[50];
  (*dbpointer)>>string;
  for(int i=0; i<n; i++, echf++){
    (*dbpointer)>>echf->stat>>echf->pedf
		>>echf->wid>>echf->q;
  }
  (*dbpointer)>>string;
  return 0;
}

#endif
