//                              -*- Mode: C++ -*-
//: ESCTCValley.C
//
//
// Author: Mauro Villa   
// ECAL group 
// HERA-B experiment, Desy
// e-mail: villa@bo.infn.it
//
// Created 18/6/97
// Last Update ........
//

#include "care/ESCTCValley.hh"
#include "arte/ArteMessage.hh"
#include "care/EMCluster.hh"

/*
CLASS
  ESCTCValley
  Concrete class to evaluate the Cluster-To-Cluster distance 
  with the help of the Valley algorithm

*/


//////////////////////////////// Static data definition

// minimum allowed value for a valley
Energy_t ESCTCValley::valleyEnergy = 0.200; // GeV 


/////////  ESCTCValley Static member function Definitions /////////////
//--
//  Static function to get access to Cut offs. First parameter is 
//  the minimum energy a peak cell must have in order to label a cluster
//  and the second is the minimum valley depth between two adjacent
//  clusters in order to label the second one.
void  ESCTCValley::setMergingCutOff(Energy_t valleyEner){
  valleyEnergy = valleyEner;
}

//--
//  Static function to print cut off
void ESCTCValley::dumpMergingCutOff(){
  arteMsg<<" Merging Cut Off  Valley energy: "<<valleyEnergy<<"  GeV ";
  arteMsg.allp();
}




/////////////////////////////////////////////////////
ESCTCValley::ESCTCValley() : ESCTCDisStrat(), mainClus(0.0), boundCell(0.0) {}
ESCTCValley::~ESCTCValley(){}

 
//-- Behavioural operations
//   set the reference cluster to which distances must be evaluated
void ESCTCValley::refCluster(EMCluster* clus){
  //--
  // Store the energy of the reference cluster
  mainClus = clus->maxEnergy();
  // find the first merging cell and get its energy
  EMClusterMember* childMerge = clus->next();
  for(; childMerge!=0 && !childMerge->isALeaf() ;  
      childMerge = childMerge->next());
  if( childMerge!=0 ){
    boundCell = childMerge->maxEnergy(); 
  } else {
    boundCell = 0.0;
  }
}

//--
//   evaluate the position for a cluster
ChiSquare_t ESCTCValley::evaluateDis(EMCluster* clus){
  // get the peak energy of this cluster and evaluate the distance
  Energy_t secEner = clus->maxEnergy();
  //
  dist = (secEner-boundCell);
  return dist;
}


//--
/// test if the clusters are different
bool ESCTCValley::areDifferent() const {
  return dist>valleyEnergy;
}
