//                              -*- Mode: C++ -*-
//: CmdCareECDB.hh
//
//
// Author: Mauro Villa   
// ECAL group 
// HERA-B experiment, Desy
// e-mail: villa@bo.infn.it
//
// Created 1/4/98
// Last Update ........
//

#ifndef CMDCAREECDB_HH_
#define CMDCAREECDB_HH_

#include "care/CmdBase.hh" 


/*
CLASS
  CmdCareECDB
  Class to deal with the care/ECDB kuip command
  Define the name of the ASCII file containing the calibration DB

*/

class CmdCareECDB : public CmdBase {
public:
  //-- Constructors
  CmdCareECDB();
  //-- Destructor
  // Destructor: currently void
  virtual ~CmdCareECDB();
  
  //-- Behavioural operations
  //   Reset stored data to default values
  virtual void defValues();
  //--
  //   Execute stored command
  virtual void execute();

# ifdef USEKUIP
  //--
  //   Read acommand from external
  virtual void read();
# endif
  //--
  //   Dump internal data
  virtual void dump();
  //--
  //   Static access member function
  char* calibfile();


  //--
  // Read the database
  void readDB();
  //--
  // Write the database
  void writeDB();

  void setDB(char *nam, int imajor, int iminor);

  //-- 
  //  Get functions
  inline int Major() const;
  inline int Minor() const;

private:

  ///////////////////// Data Part /////////////////////////////////
  //--
  static  char filename[256];
  enum  {none, dbread, dbwrite} action;
  int major;
  int minor;

};

int CmdCareECDB::Major() const {
    return major;
}

int CmdCareECDB::Minor() const {
    return minor;
}



#endif  //   CMDCAREECDB_HH_

