//                              -*- Mode: C++ -*-
//: ECClHyp.hh
//
//
// Author: Mauro Villa   
// ECAL group 
// HERA-B experiment, Desy
// e-mail: villa@bo.infn.it
//
// Created 3/4/97
// Last Update ........
//

#ifndef ECCLHYP_HH_
#define ECCLHYP_HH_

#include "care/ECALtypes.hh"

/**
class ECClHyp.
  Cluster Hypotheses

*/

class ECClHyp {
public:
  /**@name Constructors and Destructors */
  //@{
  /** Constructors.
     The construction of EMCluster is done referring to some
     parent. The depth level is adjusted accordingly. */
  inline ECClHyp(Hyp_t nhyp, Hyp_t ntrack, Hyp_t nneutr, Hyp_t ncharg);
  
  ///  The default constructor builds an empty list. 
  inline ECClHyp();

  /// Destructor.
  inline ~ECClHyp();
  //@}

  /**@name Beheavioral methods */
  //@{
  /// Reset all info in the cluster.
  inline void  reset();
  /// copy operator
  inline const ECClHyp & operator = (const ECClHyp & hypot);
  /// += operator
  inline const ECClHyp & operator += (const ECClHyp & hypot);
  //--
  ///  Dump for debugging.
  void dump() const;

  //--
  /// Set the number of hypothetical particle that gave origin to this cluster.
  inline void setHypotheses(Hyp_t num);
  //--
  ///  Set the number of linked tracks.
  inline void setTracks(Hyp_t num);
  //--
  ///  Set the number of Neutral Hypotheses.
  inline void setNeuHyp(Hyp_t num);
  //--
  /// Set the number of Charged Hypotheses.
  inline void setChargHyp(Hyp_t num);
  //--
  ///  Increase the number of linked tracks.
  ///Hyp_t newTrack() { return ++numTracks;}

  //@}

  /**@name Status set/access functions */
  //@{
  //--
  ///  Get the number of hypotheses = (showers+charged particles).
  inline Hyp_t getHypotheses() const;
  //--
  ///  Return the number of linked tracks.
  inline Hyp_t getTracks() const;
  //--
  ///  Get the number of Neutral Hypotheses.
  inline Hyp_t getNeuHyp() const;
  //--
  ///  Get the number of Charged Hypotheses.
  inline Hyp_t getChargHyp() const;
  //@}

private:
  //--
  ///  Number of particle hypotheses in this cluster and below it. 
  Hyp_t numHypot;
  //--
  ///  Number of stored extrapolated track points.
  Hyp_t numTracks;
  //--
  ///  Number of Neutral Hypotheses.
  Hyp_t numNeutr;
  //--
  ///  Number of Charged Hypotheses.
  Hyp_t numCharg;
};

#include "care/ECClHyp.icc"
#endif // ECCLHYP_HH_




