//                              -*- Mode: C++ -*-
//: EDClusterModifier.hh
//
//
// Author: Mauro Villa   
// ECAL group 
// HERA-B experiment, Desy
// e-mail: villa@bo.infn.it
//
// Created 4/4/97
// Last Update ........
//

#ifndef EDCLUSTERMODIFIER_HH_
#define EDCLUSTERMODIFIER_HH_

#include "care/ECALtypes.hh"
class  EMClusterMember;
class  EMHitCell;
class  EMCluster;
class  EMCellFraction;
class  EMTrackOnEcal;
class  EMClusterTree;


/**
class EDClusterModifier.
  Abstract base class for cluster analysis.
  It is very similar to the EAClusterAnalyser, but this class has
  the rights to modify the objects on which it operates.
  see EMClusterMember, EMClusterMember, EMHitCell, EMTrackOnEcal  
*/


class EDClusterModifier {
public:
  //-- Constructors
  /// Only default constructor is provided
  EDClusterModifier();
  /// The virtual default destructor
  virtual ~EDClusterModifier();

  /**@name Beheavioral methods */
  //@{ 
  /// EMClusterMember Modifier
  virtual void doModify(EMClusterMember * clsMem) = 0;
  //--
  /// EMHitCell Modifier
  virtual void doModify(EMHitCell * hitT) = 0;
  //--
  /// EMCellFraction Modifier
  virtual void doModify(EMCellFraction * hitF);
  //--
  /// EMCluster Modifier (no loop on childs)
  virtual void doModify(EMCluster * clus);
  //--
  /// EMClusterTree Modifier (no loop on childs)
  virtual void doModify(EMClusterTree * tree);
  //--
  /// EMTrackOnEcal Modifier
  virtual void doModify(EMTrackOnEcal * track);
  //@}
private:
};


#endif // EDCLUSTERMODIFIER_HH_
