//                              -*- Mode: C++ -*-
//: ESCTCValley.hh
//
//
// Author: Mauro Villa   
// ECAL group 
// HERA-B experiment, Desy
// e-mail: villa@bo.infn.it
//
// Created 12/6/97
// Last Update ........
//

#ifndef ESCTCVALLEY_HH_
#define ESCTCVALLEY_HH_

#include "care/ESCTCDisStrat.hh"

/**
class  ESCTCValley.
  Concrete class to evaluate the Cluster-To-Cluster distance 
  with the help of the Valley algorithm
*/

class ESCTCValley : public ESCTCDisStrat {
public:
 
  /// Constructor
  ESCTCValley();
  //-- 
  /// Destructor: currently void but needs to be virtual
  virtual ~ESCTCValley();

  /**@name Beheavioral methods */
  //@{ 
  ///   set the reference cluster to which distances must be evaluated
  virtual void refCluster(EMCluster* clus);
  //--
  ///   evaluate the position for a cluster
  virtual float evaluateDis(EMCluster* clus); 

  /**
    Static function to get access to Cut offs. First parameter is 
    the minimum energy a peak cell must have in order to label a cluster
    and the second is the minimum valley depth between two adjacent
    clusters in order to label the second one. */
  static void setMergingCutOff(Energy_t valleyEner);
  //@}
  
  //   Access functions 
  //--
  /// test if the clusters areDifferent
  virtual bool areDifferent() const;
  
  //--
  ///  Static function to print cut offs
  static void dumpMergingCutOff();

protected:
  // /////////////////// Data Part /////////////////////////////////
  //--
  /// Energy of the main cluster
  Energy_t mainClus;
  //--
  /// Energy of the boundary region cell
  Energy_t boundCell;
  //--
  ///  Threshold energy for the merging of clusters
  static Energy_t valleyEnergy;
};

#endif  //   ESCTCVALLEY_HH_
