      SUBROUTINE DGEVNT

************************************************************************
*
* DGEVNT
* written by Siegmund Nowak 08/93
*
* This routine transforms the GEANT output hits (MIMP) for all detectors
* into the digit and hit structure  foreseen in the reconstruction
* program either for all detectors (if LGEOM is not set by data cards),
* for all silicon superlayers before the magnet (LGEOM='SIDE'),
* for all chambers in the magnetic field (LGEOM = 'MAGN'),
* for the chambers in the magnetic free room  (LGEOM = 'PATT'),
* and for first level trigger detectors (LGEOM = 'FLTR')
* or for any combination of superlayers (LGEOM(i) = superlayer name).
* Calls will be to DGSIL (Silicon), DGOTR(honeycomb), DGMSGC (strip
* chambers), DGCAL (calorimeter) and DGMUOO and DGMUOI (muon chambers)
* To do this Zebra banks for digits and hits and for several relations
* between detector planes, MIMPs, digits hits and track numbers are created.
************************************************************************

      IMPLICIT NONE

      INTEGER ISLTYP,IPLAT,I,J,IGEOMV,JBIT
#include "arte/Arte.fi"
#include "arte/ArteFlag.fi"
#include "auto/GEO.fi"
#include "auto/MC.fi"
#include "gean/ZGESL.fi"
#include "digi/ZUTSEL.fi"
#include "digi/dglev.fi"

#include "arte/Arte.ff"
#include "auto/GEO.ff"
#include "auto/MC.ff"

      IF(JMIMP.LE.0 .AND. JMCAL.LE.0 ) GO TO 999

C     Define random generator numbers for digitisation

      IF(IMCIN_RDIG(1,1).EQ.0 .OR. IMCIN_RDIG(2,1).EQ.0) THEN
         CALL ALRINI
         IQ(IQMCIN_RDIG(1,1)) = LUNDSEED(1)
         IQ(IQMCIN_RDIG(2,1)) = LUNDSEED(2)
      ENDIF
      CALL GRNDMQ(IMCIN_RDIG(1,1),IMCIN_RDIG(2,1),1,'S')

c--   define GEOM version number

      IGEOMV = INT(FGEHD_GEOM(1))

c--  loop over all  superlayers


      DO 300 I = 1,NGESL()

      IF(NGSEL.NE.0) THEN
        DO J=1,NGSEL
        IF(HGESL_NAME(I).EQ.HGSEL(J)) GO TO 301
        ENDDO
        GO TO 300
      ENDIF

 301  IF(NMIMP().LE.0) GO TO 250
      ISLTYP = IGESL_CMP(I)

      IPLAT = IGESL_FPL(I) - 1

 200  IPLAT = IPLAT + 1
      IF(IPLAT.GE.IGESL_FPL(I)+IGESL_NPL(I)) GO TO 250

C*    elementary detector typ is Silicon
      IF(ISLTYP.EQ.LGESL_SIL) THEN
         CALL DGSIL(IPLAT)
          GO TO 200
      ENDIF

C*    elementary detector typ is outer tracker honeycomb chamber
      IF(ISLTYP.EQ.LGESL_WCH) THEN
            IF(IDGLEV(LGESL_WCH).EQ.0) THEN
               CALL DGO96(IPLAT)
         ELSEIF(IDGLEV(LGESL_WCH).EQ.1) THEN
               CALL DGOTR(IPLAT)
         ENDIF
         GO TO 200
      ENDIF

C*    elementary detector typ is inner tracker MSGC
      IF(ISLTYP.EQ.LGESL_MSG) THEN
         IF(IDGLEV(LGESL_MSG) .EQ. 0) THEN 
            CALL DGMSGC(IPLAT)
         ELSE IF (IDGLEV(LGESL_MSG) .EQ. 1) THEN 
            CALL DGITR(IPLAT)
         ENDIF 
         GO TO 200
      ENDIF

C*    elementary detector typ is PTC
      IF(ISLTYP.EQ.LGESL_PTC) THEN
         CALL DGPTL(IPLAT)
         GO TO 200
      ENDIF

C*    elementary detector typ is RICH
      IF(ISLTYP.EQ.LGESL_RIC) THEN
         IF(IPLAT .EQ.IGESL_FPL(I)) CALL DGRICH(IPLAT)
         GO TO 200
      ENDIF

C*    elementary detector typ is TRD
      IF(ISLTYP.EQ.LGESL_TRD) THEN
         CALL DGTRDS(IPLAT)
         GO TO 200
      ENDIF

C*    elementary detector typ is muon chamber
      IF(ISLTYP.EQ.LGESL_MUO) THEN
         CALL DGMUOO(IPLAT)
         GO TO 200
      ENDIF

      IF(ISLTYP.EQ.LGESL_MUX) THEN
         CALL DGMUOI(IPLAT)
         GO TO 200
      ENDIF

  250 CONTINUE

C*    elementary detector typ is calorimeter
      IF(ISLTYP.EQ.LGESL_ECA) CALL DGCAL(I)

 300  CONTINUE

      CALL MMFREE(JDIGB)
      CALL MMFREE(JHITB)
      CALL MMFREE(JHITC)
      CALL MMFREE(JHIDI)
      CALL MMFREE(JMIHI)
      CALL MMFREE(JMCHC)
      CALL MMFREE(JMIDI)

 999  RETURN
      END

