/* 
======================================================================== 
= DGITR                                                               =
=                                                                      =
= This routines translates GEANT output MIMP into DIGB and HITB banks  =
= called from DGEVNT                                                   =
=                                                                      =
=  itr.C : MSGC - Monte Carlo Program                                  = 
=  ----------------------------------                                  =  
=       last update : 31.03.99                                         = 
=       author      : Siegmund Nowak 08/93                             =
=                     A. Lange                                         = 
======================================================================== 
*/ 
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <math.h>
#include "arte/arte.hh"
#include "arte/ArteUtil.h"
#include "arte/ArteMessage.hh"
#include "gean/ZGESL.h"
#include "digi/msgc.h"
#include "digi/cr_HepRot.hh"
#include "digi/zgdig.hh"
#include <vector>
#include <algorithm>
extern "C" 
{  void dgitr_(int &);
   void grndm_(float*,int*);
   void granor_();
   void rnpssn_(float*, int*, int*);
   int  rnorml_(float*, int*);
}

Cluster clust;
Wire    wire;
vector<int> wire2mimp[mxwire];
vector<int> wire2mtra[mxwire];

static int first_dgitr =0,
       evt =0;
static int   e_flag;

static float vtdrift,vbdrift,ethr,mgain,s_mgain, ggain, s_ggain,
             threshold,cutoff,noise,r_time,rclust; 
static float z_bottom,sig_top,sig_bottom;

double epulse  [p_length]; 

#define ditr_max 400
int ditr[ditr_max][4];

FILE *fp;
    
///////////////////////////////////////////////////////////////////// 
///                 function declarations                         /// 
///////////////////////////////////////////////////////////////////// 

int      dedx(); 
int      diffuse(float);
int      gasgain(float);
int      gen_cluster(float*);
float    gen_position(float,float,float,float,float,float,int,int,int); 
int      get_digi(float*,int);
void     get_hit(float*, int, vector<Hit> *,float, float, float,float);
float    get_sign(float,float);
void     mimptst(int, int, int, float, float, float, float,float);
double   polya(int);
long     pulse(float, float);
void     readfilep_(int); 
void     set_epulse_();
float    tod(float);
int      wire_no(float, float, float);
float    wire_x(int, float, float);

/*===================================================================*/
void dgitr_(int &ipl) 
/*===================================================================*/
{     int   i,j;
      int   ok; 
      int   nmim, ifmimp, ncluster, nmim_f, nhit;
      float delx, dely, xleng;
      float tgx,tgy;
      float tgx_max=-999.;
      float s_offset,charge,dz;
      float x_detector[3], xv[3], xh[3],xm[3];
      vector<Hit> *dtrk=NULL; 

//---  initialisation routines at first access -------------------------------------------
      if(first_dgitr == 0) 
      { first_dgitr =1;    
        readfilep_(0);
        set_epulse_();
      }
      evt++;

//---  initialisation -------------------------------------------------------------------
//    get MIMP information (first MIMP = ifmimp,number of MIMPs nmim) 

      zgdig ZGDIG(ipl);

      nmim = ZGDIG.Get_nmimp(ifmimp);
      if(nmim <= 0)
      {   ZGDIG.Set_hit_for_nomimp();    
          return;
      }
      else
	nmim_f = nmim;

//    get information about the given plane 
      ArtePointer<GEDE> gede(ipl);
      float pitch  = gede->pit,
            zwir   = gede->zfw[0];         
      float gasgap = gede->zmax - gede->zmin;
            z_bottom = gede->zfw[1] - gede->zfw[0];

      cr_HepRot rm;       rm.set_HepRot(ipl);
      unsigned int lbigr=0; if(fabs(gede->xmax) <= fabs(gede->xmin)) lbigr=1;

      for(i=0;i<mxwire;i++)
      { wire.iwir[i]  =  0;
        wire.cwir[i]  =  0.;
        wire.digno[i] = -1;
        if(wire2mimp[i].size())
        wire2mimp[i].erase(wire2mimp[i].begin(), wire2mimp[i].end());
        if(wire2mtra[i].size())
        wire2mtra[i].erase(wire2mtra[i].begin(), wire2mtra[i].end());
      }


//-- loop over all  mimps ------------------------------------------------------------
//     loop over all  mimps - check whether the mimps are really inside the 
//     detector -  rotate mimp coordinates(front and back side) - find region of
//     interest in wire numbers - generate ions and diffuse them 

      //    ms_trace tr;

    for(i = ifmimp; i < ifmimp+nmim; i++) 
    { ncluster = -99;
//      int i0 = i - ifmimp;

      for(j=0;j<mxwire;j++) wire.dwir[j] = 0.;

//    definition of front and backside positions and rotation
      ArtePointer<MIMP> mimp(i);
      float deltaz = zwir-mimp->z;  
 
      x_detector[0] = mimp->x + mimp->tx*deltaz;          
      x_detector[1] = mimp->y + mimp->ty*deltaz;
      x_detector[2] = 0.;
      rm.rotate_Z(x_detector,xv);

      x_detector[0] = mimp->x +  mimp->tx*(deltaz+gasgap);
      x_detector[1] = mimp->y +  mimp->ty*(deltaz+gasgap);
      x_detector[2] = gasgap;
      rm.rotate_Z(x_detector,xh);

      for(j=0;j<=2;j++) xm[j] = (xh[j]+xv[j]) / 2.;

      delx = xh[0] - xv[0];
      dely = xh[1] - xv[1];

      xleng = sqrt(delx*delx + dely*dely + gasgap*gasgap);

      tgx = delx/gasgap;
      tgy = dely/gasgap;


//    check whether in effective volume
      if((xv[0]<gede->sxmn && xh[0]<gede->sxmn) || (xv[0]>gede->sxmx && xh[0]>gede->sxmx)) 
      { nmim_f--;
        continue;
      }
      if((xv[1]<gede->symn && xh[1]<gede->symn) || (xv[1]>gede->symx && xh[1]>gede->symx)) 
      { nmim_f--;
        continue;
      }

      int mtrano = mimp->mtra.index();


//    generation of ions and diffusion 
      ncluster = gen_cluster(&xleng);
      j=0;
      while(j<ncluster)
      { ok = dedx();

        if(ok) 
        { dz = gen_position(xv[0],xv[1],zwir,tgx,tgy,gasgap,i,mtrano,ipl);
          if(dz >= 0) 
          { gasgain(dz);
            diffuse(dz);
            charge = get_sign(gede->sxmn,pitch);
            j++;
          }
          else 
          { if(dz != -999.) printf("\n gen_position: Error in z Position of cluster (dz=%f)\n",dz);
         ncluster--;
          }
        }
        else    ncluster--;
      }

      if(tgx_max < fabs(tgx)) tgx_max = fabs(tgx);
        
      if(ncluster <= 0) 
      { nmim_f--;
      }
    }    /* --- end of loop over MIMPs --- */

    if(nmim_f <= 0) 
    {  ZGDIG.Set_hit_for_nomimp();
       return;
    }

//--- calculate number of digits ---------------------------------------------
    int  ndigde = get_digi(&s_offset,ipl);


    if(ndigde<=0) 
    {  ZGDIG.Set_hit_for_nomimp();
       return;
    }
  
//  fill DIGB bank and relations 
    struct DIGB digb;
    int iflag = 0, ndigb=0, mimpno;

    for(i=0;i<mxwire;i++)
    { if(wire.dwir[i] <= 0.) continue;

      digb.id = 10000*i + ipl;   
      digb.q  = wire.cwir[i];
      ArtePointer<DIGB>::append(digb); 

      ndigb = ArtePointer<DIGB>::nrow();

      wire.digno[i] = ndigb;

      if(iflag == 0)  { gede->dig1 = ndigb;  iflag = 1; } 

      vector<int>::iterator j;
      for (j=wire2mimp[i].begin(); j!=wire2mimp[i].end(); j++)
      { mimpno = *j;
        createRelation(
          ArtePointer<MIMP>(*j),ArtePointer<DIGB>(ndigb),&MIMP::digb);
      }
    }

//  fill gede table for digits
    gede->digl = ndigb;

//    fill_ditr(ipl,wire.cwir,gede->nwi[0],gede->sec,gede->lay);

//--- create hits ------------------------------------------------------------------ 
    if(dtrk == NULL ) dtrk = new vector<Hit>;
    get_hit(wire.cwir,mxwire,dtrk,threshold,pitch,gede->sxmn,s_offset);
    nhit = dtrk->size(); 
      if(nhit == 0) {
       printf("nhit = 0 \n");  
       return;
	    }
    struct HITB hitb;
    int nhitb;

    nhitb = ArtePointer<HITB>::nrow() + 1;
    gede->hit1=nhitb;

    vector<Hit>::iterator mh; 
    for (i=0;i<nhit;i++) { 
      mh = dtrk->begin() + i;
      hitb.gede = ipl;   
      hitb.z = zwir + gasgap/2.;    
      hitb.q = mh->charge;
      hitb.r = gede->acc;
      hitb.x = mh->center;

      ArtePointer<HITB>::append(hitb);
      nhitb = ArtePointer<HITB>::nrow();

      for(int k = ifmimp; k < ifmimp+nmim; k++) {
         if(mh->hit2mimp != NULL) {
            vector<int> *ph2m = mh->hit2mimp;
            if(find(ph2m->begin(),ph2m->end(),k) != ph2m->end()) createRelation(
		  ArtePointer<MIMP>(k),ArtePointer<HITB>(nhitb),&MIMP::hitb);
	 }
       }
      for(int k = mh->w_n0; k <= mh->w_n1; k++)   if(k >=0 ) 
         createRelation(
         ArtePointer<HITB>(nhitb),ArtePointer<DIGB>(wire.digno[k]),&HITB::digb);

      if(mh->hit2mimp != NULL) {
          delete mh->hit2mimp;
          mh->hit2mimp = NULL; 
       }
    }
    if(dtrk != NULL ) {
       delete dtrk;
       dtrk = NULL;
    }      
    gede->hitl = nhitb;

 }

/*===================================================================*/
void set_epulse_()
/*------------------------
  purpose  : fills array 'epulse' with the MSGC response of one
             electron.
  input    : flag:  0 response of MSGC in 'epulse' flat (for test purposes)
                   >0 'epulse' with the MSGC response of one electron
====================================================================*/
{ int  n;
  double t;

  extern int e_flag;

  if(e_flag) 
  { epulse[0] = u0a + u0b + u0c;
    for(n=1;n < p_length;n++)
    { t= (double) n;       // 1 nsec. per step

      epulse[n] = epulse[n-1] + (u0a*exp(-t/taua)+u0b*exp(-t/taub)+u0c*exp(-t/tauc));
  } }
  else
    for(n=0;n < p_length;n++)  epulse[n] = 1;
}

/*===================================================================*/ 
void readfilep_(int mod) 
/*----------------------- 
  purpose: reads parameter file 
 ===================================================================*/ 
{ const int  buflen=255; 
  char buffer[buflen],*ok; 
  FILE *fp,*utfilp(const char*,const char*); 
 
  fp=utfilp("ITR/msgc.dat",""); 
  if(fp==NULL) { arteMsg << " Error on opening parameter file"; arteMsg.error(); }

  do
  { ok=fgets(buffer,buflen,fp);
  } while(buffer[0]-'0' != mod); 

  switch(mod)
  { case 0: 
            fscanf(fp,"%f",&vtdrift);     fgets(buffer,buflen,fp);  printf("drift velocity (above GEM) : %8.3f \n",vtdrift); 
            fscanf(fp,"%f",&vbdrift);     fgets(buffer,buflen,fp);  printf("drift velocity (below GEM) : %8.3f \n",vbdrift);
            fscanf(fp,"%f",&ethr);        fgets(buffer,buflen,fp); 
            fscanf(fp,"%f",&mgain);       fgets(buffer,buflen,fp); 
            fscanf(fp,"%f",&s_mgain);     fgets(buffer,buflen,fp);  printf("MSGC gain                  : %8.3f +/- %8.3f\n",mgain,s_mgain);  
            fscanf(fp,"%f",&ggain);       fgets(buffer,buflen,fp); 
            fscanf(fp,"%f",&s_ggain);     fgets(buffer,buflen,fp);  printf("GEM  gain                  : %8.3f +/- %8.3f\n",ggain,s_ggain);
            fscanf(fp,"%f",&rclust);      fgets(buffer,buflen,fp);  printf("Cluster per cm             : %8.3f \n",rclust); 
            fscanf(fp,"%f",&cutoff);      fgets(buffer,buflen,fp);  printf("e cutoff                   : %8.3e \n",cutoff);
            fscanf(fp,"%f",&sig_top);     fgets(buffer,buflen,fp);  printf("diff. coeff.   (above GEM) : %8.3f \n",sig_top);
            fscanf(fp,"%f",&sig_bottom);  fgets(buffer,buflen,fp);  printf("diff. coeff.   (below GEM) : %8.3f \n",sig_bottom);
            fscanf(fp,"%f",&threshold);   fgets(buffer,buflen,fp);  printf("threshold                  : %8.3f \n",threshold);
            fscanf(fp,"%f",&noise);       fgets(buffer,buflen,fp);  printf("noise                      : %8.3f \n",noise);
            fscanf(fp,"%d",&e_flag);      fgets(buffer,buflen,fp);  printf("e flag                     : %4d   \n",e_flag);
            fscanf(fp,"%f",&r_time);      fgets(buffer,buflen,fp);  printf("readout time               : %8.3f \n",r_time);

            break; 
 
    case 1: break; 
 
  } 
  fclose(fp); 
} 

/*===================================================================*/
int gen_cluster(float* len) 
/*----------------------------------------------------------------
  purpose  : generates no of cluster
 ===================================================================*/
{ int   n,nclustp;
  int   err;

//--- generate no of cluster ----------------------------------
  if(rclust > 0.)         { rnpssn_(&rclust,&n,&err);  nclustp = nint(n * *len); }
  else if(rclust < 0.)                                 nclustp = nint(-rclust * *len);
  else                                                 nclustp = 1;

  return(nclustp);
}


/*===================================================================*/
int dedx() 
/*----------------------------------------------------------------
  purpose  : generates energy loss and no of electrons 
 ===================================================================*/
{ int   one = 1;
  float rndm, ampl;

  if(rclust > 0.)
  { grndm_(&rndm,&one);
    ampl=ethr/(1-rndm);
    clust.nprim = nint(ampl/ethr);
    clust.ntot  = nint(clust.nprim * polya(clust.nprim));
  }
  else
    clust.nprim = clust.ntot  = 1;

  if(clust.ntot == 0)   return(0);
  else                  return(1);
}

/*===================================================================*/
float gen_position(float x0, float y0,float z0,float tgx,float tgy,float dz, 
           int mimpno, int mtrano,int ipl) 
/*----------------------------------------------------------------
  purpose  : generates position of electrons
 ===================================================================*/
{ int   one = 1;
  float rndm;
  float del_z;

  ArtePointer<GEDE> gede(ipl);

  grndm_(&rndm,&one);

  del_z   = rndm * dz;
  clust.z = z0 + del_z;
  clust.x = x0 + del_z* tgx;
  clust.y = y0 + del_z* tgy;
  clust.t = tod(del_z);

  //check whether in effective volume
  if(clust.x < gede->sxmn || clust.x > gede->sxmx)         return(-999.);
  if(clust.y < gede->symn || clust.y > gede->symx)         return(-999.);

  clust.mimp = mimpno;
  clust.mtra = mtrano;

  return(del_z);    
}

/*===================================================================*/
float tod(float dz)
/*    ------------------------
   time of drift: this function determines the drift time of a cluster.
                  A linear drift model is used in this version of the routine.
  ===================================================================*/
{ float dtime;

  extern float  vtdrift,vbdrift;
  
  if (dz > z_bottom)
    dtime = (dz-z_bottom) / (vtdrift/10000.)  +  z_bottom / (vbdrift/10000.);
  else
    dtime = dz / (vbdrift/10000.);

  return(dtime);
}


/*===================================================================*/
int gasgain(float dz)
/*===================================================================*/
{ float rnd[2];
  int   two=2;

  extern float  mgain, ggain, s_mgain,s_ggain;
  
  rnorml_(&rnd[0],&two);
  
  clust.gampl=(mgain + rnd[1] * s_mgain) * clust.ntot;

  if(dz > z_bottom) clust.gampl = (ggain + s_ggain*rnd[0]) * clust.gampl;

  return(1);
}


/*===================================================================*/
int diffuse(float dz)
/*===================================================================*/
{ float sigm;

  float rnd[2];
  int   two=2;

  extern float  sig_top,sig_bottom;
  
  rnorml_(&rnd[0],&two);
  
  if(sig_top == 0. && sig_bottom == 0.)  
    clust.diff_x = 0.;
  else
  { if(dz > z_bottom) 
      sigm = (sig_top    * rnd[0]) * sqrt(z_bottom)     + 
             (sig_bottom * rnd[1]) * sqrt(dz-z_bottom);
    else
      sigm = (sig_bottom * rnd[1]) * sqrt(dz);

    clust.diff_x = sigm/10000.;
  } 

  return(1);
}

/*===================================================================*/
double polya(int nprim)
/*---------------------
  purpose: random generator producing polya distribution for
	   nprim (integer) input
=====================================================================*/
{ const float cut=0.20;

  int    i;
  int    one=1;
  float  rndm;
  double nue,nuemean=0,
	 randx1;
 
  if(nprim == 0) return(nprim);
  
  for(i=1;i<=nprim;i++)
  { grndm_(&rndm,&one);
    randx1 = rndm;

    nue = (randx1<cut) ? 0.7587*exp(0.6667*log(1.5*randx1))
			       : -1.0*log(1-randx1)+0.1169         ;
    nuemean += nue;
  }
  return(nuemean/nprim);
}

/*===================================================================*/
int wire_no(float x,float x0,float pitch)
/*===================================================================*/
{   return(nint((x - x0) / pitch));    
}

/*===================================================================*/
float wire_x(int n,float x0,float pitch)
/*===================================================================*/
{   return(n * pitch + x0);    
}

/*===================================================================*/
float get_sign(float x0,float pitch)
/*===================================================================*/
{ int   nx,n;
  float x,rt,pz=-1.;

  extern float r_time;

  nx = wire_no(clust.x + clust.diff_x,x0,pitch);

  if(nx < 0 && nx > -wire_off)
  { printf("get_sign: cluster : wireno. shifted (%4d --> 1)\n",nx);
    nx = 0;
  }

  if(nx >= mxwire && nx < mxwire + wire_off)
  { printf("get_sign: cluster : wireno. shifted (%4d --> 1)\n",nx);
    nx = mxwire-1;
  }

  if(nx < 0 || nx >= mxwire) 
  { printf(" get_sign: cluster lost: wireno. = %4d  ",nx );
    printf(" (without diff. =%4d) \n",wire_no(clust.x,x0,pitch));
              x = 0;
    if(nx < 0) pz = -0.01;
    else       pz = -0.02;
  } 
  else
  { rt = r_time - clust.t;
    n  = nint(rt);

    if(rt < 0.) 
    { printf("get_sign: sampling time wrong (%f)\n",r_time);
      pz = -0.03;
    }
    else
      pz = (rt >= p_length) ? 1. : epulse[n];

    wire.iwir[nx]++;
    wire.cwir[nx] += clust.gampl*pz;
    wire.dwir[nx] += clust.gampl*pz;
    if(find(wire2mimp[nx].begin(),wire2mimp[nx].end(),clust.mimp) 
       == wire2mimp[nx].end())   wire2mimp[nx].push_back(clust.mimp);
    if(find(wire2mtra[nx].begin(),wire2mtra[nx].end(),clust.mtra) 
       == wire2mtra[nx].end())   wire2mtra[nx].push_back(clust.mtra);
  }
  return(pz);

}


/*===================================================================*/
int get_digi(float *p_offs, int ipl)
/*===================================================================*/
{ int   i,n_sum = 0;
  int   rno=mxwire, nsum=0;
  float charge_mx = -1.,
        rnd[mxwire];
  float charge_sum = 0;

  extern float threshold,cutoff,noise;

// fill histograms 'charge without noise'
  for(i=0;i<mxwire;i++)
  { if(charge_mx < wire.cwir[i]) charge_mx = wire.cwir[i];
  }
  charge_mx = -1;

  if(noise > 0.)
  { rnorml_(&rnd[0],&rno);
 
    for(i=0;i<mxwire;i++)
    { wire.cwir[i] += rnd[i] * noise;
      if(wire.cwir[i] < 0.) wire.cwir[i] = 0.;
  } }

  if(cutoff > 0.) 
    for(i=0;i<mxwire;i++)
      if(wire.cwir[i] >= cutoff) wire.cwir[i] = cutoff;

  charge_mx = -1.;
  for(i=0;i<mxwire;i++)
  { if(charge_mx < wire.cwir[i]) charge_mx = wire.cwir[i];

    if(wire.cwir[i] >= threshold)
    { wire.dwir[i] = 1.;  nsum++; }
    else
      wire.dwir[i] = 0;    

    if(wire.cwir[i] < min_(threshold,4*noise)) charge_sum += wire.cwir[i]; n_sum++;
  }


  charge_sum = charge_sum/n_sum;
  *p_offs = charge_sum;

  return(nsum);
}

/*===================================================================*/
int dig_width(float *ptr,int no, float thres)
/*===================================================================*/
{ for(int i=0;i<no;i++)  if(*(ptr+i) < thres) return(i);
  return(no);
}

/*===================================================================*/
int hit_width(float *ptr,int i0, int j, vector<Hit> *dtrk, float x0, float pitch, float offs)
/*===================================================================*/
{ int k,i;
  Hit hit;
  vector<int>::iterator m;

  switch(j)
  { case 0: printf("get_hit Error: Width of hit (wire %d) width \n",i0);  
            return(-1);
  case 1:   memset(&hit,0,sizeof(hit));
            hit.hit2mimp = NULL;

            hit.width  = pitch;
            hit.center = wire_x(i0,x0,pitch);
            hit.charge = *(ptr+i0)-offs;
            hit.w_n0   = i0;
            hit.w_n1   = i0;

            for(m = wire2mimp[i0].begin();m != wire2mimp[i0].end();m++)
            { if( hit.hit2mimp == NULL) {   
                  hit.hit2mimp =  new vector<int>;
                  (hit.hit2mimp)->push_back(*m);
              }  
              else {
                 vector<int> *ph2m = hit.hit2mimp;
                 if(find(ph2m->begin(),ph2m->end(),*m)==ph2m->end())
                                                 ph2m->push_back(*m); 
              } 
            }
            dtrk->push_back(hit);
            break;

    default:memset(&hit,0,sizeof(hit));
            hit.hit2mimp = NULL;
            hit.width  = j * pitch;
            hit.center = wire_x(nint(i0 + (j-1)/2.),x0,pitch);
            for(k=i0;k<=i0+j;k++) hit.charge += *(ptr+k)-offs;
            hit.w_n0 = i0;
            hit.w_n1 = i0+j-1;
            
            for(i=i0;i<i0+j;i++)
	      for(m=wire2mimp[i].begin();m!=wire2mimp[i].end();m++) {
                if(hit.hit2mimp == NULL) {
                    hit.hit2mimp =  new vector<int>;
                     (hit.hit2mimp)->push_back(*m);
		}
                else {    
                    vector<int> *ph2m = hit.hit2mimp;
                    if(find(ph2m->begin(),ph2m->end(),*m)==ph2m->end())
                                                 ph2m->push_back(*m);
		  }
              }
            dtrk->push_back(hit);
            break;
  }

  return(j);
}

//===================================================================
void get_hit(float *ptr, int no, vector<Hit> *dtrk, float thres, 
            float pitch, float x0, float offs)
//===================================================================
{ int   i0,j,j1;

  i0 = 0;
  while(i0 < no)
  { while(*(ptr+i0) <= thres && i0 < no) i0++;
   
    if(i0 >= no) break;

    j = dig_width(ptr+i0,no-i0,thres);

    j1 = hit_width(ptr,i0,j,dtrk,x0,pitch,offs);

    if(j1< 0)   i0++;
    else        i0 += j;

  }
}








