
#ifndef _FA_HH_
#define _FA_HH_

// *****************************************************************************
//
// source:    fa.hh
//
// type:      header file
//
// created:   16 Apr 96
// updated:   14 May 99 Andreas Gellrich
//
// author:    Andreas Gellrich
//            HERA-B Collaboration
//            DESY Zeuthen
//            Platanenallee 6
//            D-15738 Zeuthen
//            Germany
//            email: Andreas.Gellrich@desy.de
//
// changes:   multiple file access (AG: 27-Nov-98)
//            prototype for faDaqLog added (AG: 15-Dec-98)
//            online include path (AG: 11 Jan 99)
//            introduced return code for faDaqLog() (AG: 15 Jan 99) 
//            target raw data include tgts.h (AG: 10 Feb 99)
//            introduce  function faCycle() (AG: 12 Feb 99)
//            include utility protoypes here (AG: 23 Feb 99)
//            polishing (AG: 11 Mar 99)
//            polishing (AG: 6 May 99)
//            no more inline functions (AG: 14 May 99)
//
// *****************************************************************************

// defines
#define _OUT_
#define _ERR_
#define _DEB_

// standard includes
#include <iostream>
#include <strstream>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include <time.h>
#include <unistd.h>
#include <math.h>

// online includes
#include "rpm.h"
#include "log.h"
#include "stage.h"
#include "grr.h"
#include "emg.h"
#include "etb.h"
#include "esp.h"
#include "event.h"
#include "tgts_raw.h"

// Arte includes
#include "arte/Arte.hh" // arte.hh is not useable here!
#include "arte/ArteFlag.h"
#include "auto/EVHD.hh"
#include "auto/DIG.hh"
#include "auto/MEMO.hh" 

// farm includes
#include "farm.hh"

// constants
const int ERR  = -1; // error occurred
const int GOOD =  0; // okay
const int OK   =  0; // okay
const int EOD  =  4; // end-of-data (no more data, next bc=0)
const int END  =  5; // end-of-file (end reached)

const int ARTE_ERR  = -1; // error occurred
const int ARTE_GOOD =  0; // okay
const int ARTE_OK   =  0; // okay
const int ARTE_EOD  =  4; // end-of-data (no more data, next bc=0)
const int ARTE_END  =  5; // end-of-file (end reached)

// C prototpyes
extern "C" int  fainit_();
extern "C" int  fastop_();
extern "C" int  faread_();
extern "C" float fatime_();

// C++ prototypes
void faCatchSIGINT ( int sig );
void faCatchSIGQUIT( int sig );
void faCatchSIGTERM( int sig );

int faLine();

FILE *faOpenFile( char *name );
int  faCloseFile( char *name, FILE *in );
int  faReadFile ();

int faInitEmg();
int faReadEmg();
int faStopEmg();

int faInitGrr();
int faStopGrr();

void faCatchSIGINT ( int sigNum );
void faCatchSIGQUIT( int sigNum );
void faCatchSIGTERM( int sigNum );

void faCycle();

int faDate( time_t *datime );
int faTime( time_t *datime );

void faPrintEvtCount( int evt, int run, int exp );

int faCopySlb( int *oldid, char *src, int *newid, char *dest, int size );

int faName2ID( int name );
int faID2Name( int id );
int faNameIDInit();
int faName2Link( int name );

char *faName( int name );
int  faName2Int( char *name );
char *faName( int name );

int  indexTable( int iLink );
int  intName( int indexTable );

int  numData( int indexTable );
int  numCols( int indexTable );
int  numRows( int indexTable );

int  *ptr2Table( int indexTable );

#endif
