/***********************************************************************
* GEOSTAT
* written by Siegmund Nowak
*
* This routines is the main routine for generating the detector geometry
*
*       called from HGINIT
***********************************************************************/
#include "stdio.h"
#include "string.h"
#include "arte/arte.hh"
#include "arte/ArteFlag.h"
#include "gean/ZGDETV.h"
#include "gean/ZGESL.h"
#include "gean/zgvers.hh"

extern "C" {
   void meallp_(char *,int);
}

void geostat() {
   char vtext[90];  char vline[90]; char vt[5]; char end='\0'; 
   memset(vline,'-',89);    vline[89]= '\0';
   meallp_(vline,88);
   sprintf(vtext,"GEOMETRY VERSION %.4f%c\n",arteflag_.geomvers,end);
   meallp_(vtext,strlen(vtext));
   meallp_("Geometry version numbers for subsystems",39);
   meallp_(
   "SUBDETECTOR   CMP   VERS-Nr   INPUT FILE                         VERSION",73);

   zgvers ZGVERS;
   for (int i=0;  i<32; i++){       
       geovers *vers = ZGVERS.verspointer(i);          
       if(vers->vers_nr >=0) {
         if(zgdetv_.DETV[vers->vers_cmp-1] == 'L') strncpy(vt,"LONG ",5);
         else if(zgdetv_.DETV[vers->vers_cmp-1] == 'S')strncpy(vt,"SHORT",5);
         else if(zgdetv_.DETV[vers->vers_cmp-1] == 'U')strncpy(vt,"DUMMY",5);
         sprintf(vtext,"%.12s   %2d   %7.4f   %-32.32s   %.5s%c\n", 
                vers->name,vers->vers_cmp,vers->vers_nr,vers->datafile,vt,end);
         meallp_(vtext,strlen(vtext));
       }
   }
   meallp_(vline,88);

   meallp_("Superlayer information",22);

   meallp_(
   "SL   cmp mfl    xmin    xmax    ymin    ymax    zmin    zmax    fpl  lpl   fwl   lwl",84);
   meallp_(vline,88);

   ArteTableIterator<NESL> gs;
   for(; gs.valid(); gs.next()){
      sprintf(vtext,
      "%.4s %3hd %3hd  %7.2f %7.2f %7.2f %7.2f %7.2f %7.2f %4d %4d %5d %5d%c\n",
       gs->name,gs->cmp,gs->mfl,gs->x1,gs->x2,gs->y1,gs->y2,gs->z1,gs->z2,
       gs->fpl,gs->fpl+gs->npl-1,gs->fwl,gs->fwl+gs->nwl-1,end); 
       meallp_(vtext,strlen(vtext)); 
       if(gs->cmp == LGESL_MOTH || gs->cmp  ==  LGESL_MAG ) {
          sprintf(vtext,"              %7.2f %7.2f %7.2f %7.2f%c\n",
	       gs->x1-gs->dx, gs->x2+gs->dx, gs->y1-gs->dy, gs->y2+gs->dy,end);
          meallp_(vtext,strlen(vtext));
       }
   }
   meallp_(vline,88);
   strcpy(vtext, 
   "All detectors with zero dimensions are not implemented as volumes");
   meallp_(vtext,strlen(vtext));
   strcpy(vtext, 
   "MOTH and MAGN are trapezoids, first/last rows give the numbers at the front/back side"); 
   meallp_(vtext,strlen(vtext));
   strcpy(vtext,
   "RICH is implemented as tube with an inner/outer radius given in xmin/xmax");
   meallp_(vtext,strlen(vtext));
   meallp_(vline,88);
}
