#include "rich/PhotonDetection.h"
#include "rich/vessel.h"
#include "rich/PhiLambda.h"

void RICHConfigDir(char *configdir) {
  
  char* RICHconfigdir=getenv("RICH_CONFIGDIR");
  if (RICHconfigdir) {
    strcpy(configdir,RICHconfigdir);
    if (configdir[strlen(configdir)-1]!='/') strcat(configdir,"/");
    }
  else {
    char *hb=getenv("HB");
    if (!hb) {
      fprintf(stderr,"RICINI:  $HB is not defined. Abort.\n");
      exit(1);
      }
    strcpy(configdir,hb);
    strcat(configdir,"/geo/RICH/");
    }
  }


extern "C" {

void F77NAME(ricini)() {

  if (RICH_Detector) return;
  char configdir[256];
  RICHConfigDir(&configdir[0]);
  printf(" === RICH configuration directory: %s\n",configdir);

  // -- find magnet status
  for (ArteTable<GESL>::iterator igesl=ArteTable<GESL>::begin();
       igesl!=ArteTable<GESL>::end(); ++igesl) {
    if ((*igesl)->cmp==12) {
      RichRun::magnet=(*igesl)->mfl;
      break;
    }
  } 


  //  -- find rich gede pointer
  for (ArteTable<GEDE>::iterator igede=ArteTable<GEDE>::begin();
       igede!=ArteTable<GEDE>::end(); ++igede) {
    if ((*igede)->cmp==5) {
      RichRun::gede=(*igede);
      break;
    }
  }


  RICH_Detector = new  RICHDetector(configdir);
  if (!RICH_Vessel)    RICH_Vessel=new RICHVessel(configdir);
  if (!RICH_LambdaPhi) RICH_LambdaPhi=new LambdaPhi();

  //cout<<"Redefinition of efficiencies"<<endl;

  //  redefine efficiencies for MIMP reduction case
  Table deff,veff;
  int n=100;  // accuracy of the linear interpolation
  //cout<<"... in RICH_Detector"<<endl;
  RICH_Detector->MaxEfficiency(deff,n,2);
  //cout<<"... in RICH_Vessel"<<endl;
  RICH_Vessel->MaxEfficiency(veff,n,2);
  //cout<<"RICH initialization finished"<<endl;
}



void F77NAME(rgnini)() {

  if (!RICH_Detector) {
    char configdir[256];
    RICHConfigDir(&configdir[0]);
    printf(" === RICH configuration directory: %s\n",configdir);

    RICH_Detector = new  RICHDetector(configdir);
    if (!RICH_Vessel)    RICH_Vessel=new RICHVessel(configdir);
    if (!RICH_LambdaPhi) RICH_LambdaPhi=new LambdaPhi();
  }


  Table eff;
  
//
// stepfunction
//
  /*
  double lmin=280e-9;
  double lmax=650e-9;
  eff.append(lmin*(1-1E-10),0);
  eff.append(lmin          ,1);
  eff.append(lmax          ,1);
  eff.append(lmax*(1+1E-10),0);
  FILE* fp=fopen("MaxEfficiency.dat","w");
  eff0.print(fp);
  */

// fold detection efficiency into generation probability
  
  int n=100;  // set accuracy of the linear interpolation
  Table deff,veff;
  RICH_Detector->MaxEfficiency(deff,n,2);
  RICH_Vessel->MaxEfficiency(veff,n,2);
 
  double lmin=1E37,lmax=0;
  int i;
  for (i=0;i<n;i++) {
    if (deff.y(i)!=0 && deff.x(i)<lmin ) lmin=deff.x(i);
    if (deff.y(i)!=0 && deff.x(i)>lmax ) lmax=deff.x(i);
    if (veff.y(i)!=0 && veff.x(i)<lmin ) lmin=veff.x(i);
    if (veff.y(i)!=0 && veff.x(i)>lmax ) lmax=veff.x(i);
  }
  double dl=(lmax-lmin)/n;
  for (i=0;i<n;i++) {
    double l=lmin+i*dl;
    eff.append(l,deff.f(l)*veff.f(l));
  }
  eff.append(lmax,0);

  RICH_Vessel->Radiator().gsckov(200,eff);
}

  
void F77NAME(ricexi)() {
  if (RICH_Detector) { delete RICH_Detector;  RICH_Detector=0;  }
  if (RICH_Vessel)   { delete RICH_Vessel;    RICH_Vessel=0;    }
  if (RICH_LambdaPhi){ delete RICH_LambdaPhi; RICH_LambdaPhi=0; }
  }


} // extern "C"
