
/***********************************************************************
*
* SIL_ARTE
* written by Siegmund Nowak
* changed by Wolfgang Wagner    01.12.1998
*
* This routines allows changes in the automatically produced NEDE/NWAL
* banks for the SVD 
*
*       called from cr_arte_user
*
* The transferred parameters are
* void sil_arte_(int nede,int nwal,char lvols[],int lcopy[],int nlev)
************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <math.h>
#include "arte/arte.hh"
#include "arte/ArteMessage.hh"
#include "arte/ArteUtil.h"
#include "gean/zgeom.h"


void sil_arte(int nede,int,char lvols[],int lcopy[],int nlev) {
    zgdet ZGDET;
    zgvol ZGVOL;
    zgrot ZGROT;
    char  volname[5];
    int up=1;
    // up: 1=up-stream, 0=down-stream  
    float x_wire[4], y1_wire[4], pol[2], d0[2];

    pol[0] = 1;
    pol[1] = 1;

    if(nede!=0) { 
      ArtePointer<NEDE> currde(nede), cr;
      cr = currde;
      
      strncpy(volname,&lvols[4*nlev],4);
      int type = lcopy[nlev];
      int itype = type>>16;
      int nr_vol = ZGVOL.Get_volnr(volname,itype);

      float angle=0.;
      int wr=ZGROT.Checkxy_Grot(currde->rot,angle);
      // Correct the xfw and y1w positions
      ZGDET.Get_x_wire(nr_vol, x_wire, cr->nsl);
      ZGDET.Get_y1_wire(nr_vol, y1_wire, cr->nsl);
      ZGDET.Get_polarity(nr_vol, pol, cr->nsl);

      if(wr && angle != 0.) {
        ZGROT.Arot_par(cr->xmin,cr->xmax,cr->ymin,cr->ymax,cr->rot); 
        ZGROT.Arot_par(cr->sxmn,cr->sxmx,cr->symn,cr->symx,cr->rot);
        float a;
        if(cr->sin >= 0){
          a = acos(cr->cos) + AR_DEGRAD*angle;
        }
        else
	{ 
          a = AR_TWOPI - acos(cr->cos) + AR_DEGRAD*angle;
        }
        cr->cos = cos(a);
	cr->sin = sin(a);
        ZGROT.Set_UnitMatrix(cr->rot);
      }
    
      //----------------------------------------------------------------------
      // Fill the gede-xfw with d_min and d_0 
      if( (cr->lay & 0xFFFF) == 1) up=1;
      else up=0;
      // Somehow I found no better way than distinguishing all the cases,
      // though it should also work with some analytic geometry.
      if(cr->nsl ==2) {            
        if(up) {
          switch(cr->sec & 0xFFFF) {
          case 1:    // module X00u
            d0[0] = (cr->sxmx - cr->sxmn)/cr->cos -
                    (cr->symx - cr->symn + (cr->sxmx - cr->sxmn) *
                    cr->sin/cr->cos) * cr->sin  + x_wire[0];
            d0[1] = (cr->symx - cr->symn + 
                    (cr->sxmx - cr->sxmn)/cr->cos * cr->sin) * 
                     cr->cos * (-1) + x_wire[1];
            break;
          case 2:    // module X03u
            d0[0] = (cr->symx - cr->symn) * (-1) * cr->sin + x_wire[0];
            d0[1] = (cr->symx - cr->symn) * (-1) * cr->cos + x_wire[1]; 
          break;
	  case 3:    // module X06u
            d0[0] = x_wire[0];
            d0[1] = x_wire[1];
            break; 
          case 4:    // module X09u
            d0[0] = (cr->sxmx - cr->sxmn) * cr->cos + x_wire[0];
            d0[1] = (cr->sxmx - cr->sxmn) * cr->sin * (-1) + x_wire[1]; 
            break;
          default:
            cerr << "sil_arte: wrong gede->sec given" << endl;
            break;   
          }
        }
        else {
          switch(cr->sec & 0xFFFF) {
            case 1:    // module X00d
              d0[0] = (cr->symx - cr->symn) * cr->sin * (-1) +
                      (cr->sxmx - cr->sxmn)/cr->sin  * cr->cos *
                       cr->cos + x_wire[0];                              
              d0[1] = (cr->sxmx - cr->sxmn + 
                      (cr->symx - cr->symn)/cr->sin * cr->cos ) /
                       cr->sin + x_wire[1];  
              break;
            case 2:    // module X03d
              d0[0] = (cr->symx - cr->symn) * (-1) * cr->sin  + x_wire[0];
              d0[1] = (cr->symx - cr->symn) * (-1) * cr->cos  + x_wire[1];
              break;
	    case 3:    // module X06d
              d0[0] = x_wire[0];
              d0[1] = x_wire[1];
              break; 
            case 4:    // module X09d 
              d0[0] = (cr->sxmx - cr->sxmn) * cr->cos + x_wire[0];
              d0[1] = (cr->sxmx - cr->sxmn) * (-1) * cr->cos + x_wire[1];
             break;
            default:
              cerr << "sil_arte: wrong gede->sec given" << endl;
            break;   
          }         
        }
      }
      else {  // single-sided
	if( (cr->lay & 0xFFFF) == 4) {   // down-stream n-side
          switch(cr->sec & 0xFFFF) {
            case 1:    // module X00d
              // Sorry, not implemented yet  
              cr->xfw[1] = 0;
              break;
            case 2:    // module X03d
              cr->xfw[1] = (cr->symx - cr->symn) * cr->cos  + x_wire[0];
            break;
	    case 3:    // module X06d
              cr->xfw[1] = x_wire[0];
              break; 
            case 4:    // module X09d 
              cr->xfw[1] = (cr->sxmx - cr->sxmn) * cr->cos + x_wire[0];
              break;
            default:
              cerr << "sil_arte: wrong gede->sec given" << endl;
            break;   
          }         
        }
        else {        
          // other than downstream n-side not implemented yet
          cerr << "sil_arte: layer of single-side differs from expectations"
               << endl; 
        }
      }
      // Caclulate d_min for the first layer of the silicon detector
      if(cr->nsl == 2)
        cr->xfw[0] = cr->sxmn*cr->cos - cr->symn*cr->sin + d0[0];
      else
        cr->xfw[0] = cr->sxmn*cr->cos - cr->symn*cr->sin + cr->xfw[1];

      //------------------------------------------------------------------
      // fill polarity into dedx
      cr->xfw[2] = pol[0];

      if(cr->nsl == 2) {
        struct NEDE gd;
        ArtePointer<NEDE>::append(gd);
        ArtePointer<NEDE> nextde(nede+1);      
        *nextde = *currde; 
        ArtePointer<NESL> currsl(zgmoth_.CURRSL);
        nextde->nesl =0;      
        createRelation(currsl,nextde,&NESL::nede);
        nextde->gmat =0;
        ArtePointer<GMAT> gm = currde->gmat;
        createRelation(gm, nextde, &GMAT::nede);  
        currsl->npl = currsl->npl +1 ;
       
        // Copy the zfw values according to the convention that
        // currde contains the smaller value and nextde the
        // higher value.
        nextde->zfw[0] = currde->zfw[0];
        currde->zfw[0] = currde->zfw[1];
        currde->zfw[1] =0.;
        nextde->zfw[1] =0.;
        nextde->xfw[0] =0.;
        currde->xfw[1] = d0[0];
        nextde->xfw[1] = d0[1];
        currde->xfw[3] = 0;
        nextde->xfw[2] = pol[1] ;
        nextde->xfw[3] = 0;
        nextde->nwi[0] = currde->nwi[1];
        currde->nwi[1] =0;
        nextde->nwi[1] =0;
        nextde->y1w[0] = currde->y1w[1];
        currde->y1w[1] = 0.;
        nextde->y1w[1] = 0.;
        nextde->y2w[0] = currde->y1w[1];
        currde->y2w[1] = 0.;
        nextde->y2w[1] = 0.;
      
        nextde->lay = nextde->lay + 1;

        float pitch[4], wire_or[4];      
        ZGDET.Get_wire_orientation(nr_vol,wire_or,currde->nsl);
        nextde->cos =cos(AR_DEGRAD*(wire_or[1]+angle));
        nextde->sin =sin(AR_DEGRAD*(wire_or[1]+angle));
        nextde->xfw[0] = nextde->sxmn * nextde->cos -
                         nextde->symn * nextde->sin +
                         nextde->xfw[1];

        ZGDET.Get_pitch(nr_vol,pitch,currde->nsl);      
        nextde->pit = pitch[1];  
        currde->nsl = 1;
        nextde->nsl = 1;
       
      }
    }

}
/* sil_arte_ */


