
         SUBROUTINE TRDG98

************************************************************************
* Author : Valeri Saveliev
*
* This routines defines the geometry for the TRD set up
* assuming a common TRD superlayer with two superlayers inside
*
* called from g98main.F
************************************************************************

      IMPLICIT NONE
#include "arte/Arte.fi"
#include "gean/gclink.fi"
#include "gean/gcvolu.fi"
#include "gean/gcflag.fi"
#include "gean/ZGMOTH.fi"
#include "auto/GEO.fi"
#include "gean/ZGESL.fi"
#include "gean/ZGDETV.fi"
C...
      INTEGER NSL, ISEC, NTST(2), NRTN(2),  NRT,  NRL,  NRS, NB, NPLT
      INTEGER NDSL(2),I,IROTM(3),MEDNUM, MATNUM,IS,ID, NMM, NN, NLVBL
      INTEGER LVOLS(15), LCOPY(15), IER, NWIR, NDET, IVOL
      REAL    XS, YS, ZS, ANGL, PRADL, RADL, FOAL, DSTR, RL(2), XMEAN 
      REAL    XRT(12), XLT(12), YRT(12), YLT(12), XROW(16), PAR(3)
      REAL A, Z, DE, AL, UB(1), XMS, XMSP, POS(3), POSM(3)
      CHARACTER*4 CSL(3),TRBL(12) 
      CHARACTER*20 CHNAM

C... Geom Definition of TRD
      DATA NTST / 78, 36 / , NRTN / 4, 2 / ,ANGL / 60. /
      DATA XLT  / -59.02, -59.02,  11.135, 11.135, -11.135, -11.135,
     +            -59.02, -59.02,  11.135, 11.135, -11.135, -11.135 /,
     +     XRT  / -11.135,-11.135, 59.02 , 59.02 ,  11.135,  11.135,
     +            -11.135,-11.135, 59.02 , 59.02 ,  11.135,  11.135 /
      DATA YLT  / -57.,     11.,   -79.,   -11.,    -68.,     11.,
     +            -79.,    -11.,   -57.,    11.,    -68.,     11.    /,
     +     YRT  /  11.,     79.,   -11.,    57.,    -11.,     68.,
     +            -11.,     57.,    11.,    79.,    -11.,     68.    /
      DATA XROW   /  0.     , -0.114375,  0.07625, -0.038125,
     +              -0.1525 ,  0.038125, -0.07625,  0.114375,
     +               0.     , -0.114375,  0.07625, -0.038125,
     +               0.1525 ,  0.038125, -0.07625,  0.114375  /
      DATA TRBL /'TRB1', 'TRB2', 'TRB3', 'TRB4', 'TRB5', 'TRB6',
     +           'TRB7', 'TRB8', 'TRB9', 'TRB0', 'TRBA', 'TRBB'/ 

#include "arte/Arte.ff"
#include "auto/GEO.ff"

      CALL TRDDAT
      CALL TRDMED

C... Rotation by +/- 60 deg around the z -axis
      CALL CRROTM(IROTM(1),90., ANGL,90.,90.+ANGL,0.,0.)
      CALL CRROTM(IROTM(2),90.,-ANGL,90.,90.-ANGL,0.,0.)
      CALL CRROTM(IROTM(3), 0.,   0.,90.,0.  , 90., 90.)
C...
      CALL GFMATE(MATNUM('TRD_FOAM'),CHNAM,A,Z,DE,RL(1),AL,UB,NB)
      CALL GFMATE(MATNUM('TRD_TUBE'),CHNAM,A,Z,DE,RL(2),AL,UB,NB)

C... Volumes definition TRD:
      DO i=1,12
       CALL CRVOLU(TRBL(I),'BOX ',LGESL_TRD,1,'TRD_FOAM',6,1)     ! FOAM
      ENDDO
      IF(DETV(LGESL_TRSL:LGESL_TRSL).EQ.'S') THEN
         CALL CRVOLU('TRTU','BOX ',LGESL_TRD,3,'TRD_TUBE',3,1)    ! BOXES
      ELSE
         CALL GSVOLU('TRTU','TUBE',MEDNUM(3,'TRD_GAS'),PAR,0,IVOL) ! TUBES     
         CALL GSDET('TRD ','TRTU',1,'TRTU',32,LGESL_TRD,50,20,IS,ID)
         CALL GSATT('TRTU','COLO',3)
         NMM = MATNUM('TRD_TUBE')
      ENDIF
      PRADL = 1.0
      RADL  = 2.3125
      DSTR  = 0.61
      NPLT  = 16
      FOAL  = PRADL+NPLT*RADL

      XMS= RADL/RL(1) + .125*PI*DSTR*(1./RL(2)-1./RL(1))
      XMSP = XMS + PRADL/RL(1)

C... TRD Geometry (TRSL common,2 TR01,TR02 part, 12 Modules, 2 Types)

      DO I =1,NNESL()
         IF(INESL_CMP(I).EQ.LGESL_TRSL) THEN
            NSL = I
         ELSEIF(INESL_CMP(I).EQ.LGESL_TRD) THEN
            NDSL(INESL_NSL(I)) = I
         ENDIF 
      ENDDO

 1    CURRSL = NSL
      CALL UHTOC(HNESL_NAME(NSL),4,CSL(1),4)
      CALL CGGESL(CSL(1),'MOTH',0)
      CALL GSATT(CSL(1),'SEEN',0)

C*** installation of TRD support
      CALL TRSGEO

      ISEC = 0 

C***  installation of superlayers 
      DO NSL=1,2
      CURRSL = NDSL(NSL)
      CALL UHTOC(HNESL_NAME(CURRSL),4,CSL(NSL+1),4)
      CALL CGGESL(CSL(NSL+1),CSL(1),IROTM(NSL))
      CALL GSATT(CSL(NSL+1),'COLO',3)
      IF(IDEBUG.NE.0)WRITE(LPRNT,*) 'TRD=',CSL(NSL+1),
     +                      FNESL_Z1(CURRSL),FNESL_Z2(CURRSL)
      
C... NRT N of Radiator 
      DO NRT = 1,2                 ! block types 
      DO NRL = 1,NRTN(NRT)         ! subblock in blocks
      ISEC = ISEC+1
******************************************
******************************************
********* different to TRDGEO*************
      IF(NRT.eq.2) GO TO 40
******************************************
******************************************
      PAR(1) = 0.5*(XRT(ISEC)-XLT(ISEC))
      PAR(2) = 0.5*(YRT(ISEC)-YLT(ISEC))
      PAR(3) = 0.5*FOAL
      XS = -0.5*(XRT(ISEC)+XLT(ISEC))
      YS = -0.5*(YRT(ISEC)+YLT(ISEC))
      ZS = 0.0
      CALL GSPOSP(TRBL(ISEC),ISEC,CSL(NSL+1),XS,YS,ZS,0,'ONLY',PAR,3)
      IF(DETV(LGESL_TRSL:LGESL_TRSL).EQ.'L') THEN
         CALL UCTOH(TRBL(ISEC),IS,4,4)
         CALL CRLVBK(IS,ISEC,ID,NLVBL,LVOLS,LCOPY)
      ENDIF

C...   loop over number of Layers in FOAM (Radiator+Straw)
      ZS     = -0.5*FOAL+PRADL-0.5*DSTR
      DO NRS = 1,NPLT

      PAR(1) = 0.5*NTST(NRT)*DSTR
      PAR(2) = 0.5*(YRT(3*NRT)-YLT(3*NRT))
      PAR(3) = 0.125*DSTR*PI
      XS     = XROW(NRS)
      YS     = 0.
      ZS     = ZS+RADL

C...     Filling the Data of TRD Layer Detectors
      IF(DETV(LGESL_TRSL:LGESL_TRSL).EQ.'S') THEN
C        short version
C        The straw tubes are implemented as a mixture of wire, gas and straw 
C        and then the volume is summed up to a box. Therefore the effective
C        thickness of this box is pi/4*RADIUS. The radiation length of the
C        radiator will be added to the detectors (NEDE-Banks)        

         CALL CRPOS('TRTU',TRBL(ISEC),NRS,ISEC,XS,YS,ZS,0,'ONLY',PAR)
      ELSE
C        long version
C        in ARTE the plane is handled as a block as in the short version 
C        in GEANT every tube is implemented individually, its detector number
c        is packed as NRS/wire number (2 I2 words) 

c        ARTE:
         CALL MMBOOK(JNEDE,1)
         NN = NNEDE()
         IQ(IQNEDE_CMP_L()) = INESL_CMP(CURRSL)
c         IQ(JNEDE+2+(NN-1)*NCNEDE()+JNEDE_NESL) = CURRSL 
         CALL MMBREL(JNSND,CURRSL,NN)
         IF(INESL_FPL(CURRSL).EQ.0) IQ(IQNESL_FPL(CURRSL)) = NN   
         IQ(IQNESL_NPL(CURRSL)) = IQ(IQNESL_NPL(CURRSL)) + 1
         IQ(IQNEDE_LAY_L()) = NRS
         IQ(IQNEDE_SEC_L()) = ISEC
         IQ(HQNEDE_GVOL_L()) = IQ(JVOLUM+IVOL)
         IQ(IQNEDE_SHP_L()) = 1
c         IQ(JNEDE+2+(NN-1)*NCNEDE()+JNEDE_GMAT) = NMM 
         CALL MMBREL(JGMND,NMM,NN)
         Q(QNEDE_SIN_L()) = 0.
         Q(QNEDE_COS_L()) = 1.
         CALL GLVOLU(NLVBL,LVOLS,LCOPY,IER)
         CALL VZERO(POS,3)
         CALL GMTOD(POS,POSM,1)
         Q(QNEDE_XMIN_L()) = XS - POSM(1) - PAR(1)
         Q(QNEDE_XMAX_L()) = XS - POSM(1) + PAR(1)
         Q(QNEDE_YMIN_L()) = YS - POSM(2) - PAR(2)
         Q(QNEDE_YMAX_L()) = YS - POSM(2) + PAR(2)
         Q(QNEDE_ZMIN_L()) = ZS - POSM(3) - PAR(3) - ZZERO
         Q(QNEDE_ZMAX_L()) = ZS - POSM(3) + PAR(3) - ZZERO
         IF(GRMAT(10,NLVBL).EQ.0.0) THEN
            Q(QNEDE_ROT_L(1))  = 1.
            Q(QNEDE_ROT_L(2))  = 0.
            Q(QNEDE_ROT_L(3))  = 0.
            Q(QNEDE_ROT_L(4))  = 0.
            Q(QNEDE_ROT_L(5))  = 1.
            Q(QNEDE_ROT_L(6))  = 0.
            Q(QNEDE_ROT_L(7))  = 0.
            Q(QNEDE_ROT_L(8))  = 0.
            Q(QNEDE_ROT_L(9))  = 1.
         ELSE
            Q(QNEDE_ROT_L(1)) = GRMAT(1,NLVBL)
            Q(QNEDE_ROT_L(4)) = GRMAT(2,NLVBL)
            Q(QNEDE_ROT_L(7)) = GRMAT(3,NLVBL)
            Q(QNEDE_ROT_L(2)) = GRMAT(4,NLVBL)
            Q(QNEDE_ROT_L(5)) = GRMAT(5,NLVBL)
            Q(QNEDE_ROT_L(8)) = GRMAT(6,NLVBL)
            Q(QNEDE_ROT_L(3)) = GRMAT(7,NLVBL)
            Q(QNEDE_ROT_L(6)) = GRMAT(8,NLVBL)
            Q(QNEDE_ROT_L(9)) = GRMAT(9,NLVBL)
         ENDIF

C        GEANT:
         PAR(1) = 0.
         PAR(2) = .5*DSTR
         PAR(3) =  0.5*(YRT(3*NRT)-YLT(3*NRT))
         DO NWIR =1, NTST(NRT)
         XS = XROW(NRS) + .5*DSTR*(2*NWIR-NTST(NRT)-1) 
         NDET =  ISHFT(NWIR,16) + NN 
         CALL GSPOSP('TRTU',NDET,TRBL(ISEC),XS,YS,ZS,
     +                           IROTM(3),'ONLY',PAR,3)
         ENDDO
      ENDIF 

C... Detector parameters 
	  Q(QNEDE_ACC_L())  = DSTR/3.       ! Accuracy
	  Q(QNEDE_EFF_L())  = 1.            ! Efficiency(real)
	  Q(QNEDE_PIT_L())  = DSTR          ! Pitch
	  Q(QNEDE_DOD_L())  = 0.5*DSTR      ! Drift Distance
	 IQ(IQNEDE_NSL_L())  = 1
	 IQ(IQNEDE_NWI_L(1))  = NTST(NRT)
c         IF(NRT.EQ.1)IQ(JPL+JNEDE_NW1)  = NTST(NRT)
          XMEAN = .5*(FNEDE_XMIN(NNEDE())+FNEDE_XMAX(NNEDE()))
	  Q(QNEDE_XFW_L(1)) = .5*DSTR*(1.-NTST(NRT)) + XMEAN 
	  Q(QNEDE_XFW_L(2)) = .5*DSTR*(1.-NTST(NRT)) + XROW(2) + XMEAN
	  Q(QNEDE_ZFW_L(1)) =.5*(FNEDE_ZMIN(NNEDE())+FNEDE_ZMAX(NNEDE())) 
          IF(NRS.EQ.1) THEN                 ! radiation length 
             Q(QNEDE_MS_L())= XMSP*(.0136*(1.+.038*ALOG(XMSP)))**2
          ELSE
             Q(QNEDE_MS_L())= XMS *(.0136*(1.+.038*ALOG(XMS )))**2
          ENDIF   

C... Superlayer
        IF(IDEBUG.NE.0) WRITE
     +  (LPRNT,'(''spl:'',2(1X,I4),2(1X,I2),4(1X,f8.3),I4,2(1X,f8.3))')
     +  NDSL(NSL),NNEDE(),INEDE_LAY(NNEDE()),INEDE_SEC(NNEDE()),
     +  FNEDE_XMIN(NNEDE()),FNEDE_XMAX(NNEDE()),
     +  FNEDE_YMIN(NNEDE()), FNEDE_YMAX(NNEDE()),
     +  INEDE_NWI(1,NNEDE()),FNEDE_XFW(1,NNEDE()),FNEDE_ZFW(1,NNEDE())
C	

       ENDDO                      ! end loop layers
******************************************
******************************************
********* different to TRDGEO*************
 40    CONTINUE
******************************************
******************************************
       ENDDO                      ! end loop subblocks
       ENDDO                      ! end loop block types 
      ENDDO                       ! end loop superlayers
C...
      RETURN
      END





