/*
*************************************************************************
**
** Description: List statistics for a given table or all tables
**
** Language:	C
**
** Called from: anywhere
**
* Author:  	Sergey Moukhine
**          	HERA-B Collaboration
**          	JINR-Dubna
**          	Russia
**
** Created: 	28 Oct 98
** Updated:  
**
*************************************************************************
*/

#include <strings.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <values.h>

#include <arte/ArteFlag.h>
#include "arte/Arte.h"
#include "auto/MEMO.h"
#include "auto/MEMA.h"

#include <arte/mecCount.h>

static void ArteTableName(int tab, char *tableName);
static void PrintLine(char *fmt,...);
static double myAbs(double val);
static char *ctplSave;
static int  lenCtplSave;

extern void FillTableAttrStatistics(int tab);
extern void meselp_(char *mess, char *ctpl, int len1, int len2);
extern void chhtot_ (int *H, char *T, int *N);

/***********************************************************************/

void listatistics_ (int *ilink, char *ctpl, int lenCtpl)

/*
************************************************************************
*
* Description: list table statictics
*
* Called from: anywhere
*
* External references:
*
* Argument  in/out description
*   ilink   in     ilink-th link; 0 -> all banks
*   ctpl    in     printer option (Terminal, Printer, Log_file)
*   lenCtpl in     length of CTPL (for calls from C)
*
************************************************************************
*/

{
int	i1;
int	i2;
int	tab;
int	acc;
double  mean;
char	errMess[500];

int	itName[namelen];
char	*tName = (char *)itName;

ctplSave = ctpl;
lenCtplSave = lenCtpl;

if (*ilink == 0)
   {
   i1 = 1;
   i2 = artelink_.narlnk;
   }
else
   {
   if (*ilink < 0 || *ilink > artelink_.narlnk)
      {
      static int errPtr;
      
      sprintf(errMess, "LISTATISTICS: invalid link address");
      mecErrorCount(&errPtr, errMess);
      return;
      }
   i1 = *ilink;
   i2 = *ilink;
   }
   
if (*ilink == 0)
   {
   PrintLine(" ");
   PrintLine("                             Statistics of Tables");
   PrintLine("                             --------------------");
   PrintLine(" ");
   }

PrintLine("----------------------------------------------------------------------");
PrintLine("   Book   Drop   Extn   Maxr  MeanRow  Nin   Nout   . . . . . TabName");
PrintLine("----------------------------------------------------------------------");

for (tab=i1; tab<=i2; tab++)
   {
   ArteTableName(tab,tName);
   if (strlen(tName) == 0)
      continue;
   
   if (!MEMO(tab).book)
      continue;
   
   FillTableAttrStatistics(tab);
   
   if (MEMO(tab).book > 1)
      {
      if (MEMO(tab).book == MEMO(tab).drop)
         acc = MEMO(tab).accrow;
      else
         acc = MEMO(tab).accrow + N_U(gcbank_.iq + artelink_.jlink[tab-1]);

      mean = (double)acc/(double)MEMO(tab).book;
      }
   else
      mean = (double)MEMO(tab).maxr;
   
   PrintLine("%7d%7d%7d%7d%7g%7d%7d   . . . . . %s",
      MEMO(tab).book,MEMO(tab).drop,MEMO(tab).extn,MEMO(tab).maxr,
      mean,MEMO(tab).nin,MEMO(tab).nout,tName);
      
   }
}

/***********************************************************************/

void liastatistics_ (int *ilink, char *ctpl, int lenCtpl)

/*
************************************************************************
*
* Description: list table attributes statictics
*
* Called from: anywhere
*
* External references:
*
* Argument  in/out description
*   ilink   in     ilink-th link; 0 -> all banks
*   ctpl    in     printer option (Terminal, Printer, Log_file)
*   lenCtpl in     length of CTPL (for calls from C)
*
************************************************************************
*/

{
int	mema;
int	nCol;
int	iCol;
int	col;
int	type;

int	i1;
int	i2;
int	tab;
int	lowSet;
int	uppSet;
char	errMess[500];

int	itName[namelen];
char	*tName = (char *)itName;

ctplSave = ctpl;
lenCtplSave = lenCtpl;

if (*ilink == 0)
   {
   i1 = 1;
   i2 = artelink_.narlnk;
   }
else
   {
   if (*ilink < 0 || *ilink > artelink_.narlnk)
      {
      static int errPtr;
      
      sprintf(errMess, "LISTATISTICS: invalid link address");
      mecErrorCount(&errPtr, errMess);
      return;
      }
   i1 = *ilink;
   i2 = *ilink;
   }
   
if (*ilink == 0)
   {
   PrintLine(" ");
   PrintLine("                       Statistics of Tables attributes");
   PrintLine("                       -------------------------------");
   PrintLine(" ");
   }

PrintLine("----------------------------------------------------------------------");
PrintLine("         Low |         Upp |         Min |         Max | AtrName",' ');
PrintLine("----------------------------------------------------------------------");

for (tab=i1; tab<=i2; tab++)
   {
   ArteTableName(tab,tName);
   if (strlen(tName) == 0)
      continue;
   
   if (!MEMO(tab).book || MEMO(tab).flag > 0)
      continue;
   
   FillTableAttrStatistics(tab);
   
   if (!MEMO(tab).maxr)
      continue;
   
   PrintLine(". . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . %s",
      tName);
      
   
   mema = MEMO(tab).mema;
   nCol = MEMO(tab).col;

   for (iCol=0; iCol<nCol; iCol++)
      {
      col = mema+iCol;
      type = MEMA(col).type;

      if (type == 9)
         type = 5;

      if (!(type == 5 || type == 7))
         continue;
   
      lowSet = 0;
      uppSet = 0;
      switch (type)
         {
         case 5: /* int or signed short */
            {
            if (MEMA(col).ilow != -MAXINT)
               lowSet = 1;
            if (MEMA(col).iupp != MAXINT)
               uppSet = 1;
            
            if (!lowSet && !uppSet)
               PrintLine("%13c|%13c|%12d |%12d | %20.20s",
                  ' ',' ',
                  MEMA(col).imin,MEMA(col).imax,MEMA(col).name);

            if (!lowSet && uppSet)
               PrintLine("%13c|%12d |%12d |%12d | %20.20s",
                  ' ',MEMA(col).iupp,
                  MEMA(col).imin,MEMA(col).imax,MEMA(col).name);

            if (lowSet && !uppSet)
               PrintLine("%12d |%13c|%12d |%12d | %20.20s",
                  MEMA(col).ilow,' ',
                  MEMA(col).imin,MEMA(col).imax,MEMA(col).name);

            if (lowSet && uppSet)
               PrintLine("%12d |%12d |%12d |%12d | %20.20s",
                  MEMA(col).ilow,MEMA(col).iupp,
                  MEMA(col).imin,MEMA(col).imax,MEMA(col).name);

            break;
            }
         case 7: /* float */
            {
            if (myAbs(MEMA(col).flow + MAXFLOAT) <= MINFLOAT)
               lowSet = 1;
            /*DEBUG*/if (lowSet) {printf("diff=%12.6G\n",
            /*DEBUG*/ MEMA(col).flow + MAXFLOAT);exit(0);}
            if (myAbs(MEMA(col).fupp - MAXFLOAT) <= MINFLOAT)
               uppSet = 1;
            
            if (!lowSet && !uppSet)
               PrintLine("%13c|%13c|%12.6G |%12.6G | %20.20s",
                  ' ',' ',
                  MEMA(col).fmin,MEMA(col).fmax,MEMA(col).name);
            
            if (!lowSet && uppSet)
               PrintLine("%13c|%12.6G |%12.6G |%12.6G | %20.20s",
                  ' ',MEMA(col).fupp,
                  MEMA(col).fmin,MEMA(col).fmax,MEMA(col).name);
            
            if (lowSet && !uppSet)
               PrintLine("%12.6G |%13c|%12.6G |%12.6G | %20.20s",
                  MEMA(col).flow,' ',
                  MEMA(col).fmin,MEMA(col).fmax,MEMA(col).name);
            
            if (lowSet && uppSet)
               PrintLine("%12.6G |%12.6G |%12.6G |%12.6G | %20.20s",
                  MEMA(col).flow,MEMA(col).fupp,
                  MEMA(col).fmin,MEMA(col).fmax,MEMA(col).name);
            
            break;
            }
            
         default:;
         }
      }
   }
}


/***********************************************************************/

static void PrintLine(char *fmt, ...)

/************************************************************************
**
** PARAMETERS
**      fmt	The format string. Type is (char *).
**      ...	The variable list of parameters
**
** FUNCTION DEFINITION
**	Print one line to ctplSave via MESELP function.
*************************************************************************
*/


{
char	mess[500];
va_list args;

va_start(args, fmt);

vsprintf(mess, fmt, args);

meselp_(mess,ctplSave,strlen(mess),lenCtplSave);

va_end(args);
}

/***********************************************************************/

static void ArteTableName(int tab, char *tableName)

/************************************************************************
**
** PARAMETERS
**	tab	  - table index (data start from iq(jlink(tab)+1) )
**	tableName - (output) table name
**
** RETURNS
**	None
**
** FUNCTION DEFINITION
**	Returns table name as a null terminated string.
*************************************************************************
*/

{
int  len = namelen;

chhtot_ ((int *)MEMO(tab).name, tableName, &len);
}

/***********************************************************************/

static double myAbs(double val)

/************************************************************************
**
** PARAMETERS
**	val	  - value
**
** RETURNS
**	abs(val)
**
** FUNCTION DEFINITION
**	Returns absolute value of val
*************************************************************************
*/

{
if (val >= 0.) return val;
return -val;
}
