//---------------------------------------------------------------------
// Created: 21.04.98 by   Olya Igonkina
// implementation of MCfunction
//---------------------------------------------------------------------



#include "marple/RcMcFunction.hh"

//------------ gamma----------------------------

int RcGamma(const ArtePointer<MTRA> mtra )
{
  if( mtra == 0 || mtra.index() > ArtePointer<MTRA>::nrow() ) return 0;
  if( abs(mtra->topp->lund) == 22 ) 
    {
      return 1;
    }
  return 0;
}

vector<ArtePointer<MTRA> > RcGamma( const vector<ArtePointer<MTRA> > mtraList )
{
  vector<ArtePointer<MTRA> > gammas;
  for( unsigned int iMtra = 0; 
       iMtra < mtraList.size(); iMtra++ )
    {
      if( RcGamma(mtraList[iMtra]) ) gammas.push_back( mtraList[iMtra]);
    }
  return  gammas;
}

//-----------bremstralung------------------------
ArtePointer<MTRA> RcOrigOfBrem( const ArtePointer<MTRA> mtra, 
				float beforeZ )
{
  if( abs(mtra->topp->lund) != 22 ) 
    {
      return 0; 
    }else{
      // cout << " origOfBrem: it is not  gamma"
    }
  if( mtra->bver->z > beforeZ )
    return 0;
  ArtePointer<MTRA> origBrem  = mtra->btra;
  if( origBrem == 0  || origBrem.index() > ArtePointer<MTRA>::nrow())
    return 0;
  int iLund = origBrem->topp->lund;
  if( abs(iLund) == 11  )
    {
      return origBrem;
    }else{
      return 0;
    }
}

// add only bremstralung origin to mtraList
vector<ArtePointer<MTRA> > RcMtraListOnlyBrem( const 
					       vector<ArtePointer<MTRA> > 
					       mtraList , 
					       float beforeZ = 200.)
{
  vector<ArtePointer<MTRA> > mtraListOnlyBrem;
  for( unsigned int iMtra = 0; 
       iMtra < mtraList.size(); iMtra++ )
    {
      ArtePointer<MTRA> brem = RcOrigOfBrem( mtraList[iMtra], beforeZ );
      if( brem.index() && !RcMtraOk(brem, mtraList) )
	mtraListOnlyBrem.push_back(brem) ; 
    }
  return mtraListOnlyBrem;
}

// add all bremstralung origin to mtraList
vector<ArtePointer<MTRA> > RcMtraListWithBrem( const vector<ArtePointer<MTRA> > 
					     mtraList,
					     float beforeZ  )
{
  vector<ArtePointer<MTRA> > mtraListWithBrem=mtraList;
  for( unsigned int iMtra = 0; 
       iMtra < mtraList.size(); iMtra++ )
    {
      ArtePointer<MTRA> brem = RcOrigOfBrem( mtraList[iMtra], beforeZ );
      if( brem.index() && !RcMtraOk(brem, mtraList) )
	mtraListWithBrem.push_back(brem) ; 
    }
  return mtraListWithBrem;
}

//---------------elepton-------------------------
int RcElepton(const ArtePointer<MTRA> mtra )
{
  if( mtra == 0 || mtra.index() > ArtePointer<MTRA>::nrow() ) return 0;
  if( abs(mtra->topp->lund) == 11 ) 
    {
      return 1;
    }
  return 0;
}

vector<ArtePointer<MTRA> > RcElepton( const vector<ArtePointer<MTRA> > 
				     mtraList )
{
  vector<ArtePointer<MTRA> > eleptons;
  for( unsigned int iMtra = 0; 
       iMtra < mtraList.size(); iMtra++ )
    {
      if( RcElepton(mtraList[iMtra]) ) eleptons.push_back( mtraList[iMtra]);
    }
  return  eleptons;
}

//---------------elepton-from-Jpsi-----------------------

int RcEleptonFromJpsi(const ArtePointer<MTRA> mtra )
{
  if( mtra == 0 || mtra.index() > ArtePointer<MTRA>::nrow() ) return 0;
  if( abs(ArtePointer<TOPP>(mtra->topp)->lund) == 11 ) 
    {
      ArtePointer<MTRA> btra = mtra->btra;
      if( btra == 0 ) return 0;
      if( abs(btra->topp->lund) == 443 )	return 1;  
    }
  return 0;
}


vector<ArtePointer<MTRA> > RcEleptonFromJpsi( const vector<ArtePointer<MTRA> > 
					     mtraList )
{
  vector<ArtePointer<MTRA> > eleptons;
  for( unsigned int iMtra = 0; 
       iMtra < mtraList.size(); iMtra++ )
    {
      if( RcEleptonFromJpsi(mtraList[iMtra]) ) 
	eleptons.push_back( mtraList[iMtra]);
    }
  return  eleptons;
}
//---------------electron-------------------------
int RcElectron(const ArtePointer<MTRA> mtra )
{
  if( mtra == 0 || mtra.index() > ArtePointer<MTRA>::nrow() ) return 0;
  if( mtra->topp->lund == 11 ) 
    {
      return 1;
    }
  return 0;
}

vector<ArtePointer<MTRA> > RcElectron( const vector<ArtePointer<MTRA> > 
				     mtraList )
{
  vector<ArtePointer<MTRA> > electrons;
  for( unsigned int iMtra = 0; 
       iMtra < mtraList.size(); iMtra++ )
    {
      if( RcElectron(mtraList[iMtra]) ) electrons.push_back( mtraList[iMtra]);
    }
  return  electrons;
}

//---------------electron-from-Jpsi-----------------------

int RcElectronFromJpsi(const ArtePointer<MTRA> mtra )
{
  if( mtra == 0 || mtra.index() > ArtePointer<MTRA>::nrow() ) return 0;
  if( mtra->topp->lund == 11 ) 
    {
      ArtePointer<MTRA> btra( mtra->btra);
      if( btra == 0 ) return 0;
      if( abs(btra->topp->lund) == 443 )	return 1;  
    }
  return 0;
}


vector<ArtePointer<MTRA> > RcElectronFromJpsi( const vector<ArtePointer<MTRA> > 
					     mtraList )
{
  vector<ArtePointer<MTRA> > electrons;
  for( unsigned int iMtra = 0; 
       iMtra < mtraList.size(); iMtra++ )
    {
      if( RcElectronFromJpsi(mtraList[iMtra]) ) 
	electrons.push_back( mtraList[iMtra]);
    }
  return  electrons;
}

//---------------positron-------------------------
int RcPositron(const ArtePointer<MTRA> mtra )
{
  if( mtra == 0 || mtra.index() > ArtePointer<MTRA>::nrow() ) return 0;
  if( mtra->topp->lund == -11 ) 
    {
      return 1;
    }
  return 0;
}

vector<ArtePointer<MTRA> > RcPositron( const vector<ArtePointer<MTRA> > 
				     mtraList )
{
  vector<ArtePointer<MTRA> > positrons;
  for( unsigned int iMtra = 0; 
       iMtra < mtraList.size(); iMtra++ )
    {
      if( RcPositron(mtraList[iMtra]) ) positrons.push_back( mtraList[iMtra]);
    }
  return  positrons;
}
//---------------positron-from-Jpsi-----------------------

int RcPositronFromJpsi(const ArtePointer<MTRA> mtra )
{
  if( mtra == 0 || mtra.index() > ArtePointer<MTRA>::nrow() ) return 0;
  if( mtra->topp->lund == -11 ) 
    {
      ArtePointer<MTRA> btra(mtra->btra);
      if( btra == 0 ) return 0;
      if( abs(btra->topp->lund) == 443 )	return 1;  
    }
  return 0;
}


vector<ArtePointer<MTRA> > RcPositronFromJpsi( const vector<ArtePointer<MTRA> > 
					     mtraList )
{
  vector<ArtePointer<MTRA> > positrons;
  for( unsigned int iMtra = 0; 
       iMtra < mtraList.size(); iMtra++ )
    {
      if( RcPositronFromJpsi(mtraList[iMtra]) ) 
	positrons.push_back( mtraList[iMtra]);
    }
  return  positrons;
}

//----------------J/psi----------------------------

int RcFromJpsi(const ArtePointer<MTRA> mtra )
{
  if( mtra == 0 || mtra.index() > ArtePointer<MTRA>::nrow() ) return 0;
  int iLund = mtra->topp->lund;
  if( abs(iLund) == 11 || abs(iLund) == 13 )
    {
      ArtePointer<MTRA> btra( mtra->btra);
      if( btra == 0 ) return 0;
      iLund = btra->topp->lund;
      if( abs(iLund) == 443 )
	{  return 1;  }
    }
  return 0;
}

int RcFromJpsi(const vector<ArtePointer<MTRA> > mtraList )
{
  for( unsigned int iMtra = 0; 
       iMtra < mtraList.size(); iMtra++ )
    {
      if( RcFromJpsi(mtraList[iMtra]) ) return 1;
    }
  return 0;
}
