//**********************************************************************/
// Created: 21.04.98 by   Olya Igonkina 
//
// implementation of SegClusBuilder class
//
//**********************************************************************/

#include <math.h>

#include "marple/marple.hh"
#include "marple/RcSegClusBuilder.hh"
#include "marple/RcMcFunction.hh"


extern "C"
{
  struct {
    int  Nevt;
    int  Nleft;
    int  Nright;
    int  Nmatch;
    int  Nbad; 
    int  Nlost;
    int  Nreal;    
    float Rbad;
    float RbadB;
    float Rlost;
    int   fromJpsi;
    float Rlmtra;
    float Rlelec ;
    float  Rlelecj;
    float  RlmtraB;
    float  RlelecB;
    float  RlelecjB;
  } FORTRAN_COMMON(mmatchnt);
}

#include "marple/RcTuples.h"

RcSegClusBuilder::RcSegClusBuilder() :
  RcMatchBuilder<RcSegPoint,RcClusPoint>(), fptRcConstBrem(0),
  fNmtra(0),  fNelectron(0),  fNjpsiElectron(0),
  fNmtraBrem(0),  fNelectronBrem(0),  fNjpsiElectronBrem(0),
  frNmtra(0),  frNelectron(0),  frNjpsiElectron(0),
  frNmtraBrem(0),  frNelectronBrem(0),  frNjpsiElectronBrem(0), 
  fNbrem(0), fNbadBrem(0)
{}

RcSegClusBuilder::RcSegClusBuilder( vector<RcSegPoint  *> vecLeft,
				vector<RcClusPoint *> vecRight,
				RcMatchConst & rcConst,
				RcMatchConst & rcConstBrem)
  : RcMatchBuilder<RcSegPoint, RcClusPoint>(vecLeft, vecRight, rcConst),
    fptRcConstBrem(&rcConstBrem),
    fNmtra(0),  fNelectron(0), fNjpsiElectron(0), 
    fNmtraBrem(0),  fNelectronBrem(0),  fNjpsiElectronBrem(0),
    frNmtra(0),  frNelectron(0),  frNjpsiElectron(0),
    frNmtraBrem(0),  frNelectronBrem(0),  frNjpsiElectronBrem(0),
    fNbrem(0), fNbadBrem(0)
{}

RcSegClusBuilder::~RcSegClusBuilder()
{
  fptRcConstBrem     = 0;
  //RcMatchBuilder<RcSegPoint,RcClusPoint>::~RcMatchPoint(*this);
}

//------------------------------------------------------
void RcSegClusBuilder::doMatch()
{
  RcMatchBuilder<RcSegPoint, RcClusPoint>::doMatch();
  statMC();
}
//------------------------------------------------------
void RcSegClusBuilder::doMatch(  MatchFunction matchFunction,
				 int markCombination = 1)
{
  int baseMark = 0;
  MatchFunction bremFunc = &matchBrem;
  RcMatchConst * ptConst = 0;
  if( matchFunction != bremFunc ) 
    ptConst = ptToMatchConst();
  else
    ptConst = ptToBremConst();

  if( markCombination ) 
    {
      for( unsigned int iLeft=0; iLeft < vectorLeft().size(); iLeft++)
	vectorLeft()[ iLeft ]->setmark( ptConst->n_pair_left);
      for( unsigned int iRight=0; iRight < vectorRight().size(); iRight++)
	vectorRight()[ iRight ]->setmark( ptConst->n_pair_right);
    }
  for( unsigned int iLeft=0; iLeft < vectorLeft().size(); iLeft++)
    {
      RcSegPoint * segment = vectorLeft()[ iLeft ];
      for( unsigned int iRight=0; iRight < vectorRight().size(); iRight++)
        {
	  RcClusPoint * cluster = vectorRight()[iRight];
	  
	  RcMatchPoint * matchPoint = 0;
	  // matchFunction will create 'new' object!! if not 0
	  //segment, cluster, matchPoint));
	  if(  ((*this).*matchFunction)(segment, cluster, matchPoint) )
	    {
	      // suppose all match points of this type
	      // should have the same mark
	      baseMark = matchPoint->mark(); 
	      fmatchList.push_back(matchPoint);
	      matchPoint = 0;
	    }
	}
    }	
  
  //**********************************************************************/
  // check the number of combinations with given RcPoints
  //**********************************************************************/
  
  // sort vector of RcMatchPoints by chi2() 
  sort( fmatchList.begin(), fmatchList.end(), comp_chi2 );

  if( ptConst->use_pair_cut ) 
    {
      for( unsigned int iMatch=0; iMatch < fmatchList.size(); iMatch++ )
	{
	  if( matchList()[iMatch]->mark() != baseMark ) continue;
	  int checkOk = 1;
	  RcSegPoint  * ptLeft   =  leftOrigin(iMatch);
	  RcClusPoint * ptRight  =  rightOrigin(iMatch);
	  
	  if( !(ptConst->pair_cut_ok( *ptLeft  )) ) checkOk = 0 ;
	  if( !(ptConst->pair_cut_ok( *ptRight )) ) checkOk = 0 ;
          if( !checkOk )
            { 
  	      delete( fmatchList[iMatch]);
              fmatchList.erase( &fmatchList[iMatch]) ; 
	      iMatch--;
            }else{
	      // mark origin
	      if( markCombination )
		{
		  int mark = ptLeft->mark();
		  mark--;
		  ptLeft->setmark(mark );
		  mark = ptRight->mark();
		  mark--;
		  ptRight->setmark(mark );
		}
	    }
	}
    }
  statMC();
}
//
void RcSegClusBuilder::matchBrem( int markCombination )
{
  doMatch(&matchBrem, markCombination);
}

int RcSegClusBuilder::matchBrem( RcSegPoint  * segment, 
				 RcClusPoint * cluster,
				 RcMatchPoint* &result)
{
  result = 0;
  if ( ptToBremConst() == 0 ) return 0;
  int lineswim = 1;

  RcSegPoint tmpPoint = *segment;
  // check for bremstralung, take if it is there
  if( ptToBremConst()->use_delta_cut ) 
    {
      tmpPoint.lineswim(cluster->z() );
      // doesn't pass delta cut, so return 0;
      if( !ptToBremConst()->energy_delta_cut_ok( tmpPoint, *cluster) )
	{ return 0; }
      //takeCombination = 0 ;       
    }
  result = new  RcMatchPoint( *segment,
			      *cluster,
			      ptToBremConst()->flag,
			      lineswim);
  if( result->mark() ||                // check mark of matching
      result->chi2() > ptToBremConst()->chi2 ) // bad chi2
    {
      delete result;
      result = 0;
      return 0;
    }
  result->setmark(3); // mark as bremstrahlung
  return 1;
}
//-------------------------------------------------------------
void RcSegClusBuilder::matchWithMomentum( int markCombination )
{
  doMatch(&matchWithMomentum, markCombination);
}

int RcSegClusBuilder::matchWithMomentum( RcSegPoint  * segment, 
					 RcClusPoint * cluster,
					 RcMatchPoint* &result)
{
  result = 0;
  if ( ptToMatchConst() == 0 ) return 0;

  HepDouble peOld = segment->pe();
  HepDouble covOld = segment->cov()(5,5);
  
  RcSegPoint tmpPoint = *segment;
  if( ptToMatchConst()->use_delta_cut ) 
    {
      tmpPoint.lineswim(cluster->z() );
      if( fabs(tmpPoint.y()-cluster->y() ) >
	  ptToMatchConst()->delta_y*2 ) return 0; // y direction doesn't fit
    }
  int iErr = 0;
  segment->estimatePEfromTAR(cluster->x(), cluster->y(),
			     cluster->z(), 
			     cluster->cov()(1,1),
			     cluster->cov()(2,2),
			     cluster->cov()(1,2),
			     3, iErr );
  if( iErr ) return 0; 

  tmpPoint = *segment;
  
  if( ptToMatchConst()->use_delta_cut ) 
    {
      tmpPoint.prolongate( cluster->z(), iErr );
      tmpPoint.setpe( fabs(tmpPoint.pe()) );
      if( ! iErr &&
	  !ptToMatchConst()->energy_delta_cut_ok( tmpPoint, *cluster) )
	{ return 0; }
    }
  
  // save combination passed cuts
  if( segment->pe() < 0 ) cluster->setpe(-cluster->pe());
  result = new RcMatchPoint( *segment, *cluster,ptToMatchConst()->flag);
  if( segment->pe() < 0 ) cluster->setpe(-cluster->pe());
  segment->setpe(peOld);
  segment->setcov(5,5,covOld);

  if(  result->mark() || result->chi2() > ptToMatchConst()->chi2 )
    { 
      delete result;
      result = 0;
      return 0;
    }
  result->setmark(1);
  return 1;
}

//-------------------------------------------------------
void RcSegClusBuilder::matchWithEcluster(int markCombination )
{
  doMatch(&matchWithEcluster, markCombination);
}

int RcSegClusBuilder::matchWithEcluster( RcSegPoint  * segment, 
					 RcClusPoint * cluster,
					 RcMatchPoint* &result)
{
  result = 0;
  if ( ptToMatchConst() == 0 ) return 0;

  int iErr = 0;
  HepDouble peOld = segment->pe();
  HepDouble covOld = segment->cov()(5,5);
  
  RcSegPoint tmpPoint = *segment;
  tmpPoint.lineswim(cluster->z() );
	  /*
	  if( (*ptConst).use_delta_cut && 
	      fabs(tmpPoint.y() - cluster->y() ) >
	      (*ptConst).delta_y )
	    {
	      if(RcMtraOk(segment->mtraList(), 
			  RcElepton(cluster->mtraList()) ))
		cout << " lost pair by lineswim: tmpPoint = " 
		     <<tmpPoint.y() << " cluster " << cluster->y()
		     << " delta_y " << (*ptConst).delta_y << endl;
	      continue;
	    }
	    */
  if( tmpPoint.x() < cluster->x() )
    {
      segment->setpe( cluster->pe() );
      segment->setcov(5,5, cluster->cov()(5,5) );
    }else{
      segment->setpe( -cluster->pe() );
      segment->setcov(5,5, cluster->cov()(5,5) );
    }
  tmpPoint = *segment;

  if( ptToMatchConst()->use_delta_cut ) 
    {
      tmpPoint.prolongate( cluster->z(), iErr );
      if( iErr ) return 0;
      if( !ptToMatchConst()->energy_delta_cut_ok( tmpPoint, *cluster) )
	{ return 0; }
    }
  if( segment->pe() < 0 ) cluster->setpe(-cluster->pe());
  result =  new RcMatchPoint( *segment, *cluster,ptToMatchConst()->flag);
  result->setpe( cluster->pe() );
  if( segment->pe() < 0 ) cluster->setpe(-cluster->pe());
  segment->setpe(peOld);
  segment->setcov(5,5,covOld);

  if(  result->mark() || result->chi2() > ptToMatchConst()->chi2 )
    { 
      delete result;
      result = 0;
      return 0;
    }
  result->setmark(2);
  return 1;
}	

//------------------------------------------------------
void RcSegClusBuilder::matchRings( int markCombination )
{
  if( maMagnetON() )
    {
      int iErr = 0;
      for( unsigned int iRight=0; iRight < vectorRight().size(); iRight++)
	vectorRight()[iRight]->estimateAngles(1,iErr);
      doMatch(&matchRingsMagnet, markCombination);
    }
  else
    doMatch(&matchRingsNoMagnet, markCombination);
}

int RcSegClusBuilder::matchRingsMagnet( RcSegPoint  * segment, 
					RcClusPoint * cluster,
					RcMatchPoint* &result)
{
  result = 0;
  if ( ptToMatchConst() == 0 ) return 0;

  int iErr = 0;
  int lineswim = 1;

  RcSegPoint tmpPoint = *segment;
	  
  HepDouble txOld = cluster->tx();
  HepDouble covtxOld = cluster->cov()(3,3);
  HepDouble pener = cluster->pe();

  // estimate TX for +charge hypothesis
  cluster->estimateAngles(2,iErr);
  int flag1 = 1;
  HepDouble tx1 = cluster->tx();
  HepDouble covtx1 = cluster->cov()(3,3);

  // estimate TX for +charge hypothesis
  cluster->settx( txOld );
  cluster->setcov(3,3, covtxOld );
  cluster->setpe( -pener );
  cluster->estimateAngles(2,iErr);
  HepDouble tx2 = cluster->tx();
  HepDouble covtx2 = cluster->cov()(3,3);

  int flag2 = 1;

  // check for rings, take if it is there
  if( ptToMatchConst()->use_delta_cut ) 
    {
      if( !ptToMatchConst()->energy_delta_cut_ok( tmpPoint, *cluster) )
	flag2 = 0;

      cluster->setpe( pener );
      cluster->settx( tx1 );
      cluster->setcov(3,3, covtx1 );	  
      if( !ptToMatchConst()->energy_delta_cut_ok( tmpPoint, *cluster) )
	flag1 = 0;
      if( !flag1 && !flag2) return 0 ;       
    }

  HepDouble zold = segment->z();
  segment->setZ( cluster->z() );

  HepDouble chi22 = MARPLE_INFINITY;
  HepDouble chi21 = MARPLE_INFINITY;
  cluster->settx( tx2 );
  cluster->setcov(3,3, covtx2 );	  
  cluster->setpe( -pener );
  RcMatchPoint * matchPair2 =  new RcMatchPoint( *segment,
						 *cluster, 
						 ptToMatchConst()->flag,
						 lineswim);
  if( matchPair2->mark() ) flag2= 0;
  cluster->setpe( pener );
  cluster->settx( tx1 );
  cluster->setcov(3,3, covtx1 );	  
  RcMatchPoint * matchPair1 =  new RcMatchPoint( *segment,
						 *cluster, 
						 ptToMatchConst()->flag,
						 lineswim);
  segment->setZ( zold );
  cluster->setpe( pener );
  cluster->settx( txOld );
  cluster->setcov(3,3, covtxOld );

  if( matchPair1->mark() ) flag1= 0;
  if( flag2 ) chi22 = matchPair2->chi2();
  if( flag1 ) chi21 = matchPair1->chi2();
  if( chi22 <= chi21 )
    {
      result = matchPair2;
      delete matchPair1;
      result->setmark(-4);
      result->setpe( -pener);
    }else{
      result = matchPair1;
      delete matchPair2;
      result->setmark(4);
      result->setpe( pener);
    }              
  if(  result->chi2() > ptToMatchConst()->chi2 )
    { 
      delete result;
      result = 0;
      return 0;
    }
  matchPair1 = 0;
  matchPair2 = 0;
  return 1;
}
//------------------------------------------------------------
int RcSegClusBuilder::matchRingsNoMagnet(RcSegPoint  * segment, 
					 RcClusPoint * cluster,
					 RcMatchPoint* &result)
{
  result = 0;
  if ( ptToMatchConst() == 0 ) return 0;

  int iErr = 0;

  // check for bremstralung, take if it is there
  if( ptToMatchConst()->use_delta_cut ) 
    {
      if( !ptToMatchConst()->energy_delta_cut_ok( *segment, *cluster) )
	return 0;
    }
  HepDouble zold = segment->z();
  segment->setZ( cluster->z() );
  int lineswim = 1;
  result =  new RcMatchPoint( *segment,
			      *cluster, 
			      ptToMatchConst()->flag,
			      lineswim);
  segment->setZ(zold);
  if( result->mark() ||                // check mark of matching
      result->chi2() > ptToMatchConst()->chi2 ) // bad chi2
    {
      delete result;
      result = 0;
      return 0;
    }
  result->setmark(4);
  return 1;
}


//----------------------------------------------------
void RcSegClusBuilder::statMC()
{
  fNrealPair         = 0;
  frNmtra             = 0;
  frNelectron         = 0;
  frNjpsiElectron     = 0;
  frNmtraBrem         = 0;
  frNelectronBrem     = 0;
  frNjpsiElectronBrem = 0;

  for( unsigned int iLeft=0; iLeft < vectorLeft().size(); iLeft++)
    {
      vector<ArtePointer<MTRA> > segMtraList = 
	(*vectorLeft()[ iLeft ]).mtraList();
      vector<ArtePointer<MTRA> > segBremList = 
	RcMtraListWithBrem(segMtraList);
      for( unsigned int iRight=0; iRight < vectorRight().size(); iRight++)
        {
	  vector<ArtePointer<MTRA> > clusMtraList = 
	    (*vectorRight()[ iRight ]).mtraList();
	  vector<ArtePointer<MTRA> > clusBremList = 
	    RcMtraListWithBrem(clusMtraList);
	  vector<ArtePointer<MTRA> > clusOnlyBremList = 
	    RcMtraListOnlyBrem(clusMtraList);

	  if( RcMtraOk( segMtraList, clusMtraList ) )
	    {
	      fNrealPair++; 
	      frNmtra++;
	      if( RcMtraOk( RcElepton(segMtraList), 
			    RcElepton(clusMtraList)) )frNelectron++;
	      if( RcMtraOk( RcEleptonFromJpsi(segMtraList),
			    RcEleptonFromJpsi(clusMtraList)) ) 
		frNjpsiElectron++;
	    }else{
	      if(      RcMtraOk(segBremList, clusOnlyBremList) ) 
		{ 
		  fNrealPair++; 
		  frNmtraBrem++;
		  if( RcMtraOk( RcElepton(segBremList), 
				RcElepton(clusOnlyBremList)) )
		    frNelectronBrem++;
		  if( RcMtraOk( RcEleptonFromJpsi(segBremList), 
				RcEleptonFromJpsi(clusOnlyBremList)))
		    frNjpsiElectronBrem++;
		}
	    }
	}
    }
  
  fNbrem = 0;
  fNmtra             = 0;
  fNelectron         = 0;
  fNjpsiElectron     = 0;
  fNmtraBrem         = 0;
  fNelectronBrem     = 0;
  fNjpsiElectronBrem = 0;

  for( unsigned int iMatch=0; iMatch < matchList().size(); iMatch++ )
    {
      if( matchList()[iMatch]->mark() == 3 ) fNbrem++;
      
      vector<ArtePointer<MTRA> > segMtraList = 
	(*leftOrigin(iMatch)).mtraList();
      vector<ArtePointer<MTRA> > segBremList = 
	RcMtraListWithBrem(segMtraList);

      vector<ArtePointer<MTRA> > clusMtraList = 
	(*rightOrigin(iMatch)).mtraList();
      vector<ArtePointer<MTRA> > clusBremList = 
	RcMtraListWithBrem(clusMtraList);
      vector<ArtePointer<MTRA> > clusOnlyBremList = 
	RcMtraListOnlyBrem(clusMtraList);

      if( matchList()[iMatch]->mark() != 3 )
	{
	  if( RcMtraOk( segMtraList, clusMtraList ) )
	    { 
	      fNmtra++;
	      if( RcMtraOk( RcElepton(segMtraList), 
			    RcElepton(clusMtraList)) )fNelectron++;
	      if( RcMtraOk( RcEleptonFromJpsi(segMtraList),
			    RcEleptonFromJpsi(clusMtraList)) )
		fNjpsiElectron++;
	    }
	}else{
	  if( RcMtraOk(segBremList, clusOnlyBremList)  )
	    {
	      fNmtraBrem++;
	      if( RcMtraOk( RcElepton(segBremList), 
			    RcElepton(clusOnlyBremList)) )fNelectronBrem++;
	      if( RcMtraOk( RcEleptonFromJpsi(segBremList), 
			    RcEleptonFromJpsi(clusOnlyBremList)))
		fNjpsiElectronBrem++;
	    }
	}
    }
  int Ngood = Nmtra() + NmtraBrem();
  fNbadBrem          = Nbrem() - NmtraBrem();
  fNbad              = matchList().size() - Nbrem() - Nmtra() ;
  fNlost             = NrealPair() - Ngood;
}


//--------------------------------------------------------------
void RcSegClusBuilder::stat( int iLong, int iType ) const
{
  unsigned int i;
  switch (iLong)
    {
    case 2 :
      arteMsg << "Left Points :" << vectorLeft().size(); 
      arteMsg.allp();
      for( i=0; i< vectorLeft().size(); i++)
        {  cout << *(vectorLeft()[i]) ; }
      
      arteMsg << "Right Points :" << vectorRight().size(); 
      arteMsg.allp();
      for( i=0; i< vectorRight().size(); i++)
        {  cout << *(vectorRight()[i]) ;  }
      
      arteMsg << "Match Points : "<< matchList().size(); 
      arteMsg.allp();
      for( i=0; i< matchList().size(); i++)
        {  cout << *(matchList()[i]) ;  }

    case 1 :
      cout << "MARPLE: *** constants used for matching ****" << endl 
	   << *ptToMatchConst(); 
      cout << "=== bremstralung constants " << endl
	   << *ptToBremConst(); 

    case 0 :
    default :     
      arteMsg << "*********** result of matching *******";
      arteMsg.allp();
      arteMsg << "Left Points      : " << vectorLeft().size(); 
      arteMsg.allp();
      arteMsg << "Right Points     : " << vectorRight().size(); 
      arteMsg.allp();
      arteMsg << "All pairs with the  same MTRA: " << NrealPair() ; 
      arteMsg.allp();
      if( iType ==  0 )
	{
      arteMsg << "All Match Points     : " << matchList().size();
      arteMsg.allp();
      arteMsg << "Brem Match Points    : " << Nbrem();
      arteMsg.allp();
	}else if(iType == -1 ){
      arteMsg << "Brem Match Points    : " << Nbrem();
      arteMsg.allp();
	}else if(iType == 1 ){
      arteMsg << "Non-Brem Match Points: " << matchList().size() - Nbrem();
      arteMsg.allp();
	}
      if( iType ==  0 || iType == 1)
	{
      arteMsg << "=======================  real = found  = lost" ;
      arteMsg.allp();
      arteMsg << "== same MTRA               " << rNmtra() << "   " 
	      << Nmtra() ;
      arteMsg.allp();
      arteMsg << "== same electron           " << rNelectron() << "   "
	      << Nelectron();
      arteMsg.allp();
      arteMsg << "== same Jpsi electron      " << rNjpsiElectron() 
	      << "   "<< NjpsiElectron();
      arteMsg.allp();
	}
      if( iType ==  0 || iType == -1)
	{
      arteMsg << "=bremstralung =========   real = found  = lost" ;
      arteMsg.allp();
      arteMsg << "== same brem MTRA          " << rNmtraBrem() << "   "
	      << NmtraBrem();
      arteMsg.allp();
      arteMsg << "== same brem electron      " << rNelectronBrem() << "   "
	      << NelectronBrem();
      arteMsg.allp();
      arteMsg << "== same brem Jpsi-electron " << rNjpsiElectronBrem() << "   "
	      << NjpsiElectronBrem();
      arteMsg.allp();
	}
      if( iType ==  0 )
	{
      arteMsg << "Bad  pairs       : " << Nbad() ; 
      arteMsg.allp();
      arteMsg << "Bad  Brem pairs  : " << NbadBrem() ; 
      arteMsg.allp();
      arteMsg << "Lost pairs       : " << Nlost(); 
      arteMsg.allp();
	}
      if( iType == 1)
	{
      arteMsg << "Bad  Non-Brem pairs  : " << Nbad() - NbadBrem() ; 
      arteMsg.allp();
	}
      if( iType == -1)
	{
      arteMsg << "Bad Brem pairs  : " << NbadBrem() ; 
      arteMsg.allp();
	}
    }
}

//----------------------------------------------------------
void RcSegClusBuilder::fillMatch( int iNtup) const
{
  ArtePointer<EVHD> event(1);
  FORTRAN_COMMON(mmatchnt).Nevt   = event->evt;
  FORTRAN_COMMON(mmatchnt).Nleft  = vectorLeft().size();
  FORTRAN_COMMON(mmatchnt).Nright = vectorRight().size();
  FORTRAN_COMMON(mmatchnt).Nmatch = matchList().size();
  FORTRAN_COMMON(mmatchnt).Nreal  = NrealPair();
  FORTRAN_COMMON(mmatchnt).Nbad   = Nbad() + NbadBrem();
  FORTRAN_COMMON(mmatchnt).Nlost  = Nlost() ;

  if( NbadBrem() )
    {
      FORTRAN_COMMON(mmatchnt).RbadB = float(NbadBrem()) /(float)Nbrem();
    }else{
      FORTRAN_COMMON(mmatchnt).RbadB     = 0;
    }
  
  if( matchList().size()-Nbrem() )
    {
      FORTRAN_COMMON(mmatchnt).Rbad = float(Nbad()) 
	/ float(matchList().size()-Nbrem());
    }else{
      FORTRAN_COMMON(mmatchnt).Rbad     = 0;
    }
  
  if( NrealPair() != 0)
    {
      FORTRAN_COMMON(mmatchnt).Rlost = (float)Nlost()/ (float)NrealPair();
    }else{
      FORTRAN_COMMON(mmatchnt).Rlost = 0;
    }
  if( rNmtra() != 0)
    {
      FORTRAN_COMMON(mmatchnt).Rlmtra = 1-
	(float)Nmtra()/ (float)rNmtra();
    }else{
      FORTRAN_COMMON(mmatchnt).Rlmtra = 0;
    }
  if( rNelectron() != 0)
    {
      FORTRAN_COMMON(mmatchnt).Rlelec = 1- 
	(float)Nelectron()/(float)rNelectron();
    }else{
      FORTRAN_COMMON(mmatchnt).Rlelec = 0;
    }
  if( rNjpsiElectron() != 0)
    {
      FORTRAN_COMMON(mmatchnt).Rlelecj = 1 - (float)NjpsiElectron()/ 
	(float)rNjpsiElectron();
    }else{
      FORTRAN_COMMON(mmatchnt).Rlelecj = 0;
    }
  if( rNmtraBrem() != 0)
    {
      FORTRAN_COMMON(mmatchnt).RlmtraB = 1 - 
	(float)NmtraBrem()/ (float)rNmtraBrem();
    }else{
      FORTRAN_COMMON(mmatchnt).RlmtraB = 0;
    }
  if( rNelectronBrem() != 0)
    {
      FORTRAN_COMMON(mmatchnt).RlelecB = 1 -
	(float)NelectronBrem()/ (float)rNelectronBrem();
    }else{
      FORTRAN_COMMON(mmatchnt).RlelecB = 0;
    }
  if( rNjpsiElectronBrem() != 0)
    {
      FORTRAN_COMMON(mmatchnt).RlelecjB = 1 - 
	(float)NjpsiElectronBrem()/ (float)rNjpsiElectronBrem();
    }else{
      FORTRAN_COMMON(mmatchnt).RlelecjB = 0;
    }



  FORTRAN_COMMON(mmatchnt).fromJpsi = NjpsiElectron();

  if( iNtup )  FORTRAN_ROUTINE( hfnt)(&iNtup); 
}


//---------------------------------------------------------
void RcSegClusBuilder::fillDeltas(int iNtup, int pulsNtup) const
{
  for( unsigned int iMatch=0; iMatch < matchList().size(); iMatch++ )
    {
      RcMatchPoint * ptMatchPoint = matchList()[iMatch];
      FORTRAN_COMMON(deltas).chi2 = (*ptMatchPoint).chi2();
      
      RcPoint * ptRp1 = leftOrigin(iMatch);
      vector<ArtePointer<MTRA> > segMtraList = (*ptRp1).mtraList();
      vector<ArtePointer<MTRA> > segBremList = 
	RcMtraListWithBrem(segMtraList);

      RcClusPoint * ptRp2 = rightOrigin(iMatch);
      vector<ArtePointer<MTRA> > clusMtraList = (*ptRp2).mtraList();
      vector<ArtePointer<MTRA> > clusBremList = 
	RcMtraListWithBrem(clusMtraList);
      vector<ArtePointer<MTRA> > clusOnlyBremList = 
	RcMtraListOnlyBrem(clusMtraList);

      int iErr;
      RcSegPoint tmpPoint =  *ptRp1;
      RcClusPoint cluster =  *ptRp2;
      //HepDouble txOld = cluster.tx();
      //HepDouble covtxOld = cluster.cov()(3,3);
      switch ( matchList()[iMatch]->mark() )
	{
	case -4 :
	  // rings for negative charge
	  tmpPoint.setZ( cluster.z() );
	  cluster.setpe( -cluster.pe() );
	  cluster.estimateAngles(2,iErr);
	  break;
	case 4 :
	  // rings
	  tmpPoint.setZ( cluster.z() );
	  cluster.estimateAngles(2,iErr);
	  break;
	case 3 :
	  // bremstralung 
	  tmpPoint.lineswim( ptRp2->z() );
	  break;
	case 2 :
	  // matching using energy of cluster
	  tmpPoint.lineswim( ptRp2->z() );      
	  if( tmpPoint.x() < ptRp2->x() )
	    {
	      tmpPoint = *ptRp1;
	      tmpPoint.setpe( ptRp2->pe() );
	      tmpPoint.setcov(5,5, ptRp2->cov()(5,5) );
	    }else{
	      tmpPoint = *ptRp1;
	      tmpPoint.setpe( -ptRp2->pe() );
	      tmpPoint.setcov(5,5, ptRp2->cov()(5,5) );
	    }
	  tmpPoint.prolongate(ptRp2->z(), iErr);
	  break;
	case 1 :
	  // matching with segment momentum estimation
	  tmpPoint.estimatePEfromTAR(ptRp2->x(), ptRp2->y(),
				     ptRp2->z(), 
				     ptRp2->cov()(1,1),
				     ptRp2->cov()(2,2),
				     ptRp2->cov()(1,2),
				     3, iErr );
	  if( iErr ) continue;
	  tmpPoint.prolongate(ptRp2->z(), iErr);
	  break;
	case 0  :
	default :
	  tmpPoint.prolongate(ptRp2->z(), iErr);
	  break;
	}

      HepDouble energyCorr = sqrt(fabs(ptRp2->pe()));
      if( energyCorr < MARPLE_INFINITY ) energyCorr = 1;

      FORTRAN_COMMON(deltas).deltaX  = (tmpPoint.x()  - (*ptRp2).x())
                	                *energyCorr ;
      FORTRAN_COMMON(deltas).deltaY  = (tmpPoint.y()  - (*ptRp2).y() )
                	                *energyCorr ;
      FORTRAN_COMMON(deltas).deltaTX = (tmpPoint.tx() - (*ptRp2).tx() )
                	                *energyCorr ;
      FORTRAN_COMMON(deltas).deltaTY = (tmpPoint.ty() - (*ptRp2).ty() )
                	                *energyCorr ;
      FORTRAN_COMMON(deltas).deltaPE = (fabs(tmpPoint.pe()) - (*ptRp2).pe() )
                	                *energyCorr ;

      if( matchList()[iMatch]->mark() != 3 )
	{
	  if( RcMtraOk( segMtraList, clusMtraList ) )
	    { 
	      FORTRAN_COMMON(deltas).flag    = 11;
	      if( RcMtraOk( RcElepton(segMtraList), 
			    RcElepton(clusMtraList)) )
		FORTRAN_COMMON(deltas).flag    = 12;
	      if( RcMtraOk( RcEleptonFromJpsi(segMtraList),
			    RcEleptonFromJpsi(clusMtraList)) )		
		FORTRAN_COMMON(deltas).flag    = 13;
	    }else{
	      FORTRAN_COMMON(deltas).flag    = 0;
	    }
	}else{
	  if( RcMtraOk(segBremList, clusOnlyBremList)  )
	    {
	      FORTRAN_COMMON(deltas).flag    = 1;
	      if( RcMtraOk( RcElepton(segBremList), 
			    RcElepton(clusOnlyBremList)) )
		FORTRAN_COMMON(deltas).flag    = 2;
	      if( RcMtraOk( RcEleptonFromJpsi(segBremList), 
			    RcEleptonFromJpsi(clusOnlyBremList)))
		FORTRAN_COMMON(deltas).flag    = 3;
	    }else{
	      FORTRAN_COMMON(deltas).flag    = -1;
	    }
	}

      int ipart;
      ArtePointer<MTRA> mtra;
      if( ((*ptRp1).mtraList()).size() !=0)
        {
          mtra  = (*ptRp1).mtraList()[0];
          ipart = mtra->topp->lund;
          FORTRAN_COMMON(deltas).ipart1  = ipart;
          mtra  = mtra->btra;
          ipart = mtra->topp->lund;
          FORTRAN_COMMON(deltas).imoth1  = ipart;
        }else{
          FORTRAN_COMMON(deltas).ipart1  = 0;
          FORTRAN_COMMON(deltas).imoth1  = 0;
        }
      
      if( ((*ptRp2).mtraList()).size() != 0 )
        {
          mtra  = (*ptRp2).mtraList()[0];
          ipart = mtra->topp->lund;
          FORTRAN_COMMON(deltas).ipart2  = ipart;
          mtra  = mtra->btra;
          ipart = mtra->topp->lund;
          FORTRAN_COMMON(deltas).imoth2  = ipart;
        }else{
          FORTRAN_COMMON(deltas).ipart2  = 0;
          FORTRAN_COMMON(deltas).imoth2  = 0;
        }
      if( iNtup ) FORTRAN_ROUTINE( hfnt)(&iNtup); 

      //--------------------------------------------------
      // fill Pulses if specified
      //--------------------------------------------------

      if( pulsNtup == 0 )   continue;

      if( ptToMatchConst()->flag.ifmarked_X() )
	{
	  FORTRAN_COMMON(deltas).lpulsX = ( ptMatchPoint->x() - tmpPoint.x() )
	    / sqrt(tmpPoint.cov()(1,1));
	  FORTRAN_COMMON(deltas).rpulsX = ( ptMatchPoint->x() - cluster.x() )
	    / sqrt(cluster.cov()(1,1));
	}else{
	  FORTRAN_COMMON(deltas).lpulsX = 0;
	  FORTRAN_COMMON(deltas).rpulsX = 0;
	}

      if( ptToMatchConst()->flag.ifmarked_Y() )
	{
	  FORTRAN_COMMON(deltas).lpulsY = ( ptMatchPoint->y() - tmpPoint.y() )
	    / sqrt(tmpPoint.cov()(2,2));
	  FORTRAN_COMMON(deltas).rpulsY = ( ptMatchPoint->y() - cluster.y() )
	    / sqrt(cluster.cov()(2,2));
	}else{
	  FORTRAN_COMMON(deltas).lpulsY = 0;
	  FORTRAN_COMMON(deltas).rpulsY = 0;
	}

      if( ptToMatchConst()->flag.ifmarked_TX() )
	{
	  FORTRAN_COMMON(deltas).lpulsTX = ( ptMatchPoint->tx() - tmpPoint.tx())
	    / sqrt(tmpPoint.cov()(3,3));
	  FORTRAN_COMMON(deltas).rpulsTX = ( ptMatchPoint->tx() - cluster.tx())
	    / sqrt(cluster.cov()(3,3));
	}else{
	  FORTRAN_COMMON(deltas).lpulsTX = 0;
	  FORTRAN_COMMON(deltas).rpulsTX = 0;
	}

      if( ptToMatchConst()->flag.ifmarked_TY() )
	{
	  FORTRAN_COMMON(deltas).lpulsTY = ( ptMatchPoint->ty() - tmpPoint.ty())
	    / sqrt(tmpPoint.cov()(4,4));
	  FORTRAN_COMMON(deltas).rpulsTY = ( ptMatchPoint->ty() - cluster.ty())
	    / sqrt(cluster.cov()(4,4));
	}else{
	  FORTRAN_COMMON(deltas).lpulsTY = 0;
	  FORTRAN_COMMON(deltas).rpulsTY = 0;
	}

      if( ptToMatchConst()->flag.ifmarked_PE() )
	{
	  FORTRAN_COMMON(deltas).lpulsPE = ( ptMatchPoint->pe() - tmpPoint.pe())
	    / sqrt(tmpPoint.cov()(5,5));
	  FORTRAN_COMMON(deltas).rpulsPE = ( ptMatchPoint->pe() - cluster.pe())
	    / sqrt(cluster.cov()(5,5));
	}else{
	  FORTRAN_COMMON(deltas).lpulsPE = 0;
	  FORTRAN_COMMON(deltas).rpulsPE = 0;
	}
      
      FORTRAN_ROUTINE( hfnt)(&pulsNtup);      

    } 
}

