//**********************************************************************/
// Created: 15.08.97  by   Olya Igonkina
//
// MARPLE initialization 
//
// fills the msGeslList with average Multiple Scattering for GESL 
//
//**********************************************************************/
#include <algo.h>

#include "marple/marple.hh"
#include "arte/ArteMessage.hh"
#include <gean/ZGESL.h>

vector<RcMsOfGesl> msGeslList;

#include "marple/marpleobjects.hh"
RcMatchBuilder< RcSegPoint, RcSegPoint>   SegmentsMatch;
RcMatchBuilder< RcSegPoint, RcClusPoint>  SegmentClusterMatch;
RcMatchBuilder< RcTraPoint, RcSegPoint>   TrackSegmentMatch;
RcMatchBuilder< RcTraPoint, RcClusPoint>  TrackClusterMatch;
RcMatchBuilder< RcTraPoint, RcTraPoint>   TracksMatch;
RcMatchBuilder< RcClusPoint, RcClusPoint> ClustersMatch;

//#include <algorithm>
bool comp_GESL_z (const RcMsOfGesl rms1, const RcMsOfGesl rms2)
{  return rms1.zcenter() < rms2.zcenter(); 
} 

HepDouble const MARPLE_INFINITY =  1048576;
HepDouble const MARPLE_NEGLIGIBLE =  1.e-30;

bool MARPLE_MagnetON = 1; // by default magnet is ON

// initialization itself

extern "C"
{
  void FORTRAN_ROUTINE( marpleinit)()
    {
      static int firstMarpleInit = 1;
      if( !firstMarpleInit )
	{
	  arteMsg << "MARPLE: marpleinit() is called twice";
	  arteMsg.allp();
	  return;
	}
      firstMarpleInit = 0;

      // initialization of Multiple Scattering array
      for( ArteTable<GESL>::iterator igesl = ArteTable<GESL>::begin();
	   igesl != ArteTable<GESL>::end(); ++igesl )
	{
	  if( (*igesl)->shp == 0 ) continue; // zero volume
	  RcMsOfGesl msEntry( *igesl);
	  msGeslList.push_back( msEntry);
	}
      sort(msGeslList.begin(),msGeslList.end(), comp_GESL_z); 
    }
}

int const  c_matrix_pack[5][5] = {
  {  0, 1, 3, 6,10},
  {  1, 2, 4, 7,11},
  {  3, 4, 5, 8,12},
  {  6, 7, 8, 9,13},
  { 10,11,12,13,14}
};

int const  fortran_matrix_pack[5][5] = {
  {  1, 2, 4, 7,11},
  {  2, 3, 5, 8,12},
  {  4, 5, 6, 9,13},
  {  7, 8, 9,10,14},
  { 11,12,13,14,15}
};

int const  c_rotation_matrix[3][3] = {
  {  0, 1, 2 },
  {  3, 4, 5 },
  {  6, 7, 8 }
};

int const  fortran_rotation_matrix[3][3] = {
  {  1, 2, 3 },
  {  4, 5, 6 },
  {  7, 8, 9 }
};

