// Author: Vitaly Eiges,  22.03.99
// Set of Functions to facilitate 
// muon digi-->hitb translation

int GetOffset(int FED_id)

  // Initial address in table FED_pl for plane
  // numbers for FED with given FED_id

{
  if (FED_id == 0x111) return  0*32;
  if (FED_id == 0x112) return  1*32;
  if (FED_id == 0x121) return  2*32;
  if (FED_id == 0x122) return  3*32;
  if (FED_id == 0x131) return  4*32;
  if (FED_id == 0x132) return  5*32;
  if (FED_id == 0x211) return  6*32;
  if (FED_id == 0x212) return  7*32;
  if (FED_id == 0x231) return  8*32;
  if (FED_id == 0x232) return  9*32;
  if (FED_id == 0x411) return 10*32;
  if (FED_id == 0x412) return 11*32;
  if (FED_id == 0x413) return 12*32;
  if (FED_id == 0x422) return 13*32;
  if (FED_id == 0x431) return 14*32;
  if (FED_id == 0x441) return 15*32;
  if (FED_id == 0x442) return 16*32;
  if (FED_id == 0x451) return 17*32;
  if (FED_id == 0x452) return 18*32;
  if (FED_id == 0x512) return 19*32;
  if (FED_id == 0x513) return 20*32;
  if (FED_id == 0x521) return 21*32;
  if (FED_id == 0x522) return 22*32;
  if (FED_id == 0x523) return 23*32;

  return 1;
 }

int GetPosition(int FED_id)

  // Returns 0 - bottom, 1 - top

{
  //  +X
  if (FED_id == 0x111) return 0;  // AB    Mt1.4, Mt1.2, Mt1.6 
  if (FED_id == 0x112) return 0;  // AB    Mt1.6
  if (FED_id == 0x121) return 1;  // ABCD  Mp3.1
  if (FED_id == 0x122) return 0;  // CDAB  Mp3.2
  if (FED_id == 0x131) return 1;  // BA    Mt3.1, Mt4.1 (watch 19's !!)
  if (FED_id == 0x132) return 0;  // BA    Mt3.2, Mt4.2
  if (FED_id == 0x211) return 1;  // BA    Mt1.5, Mt1.3, Mt1.1
  if (FED_id == 0x212) return 1;  // BA    Mt1.1
  if (FED_id == 0x231) return 1;  // ABCD  Mp4.1
  if (FED_id == 0x232) return 0;  // CDAB  Mp4.2

  // -X
  if (FED_id == 0x411) return 1;  // BA    Mt1.5
  if (FED_id == 0x412) return 1;  // BA    Mt1.3, Mt1.1
  if (FED_id == 0x413) return 0;  // AB    Mt1.2, Mt1.4, Mt1.6
  if (FED_id == 0x422) return 0;  // AB    Mt1.6
  if (FED_id == 0x431) return 2;  // CDAB  Mp3.1
  if (FED_id == 0x441) return 2;  // ABCD  Mp3.2
  if (FED_id == 0x442) return 3;  // CDAB, ABCD Mp3.1, Mp3.2 (mix !!)
  if (FED_id == 0x451) return 1;  // AB    Mt3.1
  if (FED_id == 0x452) return 0;  // AB    Mt3.2
  if (FED_id == 0x512) return 1;  // AB    Mt4.1
  if (FED_id == 0x513) return 2;  // ABCD  Mp4.2
  if (FED_id == 0x521) return 0;  // AB    Mt4.2
  if (FED_id == 0x522) return 2;  // CDAB  Mp4.1
  if (FED_id == 0x523) return 4;  // CDAB, ABCD Mp4.1, Mp4.2 (mix !!)

  return 99;
}

#include <arte/Arte.hh>
#include <auto/GESL.hh>

int GetGede(int icode)

{
  // Decodes GEDE plane number from the code

  char s[4];
  char t[] = {'0', '1', '2', '3', '4'};
  
  if (icode == 0) return 0;
  for ( ArteTableIterator<GESL> iG; iG.valid(); iG.next() ) 
    {
      for(int i=0; i<4; i++) s[i] = *(ArtePointer<GESL>(iG)->name + i);
 
      if (s[0]=='M' && s[1]=='U' && s[2]=='D' &&  
          s[3]==t[(int )(((float)icode)/1000.)]) 
	{return ArtePointer<GESL>(iG)->fpl + icode - 1000*(int )(((float)icode)/1000.);};
    }
  
  return 1;
}

#include <iostream.h>
#include <fstream.h>
#include <stdlib.h>
#include <arte/Arte.hh>
#include <auto/GEDE.hh>
#include <sys/time.h>

unsigned int GetHot(int iCH, int FEDid, int plane, int tube_pad)
  
  // Returns 0 - noisy channel, 1 - good channel
  // 0 - tube readout, 1 - pad readout
  // MU1 tube   15% or 15%*(50/X)^2
  // MU3, MU4 tube  1.8%
  // Pads 1.5%
{

#define N_CRATES 24

  int i,j;
  static int FED[N_CRATES];
  static int first=0;
  struct hitmap
  {
    int fed;
    float occup;
  };
  static hitmap hot[N_CRATES*1024];

  if (first==0)
    {  
      const char *file = (char *)getenv("MUON_BADFILE");
      ifstream HotMap(file);
      
      for (i=0; i<N_CRATES; i++)
	{
	  for(j=0;j<1024;j++)
	    {
	      HotMap >> hot[1024*i+j].fed >> hot[1024*i+j].occup;
	      // cout << i*1024+j << " " << hot[j].fed << " " << hot[j].occup << endl;
	      if (j==0) 
		{
		  FED[i] = hot[1024*i].fed; 
		  cout << hex << "FED " << hot[1024*i].fed << " hot map read" << dec << endl;
		}
	      //  cout << i << " " << hot[j].fed << " " << " " << hot[j].occup << endl;
	    }
	}
      //  sleep(5);
      
      HotMap.close();
    }
  first = 1;

  j = 0;
  for (i=0; i<N_CRATES; i++)
    {
      if (FED[i] == FEDid) 
	{
	  j = i;
	  break;
	}
    }
  
  // cout << "fff " << hex << j << " " << FED[j] << " " << FEDid << " "  << iCH 
  //      << " " << dec << hot[iCH+j*1024].occup << " " 
  //      << ArtePointer<GEDE>(plane)->hwc << endl;

  iCH += 1024*j;  // channel is shifted to the FED position in the hot table

  if (tube_pad == 1) // pad
    {
      if (hot[iCH].occup >= 0.015) return 0;
      if (hot[iCH].occup <  0.015) return 1;
    }
  if (tube_pad == 0) // tube
    {
      if (ArtePointer<GEDE>(plane)->zmax < 1800.) // MU1
	{
	  if (hot[iCH].occup >= 0.15) return 0;
	  if (hot[iCH].occup <  0.15)
	    {
	      if (hot[iCH].occup >= 0.15*(2500./((ArtePointer<GEDE>(plane)->xmax)*
						 (ArtePointer<GEDE>(plane)->xmax)))) return 0;
	      if (hot[iCH].occup <  0.15*(2500./((ArtePointer<GEDE>(plane)->xmax)*
						 (ArtePointer<GEDE>(plane)->xmax)))) return 1;
	    }
	}
      if (ArtePointer<GEDE>(plane)->zmax > 1800.) // MU3, MU4
	{
	  if (hot[iCH].occup >= 0.018) return 0;
	  if (hot[iCH].occup <  0.018) return 1;	  
	}
    }
  return 0;
}

int GetPlane(int i)

  // Returns plane code in the table 
{
  int Plane[] = 
  { 
    // FED 111 offset 0x32    tube bottom
    1096, 1026, 1028, 1030, 1032, 1034, 1036, 1038,   
    1040, 1042, 1044, 1046, 1072, 1074, 1076, 1078,   
    1080, 1082, 1084, 1086, 1088, 1090, 1092, 1094,   
    1024, 1048, 1128, 1130, 1132, 1134, 1136, 1098,   
    // FED 112 offset 1x32   tube bottom
    1138, 1140, 1142, 1144, 1146, 1148, 1150,    0,   
       0,    0,    0,    0,    0,    0,    0,    0,   
       0,    0,    0,    0,    0,    0,    0,    0,   
       0,    0,    0,    0,    0,    0,    0,    0,   
    // FED 121 offset 2x32   pad  top
    3000,    0, 3002,    0, 3004,    0, 3006,    0,   
    3008,    0, 3010,    0, 3012,    0, 3014,    0,   
    3016,    0, 3018,    0, 3020,    0, 3022,    0,   
    3024,    0, 3026,    0, 3028,    0,    0,    0,   
    // FED 122 offset 3x32   pad  bottom
    3030,    0, 3032,    0, 3034,    0, 3036,    0,   
    3038,    0, 3040,    0, 3042,    0, 3044,    0,   
    3046,    0, 3048,    0, 3050,    0, 3052,    0,   
    3054,    0, 3056,    0, 3058,    0,    0,    0,
    // FED 131 offset 4x32  tube top
    3000, 3002, 3004, 3006, 3008, 3010, 3012, 3014,   
    3016, 3018, 3020, 3022, 3024, 3026, 3028,    0,   
    4000, 4002, 4004, 4006, 4008, 4010, 4012, 4014,   
    4016, 4018, 4020, 4022, 4024, 4026, 4028,    0,   
    // FED 132 offset 5x32  tube bottom
    3060, 3030, 3032, 3034, 3036, 3038, 3040, 3042,   
    3044, 3046, 3048, 3050, 3052, 3054, 3056, 3058,   
    4060, 4030, 4032, 4034, 4036, 4038, 4040, 4042,   
    4044, 4046, 4048, 4050, 4052, 4054, 4056, 4058,   
    // FED 211 offset 6x32  tube top
    1100, 1102, 1104, 1106, 1108, 1110, 1112, 1114,   
    1116, 1118, 1120, 1122, 1124, 1126, 1050, 1052,   
    1054, 1056, 1058, 1060, 1062, 1064, 1066, 1068,   
    1070, 1000, 1002, 1004, 1006, 1008, 1010, 1012,   
    // FED 212 offset 7x32  tube top
    1014, 1016, 1018, 1020, 1022,    0,    0,    0,   
       0,    0,    0,    0,    0,    0,    0,    0,   
       0,    0,    0,    0,    0,    0,    0,    0,   
       0,    0,    0,    0,    0,    0,    0,    0,   
    // FED 231 offset 8x32  pad  top
    4000,    0, 4002,    0, 4004,    0, 4006,    0,   
    4008,    0, 4010,    0, 4012,    0, 4014,    0,   
    4016,    0, 4018,    0, 4020,    0, 4022,    0,   
    4024,    0, 4026,    0, 4028,    0,    0,    0,   
    // FED 232 offset 9x32  pad  bottom
    4030,    0, 4032,    0, 4034,    0, 4036,    0,   
    4038,    0, 4040,    0, 4042,    0, 4044,    0,   
    4046,    0, 4048,    0, 4050,    0, 4052,    0,   
    4054,    0, 4056,    0, 4058,    0,    0,    0, 
    // FED 411 offset 10x32  tube top
    1276, 1278, 1280, 1282, 1284, 1286, 1288, 1290,   
    1292, 1294, 1296, 1298, 1300, 1302,    0,    0,   
       0,    0,    0,    0,    0,    0,    0,    0,   
       0,    0,    0,    0,    0,    0,    0,    0, 
    // FED 412 offset 11x32  tube top
    1212, 1214, 1216, 1218, 1220, 1222, 1224, 1226,   
    1228, 1230, 1232, 1234, 1236, 1238, 1240, 1242,   
    1244, 1152, 1154, 1156, 1158, 1160, 1162, 1164,   
    1166, 1168, 1170, 1172, 1174, 1176, 1178, 1180, 
    // FED 413 offset 12x32  tube bottom
    1182, 1184, 1186, 1188, 1190, 1192, 1194, 1196,   
    1198, 1200, 1202, 1204, 1206, 1208, 1210, 1336,   
    1246, 1248, 1250, 1252, 1254, 1256, 1258, 1260,   
    1262, 1264, 1266, 1268, 1270, 1272, 1274,    0, 
    // FED 422 offset 13x32  tube bottom
    1304, 1306, 1308, 1310, 1312, 1314, 1316, 1318,   
    1320, 1322, 1324, 1326, 1328, 1330, 1332, 1334,   
       0,    0,    0,    0,    0,    0,    0,    0,   
       0,    0,    0,    0,    0,    0,    0,    0, 
    // FED 431 offset 14x32  pad top
    3062,    0, 3064,    0, 3066,    0, 3068,    0,   
    3070,    0, 3072,    0, 3074,    0, 3076,    0,   
    3078,    0, 3080,    0, 3082,    0, 3084,    0,   
    3086,    0, 3088,    0, 3090,    0,    0,    0, 
    // FED 441 offset 15x32  pad bottom
    3098,    0, 3100,    0, 3102,    0, 3104,    0,   
    3106,    0, 3108,    0, 3110,    0, 3112,    0,   
    3114,    0, 3116,    0, 3118,    0, 3120,    0,   
    3122,    0, 3124,    0, 3126,    0,    0,    0, 
    // FED 442 offset 16x32  pad mix
       0,    0,    0,    0,    0,    0,    0,    0,   
       0,    0,    0,    0,    0,    0,    0,    0,   
    3092,    0, 3094,    0, 3096,    0,    0,    0,   
    3128,    0, 3130,    0, 3132,    0,    0,    0, 
    // FED 451 offset 17x32  tube top
       0,    0,    0,    0,    0,    0,    0,    0,   
    3062, 3064, 3066, 3068, 3070, 3072, 3074, 3076,   
    3078, 3080, 3082, 3084, 3086, 3088, 3090,    0,   
    3092, 3094, 3096,    0,    0,    0,    0,    0, 
    // FED 452 offset 18x32  tube bottom
       0,    0,    0,    0,    0,    0,    0,    0,   
    3098, 3100, 3102, 3104, 3106, 3108, 3110, 3112,   
    3114, 3116, 3118, 3120, 3122, 3124, 3126, 3128,   
    3130, 3132,    0,    0,    0,    0,    0,    0, 
    // FED 512 offset 19x32  tube top
       0,    0,    0,    0,    0,    0,    0,    0,   
    4062, 4064, 4066, 4068, 4070, 4072, 4074, 4076,   
    4078, 4080, 4082, 4084, 4086, 4088, 4090,    0,   
    4092, 4094, 4096,    0,    0,    0,    0,    0, 
    // FED 513 offset 20x32  pad bottom
    4098,    0, 4100,    0, 4102,    0, 4104,    0,   
    4106,    0, 4108,    0, 4110,    0, 4112,    0,   
    4114,    0, 4116,    0, 4118,    0, 4120,    0,   
    4122,    0, 4124,    0, 4126,    0,    0,    0, 
    // FED 521 offset 21x32  tube bottom
       0,    0,    0,    0,    0,    0,    0,    0,   
    4098, 4100, 4102, 4104, 4106, 4108, 4110, 4112,   
    4114, 4116, 4118, 4120, 4122, 4124, 4126, 4128,   
    4130, 4132,    0,    0,    0,    0,    0,    0, 
    // FED 522 offset 22x32  pad top
    4062,    0, 4064,    0, 4066,    0, 4068,    0,   
    4070,    0, 4072,    0, 4074,    0, 4076,    0,   
    4078,    0, 4080,    0, 4082,    0, 4084,    0,   
    4086,    0, 4088,    0, 4090,    0,    0,    0, 
    // FED 523 offset 23x32  pad mix
       0,    0,    0,    0,    0,    0,    0,    0,   
       0,    0,    0,    0,    0,    0,    0,    0,   
    4092,    0, 4094,    0, 4096,    0,    0,    0,   
    4128,    0, 4130,    0, 4132,    0,    0,    0 
  };
  
  return Plane[i];
}
