        subroutine dicell
*********************************************************
*
* Displays a cut view of OTR honeycomb cells in Prism framework.
*
* Only cells with sense wires are drawn!
* Only fully cross-sected cells are drawn!
*
* Parameters read from KUIP:
*   cutval - cut value for section
*   filcol - fill color (or S - to assign colors according to sector number)
*   lincol - line color
*   lincol - line width
*   SL     - superlayer name
*
* Called routines:
*   PRISM:   dipro
*   KUIP:    kugetr kugetc
*   HIGZ:    igqwk isclip iuwk igsg igset ifa ipl islwsc igarc
*
* Author:       Alexander Lanyov  Apr 97
*
* Modification: Andrey Moshkin    Jul 97 
* Modification: Alexander Lanyov  Mar 99
*
*********************************************************
        implicit none

#include "arte/Arte.fi"
#include "gean/gcflag.fi"
#include "auto/GEO.fi"
#include "pris/zmh.fi"
#include "pris/zgraf.fi"
#include "gean/ZGESL.fi"

        integer k
        real ro,pit,dia,wn(4),rwire,R(9),xmin,xmax,ymin,ymax
        real cutval,filcol,lincol,linwid
        real xoffoct(7),zoffoct(7),xr(7),yr(7),p(7)
        real zx,zn,xwx,xwn,yx,yn,xx,xn,x_x,y_x,z_x,x_n,y_n,z_n
        real xpx,ypx,zpx,xpn,ypn,zpn,x1,y1,z1,x2,y2,z2
        integer iwkid,i,l,isl,pl,pl1,pl2,nl,il,nw,iw,isec,slnamh

        character*4 slnam

#include "arte/Arte.ff"
#include "auto/GEO.ff"

        data xoffoct/+1., +0.5, -0.5, -1., -0.5, +0.5, +1./
        data zoffoct/ 0., +0.5, +0.5,  0., -0.5, -0.5, +0./

        call kugetr(cutval)
cc Probably program should suggest some reasonable value for cutval?

        call kugetr(filcol)
        call kugetr(lincol)
        call kugetr(linwid)
        call kugetc(slnam,l)
        call uctoh(slnam,slnamh,4,4)

        call iselnt(52)         ! switch to Prism NT
cc        call igsg(1)
cc        call isclip(1)
        iwkid=1
        call igqwk(iwkid,'NTWN',wn)
        xmin=wn(1)
        xmax=wn(2)
        ymin=wn(3)
        ymax=wn(4)

        if (iviewzg.ge.9) then
	 c78 = 'DICELL: cannot display view'
	 call ch78bi(iviewzg)
         call me78sp('T')
         goto 999
	end if

c------------------------------------------------------ Loop on superlayers
        do isl=1,ngesl()
         pl1 = igesl_fpl(isl)
         pl2 = pl1 + igesl_npl(isl) - 1
         if (igesl_cmp(isl) .ne. lgesl_wch) pl2=-1
         if (slnam.ne.'    '.and.slnamh.ne.iq(hqgesl_name(isl)))
     @    pl2=-1

c------------------------------------------------------ Loop on planes
         do pl=pl1,pl2

          nl  = igede_nsl(pl)
          pit = fgede_pit(pl)
          dia = 0.1 * int(pit/0.174 + 0.5)  ! Diameter in cm (rounded)
          ro = 0.5*dia                      ! outer radius of hexagon

          do k =1,9
           R(k)  = fgede_rot(k,pl)
          end do

c Probably here we should throw away some planes which cannot be
c displayed at the current screen?
c (to make program faster)

          isec = igede_sec(pl)          ! to use this for color output?

c---------------------------------------------------- Loop on monolayers
          do il=1,nl
           xwx = fgede_xfw(il,pl)
           zx  = fgede_zfw(il,pl)
           xwn = xwx
           zn  = fgede_zfw(il,pl)
           yx  = fgede_ymin(pl)
           yn  = fgede_ymax(pl)
           nw = igede_nwi(il,pl)

c--------------------------------------------------- Loop on wires
           do iw=1,nw
            xx = xwx + (iw-1)*pit
            xn = xwn + (iw-1)*pit

c--------------------------------------------------- Loop on corners
            do i=1,7  ! corners of hexagon
             z_x = zx + dia    * zoffoct(i)
             x_x = xx + pit/3. * xoffoct(i)
             z_n = zn + dia    * zoffoct(i)
             x_n = xn + pit/3. * xoffoct(i)
             y_x = yx
             y_n = yn

c upper end of tube
             xpx = R(1)*x_x + R(2)*y_x + R(3)*z_x
             ypx = R(4)*x_x + R(5)*y_x + R(6)*z_x
             zpx = R(7)*x_x + R(8)*y_x + R(9)*z_x

c lower end of tube
             xpn = R(1)*x_n + R(2)*y_n + R(3)*z_n
             ypn = R(4)*x_n + R(5)*y_n + R(6)*z_n
             zpn = R(7)*x_n + R(8)*y_n + R(9)*z_n

             call dipro3(xpx,ypx,zpx,x2,y2,z2)
             call dipro3(xpn,ypn,zpn,x1,y1,z1)

             p(i) = (cutval-z1)/(z2-z1)
             xr(i) = x1 + p(i)*(x2-x1)
             yr(i) = y1 + p(i)*(y2-y1)

             if (p(i).gt.1. .or. p(i).lt.0.) goto 99

             if ((xr(i)-ro) .gt. xmax) goto 99
             if ((xr(i)+ro) .lt. xmin) goto 99
             if ((yr(i)-ro) .gt. ymax) goto 99
             if ((yr(i)+ro) .lt. ymin) goto 99
            end do
c--------------------------------------------------- End of loop on corners

            if (idebug.ne.0 .and. (iw.eq.1 .or. iw.eq.nw)) then
             write (c78,
     @        '('' dicell: Drawing pl='',i4,'' il='',i1,'', iw='',i2)')
     @             pl,il,iw
             call me78sp('T')
            end if

cccc            filcol = cocevzg ! in Prism
            if (filcol.gt.0) then
             call igset('FACI',filcol)  ! HIGZ, p.27 - fill area color index
             call igset('FAIS',1.)      ! HIGZ, p.28 - fill area style index
             call ifa(7,xr,yr)          ! HIGZ, p.25 - fill area
	    end if

cccc            lincol = cocebzg ! in Prism
            call igset('PLCI',lincol)  ! HIGZ, p.90 - polyline color index
            call islwsc(linwid)        ! HIGZ, p.31 - line width scale factor
            call ipl(7,xr,yr)

            rwire = 25.e-4
            if ((wn(4)-wn(3)) .lt. 20.) then
             rwire = max(rwire, 0.0015*(wn(4)-wn(3)))
            end if

            call igarc(0.5*(xr(1)+xr(4)),0.5*(yr(1)+yr(4)),
     @           rwire,rwire,0.,0.) ! draw an arc - here circle

99          continue
           end do
c--------------------------------------------------- End of loop on wires
          end do
c--------------------------------------------------- End of loop on monolayers
         end do
c--------------------------------------------------- End of loop on planes
        end do
c--------------------------------------------------- End of loop on superlayers

 999    continue
        call iuwk(0,1)          ! Flush buffer
        return
        end

      subroutine dipro3(xp,yp,zp,xx,yy,zz)
*********************************************************
* Transforms 3-vector (xp,yp,zp) in lab. system
* to "screen system" (xx,yy,zz)
* (where xx,yy - screen corrdinates,
*           zz - orthogonal coordinate)
*********************************************************
      implicit none

#include "pris/zgraf.fi"

      real  xp,yp,zp,xx,yy,zz,xs,ys,zs
      real cp, sp, ct, st, radeg
      data radeg /57.29578/

      if (iviewzg.eq.1) then
*      **** xz view
       xx =  zp
       yy =  xp
       zz =  yp
      elseif (iviewzg.eq.0) then
*      **** xy view
       xx =  xp
       yy =  yp
       zz =  zp
      elseif (iviewzg.eq.2) then
*      **** yz view
       xx =  zp
       yy =  yp
       zz = -xp
      elseif (iviewzg.ge.3.and.iviewzg.le.6) then
       if (iviewzg.eq.3) then
*       **** spatial view
        ct = cos(thetzg / radeg)
        st = sin(thetzg / radeg)
        cp = cos(phizg / radeg)
        sp = sin(phizg / radeg)
       elseif (iviewzg.eq.4) then
*       **** isometric view
        ct = sqrt(2./3.)
        st = sqrt(1./3.)
        cp = sqrt(0.5)
        sp = cp
       elseif (iviewzg.eq.5) then
*       **** dimetric view
        ct = sqrt(0.8)
        st = sqrt(0.2)
        cp = sqrt(0.75)
        sp = 0.5

       elseif (iviewzg.eq.6) then
*       **** trimetric view
        ct = sqrt(5./6.)
        st = sqrt(1./6.)
        cp = sqrt(0.7)
        sp = sqrt(0.3)
       endif

*      **** scale factors
       xs = xp * fscalzg(1)
       ys = yp * fscalzg(2)
       zs = zp * fscalzg(3)
       xx =   cp      * xs            - sp      * zs
       yy = - st * sp * xs  + ct * ys - st * cp * zs
       zz =   ct * sp * xs  + st * ys + ct * cp * zs

      elseif (iviewzg.eq.7) then
*       **** x-stereo view (u)
       xx = zp
       yy = ucoszg * xp - usinzg * yp
       zz = usinzg * xp + ucoszg * yp
      elseif (iviewzg.eq.8) then
*      **** x-stereo view (v)
       xx = zp
       yy = ucoszg * xp + usinzg * yp
       zz =-usinzg * xp + ucoszg * yp
      endif

      return
      end
