/* Use these staff in the form:

   call_interactively(F f, (Arg1*)0, ..., (ArgN*)0);

   where F f is such a function object that expression 
   f(a1, .., aN) is valid.

   This function returns the the pointer to the object (created by "new") 
   of the class MakeInteractiveCommand<...>, which is derived from
   InteractiveCommand class. Therefore this pointer can be inserted in the
   InteractiveMenu like 
   menu["sub_menu_name"]["sub_sub_menu_name"]..("command_name") =
   call_interactively(f, (Arg1*)0, ..., (ArgN*)0);

   When this InteractiveCommand is called it invokes function
   f(a1, .., aN), where a1, .., aN are extracted from the corresponding 
   istream.
*/

// Author V.Balagura (balagura@mail.desy.de)   (10.04.98).

#ifndef _call_interactively_hh
#define _call_interactively_hh 1

#include "ptset/call_via_istr.hh"
#include "ptset/interactive.hh"

/* -------------------- 0 args -------------------- */
template <class F>
inline MakeInteractiveCommand<CallViaIstr0<F, _CheckIstr_EOF> >*
call_interactively(F f) { 
  return make_interactive_command(call_via_istr_check_EOF(f));
}
/* -------------------- 1 arg -------------------- */
template <class F, class A1>
inline MakeInteractiveCommand<CallViaIstr1<F, A1, _CheckIstr_EOF> >*
call_interactively(F f, A1*) { 
  return make_interactive_command(call_via_istr_check_EOF(f, (A1*)0));
}
/* -------------------- 2 args -------------------- */
template <class F, class A1, class A2>
inline MakeInteractiveCommand<CallViaIstr2<F, A1, A2, _CheckIstr_EOF> >*
call_interactively(F f, A1*, A2*) { 
  return make_interactive_command
    (call_via_istr_check_EOF(f, (A1*)0, (A2*)0));
}
/* -------------------- 3 args -------------------- */
template <class F, class A1, class A2, class A3>
inline MakeInteractiveCommand<CallViaIstr3<F, A1, A2, A3, _CheckIstr_EOF> >*
call_interactively(F f, A1*, A2*, A3*) { 
  return make_interactive_command
    (call_via_istr_check_EOF(f, (A1*)0, (A2*)0, (A3*)0));
}
/* -------------------- 4 args -------------------- */
template <class F, class A1, class A2, class A3, class A4>
inline MakeInteractiveCommand<CallViaIstr4<F, A1, A2, A3, A4, 
  _CheckIstr_EOF> >*
call_interactively(F f, A1*, A2*, A3*, A4*) { 
  return make_interactive_command
    (call_via_istr_check_EOF(f, (A1*)0, (A2*)0, (A3*)0, (A4*)0));
}
/* -------------------- 5 args -------------------- */
template <class F, class A1, class A2, class A3, class A4, class A5>
inline MakeInteractiveCommand<CallViaIstr5<F, A1, A2, A3, A4, A5, 
  _CheckIstr_EOF> >*
call_interactively(F f, A1*, A2*, A3*, A4*, A5*) { 
  return make_interactive_command
    (call_via_istr_check_EOF(f, (A1*)0, (A2*)0, (A3*)0, (A4*)0, (A5*)0));
}

#endif
