/* Defines Delimiter class.
   Usage:
      Delimiter delim(" ", 10, "\n");
      vector<double> v(100, 3.14);
      copy(v.begin(), v.end(), ostream_iterator<double>(v, delim));

   this should copy vector of doubles to cout: 10 doubles per line.
   "delim" object will produce 9 spaces between doubles, and every tenth 
   it will give a new line.

   delim.flush(cout); // new line is written; the internal counter in "delim"
                      // is reset.
   */   

// Author V.Balagura (balagura@mail.desy.de)   (23.03.98).

#ifndef _delimit_hh
#define _delimit_hh 1

#include <iostream.h>
#include <string>

class Delimiter {
  string delimiter1_;
  string delimiter2_;
  int n;
  int count;
public:
  Delimiter(char *delimiter1, int n_repetitions,
	    char *delimiter2);
  void flush(ostream& os);
  friend ostream& operator<< (ostream& os, Delimiter& d);
};


inline Delimiter::Delimiter(char *delimiter1, int n_repetitions,
                            char *delimiter2) : delimiter1_(delimiter1),
  delimiter2_(delimiter2),
  n(n_repetitions-1), count(0) { }
inline void Delimiter::flush(ostream& os) {
  if ( count ) os << delimiter2_;
  count = 0;
}
inline ostream& operator<< (ostream& os, Delimiter& d) {
  if ( d.count < d.n ) {
    ++d.count;
    return os << d.delimiter1_;
  }
  else {
    d.count = 0;
    return os << d.delimiter2_;
  }
}

#endif
