// 
//  Author:  Rainer Mankel                     Created:  29-Sep-98
//           Humboldt University Berlin        Modified:  
//  Email:   mankel@ifh.de                     HERA-B Collaboration
// 
#include <iostream.h>
#include <math.h>
#include "CLHEP/Matrix/Matrix.h"
#include "CLHEP/Matrix/SymMatrix.h"
#include "rang/StateL.hh"
#include "rang/zfpar.hh"
#include "rang/rkmain.hh"
#include "rang/utfeld.hh"
#include "rang/FitMode.hh"
#include "rang/hfill.hh"
#include "rang/zfconst.hh"


//-- Constructor
rangerStateL::rangerStateL() : p(4), c(4) {
//  cout << "rangerState default constructor" << endl;
  zState = 0;
  qPEstimate = 0.1;
#ifdef FULL
  cout << "rangerState rhit constructor" << endl;
  cout << "rangerState: the state vector is now " << endl;
  cout << p << endl;
#endif
}

rangerStateL::rangerStateL(ArtePointer<RHIT> rhit) 
  : rangerState(rhit), p(4), c(4) {
  setX() = rhit->x;
  setY() = rhit->y;
  setTx() = rhit->tx;
  setTy() = rhit->ty;
}

rangerStateL::rangerStateL(ArtePointer<HITB> hitb) 
  : rangerState(hitb), p(4), c(4) {
  setX() = 0;
  setY() = 0;
  setTx() = 0;
  setTy() = 0;
}

rangerStateL::rangerStateL(ArtePointer<RCCL> rccl) 
  : rangerState(rccl), p(4), c(4) {
  setX() = rccl->x;
  setY() = rccl->y;
  setTx() = rccl->tx;
  setTy() = rccl->ty;
}

rangerStateL::rangerStateL(const rangerStateL& s) 
  : rangerState(s), p(4), c(4) {
#ifndef FULL
  cout << "rangerStateL::rangerStateL: here is the copy constructor" << endl;
#endif
  zState = s.zState;
  p = s.p;
  c = s.c;
  qPEstimate = s.qPEstimate;
#ifdef FULL
  cout << "rangerStateL::rangerStateL: Copy constructor done " << endl;
#endif
}

rangerStateL::~rangerStateL(){
//  cout << "rangerStateL destructor has been called" << endl;
}

rangerState& rangerStateL::operator=(const rangerStateL& s) {
#ifdef FULL
  cout << "rangerState& rangerStateL::operator=(const rangerStateL& s): "
       << " here is the assignment operator" << endl;
  double zz = s.z();
  cout << "   the given z is " << zz << endl;
#endif
  zState = s.zState;
  qPEstimate = s.qPEst();
  p = s.p;
  c = s.c;
  return *this;
}

rangerState& rangerStateL::operator=(const rangerState& s) {
#ifdef FULL
  cout << "rangerState& rangerStateL::operator=(const rangerState& s)" << endl;
#endif
  zState = s.z();
  qPEstimate = s.qPEst();
  p = s.vec();
  c = s.cov();
  return *this;
}

rangerState& rangerStateL::operator=(const rangerStateP& ) {

  cout << "Oh no!!!" << endl; 
  cout << "rangerState& rangerStateL::operator=(const rangerStateP& s)" 
       << endl;
  exit(1);
}

rangerState& rangerStateL::operator=(const rangerStatePI& ) {

  cout << "Oh no!!!" << endl; 
  cout << "rangerState& rangerStateL::operator=(const rangerStatePI& s)" 
       << endl;
  exit(1);
}

void rangerStateL::covInit() {
#ifdef FULL
  cout << "rangerStateL::covInit: z=" << z << endl;
  cout << "                     state matrix has " 
       << c.num_row() << " " << c.num_col() << endl;
#endif
  c(1,1) = zfpar_.cxxrf;
  c(2,2) = zfpar_.cyyrf;
  c(3,3) = zfpar_.ctxtxrf;
  c(4,4) = zfpar_.ctytyrf;
  for (int i=1; i<=dim(); i++) {
    for (int j=1; j<i; j++) {
//      cout << "initialize c(" << i << "," << j << ")" << endl;
      c(i,j) = 0;
    }
  }
#ifdef FULL
  cout << "cov matrix is " << endl << c << endl;
#endif
}



int rangerStateL::predict(const double& zDestination,
			 const double& ,rangerState*& s,
			 HepMatrix*& F) {

  int newState     = (s == 0);  // denote if memory allocations exist

#ifdef FULL
  cout << "rangerStateL::predict: zDestination=" << zDestination << endl;

  cout << "rangerStateL::predict: targetState=" << s
       << " newState= " <<  newState << endl;
#endif

  if (newState) {
    s = new rangerStateL();
    if (!s) {
      cout << "rangerState::predict: allocation error" << endl; 
      exit(1);
    }
  }
  if (!F) {
    F = new HepMatrix(dim(),dim());
    if (!F) {
      cout << "rangerState::predict: allocation error" << endl; 
      exit(1);
    }
  }

#ifdef FULL
  cout << "rangerStateL::predict: after allocation: s->vec()=" << s->vec() 
       << endl << "       *F=" << *F << endl;
#endif

  //double deltaZ = fabs(zDestination - zState);
  s->setZ() = zDestination;      // linear transport
  s->setX() = x() + (zDestination - zState) * tx();
  s->setY() = y() + (zDestination - zState) * ty();
  s->setTx() = tx();
  s->setTy() = ty();
  s->setQpEst() = qPEst();

  for (int i=1; i<=dim(); i++) {
    for (int j=1; j<=dim(); j++) {
      (*F)(i,j) = 0.;
    }
  }
  (*F)(1,1) = 1;
  (*F)(2,2) = 1;
  (*F)(3,3) = 1;
  (*F)(4,4) = 1;
  (*F)(1,3) = zDestination - zState;
  (*F)(2,4) = zDestination - zState;

#ifdef FULL
  cout << "rangerStateL::predict: after calculation, *F=" << *F << endl;

  cout << "rangerStateL::predict: the transported vector is"
       << s->vec() << endl;
#endif

  s->cov() = c.similarity(*F);

#ifdef FULL
  cout << "rangerStateL::predict: the transported cov is"
       << s->cov() << endl;
#endif

  return 0;
}


void rangerStateL::applyScat(const rangerScatThin& scat) {

  cov()(3,3) += scat.cTxTx;
  cov()(4,4) += scat.cTyTy;
  cov()(3,4) += scat.cTxTy;   // exploit symmetric matrix
}

void rangerStateL::applyScat(const rangerScatThick& scat) {

  cov()(3,3) += scat.cTxTx;
  cov()(4,4) += scat.cTyTy;
  cov()(3,4) += scat.cTxTy;   // exploit symmetric matrix

  cov()(1,1) += scat.cXX;     // correction for 
  cov()(1,2) += scat.cXY;     // thick scatterer
  cov()(2,2) += scat.cYY;
  cov()(1,3) += scat.cXTx;
  cov()(1,4) += scat.cXTy;
  cov()(2,3) += scat.cYTx;
  cov()(2,4) += scat.cYTy;
}



void rangerStateL::storeRseg(ArtePointer<RSEG> rseg,int firstLast){
 switch (firstLast) {
 case 1:
   rseg->zf = zState;
   rseg->xf = x();
   rseg->yf = y();
   rseg->txf = tx();
   rseg->tyf = ty();
   rseg->pf = qPEst();
   rseg->cvf[ 0] = cov()(1,1);
   rseg->cvf[ 1] = cov()(1,2);
   rseg->cvf[ 2] = cov()(2,2);
   rseg->cvf[ 3] = cov()(3,1);
   rseg->cvf[ 4] = cov()(3,2);
   rseg->cvf[ 5] = cov()(3,3);
   rseg->cvf[ 6] = cov()(4,1);
   rseg->cvf[ 7] = cov()(4,2);
   rseg->cvf[ 8] = cov()(4,3);
   rseg->cvf[ 9] = cov()(4,4);
   rseg->cvf[10] = 0;
   rseg->cvf[11] = 0;
   rseg->cvf[12] = 0;
   rseg->cvf[13] = 0;
   rseg->cvf[14] = 99999.;
   rseg->time = 0;
   rseg->dtim = 0;
   break;
 case 2:
   rseg->ze = zState;
   rseg->xe = x();
   rseg->ye = y();
   rseg->txe = tx();
   rseg->tye = ty();
   rseg->pe = qPEst();
   rseg->cve[ 0] = cov()(1,1);
   rseg->cve[ 1] = cov()(1,2);
   rseg->cve[ 2] = cov()(2,2);
   rseg->cve[ 3] = cov()(3,1);
   rseg->cve[ 4] = cov()(3,2);
   rseg->cve[ 5] = cov()(3,3);
   rseg->cve[ 6] = cov()(4,1);
   rseg->cve[ 7] = cov()(4,2);
   rseg->cve[ 8] = cov()(4,3);
   rseg->cve[ 9] = cov()(4,4);
   rseg->cve[10] = 0;
   rseg->cve[11] = 0;
   rseg->cve[12] = 0;
   rseg->cve[13] = 0;
   rseg->cve[14] = 9999;
   break;
 default:
   cout << "rangerStateL::storeRseg: invalid firstLast " << firstLast << endl;
   exit(1);
   break;
 }
}


void rangerStateL::storeRtra(ArtePointer<RTRA> rtra,int firstLast){
 switch (firstLast) {
 case 1:
   rtra->zf = zState;
   rtra->xf = x();
   rtra->yf = y();
   rtra->txf = tx();
   rtra->tyf = ty();
   rtra->pf = qPEst();
   rtra->cvf[ 0] = cov()(1,1);
   rtra->cvf[ 1] = cov()(1,2);
   rtra->cvf[ 2] = cov()(2,2);
   rtra->cvf[ 3] = cov()(3,1);
   rtra->cvf[ 4] = cov()(3,2);
   rtra->cvf[ 5] = cov()(3,3);
   rtra->cvf[ 6] = cov()(4,1);
   rtra->cvf[ 7] = cov()(4,2);
   rtra->cvf[ 8] = cov()(4,3);
   rtra->cvf[ 9] = cov()(4,4);
   rtra->cvf[10] = 0;
   rtra->cvf[11] = 0;
   rtra->cvf[12] = 0;
   rtra->cvf[13] = 0;
   rtra->cvf[14] = 99999.;
   break;
 default:
   cout << "rangerStateL::storeRtra: invalid firstLast " << firstLast << endl;
   exit(1);
   break;
 }
}


void rangerStateL::storeRhit(ArtePointer<RHIT> rhit) {
  rhit->x = x();
  rhit->y = y();
  rhit->tx = tx();
  rhit->ty = ty();
  rhit->p = qPEst();
}


void rangerStateL::copyVec( const rangerState& s) {
  zState = s.z();
  p = s.vec();
  qPEstimate = s.qPEst();
}




void rangerStateL::linExtrapVec( const rangerState& s, float z) {

  zState = z;

  setX() = s.x() + (z - s.z()) * s.tx();
  setY() = s.y() + (z - s.z()) * s.ty();
  setTx() =  s.tx();
  setTy() =  s.ty();
  setQpEst() =  s.qPEst();
}


void rangerStateL::dump() {
  cout << "     zState=" << zState << "  vec=" << p << endl;
  cout << "                              cov=" << c << endl;
}
