#include "rich/RICHDAQ.h"
#include "rich/PhotonDetection.h"

extern int RICHdebug;
// --------------------------------------------------------------------

RICHDAQ *RICH_DAQ=0;
ArtePointer<GEDE> RICHDAQ::RichGede=0;

// --------------------------------------------------------------------

M16Base::M16Base(ScriptFile& f) {
  int pmi,pmc,asd,asdchan;
  numchan=0;
  char s[256];
  char sasd[5];
  while (f.getline(s)) {
    if (sscanf(s,"%d %d %s %d\n",&pmi,&pmc,sasd,&asdchan)==4) {
      sasd[0]=sasd[3];
      sasd[1]='\0';
      asd=atoi(sasd);
      pm[index(asd,asdchan)]=pmi;
      pmChannel[index(asd,asdchan)]=pmc;
      asdID     [pmi*16+pmc]=asd;
      asdChannel[pmi*16+pmc]=asdchan;
      numchan++;
      }
    }
  if (numchan!=64) {
    fprintf(stderr,"M16Base: missing channels: check file m16wiremap\n");
    Exit(1);
    }
  }
  
// --------------------------------------------------------------------

M4Base::M4Base(ScriptFile& f) {
  int pmi,pmc,asdchan;
  numchan=0;
  char s[256];
  while (f.getline(s)) {
    if (sscanf(s,"%d %d %d\n",&pmi,&pmc,&asdchan)==3) {
      pm[index(asdchan)]=pmi;
      pmChannel[index(asdchan)]=pmc;
      asdChannel[pmi*4+pmc]=asdchan;
      numchan++;
      }
    }
  if (numchan!=16) {
    fprintf(stderr,"M4Base: missing channels: check file m4wiremap\n");
    Exit(1);
    }
  }

  

// -------------------------------------------------------------------------

RICHDAQ::RICHDAQ(char *path) {
  if (RICH_DAQ) {
    memcpy(this,RICH_DAQ,sizeof(RICHDAQ));
  }
  else {  
  if (!RICH_Detector)  RICH_Detector =new RICHDetector(path);
  if (!RICH_Vessel)    RICH_Vessel   =new   RICHVessel(path);
  if (!RICH_LambdaPhi) RICH_LambdaPhi=new LambdaPhi();

  RICHDetector& det=*RICH_Detector;

  nhit=0;
  numfed=0;
  numchannel=RICH_Detector->NumChannel();
  allcha =new RICHChannel[numchannel];
  hitlist=new unsigned short[numchannel];
  numhit =new unsigned long [numchannel];
  
  if (!numhit) { // is enough to check, since it is last and largest array
    fprintf(stderr,"RICHDAQ: Out of memory: Memory allocation error.\n");
    Exit(1);
    }
  ZeroNumHit();
  memset(allcha,0xFF,sizeof(RICHChannel)*numchannel);
 
  int sm,r,c,p;
  for (det.NewChannelLoop();det.ChannelLoop(&sm,&r,&c,&p);) {
    RICHChannel& ch=allcha[det.ChannelIndex()];
    ch.sm() =sm;
    ch.row()=r;
    ch.col()=c;
    ch.pm_channel()=p;
    ch.pm_numchannel()=det.pm().numchannel();
    if (det.efficiency()<0) ch.status()=3;
    else if (det.efficiency()>0) ch.status()=1;
    else ch.status()=1;
  }  


  char fname[256];
  strcpy(fname,path);
  strcat(fname,"m16wiremap.dat");
  ScriptFile m16(fname);
  m16bas=new M16Base(m16);

  strcpy(fname,path);
  strcat(fname,"m4wiremap.dat");
  ScriptFile m4(fname);
  m4bas= new M4Base(m4);
 
  strcpy(fname,path);
  strcat(fname,"ASD.dat");
  ScriptFile f(fname);
  char s[256],tag[256];


  int *fid=0;
  numfed=0;

  int *cnt=new int[NumChannel()];
  memset(cnt,0,sizeof(int)*NumChannel());
  
  while (f.getline(s)) {
    if(sscanf(s,"%s",tag)!=1)continue;
    int sm,r,c,fedid,d,con;
    char asd[5];

    if (sscanf(s,"%s %s %d %d %d %d %d %d",
	    tag,asd,&sm,&r,&c,&fedid,&d,&con)!=8) {
      fprintf(stderr,"ASD.dat: Error scanning %s\n",s);
      Exit(1);
      }
    asd[0]=asd[3];
    asd[1]='\0';
    int asdid=atoi(asd);
    r*=2;
    c*=2;   
    if (!strcmp(tag,"M16")) {    
      int asdchannel;
      for (asdchannel=0;asdchannel<NumChannelPerASD();asdchannel++) {
        int pm =m16bas->pmt(asdid,asdchannel);
        int p  =m16bas->pmtChannel(asdid,asdchannel);
        if (sm>6) { // on lower detector baseboard are rotated by 180 degree
	  pm=3-pm; 
	  p=15-p;
	  }
        int rr=r+pm/2;
        int cc=c+pm%2;
	
	if (!det.ChannelExist(sm,rr,cc,p)) {
          fprintf(stderr,"RICHDAQ(char*) initialisation error: \n");
          fprintf(stderr,"  ASD.dat requires channel without pmt: %2d %2d %2d %2d\n",
                sm,rr,cc,p);
          fprintf(stderr,"  last line: %s\n",s);
          Exit(1);
	  }
	
        if (++cnt[det.ChannelIndex()]>1) {
	  printf("Channel double assigned: %2d %2d %2d %2d asd: %d %2d  ncount: %d\n",
	  sm,rr,cc,p,asdid,asdchannel,cnt[det.ChannelIndex()]);
	  }
	
	allcha[det.ChannelIndex()].fedid()=fedid;
	
	int i=(asdchannel&0x8)|(7-(asdchannel&0x7)); // hardware map on asd card
	
	allcha[det.ChannelIndex()].bit()
	  =(d*NumConnectorPerDaughter()+con)*NumChannelPerConnector()+i;
        }
      }
    else if (!strcmp(tag,"M4")) {
      int asdchannel;
      for (asdchannel=0;asdchannel<NumChannelPerASD();asdchannel++) {
        int pm =m4bas->pmt(asdchannel);
        int p  =m4bas->pmtChannel(asdchannel);
        if (sm>6) { // on lower detector baseboard are rotated by 180 degree
	  pm=3-pm; 
	  p=3-p;
	  }
	
        int rr=r+pm/2;
        int cc=c+pm%2;
	
	if (!det.ChannelExist(sm,rr,cc,p)) {
          fprintf(stderr,"RICHDAQ(char*) initialisation error: \n");
          fprintf(stderr,"  ASD.dat requires channel without pmt: %2d %2d %2d %2d\n",
                sm,rr,cc,p);
          fprintf(stderr,"  last line: %s\n",s);
          Exit(1);
	  }
	
        if (++cnt[det.ChannelIndex()]>1) {
	  printf("Channel double assigned: %2d %2d %2d %2d asd: %d %2d  ncount: %d\n",
	  sm,rr,cc,p,asdid,asdchannel,cnt[det.ChannelIndex()]);
	  }
	allcha[det.ChannelIndex()].fedid()=fedid;
	
	int i=(asdchannel&0x8)|(7-(asdchannel&0x7)); // hardware map on asd card
	
	allcha[det.ChannelIndex()].bit()
	  =(d*NumConnectorPerDaughter()+con)*NumChannelPerConnector()+i;
        }
      } 
    else {
      fprintf(stderr,"RICHDAQ: Unknown Base type: %s\n",tag);
      Exit(1);
      }
// see if new fed is involved
 
    int i;
    for (i=0;i<numfed;i++) 
      if (fid[i]==fedid) break;
    if (i==numfed) {
      numfed++;
      fid=(int*)realloc(fid,sizeof(int)*numfed);
      fid[numfed-1]=fedid;
      }
    }
  free(fid);
    
  int i;
  for (i=0;i<NumChannel();i++) {
    if (allcha[i].bit()==(short)0xFFFF) {
      allcha[i].fedid()=numfed;
      allcha[i].bit()=0;
      allcha[i].status()=0;
      printf("Channel not assigned: %2d %2d %2d %2d\n",
        allcha[i].sm(),allcha[i].row(),allcha[i].col(),allcha[i].pm_channel());
      fflush(stdout);
      }
    }
     
  fedbitindex= new unsigned short[numfed*NumChannelPerFED()+1];
  memset(fedbitindex,0xFF,sizeof(unsigned short)*(numfed*NumChannelPerFED()));
  
  for (i=0;i<NumChannel();i++) {
    if (!ChannelExist(i)) continue;
    fedbitindex[fedid()*NumChannelPerFED()+bit()]=i;
    }
  
  int n=0;
  for (i=0;i<numfed*NumChannelPerFED();i++) {
    if (fedbitindex[i]!=(unsigned short)0xFFFF) n++;
    }
  if (n!=NumChannel()) {
    fprintf(stderr,"RICHDAQ: error initializing fedbitindex\n");
    Exit(1);
    }
    
  etb_fed=new ETB_fed[numfed];
  
  for (i=0;i<256;i++) {
    pattern[i].bit[7]=(i&0x01);
    pattern[i].bit[6]=(i&0x02);
    pattern[i].bit[5]=(i&0x04);
    pattern[i].bit[4]=(i&0x08);
    pattern[i].bit[3]=(i&0x10);
    pattern[i].bit[2]=(i&0x20);
    pattern[i].bit[1]=(i&0x40);
    pattern[i].bit[0]=(i&0x80);
    }
  }
}

// -------------------------------------------------------------------------

/*int RICHDAQ::NextEvent() {

  // -- Check if MC event
  if (!ArtePointer<MTRA>::nrow()) {
    fprintf(stderr," $$$RICH: RICHDAQ::NextEvent(): Function used only in Monte Carlo\n");
    return 1;
  }

  if (RichGede==0) 
    for (ArteTable<GEDE>::iterator igede=ArteTable<GEDE>::begin();
	 igede!=ArteTable<GEDE>::end(); ++igede) {
      if ((*igede)->cmp==5) {
	RichGede=(*igede);
	break;
      }
    }

  // -- Fill RICHDAQ event variables
  numevent++;
  nhit=0;
  RICHDetector &rd=*RICH_Detector;
  for (ArteTable<DIGB>::iterator idigb=RichGede->dig1;
       idigb!=RichGede->digl; ++idigb) {
    int id=(*idigb)->id;
    rd.ChannelExist(id);
    int ci=rd.ChannelIndex();
    numhit[ci]++;
    hitlist[nhit++]=ci;
  }
  return 0;
  }*/



int RICHDAQ::NextEvent() {

  // -- Check if MC event
  if (!ArtePointer<MTRA>::nrow()) {
    fprintf(stderr," $$$RICH: RICHDAQ::NextEvent(): Function used only in Monte Carlo\n");
    return 1;
  }

  // -- Fill RICHDAQ event variables
  numevent++;
  nhit=0;
  RICHDetector &rd=*RICH_Detector;
  for (ArteTable<DIGB>::iterator idigb=RichRun::gede->dig1;
       idigb!=RichRun::gede->digl; ++idigb) {
    int id=(*idigb)->id;
    rd.ChannelExist(id);
    int ci=rd.ChannelIndex();
    numhit[ci]++;
    hitlist[nhit++]=ci;
  }
  return 0;
}



int RICHDAQ::NextEventL4(FILE *fp) {
  int ret=0;
  static int* evtmem=0;
  static EVENT_HDR* event_ptr;
  extern int  SEG_LEN;
  
  if (!evtmem) {
    if (feof(fp)) return 0;
    evtmem=new int[SEG_LEN];
    event_ptr=(EVENT_HDR*) evtmem;
    ZeroNumHit();
  }

  int bc = 0;
  LogHdr hdr;
  l4aGetEvt( fp, event_ptr, SEG_LEN, &bc, &hdr);

  if (feof(fp)||!(ret=NextEvent(event_ptr))) {
    free(evtmem);
    evtmem=0;
  }
  return ret;
}


int RICHDAQ::NextEvent(EVENT_HDR* event_ptr)
{
  event=event_ptr;
  fltevt=event->sltevt;
  numevent++;
  ETB_get_feds(event, COMP_ID_RICH, numfed, etb_fed, &numfedevt);
  
  register int j1=NumChannelPerFED()/8;
  
  nhit=0;
  for (int i=0;i<numfedevt;i++)
    {
      
      if (etb_fed[i].hdr.fedid!=i)
	{
	  static int errorcount=0;
	  errorcount++;
	  if (errorcount<5)
	    {
	      cerr << "RICH:  Error with FED " << i << "\n";
	    }
	  else if (errorcount == 5)
	    {
	      cerr << "RICH:  Error with FED " << i << "\n";
	      cerr << "RICH:  Error message will be suppressed now.\n";
	    }
	  continue;
	}
      register int  f0=etb_fed[i].hdr.fedid*NumChannelPerFED();
      register unsigned char *data=(unsigned char*)etb_fed[i].data;
      register int bit=0;
      for (register int j=0;j<j1;j++)
	{
	  if (*data)
	    {
	      register unsigned char* h=&pattern[*data].bit[0];
	      //cout<<"RICHDAQ: data="<<hex<<int(*data)<<dec<<" "<<hex<<int(*h)<<dec<<endl;
	      for (register int k=0;k<8;k++)
		{
		  if (*h)
		    {
		      register unsigned short ci=fedbitindex[f0+bit];
		      if ((ci!=(unsigned short)0xFFFF))
			{
			  numhit[ci]++;
			  hitlist[nhit++]=ci;
			}
		    }
		  bit++;
		  h++;
		}
	    }
	  else bit+=8;
	  data++;
	}
    }
  return 1;
}
 
 
// -------------------------------------------------------------------------


int RICHDAQ::NextEvent(FILE *fp) {
  int ret=0;
  static int* evtmem=0;
  static EVENT_HDR* event_ptr;
  
  if (!evtmem) {
    if (feof(fp)) return 0;
    evtmem=new int[0x4000];
    event_ptr=(EVENT_HDR*) evtmem;
    ZeroNumHit();
  }
  ETB_read_event(fp,event_ptr,0x4000);
  if (feof(fp)||!(ret=NextEvent(event_ptr))) {
    free(evtmem);
    evtmem=0;
  }
  return ret;
}

// -------------------------------------------------------------------------

// -------------------------------------------------------------------------

void RICHDAQ::printevent(FILE *fp) {

  fprintf(fp,"\nEvent Header:\n");
  fprintf(fp,"  Length ...........: %08X  %6d\n",event->length,event->length);
  fprintf(fp,"  Record ID ........: %04X\n",event->recid);
  fprintf(fp,"  reserved .........: %04X\n",event->format);
  fprintf(fp,"  SLT Event No .....: %08X\n",event->sltevt);
  fprintf(fp,"  Component Mask ...: %08X\n",event->compmask);
  fprintf(fp,"  Event Mask .......: %08X\n",event->evtmask);
  fflush(fp);
  int i;
  for (i=0;i<numfedevt;i++) {
    SLB_SUBHDR* slb_header=&etb_fed[i].hdr;
    char* data=(char*)etb_fed[i].data;
    if (slb_header==0) {
      fprintf(fp,"\n No data in this event for fed %08X.\n",i);
      return;
      }
    fprintf(fp,"\n");
    fprintf(fp,"    Length ................: %04X %6d\n",
  	  slb_header->length,slb_header->length);
    fprintf(fp,"    MotherID (BoardID) ....: %04X\n",slb_header->fedid);
    fprintf(fp,"    RecordID ..............: %04X\n",slb_header->recid);
    fprintf(fp,"    BX-ID .................: %04X\n",slb_header->bxid);
    fflush(fp);
    int i;
    for (i=0;i<NumChannelPerFED()/8;i++) {
      fprintf(fp,"%02X",data[i]);
      if (i%32==31) fprintf(fp,"\n");
      else if (i%2==1) fprintf(fp," ");
      }
    fflush(fp);
    }
  }


// -------------------------------------------------------------------------

extern "C" {
  int ARTE_get_feds_DRIC( int max, ETB_fed *feds, int *n );
	   }

int RICHDAQ::fillHITBfromDRIC() {

  static int noDric=0;
  if (!ArteTable<DRIC>::nrow()) {
     if (noDric<10) { 
       fprintf(stderr," $$$RICH: DRIC table not filled\n");
       if (noDric==9) fprintf(stderr," $$$RICH: the last message of this kind ...\n"); 
       noDric++;
     }
     return 0;
  }

  ArtePointer<HITB> hitb;
  ArtePointer<DIGB> digb;
  RichRun::gede->hit1=ArteTable<HITB>::end();
  RichRun::gede->dig1=ArteTable<DIGB>::end();
 
  RICHDetector &rd=*RICH_Detector;
  HITB hdat;
  DIGB hdig;
  numevent++;

  ARTE_get_feds_DRIC(numfed,etb_fed,&numfedevt); // similar to ETB_get_feds()

  register int l=NumChannelPerFED()/8;

  nhit=0;
  for( int i=0; i<numfedevt; i++ ) {

    if (etb_fed[i].hdr.fedid!=i)
      {
        static int errorcount=0;
        errorcount++;
        if (errorcount<5)
          {       
            cerr << "RICH:  Error with FED " << i << "\n";
          }
        else if (errorcount == 5)
          {
            cerr << "RICH:  Error with FED " << i << "\n";
            cerr << "RICH:  Error message will be suppressed now.\n";
          }
	continue;
      }


    register int  f0 = etb_fed[i].hdr.fedid*NumChannelPerFED();   
    register unsigned char *p = (unsigned char *)etb_fed[i].data;
    register int  bit= 0;
    for (int j=0; j<l; j++) {
      register unsigned char adc=*(p+j);
      if (adc) {
        //cout << "j=" << j << " 0x" << hex << int( adc ) << dec <<endl;
        register unsigned char* h=&pattern[adc].bit[0];
	//cout<<"RICHOnline: adc="<<hex<<int(adc)<<dec<<" "<<hex<<int(*h)<<dec<<endl;
        for (register int k=0;k<8;k++) {
          if (*h) {
            register unsigned short ci=fedbitindex[f0+bit];
	    if ((ci!=(unsigned short)0xFFFF)) {
	      ChannelExist(ci);
	      if (!bad()) {
		numhit[ci]++;
		hitlist[nhit++]=ci;
		rd.ChannelExist(sm(),row(),col(),pm_channel());
		Vec3 x=rd.pos();
		hdat.z=x.z()/10.;
		hdat.x=x.x()/10.;
		hdat.y=x.y()/10.;
		hdat.q=RICH_LambdaPhi->phi(ci);
		hdat.r=RICH_LambdaPhi->lambda(ci);
		hdat.gede=RichRun::gede.index();
		hdig.id=sm()*1000000+row()*10000+col()*100+pm_channel();
		digb=ArteTable<DIGB>::push_back(hdig);
		hitb=ArteTable<HITB>::push_back(hdat);
		createRelation(hitb,digb,&HITB::digb);
	      }
	    }
	  }
          bit++;
	  h++;
	}
      }
      else bit+=8;
    }
  }
  RichRun::gede->hitl=ArteTable<HITB>::end();
  RichRun::gede->digl=ArteTable<DIGB>::end();
  return 1;
}

// -------------------------------------------------------------------------
