
/*--------------------------------------------------------------------------*
 *                                                                          *
 *   Routine to calculate hot/dead channels.   Convencion:                  *
 *                                                                          *
 *              April-99                                                    *
 *--------------------------------------------------------------------------*/


#include <stdlib.h>
#include "rich/RICHDAQ.h"
#include <fstream.h>
 
	/* Constants for the 99 run */

const int N16 = 4; //Number of sigma deviations
const int N4 = 3; //Number of sigma deviations
const float M16 = 0.5; //
const float M4 = 0.75; //
 	/*     *****     */

const double LEVEL_LOW = 0.0 ;
#define NCHS  26816
int st1[NCHS];


unsigned long loopinequich(int ss, int rr, int cc, double &occupancy, int pmtype);
 
void hotdead() {
  
  if (!RICH_DAQ) {
    fprintf(stderr," $$$RICH:  hotdead(): Initialize RICH before removing hot channels!\n");
    return;
  };

  extern char *IN_FILE; // -- input file name
  FILE *fp;             // -- input file
  if (!(fp=fopen(IN_FILE,"rb"))) {
    fprintf(stderr," $$$RICH:  hotdead(): Cannot open data-file\n");
    fprintf(stderr," $$$RICH:  hotdead(): BadChannels not removed !!!!\n");
    return;
  }

  RICHDAQ &daq =*RICH_DAQ;
  printf("from a DAQ file: %s\n",IN_FILE);fflush(stdout);
  daq.ZeroNumHit();
  extern int OLD;
  int n=0,ncut=2000;
  int uphot=0, downhot=0, upcold=0, downcold=0;

  if (OLD) 
    while (daq.NextEvent(fp)&&daq.NumEvents()<ncut) n++;
  else 
    while (daq.NextEventL4(fp)&&daq.NumEvents()<ncut) n++;

  // Mean occupancy.

  unsigned long sumhits=0;
 
  for (int i=0; i<daq.NumChannel(); ++i) 
  {
    if (!daq.ChannelExist(i)) continue;
    
    sumhits=sumhits+daq.NumHit();
  }
    double oc=double(sumhits)/ ( daq.NumEvents() * daq.NumChannel()) ;
    
   
  // Establishing the first cut for hot channels.

    double LEVEL_HIGH = oc / 0.23 ;

  // *** First classification. ***

  for (int i=0; i<daq.NumChannel(); ++i) 
  {
    if (!daq.ChannelExist(i)) continue;

    double oc=double(daq.NumHit())/daq.NumEvents();

    if (oc >= LEVEL_HIGH)
      {
	st1[i]=2;
	daq.Channel().status()=2; // hot
      }

    else if (oc <= LEVEL_LOW)  
      {
	st1[i]=3;
	daq.Channel().status()=3; // cold
      }
    else 
      {
	st1[i]=1;
	daq.Channel().status()=1; // normal
      }
      }

 //  ***  Second classification. ***


  for (int i=0;i<daq.NumChannel();i++) { 
    if (!daq.ChannelExist(i)) continue;

    unsigned long  ocupinch= daq.NumHit(), numeq=0;
    double effich, ocupineq=0;
    int pmtype;
    pmtype = daq.pm_numchannel();
    int sm=daq.sm(), ro=daq.row() , co=daq.col() ;
    numeq = loopinequich(sm, ro, co, ocupineq, pmtype);

    if(st1[i]==2)
	effich=ocupinch/ocupineq*(numeq) ;
    else if (st1[i]==3)
      effich= 0;
    else
      {
	if (ocupineq >= ocupinch)
	  effich = ocupinch/(ocupineq-ocupinch)*(numeq-1);
	else
	  effich= -1;
      }

    if (ocupineq != ocupinch)
      {
	double nmed=0;
	double c1=0, sigma1=0;
	double c2=0, sigma2=0;

	if(st1[i]==1)
	  nmed = (numeq-1)/(ocupineq-ocupinch);
	else if (ocupineq !=0)
	  nmed = (numeq)/(ocupineq);

	if(ocupineq !=0 && numeq !=0)
	  {

	      if (pmtype == 16) {
				 
		sigma1 = sqrt ( (nmed+nmed*M16) * (1 + 1/ (numeq)) );
		sigma2 = sqrt ( (nmed-nmed*M16) * (1 + 1/ (numeq)) );
		c1 = 1 + M16 + N16*sigma1;
		c2 = 1 - M16 - N16*sigma2;
	      }
	      else {
				 
		sigma1 = sqrt ( (nmed+nmed*M4) * (1 + 1/ (numeq)) );
		sigma2 = sqrt ( (nmed-nmed*M4) * (1 + 1/ (numeq)) );
		c1 = 1 + M4 + N4*sigma1;
		c2 = 1 - M4 - N4*sigma2;
	      }

	    if( effich > c1 )
	      {
		daq.Channel().status() = 2;  //Hot.
		if ( daq.sm()<7) uphot++;
		if ( daq.sm()>6) downhot++;
	      }
	        
	    else if ( effich < c2 && effich >=0)
	      {
		daq.Channel().status() = 3;  // Cold.
		if (daq.sm()<7) upcold++;
		if (daq.sm()>6) downcold++;
	      }
	    else daq.Channel().status() = 1;
	  }
	else daq.Channel().status() = 1;
      }
    else 

      daq.Channel().status() = 1;


    }  // End of loop in channels 

  cout <<"Bad channel supression based on "<<n<<" events"<<endl<< endl;
  cout <<"  Up good="<<(NCHS/2)-upcold-uphot<<endl;
  cout <<"  Up hot="<<uphot<<endl;
  cout <<"  Up cold="<<upcold<<endl << endl;
  cout <<"  Down good="<<(NCHS/2)-downcold-downhot<<endl;
  cout <<"  Down hot="<<downhot<<endl;
  cout <<"  Down cold="<<downcold<<endl;
  fclose(fp);
  daq.ZeroNumHit();
}


unsigned long loopinequich(int ss, int rr, int cc, double &occupancy, int pmtype)
{

  int i, j;
  int nsm, nro, nco;
  unsigned long numeq = 0;

  nsm = ss;
  nro = rr;
  nco = cc;

  RICHDAQ &daqn =*RICH_DAQ;
  RICHDetector& rd=*RICH_Detector;

  // For the same PM.

  for (i = 0; i < pmtype; i++)
    {
      j = rd.ChannelIndex(nsm, nro, nco, i);
      if(!daqn.ChannelExist(j))  continue;
      if(st1[j]==1)
        {
          numeq++;
          occupancy += daqn.NumHit();
        }
    }
  
  // Simetric for x and y (x -> -x and y -> -y)
  if (nsm < 7)
    nsm += 7;
  else
    nsm -= 7;
  for (i = 0; i < pmtype; i++)
    {
      j = rd.ChannelIndex(nsm, nro, nco, i);
      if(!daqn.ChannelExist(j)) continue;
      if(st1[j]==1)
        {
          numeq++;
          occupancy +=daqn.NumHit();
        }
    }
  
  // the simetric pm for y (x -> x, y -> -y)
  
  if (nsm < 7)
    nsm = 6 - nsm;
  else
    nsm = 20 - nsm;
    nco = 11 - nco;
  for (i = 0; i < pmtype; i++)
    {
      j = rd.ChannelIndex(nsm, nro, nco, i);
      if(!daqn.ChannelExist(j)) continue;
      if(st1[j]==1)
        {
          numeq++;
          occupancy += daqn.NumHit();
        }
    }
  // The simetric pm for x (x -> -x, y -> y)
  
  if (nsm < 7)
    nsm += 7;
  else
    nsm -= 7;
  for (i = 0; i < pmtype; i++)
    {
      j = rd.ChannelIndex(nsm, nro, nco, i);
      if(!daqn.ChannelExist(j)) continue;
      if(st1[j]==1)
        {
          numeq++;
          occupancy += daqn.NumHit();
        }
    }
  return(numeq);
}


// -------------------------------------------------------------------------
/// Removal of bad channels: channels are read from file specified with
/// $RICH_BADFILE

void badChannels(char *file) {

  if (!RICH_DAQ) {
    fprintf(stderr," $$$RICH:  badChannels: Initialize RICH before removing hot channels!\n");
    return;
  }

  FILE *in=fopen(file,"rt");
  if (!in) {
    fprintf(stderr," $$$RICH: badChannels(char *file): File RICH_BADFILE=%s specified but not found!!\n\n",file);
    fprintf(stderr," $$$RICH: Bad channels not removed !!!!\n\n");
    return;
  }
  printf("from a TXT file: %s\n",file);
  int id,status;
  RICHDAQ       &daq=*RICH_DAQ;
  RICHDetector  &rd =*RICH_Detector; 
  while (fscanf(in,"%d%d",&id,&status) != EOF) {
    if (!rd.ChannelExist(id)) continue;
    int ci=rd.ChannelIndex();
    if (!daq.ChannelExist(ci)) continue;
    // 1=hot 2=dead (Patricia Conde convention)
    // 2=hot 3=dead (Reinhard Eckmann convention)
    switch (status) {
    case 1: daq.Channel().status()=2; break; 
    //case 2: daq.Channel().status()=3; break;
    }
  }
}

extern "C" {

void badrich_() {
  char f[256];
  f[0]=0;
  char *badfile=getenv("RICH_BADFILE");
  if (badfile) {
    strcpy(f,badfile);
    badChannels(f);
  }
  else hotdead();
}

          } // extern "C"
