#include "rich/rich.h"


extern "C" void badrich_();



// -------------------------------------------------------------------------
// --- Functions for RICH event processing
// --- a) richinit_() 
// --- b) richexit_()
// --- c) dric2hitb_();
// --- d) richprint_()
//

void initrich_() {

  if (RICH_DAQ) return;
  cout<<"$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"<<endl;
  cout<<"$ RICH initialization ";


  // --- Select one of three possible options for RICH initialization:
  // --- a) database, 
  // --- b) custom config files, 
  // --- c) default config files.
  //
  RichRun::type=0;
  
  char* RICHconfigdir= getenv("RICH_CONFIGDIR");
  char* RICHdatabase = getenv("RICH_INSTALLDB");
  char* RICHherab    = getenv("HB");
  char datapath[256];
  datapath[0]=0;
  if (RICHdatabase) {
    RichRun::type=1;
  }
  else if (RICHconfigdir) {
    datapath[0]=0;
    strcpy(datapath,RICHconfigdir);
    if (datapath[strlen(datapath)-1]!='/') strcat(datapath,"/");
    RichRun::type=2;
  }
  else {
    datapath[0]=0;
    strcpy(datapath,RICHherab);
    strcat(datapath,"/geo/RICH");
    if (datapath[strlen(datapath)-1]!='/') strcat(datapath,"/");
    RichRun::type=3;
  }
  if (!RichRun::type) return;


  // --- find RICH gede number
  if (RichRun::gede==0) { 
    for (ArteTable<GEDE>::iterator igede=ArteTable<GEDE>::begin(); 
	 igede!=ArteTable<GEDE>::end(); ++igede) {
      if ((*igede)->cmp == 5) {
	RichRun::gede=(*igede);
	break;
      }
    }
  }
  
  // --- find magnet status
  ArtePointer<EVHD> evhd=ArteTable<EVHD>::begin();
  if (evhd->run>0) { // DATA
    if (evhd->date<990408) RichRun::magnet=0;
    else RichRun::magnet=1;
  }
  else if (RichRun::gede!=0) { // MC
    for (ArteTable<GESL>::iterator igesl=ArteTable<GESL>::begin();
	 igesl!=ArteTable<GESL>::end(); ++igesl) {
      if ((*igesl)->cmp==12) {
	RichRun::magnet=(*igesl)->mfl;
	break;
      }
    } 
  }


  switch (RichRun::type) {
  case 1: cout<<"from database"<<endl; break;
  case 2: cout<<"from custom datafiles "<<datapath<<endl; break;
  case 3: cout<<"from default datafiles "<<datapath<<endl; break;
  dafault: cout<<"not yet done"<<endl; return; break;
  };


  // --- initialize basic classes
  RICH_DAQ   = new RICHDAQ(datapath);

  cout<<"$ Classes initialized:";
  if (RICH_Vessel)    cout<<" RICH_Vessel";
  if (RICH_Detector)  cout<<" RICH_Detector";
  if (RICH_LambdaPhi) cout<<" RICH_LambdaPhi";
  if (RICH_DAQ)       cout<<" RICH_DAQ";
  cout<<endl;
  cout<<"$ Run conditions:";
  if (RichRun::magnet) cout<<" magnet ON ";
  else cout<<" magnet OFF ";
  cout<<endl;

  // --- check if hot channels need to be removed
  // --- a) from txt file RICH_BADFILE
  // --- b) from current DAQ file
  // --- c) from database
  char* RICHbadfile = getenv("RICH_BADFILE");
  if (RICHbadfile) RichRun::cleanHot=1;
  else RichRun::cleanHot=2;
  
  cout<<"$ Hot channels ";
  badrich_();

  cout<<"$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"<<endl;

}


void exitrich_() {
  if (!RICH_DAQ) return;
  if (RICH_DAQ)       { delete RICH_DAQ;       RICH_DAQ=0; }
  if (RICH_Vessel)    { delete RICH_Vessel;    RICH_Vessel=0; }
  if (RICH_Detector)  { delete RICH_Detector;  RICH_Detector=0; }
  if (RICH_LambdaPhi) { delete RICH_LambdaPhi; RICH_LambdaPhi=0; }
}


void printrich_() {

  cout<<"$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"<<endl;
  cout<<"$ RICH initialization ";
  switch (RichRun::type) {
  case 1: cout<<"from database"<<endl; break;
  case 2: cout<<"from custom datafiles"<<endl; break;
  case 3: cout<<"from default datafiles"<<endl; break;
  dafault: cout<<"not yet done"<<endl; return; break;
  };
  cout<<"$ Classes initialized:";
  if (RICH_Vessel)    cout<<" RICH_Vessel";
  if (RICH_Detector)  cout<<" RICH_Detector";
  if (RICH_LambdaPhi) cout<<" RICH_LambdaPhi";
  if (RICH_DAQ)       cout<<" RICH_DAQ";
  cout<<endl;
  cout<<"$ Run conditions:";
  if (RichRun::magnet) cout<<" magnet ON ";
  else cout<<" magnet OFF ";
  cout<<endl;
  cout<<"$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"<<endl;

}

void dric2hitb_() {
  if  (ArteTable<MTRA>::nrow()) return;
  if  (!RICH_DAQ) initrich_();
  RICH_DAQ->fillHITBfromDRIC();
}

void hidric_(int n) {
  n++; // dummy n
  dric2hitb_();
}



