
//**********************************************************************/
//   RICH reconstruction (rire), version 2.00
//
// Created: 08.03.99, by Ignacio Ari\~no. 
//        
// RichTrack: main class for the RICH reconstruction from tracks.
//  *Initialisation: from a segment (RcSegPoint or RcMatchPoint), an 
//          ECAL cluster, a rec. track(RTRA), or a set of parameters
//  *To find the likelihood for a given mass hypothesis call the function
//       FindLikelihood(int masshyp,RichHits *richhits);
//  * The class Richhits has to be initialised before in order to fill it 
//      with the Rich hits. (in future versions it will be automatic)
//
//
//**********************************************************************/


#ifndef _RICHRECO_HH_
#define _RICHRECO_HH_

//Marple
#include "marple/marple.hh"
#include "marple/RcMatchBuilder.hh"

#include <arte/arte.hh>

#include "CLHEP/Vector/ThreeVector.h"
#include "CLHEP/Matrix/SymMatrix.h"

//#include "rise/Corrections.h" //old
#include "rich/PhiLambda.h"

#include "rire/richhits.hh"

extern float Meanrefrindex;
extern float Nphotbeta1;


class RichTrack
{
public:
  //constructors
  RichTrack();
  RichTrack(RcSegPoint);
  RichTrack(RcMatchPoint);
  RichTrack(Hep3Vector point, Hep3Vector direction, float momentum );

  //constructor from cluster (straight line to target)
  RichTrack(ArtePointer<RCCL> aprccl);

  RichTrack(ArtePointer<RTRA> aprtra);

  //destructor
  ~RichTrack();
  
  void Readconstantsfromfile();
  void SetShadow();


  double NumExpPhotons(int masshyp);
  int NumBackPhotons(int masshyp);

  int NumCandidates(int masshyp, RichHits *richhits);

  void FindLikelihoods();
  double FindLikelihood(int masshyp,RichHits *richhits);

  double lnLike(vector<RHit*> VecCand, double nexpp,double a ,int ihyp, 
		double *d1lnlike,double *d2lnlike);

  inline double masshyp(int hyp)
  {
    switch(hyp)
      {
      case 1: return 0.000511;  //electron
      case 2: return 0.1056584; //muon
      case 3: return 0.1395699; //pion
      case 4: return 0.493677;  //kaon
      case 5: return 0.938272;  //proton
      default: return 0;
      }
  }	

  inline double ThetaCer(int ihyp)
    {
      //double beta=sqrt(1-pow(masshyp(ihyp)*track->pe(),2));
      double beta=1/sqrt(1+pow(masshyp(ihyp)*track->pe(),2));
      if(beta<1/Meanrefrindex) 
	return 0;
      else 
	return acos(1/(beta*Meanrefrindex));
    }

  inline Hep3Vector RichEntryPoint(){
    Hep3Vector entpoint(track->x(),track->y(),track->z());
    return entpoint;
    //track assumed propagated to entry point in the initialisation
  }

  inline Hep3Vector RichDir(){
    Hep3Vector dir(track->tx(),track->ty(),1);
    return dir.unit();
  }

  inline double SigmaPhiTrack(){ return sqrt(track->cov()(3,3))/
				   (1+track->tx()*track->tx());}
  inline double SigmaLambdaTrack(){ return sqrt(track->cov()(4,4))/
				      (1+track->ty()*track->ty());}
  inline double TrackPhi(){ return atan(track->tx());}
  inline double TrackLambda(){ return atan(track->ty());}

  inline RcSegPoint* Track(){ return track;}

  inline double Mom() { return fabs(1/track->pe());}

  //now is the same as RichDir, but will be the initial direction, 
  //when magnet is on
  inline Hep3Vector Dir(){
    Hep3Vector dir(track->tx(),track->ty(),1);
    return dir.unit();
  }

  double RelTheta(double hitphi,double hitlambda,int up=0);

  inline float Minazi(){return minazi;}
  inline float Maxazi(){return maxazi;}

  inline float ZRichMin(){return zRichmin;}
  inline float ZRichMax(){return zRichmin;}
  inline float ZMidRich(){return (zRichmax+zRichmin)/2;}
  inline float Radlength(){return zRichmax-zRichmin;}

  inline double Likelihood(int i) {return lnlikelihoods[i];}

private:
  RcSegPoint *track;
  double lnlikelihoods[5];

  //beam pipe shadow
  float minazi;
  float maxazi;

  //parameters to be read from ARTE tables
  float zRichmin;
  float zRichmax;

  //parameters to be read from a file
  //  double Meanrefrindex;  
  //  double Nphotbeta1;
  double Sigma_spherical;
  double Sigma_cromatic;
  double Sigma_cellsize;

  // corrections parameters
  //Corrections corr; //old
  
};

#endif







