//-----------------------------------------------------------------------//
//
//  RingSearchARTE: Program for stand alone RICH reconstruction
//
//  Author: Reinhard Eckmann
//
//  Created: May-98
//
//  All Files: RingSearchARTE.C, RingSearchARTE.h, ring.C, ring.h,
//             phila.C,  phila.h    
//
//-----------------------------------------------------------------------//

#include "rise/RingSearchARTE.h"
#include "rich/PhotonDetection.h"
#include "rich/PhiLambda.h"
#include "arte/RSEG_cmp.hh"
#include "CLHEP/Hist/HBookHistogram.h"


#include "rise/RICHFit.h"


extern "C" {

  void myhfithn_(int const &id,
		 int const &np,
		 float *par,
		 float const *step,
		 float const *pmin,
		 float const *pmax,
		 float *sigpar,
		 float &chi2);


  void myhbook1_(int   const &id,
		 int   const &nx,
		 float const &xmi,
		 float const &xma);

  void hfill_(int const &id,
	      float const &x,
	      float const &y,
	      float const &weight);

  void hprint_(int const &id);
  void hdelet_(int const &id);

  float hx_(int   const &id,
	    float const &x);

  void myhreset_(int const &id);

	   }



RingSearchARTE::RingSearchARTE() {

  // obsolete
  for (unsigned i=1;i<=ArteTable<GESL>::nrow();i++) {
    ArtePointer<GESL> gesl(i);
    if (gesl->cmp==12) {
      RichRun::magnet=gesl->mfl;
      break;
    }
  } 
  if (RichRun::magnet) printf("RingSearch with magnet ON\n");
  else printf("RingSearch with magnet OFF\n");

  for (ArteTable<GEDE>::iterator igede=ArteTable<GEDE>::begin();
       igede!=ArteTable<GEDE>::end(); ++igede) {
    if ((*igede)->cmp==5) {
      RichRun::gede=(*igede);
      break;
    }
  }

}
  
void RingSearchARTE::makeHitMap() {

  h.reset();

  unsigned int igede;
  static unsigned int rich_gede=0;
  int plane,imtra;

  if (!rich_gede) 
    for (igede=1;igede<=ArtePointer<GEDE>::nrow();igede++)
      if (ArtePointer<GEDE>(igede)->cmp==5) {
	rich_gede=igede;
	break;
      }

  if (!rich_gede) {
    fprintf(stderr,"RingSearchARTE: GEDE number for RICH not assigned\n\n");
    return;
  }

  ArtePointer<GEDE> gede(rich_gede);
  int n0=gede->hit1.index();
  int n1=gede->hitl.index();

  int i;
  for (i=n0;i<n1;i++) {

    ArtePointer<HITB> hitb(i);
    ArteRelationIterator<DIGB> idigb=hitb->digb.begin();
    ArtePointer<DIGB> digb(idigb);
    int id=digb->id;
    plane = (smFromChannelID(id)>6) ? 1 : 0 ;
    if (ArtePointer<MTRA>::nrow()) { // MC mode
      ArteRelationIterator<MIMP> imimp=hitb->mimp;
      ArtePointer<MTRA> mtra=(*imimp)->mtra;
      imtra = mtra.index();
    }
    else { // DAQ mode
      imtra=99999;
    }
    h.append(d.dualx(hitb->q*1000),d.dualy(hitb->r*1000),plane,imtra);
  }
  h.randomize(); 
  d.nextevent();
}


void RingSearchARTE::getrings(float r1,float r2,float dr,int t) {
  makeHitMap();
  if (dr<0)
    for (float radius=r2;radius>r1;radius+=dr) {// mrad /////
      d.define(radius,t);
      d.ringsearch(h);
    }
  else
    for (float radius=r1;radius<r2;radius+=dr) {// mrad /////
      d.define(radius,t);
      d.ringsearch(h);
    }
  reFit();
  FillRSEG();
}

void RingSearchARTE::getrings(double radius) {
  makeHitMap();
  d.define(radius,5);
  d.ringsearch(h);
  reFit();
  FillRSEG();
}


void RingSearchARTE::FillRSEG() {

  RSEG r;
  ArtePointer<RSEG>rseg;
  //cout<<"RingSearchARTE::FillRSEG() numrings="<<numrings()<<endl;
  for (int i=0;i<numrings();i++) {
    Ring &ring=getRing(i);

    //if (!ring.mctrack()) continue;

    r.xf=ring.phi();
    r.yf=ring.lambda();
    r.time=ring.thetaC();

    float beta=(1/1.00138/cos(ring.thetaC()));
    r.dtim=beta;

    r.txf=tx(i);
    r.tyf=ty(i);

    r.pf=float(ring.numHits());
    r.ze=ring.numhit_refit;
    r.ye=ring.numhit_bgrd;
    r.xe=ring.numhit_3sigma;
    r.zf=ring.f;
    //r.txe=ring.N;
    //r.tye=ring.mu;
    //r.pe=ring.sigma;

    r.chi=ring.chi2;

    r.cvf[ 0]=0;
    r.cvf[ 1]=0;
    r.cvf[ 2]=0;
    r.cvf[ 5]=ring.covariance(0,0);
    r.cvf[ 8]=ring.covariance(0,1);
    r.cvf[12]=ring.covariance(0,2);
    r.cvf[ 9]=ring.covariance(1,1);
    r.cvf[13]=ring.covariance(1,2);
    r.cvf[14]=ring.covariance(2,2);
    r.cvf[ 3]=0;
    r.cvf[ 6]=0;
    r.cvf[10]=0;
    r.cvf[ 4]=0;
    r.cvf[ 7]=0;
    r.cvf[11]=0;

    /*cout<<r.cvf[5]<<" "<<r.cvf[8]<<" "<<r.cvf[12]<<endl;
    cout<<r.cvf[8]<<" "<<r.cvf[9]<<" "<<r.cvf[13]<<endl;
    cout<<r.cvf[12]<<" "<<r.cvf[13]<<" "<<r.cvf[14]<<endl<<endl;;*/

    r.cmp = Rsegc::rich;

    if (ArtePointer<MTRA>::nrow()) r.fit = ring.mctrack();

    rseg=ArteTable<RSEG>::push_back(r);

    if (imtra(i)!=0 && ArtePointer<MTRA>::nrow())
      createRelation(rseg,ArtePointer<MTRA>(imtra(i)),&RSEG::mtra);
  }
}



void RingSearchARTE::reFit() {
;
}

RingSearchARTE *RICH_Search=0;



void rcrings_(float r1,float r2,float dr,int t) {
  if (!RICH_Search) RICH_Search=new RingSearchARTE;
  RICH_Search->getrings(r1,r2,dr,t);
}

