#ifndef _RICHFIT_H_
#define _RICHFIT_H_

#include <vector.h>


class Histo {
 private:
  int n;
  float min,max;
  float dx;
  float *m;
  float sum;
 public:
  inline Histo(int ni,float mini,float maxi) { 
    n=ni; 
    min=mini; 
    max=maxi; 
    m=new float[ni]; 
    memset(&m[0],0,sizeof(float)*n);
    dx=(max-min)/float(n);
    sum=0;
  }
  inline ~Histo() { delete m; }
  inline int &N()   { return n; }
  inline float &Min() { return min; }
  inline float &Max() { return max; }
  inline float  Sum() { return sum; } 
  inline void fill(float x,float weight=1) {
    if (x<min || x>=max) return;
    int bin=int((x-min)/dx);
    m[bin]+=weight;
    sum+=weight;
  }
  inline void setY(float x,float weight=1) {
    if (x<min || x>=max) return;
    int bin=int((x-min)/dx);
    sum+=weight-m[bin];
    m[bin]=weight;
  }
  inline void setY(int i,float weight=1) {
    if (i<0 || i>=n) return;
    sum+=weight-m[i];
    m[i]=weight;
  }
  inline float getY(int i) {
    if (i<0 || i>=n) {
      fprintf(stderr,"Histo: index out of range\n");
      return 0;
    }
    return m[i];
  }
  inline float getY(float x) {
    if (x<min || x>=max) {
      fprintf(stderr,"Histo: value out of range\n");
      return 0;
    }
    int i=int((x-min)/dx);
    return m[i];
  }
  inline float getX(int i) { return min+(i+.5)*dx; } 
  inline void print() { for (int i=0; i<n; i++) cout<<i<<" "<<m[i]<<endl; cout<<"Sum="<<sum<<endl; }
  inline void reset() { memset(&m[0],0,sizeof(float)*n); sum=0; }
};



class RICHFit {
private:

  Histo *histo;           // radius histogram
  int   nx;               // number of bins
  float xmi;              // lower histo limit
  float xma;              // upper histo limit
  float f;                // photons: (up-lo)/(up+lo)

  float sigma;            // ring sigma
  float tRad;             // ring radius
  float tPhi,tLam;        // ring center
  float chi2;             // ring chi2
  float n;                // ring photons
  float mu;               // ring radius shift
  float s;                // signal photons
  float s3s;              // signal photons inside 3 sigma
  float b;                // background photons


public:
  // --- constructor
  RICHFit(int nbin,float minx,float maxx);
  // --- destructor
  ~RICHFit();


  // --- fitting procedures
  void   fillRadiusHistogram(float tphi,float tlam,float trad);
  void   estimateSN();
  void   fitS();
  inline void resetRadiusHistogram() { histo->reset(); }
  inline void printRadiusHistogram() { histo->print(); }


  // --- parameters
  inline float  &Sigma()  { return sigma; }
  inline float  &Mu()     { return mu; }
  inline float  &Radius() { return tRad; }
  inline float  &Phi()    { return tPhi; }
  inline float  &Lambda() { return tLam; }
  inline float  &N()      { return n; }
  inline float  &Signal()       { return s; }
  inline float  &Signal3Sigma() { return s3s; }
  inline float  &Background()   { return b; }
  inline float  &Chi2()   { return chi2; } 
  inline float  &F()      { return f; } 
};


#endif







