#include <stdio.h>
#include <iostream.h>
#include <iomanip.h>
#include <strings.h>
#include <stdlib.h>

// some Arte stuff
#include "arte/arte.hh"
#include "farm/farm.hh"
#include "silarte/silarte.hh"
#include "silarte/EVinfo.hh"

// prototype declarations
#include "aframelib.h"
#include "vdsleda.h"
#include "silraw.h"
#include "marvin.h"

// return codes
#include "grr.h"

// sparse event definition
#include "event.h"
#include "esp.h"

// flag to indicate first call to SvdEvnt 
static int first = 1;

// keep record ID for SvdStop
int DSVDrecid = 0;
extern int DSVDrecid;


// size and location of sparsified event
#define NBUFF         32768     // size of event buffers
char sparsevt[NBUFF];


//-----------------------------------------------------------------------------

int SvdEvnt()
{
  char  *routine = "SvdEvnt";

  //
  // No SVD Digitization in case of MC  
  //
  if ( CheckMC() )
    return GRR_OK;

  //
  // REAL DATA
  //

  // get DSVD record id and return, if 0 (no SVD data there) 
  ARTE_get_recid_DSVD( &DSVDrecid );
  if ( silarte_iol.debug_mode.value )
    aframe_mess(routine,GRR_SINFO, "DSVD record id = 0x%x", DSVDrecid );
  if ( !DSVDrecid )
    return GRR_OK;

  // I'd really like to do this in SvdInit, but we don't have a run# there yet.
  // The same holds for the exp # in EVHD: Thats why we fill the geometry
  // here rather than in SvdInit. 
  if (first) {

    //    Get constants from database
    //
    // Time stamp is evaluated in this order:
    //  1. date/time   from vdsleda IOL
    //  2. major/minor from vdsleda IOL
    //  3. event time from DDAQ (if all IOL parameters are =0)
    ArtePointer<DDAQ> ddaq( 2 );
    EVT_DATA *p = (EVT_DATA *) &ddaq->bits;
    int major;
    ESP_get_int( &p->event_time, &major, 1 );
    int rc = vdsleda_startprogram( major, 0 );
    if ( rc != GRR_OK )
      return rc;

    // start silraw
    rc = silraw_startprogram();
    if ( rc != GRR_OK )
      return rc;

    // start marvin
    marvin_init();

    // fill GEDE table with geometry from file
    silarte_updateGEDE();

  }

  //
  // Now the actual event routines
  //

  // fill SILRAW structure and call marvin in case of raw data format
  if ( DSVDrecid == RID_NORMAL_SLB0 ) {

    // get whole event
    EVENT_HDR *event = ARTE_get_event();
    if ( !event ) {
      aframe_mess(routine,GRR_SERROR,"Could not find event!");
      return GRR_FISHY;
    }      
    
    // process event with silraw
    int rc = silraw_event( (char*) event, 0 );
    if ( rc != GRR_OK)
      return GRR_FISHY;

    // Do cluster finding
    Marvin_Sevt_t *sevt = marvin_fetch_sevt();
    if ( !sevt ) {
      aframe_mess(routine,GRR_SERROR,
		  "no Marvin_Sevt_t available (marvin not initialized?)");
      return GRR_FISHY;
    }
    sevt->data   = sparsevt; 
    sevt->size   = NBUFF;
    sevt->next   = sevt->data; 
    marvin_event();
  }

  // get SPARSEVT w/o calling marvin in case of sparsified data format
  else if ( DSVDrecid == RID_NORMAL_SPARSE ) {
    int rc = fillSPARSEVTfromDSVD();
    if ( rc != GRR_OK )  return GRR_FISHY;
  }

  // no valid DSVD record id
  else {
    aframe_mess(routine,GRR_SWARNING,
		"Cannot interprete DSVD recid 0x%x", DSVDrecid );
    first = 0;
    return GRR_FISHY;
  }
  
  // Fill HITB table
  fillHITBfromSPARSEVT();

  // we are definitely through the first event now 
  first = 0;

  return GRR_OK;

}
