#include <stdio.h>
#include <time.h>
#include <stdlib.h>

// Arte banks,  C++ version
#include "arte/arte.hh"
#include "silarte/silarte.hh"

#include "aframelib.h"
#include "marvin.h"

#include "event.h"      // for identification of sparse event
#include "grr.h"        // for error codes

/*-----------------------------------------------------------------
 *
 *  Read in sparse data from DSVD bank into marvin's SPARSEVT structure.
 *  (NO copying is done! The Marvin_Sevt_t structure just points to 
 *   the DSVD bank!)
 *
 *-----------------------------------------------------------------*/ 
int fillSPARSEVTfromDSVD()
{
  char *routine = "fillSPARSEVTfromDSVD";

  // get pointer to DVSD bank and check, if we have a sparsified event
  int DSVDrecid = *((int*) ArtePointer<DSVD>(1)->bits);
  if ( DSVDrecid != RID_NORMAL_SPARSE ) {
    aframe_mess(routine,GRR_SERROR,
		"Wrong data format, found recid = 0x%x", DSVDrecid);
    return GRR_FISHY;
  }

  // Dump out record id, if in debug mode
  if ( silarte_iol.debug_mode.value )
    aframe_mess(routine,GRR_SINFO,
		"Here we go.... DSVD recid = 0x%x", DSVDrecid);

  // get the sevt structure from marvin, if he has declared it already
  // If not: allocate space and declare the structure to marvin
  Marvin_Sevt_t *sevt = marvin_fetch_sevt();
  if ( !sevt ) {
    sevt = (Marvin_Sevt_t*) malloc( sizeof(Marvin_Sevt_t) );
    if ( !sevt ) {
      aframe_mess(routine,GRR_SERROR,
		  "Unable to allocate space for sparsified event.");
      return GRR_FISHY;
    }
    if ( silarte_iol.debug_mode.value )
      aframe_mess(routine,GRR_SINFO,
		  "Allocated space for sparsified event successfully." );

    // tell marvin about it
    marvin_declare_sevt( sevt );
  }

  // Fill Marvin_Sevt_t structure (i.e.: just set its pointers!)
  sevt->size = ArteTable<DSVD>::size() - 1; 
  sevt->data = ArtePointer<DSVD>(2)->bits;
  sevt->next = sevt->data + sevt->size;

  return GRR_OK;
}
