// --------------------------------------------------------------
//
//   fill VDS database table VDS_Side with GEDE table and store it
//
// --------------------------------------------------------------

#include <stdio.h>
#include <time.h>

#include <arte/arte.hh>
#include "silarte/silarte.hh"

#include "grr.h"

#include "aframelib.h"
#include "vdsleda.h"

// --------------------------------------------------------------
//
//   Store GEDE table in new VDS_Side record for time (major,minor)
//
// --------------------------------------------------------------
int silarte_store_VDS_Side( int major, int minor, int flag )
{
  // fill VDS_Side table from GEDE
  fillVDSSidefromGEDE( major, minor, flag );

  // Now store database
  vdsleda_store_sides( major, minor );

  return GRR_OK;
}


// --------------------------------------------------------------
//
//   Find last VDS_Side record with (major',minor') <= (major,minor)
//   and store GEDE table in new VDS_Side record for time (major',minor'+1)
//
// --------------------------------------------------------------
int silarte_update_VDS_Side( int major, int minor, int flag )
{
  // fill VDS_Side table from GEDE
  fillVDSSidefromGEDE( major, minor, flag );

  // Now update database
  vdsleda_update_sides( major, minor );

  return GRR_OK;
}


int fillVDSSidefromGEDE( int major, int minor, int flag )
{
  char *routine = "fillVDSSidefromGEDE";

  // check major/minor number
  if ( major == 0 && minor == 0 )
    major = time( NULL );

  // initialize Leda (if not yet done)
  vdsleda_param_init();
  vdsleda_startprogram( major, minor );

  // Now loop over ARTE GEDE tables
  for ( ArteTable<GEDE>::iterator igede = ArteTable<GEDE>::begin();
	igede != ArteTable<GEDE>::end(); igede++ ) {
    ArtePointer<GEDE> gede = *igede;
  
    if ( gede->cmp != 1 )
      continue;

    // loop over VDS_Side tables to get correspondend one
    int found = 0;
    for ( LedaTableIterator<class VDS_Side> side; side.valid(); side.next() ) {

      if (   strncmp( gede->gesl->name, side->slname, 4 ) == 0
	  && gede->lay == (int) side->lay
	  && gede->sec == (int) side->sec ) {

	// update VDS_Side with new alignment information
	// The ONLY variables are lateral movements and rotations!
	side->sxmn       = gede->sxmn;
	side->symn       = gede->symn;
	side->z          = gede->zfw[0];
	for ( int i=0; i<9; i++ )
	  side->rot[i]   = gede->rot[i];
	side->align_flag = flag;

	found = 1;
      }
    }
    if ( !found ) {
      char geslname[5];
      strncpy( geslname,  gede->gesl->name, 4 );
      geslname[4] = '\0';
      aframe_mess(routine,GRR_SWARNING,
		  "GEDE row %d (SL=%s, lay=%d, sec=%d) not found in VDS database", 
		  gede.index(), geslname, gede->lay, gede->sec );
    }  

  }  // end loop over GEDE row

  // debug print out if requested
  if ( silarte_iol.debug_mode.value ) {
    aframe_mess(routine,GRR_SINFO, "Updated VDS_Side table:" );
    vdsleda_dump_sides();
  }

  return GRR_OK;
}
