#include <iostream.h>
#include <stdlib.h>
#ifdef USE_ROOT
 #include "TProfile.h"
 #include "TH1.h"
 #include "TString.h"
#endif
#include "wax/OTrAsdBoard.hh"
#include "wax/OTrChannel.hh"

int OTrAsdBoard::fgNChannels=16 ;

OTrAsdBoard::OTrAsdBoard(): OTrDaqComponent(), fChannels() {} ;

OTrAsdBoard::OTrAsdBoard(ComponentId_t id) : 
  OTrDaqComponent(id), fChannels(fgNChannels)
{
  // cout << "OTrAsdBoard::OTrAsdBoard(...)" << endl ;
  for(int chanId=0; chanId<fgNChannels; chanId++) {
    fChannels[chanId] = OTrChannel(SetChannelId(GetId(),chanId)) ;
  };
};

OTrAsdBoard::~OTrAsdBoard() { 
  //cout << "OTrAsdBoard::~OTrAsdBoard() " << GetAsdId() << endl ;
} ;

// Now in OTrDaq.C ?!?
void
OTrAsdBoard::ProcessAsdBuffer(const OTrAsdBoardDataBuffer* data, 
			      OTrEvent* event)
{
  //  for(int chanId=0; chanId<GetNChannels(); chanId++) {
  for(unsigned int chanId=0; chanId<fChannels.size(); chanId++) {
    unsigned char tdcCount = *(data+chanId) ;
    fChannels[chanId].ProcessChannelBuffer(tdcCount,event) ;
  }
}

#ifdef USE_ROOT
TString OTrAsdBoard::GetName() const
{
  //cout << "OTrAsdBoard::GetName()" << endl;
  char name[12];
  sprintf(name,"Asd_%d.%d",GetTdcId(),GetAsdId()) ;
  return TString(name) ;
}
#endif

#ifdef USE_ROOT
TH1*
OTrAsdBoard::GetTimeH1() const 
{
  //cout << "OTrAsdBoard::GetTimeH1()" << endl;
  TString timeH1Name(GetName());
  timeH1Name.Append("::TimeH1") ;
  TH1* timeH1 = new TH1F(timeH1Name,timeH1Name,OTrChannel::GetNTimeBins(),
			 OTrChannel::GetMinimumTimeAxis(),
			 OTrChannel::GetMaximumTimeAxis()) ;
  for(ChannelContainer::const_iterator chanIt=fChannels.begin();
      chanIt!=fChannels.end(); chanIt++) {
    TH1D* histo = chanIt->GetTimeH1() ;
    timeH1->Add(histo,1) ;
    delete histo ;
  };
  return timeH1;
}
#endif

void OTrAsdBoard::Reset()
{
  for(ChannelContainer::iterator chanIt=fChannels.begin();
      chanIt!=fChannels.end(); chanIt++)
    chanIt->Reset() ;
}

/*
void OTrAsdBoard::Print() {
  cout << "I am " << fName << ". I am an OTrAsdBoard object." << endl;
  cout << fNHits[1] << " " << fAvTime[1] << endl ;
  TObject::Print() ;
};

void OTrAsdBoard::Reset(Bool_t resetMask)
{
  for(int chanId=0; chanId<GetNChannels() ; chanId++) {
    fNHits[chanId]=0;
    fAvTime[chanId]=0;
    fAvTime2[chanId]=0;
    if(resetMask) fMask[chanId]=0;
  };
  for(int timeBin=0; timeBin<GetNTimeBins(); timeBin++)
    fTimes[timeBin]=0;
};

UChar_t OTrAsdBoard::Fill(const OTrAsdBoardDataBuffer* data)
{
  //cout << fName << "::Fill(..)" << endl; 
  UChar_t nValidHits(0);
  for(int chanId=0; chanId<GetNChannels() ; chanId++)
    {
      UChar_t tdcValue = *(data + chanId) ;
      if(tdcValue >=GetMinimumTime() && tdcValue<GetMaximumTime())
	{
	  //cout << chanId << " " << (Int_t)tdcValue << " " ;
	  nValidHits++ ;
	  fNHits[chanId]++ ;
	  fAvTime2[chanId] = 
	    ((fNHits[chanId]-1)*fAvTime2[chanId]+tdcValue*tdcValue)
	    /fNHits[chanId] ;
	  fAvTime[chanId] = 
	    ((fNHits[chanId]-1)*fAvTime[chanId]+tdcValue)/fNHits[chanId] ;
	  //cout << fAvTime[chanId] << " " ;
	  if(!fMask[chanId]) {
	    fTimes[((tdcValue-GetMinimumTime())*GetNTimeBins())
		  /(GetMaximumTime()-GetMinimumTime())]++ ;
	    for(int chanId2=chanId+1; chanId2<GetNChannels() ; chanId2++)
	      if(!fMask[chanId2]) {
		UChar_t timeDif=  abs(tdcValue- *(data + chanId2)) ;
		fTimeDifs[((timeDif-GetMinimumTime())*GetNTimeBins())
		  /(GetMaximumTime()-GetMinimumTime())]++ ;
	      };
	  };
	};
    };
  //cout << endl ;
  return nValidHits ;
};

TProfile OTrAsdBoard::GetTimeProfile() 
{
  TString timeProfileName(GetName());
  timeProfileName.Append("::TimeProfile") ;
  TProfile
    timeProfile((char*)timeProfileName.Data(),(char*)timeProfileName.Data(),
		(Int_t)GetNChannels(),(Float_t)0,(Float_t)GetNChannels(),"") ;
  //TProfile timeProfile("","",GetNChannels(),0,GetNChannels(),"") ;
  for(Int_t bin=0; bin<GetNChannels(); bin++) {
    timeProfile.SetBinContent(bin+1,fAvTime[bin]);
    timeProfile.SetBinError(bin+1,
			    sqrt(fAvTime2[bin]-fAvTime[bin]*fAvTime[bin]));
  };
  return timeProfile;
};

TH1S OTrAsdBoard::GetOccupancyH1()
{
  TString occupancyH1Name(GetName());
  occupancyH1Name.Append("::OccupancyH1") ;
  TH1S occupancyH1(occupancyH1Name,occupancyH1Name,GetNChannels(),0,GetNChannels()) ;
  for(Int_t bin=0; bin<GetNChannels(); bin++) {
      occupancyH1.SetBinContent(bin+1,fNHits[bin]);
    };
  return occupancyH1 ;
};

TH1D OTrAsdBoard::GetTimeH1()
{
  TString timeH1Name(GetName());
  timeH1Name.Append("::TimeH1") ;
  TH1D timeH1(timeH1Name,timeH1Name,GetNTimeBins(),
	      GetMinimumTime(),GetMaximumTime()) ;
  for(Int_t bin=0; bin<GetNTimeBins(); bin++) {
    timeH1.SetBinContent(bin+1,(Double_t)fTimes[bin]);
  };
  return timeH1 ;
};

TH1D OTrAsdBoard::GetTimeDifH1()
{
  TString timeH1Name(GetName());
  timeH1Name.Append("::TimeDifH1") ;
  TH1D timeH1(timeH1Name,timeH1Name,GetNTimeBins(),
	      GetMinimumTime(),GetMaximumTime()) ;
  for(Int_t bin=0; bin<GetNTimeBins(); bin++) {
    timeH1.SetBinContent(bin+1,(Double_t)fTimeDifs[bin]);
  };
  return timeH1 ;
};

void OTrAsdBoard::Mask(Int_t nEvent)
{
  for(UChar_t chanId=0; chanId<GetNChannels(); chanId++) {
    const Int_t nHits = fNHits[chanId]; 
    fMask[chanId] = 100*(fNHits[chanId]/(Double_t)nEvent) ;
    cout << nHits << " " << fNHits[chanId] << " "
	 << nEvent << " " 
	 << 100*(fNHits[chanId]/(Double_t)nEvent) << " "
	 << fMask[chanId] << endl;
  };
};

// void OTrAsdBoard::Streamer(TBuffer &R__b)
// {
//    // Stream an object of class OTrAsdBoard.
//    if (R__b.IsReading()) {
//       Version_t R__v = R__b.ReadVersion(); if (R__v) { }
//       TObject::Streamer(R__b);
//       fName.Streamer(R__b);
//       fNHits.Streamer(R__b);
//       fTimes.Streamer(R__b);
//       fAvTime.Streamer(R__b);
//       fAvTime2.Streamer(R__b);
//       for(int chanId=0; chanId<GetNChannels(); chanId++) {
// 	R__b >> fMask[chanId] ;
//       };
//    } else {
//       R__b.WriteVersion(OTrAsdBoard::IsA());
//       TObject::Streamer(R__b);
//       fName.Streamer(R__b);
//       fNHits.Streamer(R__b);
//       fTimes.Streamer(R__b);
//       fAvTime.Streamer(R__b);
//       fAvTime2.Streamer(R__b);
//       for(int chanId=0; chanId<GetNChannels(); chanId++) {
// 	R__b << fMask[chanId] ;
//       };
//    }
// }

*/
