#include <math.h>
#ifdef USE_ROOT
#include <TH1.h>
#endif
#include "wax/OTrChannelHistogrammer.hh"
#include "wax/OTrChannel.hh"
#include "wax/OTrDaq.hh"
#include "wax/OTrNodeProjection.hh"
#include "wax/OTrReconstructedHit.hh"
#include "wax/OTrHit.hh"
#include "wax/OTrParameterizedRTRelation.hh"
#include "wax/OTrWire.hh"

//__________________________________________________________________
// OTrChannelHistogrammer
//
//
// Author: Ivar Siccama && Wouter Hulsbergen
//__________________________________________________________________


#ifdef __OTR_CLASSDEF__
ClassImp(OTrChannelHistogrammer) ;
//void OTrChannelHistogrammer::ShowMembers(TMemberInspector &, char *) {};
//void OTrChannelHistogrammer::Streamer(TBuffer &) {} ;
#endif

//int OTrChannelHistogrammer::fgNTimeBins=256 ;
int OTrChannelHistogrammer::fgNTimeBins=64 ;
int OTrChannelHistogrammer::fgMaximumTimeAxis= 256 ;
int OTrChannelHistogrammer::fgMinimumTimeAxis= 0 ;
int OTrChannelHistogrammer::fgTZeroUpdateFrequency = 100 ;

bool OTrChannelHistogrammer::fgTimeHistogramming=true;
int OTrChannelHistogrammer::fgHistogrammingMode=0 ;

OTrChannelHistogrammer&
OTrChannelHistogrammer::operator=(const OTrChannelHistogrammer& rhs) 
{
  fChannel = rhs.fChannel ;
  fNHit = rhs.fNHit ;
  fSumTdcCount = rhs.fSumTdcCount ;
  fSumTdcCount2 = rhs.fSumTdcCount2 ;
  if(rhs.fTimeBins) {
    //    if(!fTimeBins) fTimeBins = new UShort_t[fgNTimeBins] ;
    if(!fTimeBins) fTimeBins = new unsigned short[fgNTimeBins] ;
    for(int bin=0; bin<fgNTimeBins ; bin++)
      fTimeBins[bin] = rhs.fTimeBins[bin] ;
  } else {
    if(fTimeBins) {
      delete[] fTimeBins ;
      fTimeBins=0;
    };
  };
// #ifdef USE_ROOT
//   if(rhs.fTResidualH1) {
//     if(!fTResidualH1) {
//       fTResidualH1 = new TH1F(*rhs.fTResidualH1) ;
//     } else {
//       *fTResidualH1 = *rhs.fTResidualH1 ;
//     }
//   } else {
//     if(fTResidualH1) {
//       delete fTResidualH1 ;
//       fTResidualH1=0;
//     };
//   };
// #endif //USE_ROOT
  return *this ;
} ;

OTrChannelHistogrammer::OTrChannelHistogrammer() 
  : fChannel(0), fNHit(0),fSumTdcCount(0),fSumTdcCount2(0),fTimeBins(0),
    fDChi2DTZero(0),fD2Chi2DTZero2(0),fNNodeP(0),fChi2(0),fSumWeight(0),
    fSumTResidual(0),fSumTResidual2(0)
// #ifdef USE_ROOT
//   ,fTResidualH1(0)
// #endif
{} ;

OTrChannelHistogrammer::OTrChannelHistogrammer(const OTrChannelHistogrammer& rhs) 
  : fChannel(0), fNHit(0),fSumTdcCount(0),fSumTdcCount2(0),fTimeBins(0) ,
    fDChi2DTZero(0),fD2Chi2DTZero2(0),fNNodeP(0),fChi2(0),fSumWeight(0),
    fSumTResidual(0),fSumTResidual2(0)
// #ifdef USE_ROOT
//   ,fTResidualH1(0)
// #endif
{
  operator=(rhs) ;
} ;


OTrChannelHistogrammer::OTrChannelHistogrammer(OTrChannel* channel) :
  fChannel(channel),fNHit(0),fSumTdcCount(0),fSumTdcCount2(0),fTimeBins(0),
  fSumWeight(0),fSumTResidual(0),fSumTResidual2(0)
{
  if(fgTimeHistogramming) {
    //fTimeBins = new UShort_t[fgNTimeBins] ;
    fTimeBins = new unsigned short[fgNTimeBins] ;
    for(int i=0; i<fgNTimeBins; i++) fTimeBins[i]=0;
  } ;
// #ifdef USE_ROOT
//   TString name = GetName() ;
//   name.Prepend("TResidualH1") ;
//   fTResidualH1 = new TH1F(name,name,50,-25,25) ;
// #endif
} ;

OTrChannelHistogrammer::~OTrChannelHistogrammer() 
{
  if(fTimeBins) delete fTimeBins ;
  //if(fTResidualH1) delete fTResidualH1 ;
} ;

void 
OTrChannelHistogrammer::Reset()
{
  //cout << " reset channel " << flush;
  fNHit=0; fSumTdcCount=0; fSumTdcCount2=0 ;
  if(fgTimeHistogramming) {
    for(int i=0; i<fgNTimeBins; i++) fTimeBins[i]=0;
  };
  fNNodeP = 0 ;
  fDChi2DTZero = fD2Chi2DTZero2 = fChi2 = 0 ;
  //fSumWeight = fSumTResidual = fSumTResidual2 = 0 ;
  //if(fTResidualH1) fTResidualH1->Reset() ;
};
 
void
OTrChannelHistogrammer::PerformHistogramming( const TdcCount_t count )
{
  if(fgTimeHistogramming) {
    int bin = int(0.5 + GetNTimeBins()*(int(count)-GetMinimumTimeAxis())/
		  (double)(GetMaximumTimeAxis()-GetMinimumTimeAxis())) ;
    if(bin>=0 && bin<fgNTimeBins) fTimeBins[bin]++ ;
  } 
} 

void
OTrChannelHistogrammer::ProcessNodeProjection(const OTrNodeProjection* node )
{
  // if(node->GetReconstructedHit()->GetMeasuredHit()->GetChannel()==GetChannel()) {
  float residual = node->GetResidualT() ;
  float weight   = 1/node->GetSigmaT2() ;
  if(fabs(node->GetD()) < 0.7*GetChannel()->GetRTRelation()->GetRMax()
     && residual*residual*weight < 2) {
    fChi2          += weight*residual*residual ;
    fDChi2DTZero   += -2*weight*residual ;
    fD2Chi2DTZero2 += 2*weight ;
    fNNodeP++ ;
    fSumWeight     += weight ;
    fSumTResidual  += weight*residual ;
    fSumTResidual2 += weight*residual*residual ;
  } 
  //if(fNNodeP==fgTZeroUpdateFrequency) UpdateTZero() ;
} 

float
OTrChannelHistogrammer::UpdateTZero()
{
  float rv = -1000 ;
  if(fNNodeP>=fgTZeroUpdateFrequency) {
    float dtzero = - fDChi2DTZero/ fD2Chi2DTZero2 ;
    float tzero_new_value = 
      GetChannel()->GetTZero().GetValue() + dtzero ;
    float tzero_new_sigma = sqrt(2/fD2Chi2DTZero2) ;
    GetChannel()->SetTZero(OTrChannel::TZero_t(tzero_new_value,tzero_new_sigma)) ;
    rv = dtzero ;
    Reset() ;
  }
  return rv ;
} ;

float 
OTrChannelHistogrammer::GetOccupancy() const {
  float occup=0;
  int n;
  if((n=GetNProcessed())>0) occup = GetNHit()/double(n) ;
  return occup ;
} ;

float OTrChannelHistogrammer::GetWidthTResidual() const {
  float w2 = GetAverageTResidual2()-GetAverageTResidual()*GetAverageTResidual() ;
  if(w2>0) return sqrt(w2) ;
  return 0;
  //return sqrt(GetAverageTResidual2()-GetAverageTResidual()*GetAverageTResidual()) ;
}

float
OTrChannelHistogrammer::GetSigma() const {
  float sigma=0;
  if(GetNHit()>1)
    sigma = sqrt( float(GetNHit())/(GetNHit()-1) * 
		  (GetAverageTime2() - GetAverageTime()*GetAverageTime())
		  ) ;
  return sigma ;
} ;

int OTrChannelHistogrammer::GetNProcessed() const {
  // niet zo netjes, maar het werkt ...
  return GetChannel()->GetDaq()->GetTdcBoard(GetChannel()->GetTdcId())->GetNProcessedEvents() ;
}

int OTrChannelHistogrammer::GetTZeroBin() const {
  return int(GetChannel()->GetTZero().GetValue()*fgNTimeBins/
	     double(GetMaximumTimeAxis()-GetMinimumTimeAxis())) ; 
};

float 
OTrChannelHistogrammer::GetAverageFromTimeH1() const
{
  float average = 0;
  int ntot = 0;
  int tot = 0;
  for (int bin=0; bin<fgNTimeBins ; bin++ ) {
    int count = Bin2Count(bin);
    int n = fTimeBins[bin] ;
    ntot += n;
    tot  += count*n;
  }
  if ( ntot > 0 ) average = float(tot)/float(ntot);
  //cout << " av = " << average << " " 
  //   << GetAverageTime() << flush;
  return average;
}


#ifdef USE_ROOT

TString OTrChannelHistogrammer::GetName() const
{
  //return GetChannel()->GetName() ;
  char name[20];
  sprintf(name,"Wire_%d",GetChannel()->GetWire()->GetId() ) ;
  return TString(name) ;
}

TH1D* OTrChannelHistogrammer::GetTimeH1() const 
{
  if(!fgTimeHistogramming) {
    cout << "ERROR in OTrChannel::GetTimeH1() : fgTimeHistogramming == false"
	 << endl;
    return 0 ;
  }
  TString timeH1Name(GetName());
  timeH1Name.Append("_TimeH1") ;
  TH1D* timeH1 = new TH1D(timeH1Name,timeH1Name,GetNTimeBins(),
			  GetMinimumTimeAxis(),GetMaximumTimeAxis()) ;
  for(int bin=0; bin<GetNTimeBins(); bin++) {
    timeH1->SetBinContent(bin+1,(Float_t)(fTimeBins[bin]));
    timeH1->SetBinError(bin+1,sqrt((Float_t)(fTimeBins[bin]))) ;
  };
  return timeH1 ;
}

TH1D* OTrChannelHistogrammer::GetTimeH1_B( float tshift ) const 
{
  if(!fgTimeHistogramming) return 0;
  TString timeH1Name(GetName());
  timeH1Name.Append("::TimeH1") ;
  TH1D* timeH1 = new TH1D(timeH1Name,timeH1Name,GetNTimeBins(),
			  GetMinimumTimeAxis(),GetMaximumTimeAxis()) ;

  int binshift = 0;
  if ( tshift != 0 ) {
    float binwidth = float(fgMaximumTimeAxis-fgMinimumTimeAxis)/
                     float(fgNTimeBins);
    binshift = int(tshift / binwidth ) ;
  }

  for(int bin=0; bin<GetNTimeBins(); bin++) {
    //cout << bin << " " << fTimeBins[bin] << flush ;
    int newbin = bin+1 + binshift;
    if ( newbin > 0 && newbin < GetNTimeBins() ) {
      timeH1->SetBinContent(newbin,(float)fTimeBins[bin]);
      timeH1->SetBinError(newbin,sqrt((float)fTimeBins[bin])) ;
    }
  };
  return timeH1 ;
};

// TH1D OTrChannel::GetBinominalCorrectedTimeH1() const 
// {
//   if(!fgTimeHistogramming) {
//     cout << "ERROR in OTrChannel::GetTimeH1() : fgTimeHistogramming == false"
// 	 << endl;
//     return TH1D() ;
//   }
//   TString timeH1Name(GetName());
//   timeH1Name.Append("::BinominalCorrectedTimeH1") ;
//   TH1D timeH1 = TH1D(timeH1Name,timeH1Name,GetNTimeBins(),
// 		     GetMinimumTimeAxis(),GetMaximumTimeAxis()) ;
//   int N = 0; // total number of entries (== `tries')
//   for(int bin=0; bin<GetNTimeBins(); bin++) N += fTimeBins[bin] ;
//   for(int bin=0; bin<GetNTimeBins(); bin++) {
//     float nprime = N*(fTimeBins[bin]+1)/float(N+2) ;
//     float sigmaprime = N*
//       sqrt((N+1)*(N+1-fTimeBins[bin])/float((N+2)*(N+2)*(N+3))) ;
//     timeH1.SetBinContent(bin+1,nprime) ;
//     timeH1.SetBinError(bin+1,sigmaprime) ;
//   };
//   return timeH1 ;
// };

#endif //USE_ROOT

