#include "wax/OTrGede.hh"

#ifdef USE_ROOT
#include <TBRIK.h>
#include <TNode.h>
#endif

//__________________________________________________________________
// OTrGede
//
// Container for:
// * pointer to arte GEDE
//
// Functionality:
// * geometry decoding
//
// In the near future this class wil be `reduced' to a pointer to a
// GEDE/NEDE structure (ARTE or standalone) and maybe some alignment
// constants. I would also like to rename it :-)
//
// Author: Wouter Hulsbergen && Ivar Siccama
//__________________________________________________________________

#ifdef __OTR_CLASSDEF__
ClassImp(OTrGede) ;
//void OTrGede::Streamer(TBuffer &) {} ;
//void OTrGede::ShowMembers(TMemberInspector &, char *) {} ;
#endif

const float OTrGede::fgLayerPitch[2] = {0.500, 1.000} ;
const float OTrGede::fgWirePitch[2]  = {0.870, 1.740} ;
//const float OTrGede::fgStereoAngle[3] = {-0.08000,0.00000,0.08000 } ;

#ifndef USE_ARTE
#define NEW_GEOMETRY_FILE
#endif

OTrGede::~OTrGede()
{
  //  cout << "**** deleting an OTrGede object *******" << endl;
#ifdef USE_ROOT
  if(fNode) delete fNode ;
  if(fShape) delete fShape ;
  if(fTRotMatrix) delete fTRotMatrix ;
  if(fWireTRotMatrix) delete fWireTRotMatrix ;
#endif
#ifndef USE_ARTE
  if(fGEDE) delete fGEDE ;
#endif
}

OTrGede::OTrGede() 
  : fGEDE(0), fGEDEIndex(0), fSuperLayerId(0)
#ifdef USE_ROOT
  ,fShape(0),fNode(0),fTRotMatrix(0),fWireTRotMatrix(0)
#endif
{
}

const OTrGede& OTrGede::operator=(const OTrGede& rhs)
{
#ifdef USE_ARTE
  fGEDE = rhs.fGEDE ;
#else
  if(rhs.fGEDE) {
    if(!fGEDE) fGEDE = new GEDE ;
    *fGEDE = *rhs.fGEDE ;
  } else if(fGEDE) {
    delete fGEDE ;
    fGEDE = 0;
  };
#endif
  fGEDEIndex = rhs.fGEDEIndex ;
  fSuperLayerId = rhs.fSuperLayerId ;
  fSuperLayerName = rhs.fSuperLayerName ;
  fEStereoAngle = rhs.fEStereoAngle ;
  fEPitch = rhs.fEPitch ;
  fEXSide = rhs.fEXSide ;
#ifdef USE_ROOT
  if(rhs.fNode) CreateNode() ;
#endif
  return *this ;
}

OTrGede::OTrGede( const OTrGede& rhs ) :
  fGEDE(0), fGEDEIndex(0), fSuperLayerId(0)
#ifdef USE_ROOT
  ,fShape(0),fNode(0),fTRotMatrix(0),fWireTRotMatrix(0)
#endif
{
  operator=(rhs) ;
}


#ifdef USE_ARTE
OTrGede::OTrGede( ArtePointer<GEDE> g ) :
  fGEDE(g), fGEDEIndex(g.index()), fSuperLayerId(0)
#ifdef USE_ROOT
  ,fShape(0),fNode(0),fTRotMatrix(0),fWireTRotMatrix(0)
#endif
{
  fSuperLayerName = ArtePointer<GESL>(GetGEDE()->gesl) -> name;
  //  fSuperLayerId = gede()->gesl.index();
  SetSuperLayerId();
  DetermineStereoAndPitch();
}
#endif  //USE_ARTE

OTrGede::OTrGede( istream& is ) :
  fGEDE(0), fGEDEIndex(0), fSuperLayerId(0)
#ifdef USE_ROOT
  ,fShape(0),fNode(0),fTRotMatrix(0),fWireTRotMatrix(0)
#endif
{ 
  is >> fGEDEIndex;    
  if ( fGEDEIndex < 0 ) {
    cout << "ERROR in OTrGede: fGEDEIndex = " << fGEDEIndex << endl;
    exit(1);
  }
#ifdef USE_ARTE
  fGEDE = ArtePointer<GEDE>( fGEDEIndex) ;
#else
  fGEDE = new GEDE ;
#endif
  is >> fGEDE->cmp;
  is >> fGEDE->lay;
  is >> fGEDE->shp;
  is >> fGEDE->xmin >> fGEDE->xmax ;
  is >> fGEDE->ymin >> fGEDE->ymax ;
  is >> fGEDE->zmin >> fGEDE->zmax ;
  is >> fGEDE->sxmn >> fGEDE->sxmx ;
  is >> fGEDE->symn >> fGEDE->symx ;
  is >> fGEDE->cos  >>  fGEDE->sin ;
  for ( int i = 0; i < 9; i++ ) is >> fGEDE->rot[i];
  is >> fGEDE->acc >>  fGEDE->eff ;
  is >> fGEDE->dod ;
  is >> fGEDE->pit ;
  is >> fGEDE->ms ;
  for ( int i = 0; i < 4; i++ ) is >> fGEDE->xfw[i];
  for ( int i = 0; i < 4; i++ ) is >> fGEDE->zfw[i];
  for ( int i = 0; i < 4; i++ ) is >> fGEDE->nwi[i];
  is >> fGEDE->nsl ;
#ifdef USE_ARTE
  int gesl, gmat ;
  is >> gesl >> gmat ;
  fGEDE->gesl= ArtePointer<GESL>(gesl) ;
  fGEDE->gmat= ArtePointer<GMAT>(gmat) ;
#else
  is >> fGEDE->ogesl >> fGEDE->ogmat ;
#endif
  is >> fGEDE->nwl ;
  is >> fGEDE->fwl ;
  is >> fGEDE->xmad >> fGEDE->ymad >> fGEDE->zmad;

  for(  int i = 0; i < 4; i++ ) is >> fGEDE->y1w[i];
  for(  int i = 0; i < 4; i++ ) is >> fGEDE->y2w[i];
 
  // is >> gvol;
  // is >> hwc;
  is >> fGEDE->dedx ;
  is >> fGEDE->gcop ;

  is >> fGEDE->sec ;
  is >> fSuperLayerName ;
#ifdef NEW_GEOMETRY_FILE
  is >> fGEDE->hwc ;
#endif

  SetSuperLayerId();
  DetermineStereoAndPitch();
  if ( !is ) { cout << "OTrGede: input stream is at end" << endl; }
}

void OTrGede::DetermineStereoAndPitch()
{
  // Set EStereoAngle
  if(GetSinStereoAngle() > 0.050 /*rad*/) fEStereoAngle = kPlusFiveDegrees ;
  else if(GetSinStereoAngle() < -0.050) fEStereoAngle = kMinusFiveDegrees ;
  else fEStereoAngle = kZeroDegrees ;
  // Set EPitch
  if(GetLayerPitch() < 1.0/*cm*/) fEPitch = kFiveMM ;
  else fEPitch = kTenMM ;
  // Set ESide
  if(GetGEDE()->xfw[0] < fGEDE->xfw[1]) fEXSide = kXSideMinus ;
  else fEXSide = kXSidePlus ;
}

#ifdef USE_ROOT
int
OTrGede::CreateNode()
{ 
  if(fShape) {
    delete fShape ;
    delete fNode ;
    delete fTRotMatrix ;
    delete fWireTRotMatrix ;
  };
  char brikname[20];
  sprintf(brikname,"%s%d","brik_",GetGEDEIndex());
  fShape = new TBRIK( brikname, brikname,"void", 
		      ( GetGEDE()->xmax - GetGEDE()->xmin )/2,
		      ( GetGEDE()->ymax - GetGEDE()->ymin )/2,
		      ( GetGEDE()->zmax - GetGEDE()->zmin )/2 );
  char name[20];
  sprintf(name,"%s%d","gede_",GetGEDEIndex());
  fNode = new TNode( name, name, fShape);
  //fNode->Draw();

  // The root, geant and arte matrix are defined in the same
  // way. However, the ROOT matrix is of type `double*', so we have to
  // copy it. Unfortunately we then have two instants of the rotation
  // matrix !!
  char rotname[20]; sprintf(rotname,"%s%d","gederotation_",fGEDEIndex);
  double doubleRot[9] ; // copy of fRot in doubles
  for(int i=0; i<9; i++) doubleRot[i]=GetGEDE()->rot[i] ;
  fTRotMatrix = new TRotMatrix(rotname,rotname,doubleRot) ;
  fNode->SetMatrix(fTRotMatrix);
  fNode->SetLineColor(1);//+(int)fEStereoAngle);
  float xco = ( GetGEDE()->xmax + GetGEDE()->xmin )/2 ;
  float yco = ( GetGEDE()->ymax + GetGEDE()->ymin )/2 ;
  fNode->SetPosition( +xco*GetGEDE()->rot[0] + yco*GetGEDE()->rot[1],
		      -xco*GetGEDE()->rot[1] + yco*GetGEDE()->rot[0],
		      ( GetGEDE()->zmax + GetGEDE()->zmin )/2 );
  
  //The TTube is normaly oriented in z (which is quite logical!). We
  //therefore need actually two rotations to turn the wire in the
  //direction of the gede:
  float rotationOfAxis[9] = {0,0,-1,1,0,0,0,-1,0} ; 
  // The matrix multiplication fWireTRotMatrix=doubleRot*rotationOfAxis
  fWireTRotMatrix = new TRotMatrix(rotname,rotname,doubleRot) ;
  for(int i=0; i<9; i++) {
    fWireTRotMatrix->GetMatrix()[i] = 0 ;
    int icol=i/3; int irow=i%3 ;
    for(int k=0; k<3; k++)
      fWireTRotMatrix->GetMatrix()[i] += doubleRot[irow+k*3]*rotationOfAxis[k+icol*3] ;
  };
  return 0;
}
#endif

int OTrGede::SetSuperLayerId() 
{
  int id = 0;
  const char* namechar = GetSuperLayerName().c_str() ;
  if     (!strncmp(namechar,"MC01",4)) id = 1; 
  else if(!strncmp(namechar,"MC02",4)) id = 2; 
  else if(!strncmp(namechar,"PC03",4) || !strncmp(namechar,"PC04",4)  ) id = 31; // MSL1
  else if(!strncmp(namechar,"TC01",4)) id = 32 ; // MSL2
  else if(!strncmp(namechar,"TC02",4)) id = 33;
  else {
    cout << "** ERROR in OTrGede::GetSuperLayerId(), name = " 
	 << GetSuperLayerName() << endl; 
    exit(1);
  }
  fSuperLayerId = id;
  return id;
}

int OTrGede::determineSection() const
{
  // some ARTE GEDE sections are "unphysical", they are used in GEANT
  // they can easily recognized, since they are very short: 
  // ... and they have no monolayers !! (wouter)
  if ( GetNMonoLayers()<1 || GetPMax().y() - GetPMin().y()< 10 ) { return -2; }

#ifdef NEW_GEOMETRY_FILE
  char* sulName = "000";
  int layid = atoi(GetGEDE()->hwc+4) ;
  int modid = atoi(GetGEDE()->hwc+6) ;
  int secid = atoi(GetGEDE()->hwc+9) ;
  //cout << GetGEDE()->hwc << " " << layid << " " 
  //     << secid << " " << modid << " |  " << layerId() << " " << endl ;
  if(secid != GetSectorNumber())
    cout << "Error in geometry file: sector numbers not equal ! " << endl ;
  int s = secid*100 + modid ;
#else
  
  // October 1998
  // if(superLayerId()==14) xtmp += 30.6 ; //cm
  // 17 november 1998
  //if(superLayerId()==14) xtmp += 11.6 ; //cm
  //if(superLayerId()==15) xtmp += 26.6 ; //cm
  //double xtmp = GetMean().x() ; 
  //if(superLayerId()==31) xtmp += 11.6 ; //cm    used to be SL11
  //if(superLayerId()==32) xtmp += 26.6 ; //cm    used to be SL12

  float xfw = GetGEDE()->xfw[0] ;
  if(superLayerId()==31) xfw += 11.6 ; //cm    used to be SL11
  if(superLayerId()==32) xfw += 26.6 ; //cm    used to be SL12
  if(superLayerId()==33) xfw += 35   ; 
  
  int s = GetSectorNumber()*100;
  if(GetSectorNumber()==1) {
    if(-47 < xfw && xfw < -40) 
      s += 2 ;
    else if(-20 < xfw && xfw < -10)
      s += 4 ;
    else if( 10 < xfw && xfw <  12)
      s += 5 ;
    else if( 14 < xfw && xfw <  20)
      s += 6 ;
    else
      cout << "OTrGede::determineSection: strange sector at strange position :) "
	   << xfw << endl ;
  } else if(GetSectorNumber()==2) {
    if(-47 < xfw && xfw < -40) 
      s += 2 ;
    else if(-20 < xfw && xfw < -10)
      s += 3 ;
    else if( 10 < xfw && xfw <  12)
      s += 5 ;
    else if( 14 < xfw && xfw <  20)
      s += 6 ;
    else
      cout << "OTrGede::determineSection: strange sector at strange position :) "
	   << xfw << endl ;
  }
  // not needed anymore
  if ( (GetSectorNumber()==11 || GetSectorNumber()==12) ) {
    if ( GetMean().y() < -10 ) {
      s = 100;
    } else {
      s = 90;
    }
  }
#endif
  return s;
}

void OTrGede::Translate( const ThreeVector& p )
{
  GetGEDE()->xmin += p.x() ; 
  GetGEDE()->xmax += p.x() ; 
  GetGEDE()->ymin += p.y() ;
  GetGEDE()->ymax += p.y() ; 
  GetGEDE()->zmin += p.z() ;
  GetGEDE()->zmax += p.z() ; 

  for ( int i=0; i < 4; i++ ) {
    GetGEDE()->xfw[i] += GetCosStereoAngle()*p.x() - GetSinStereoAngle()*p.y() ;
    GetGEDE()->y1w[i] += GetSinStereoAngle()*p.x() + GetCosStereoAngle()*p.y() ;
    GetGEDE()->y2w[i] += GetSinStereoAngle()*p.x() + GetCosStereoAngle()*p.y() ;
    GetGEDE()->zfw[i] += p.z();
  };
}

void OTrGede::LongDump() const
{
  cout << "GEDE index = " << GetGEDEIndex() << " " 
       << GetSuperLayerName() << endl;
  cout << "sl = " << superLayerId() 
       << " lay = " << layerId() 
       << " section = " << determineSection() << endl; 
  cout << "pmin = " << GetPMin() << " pmax = " << GetPMax() << endl;
  cout << "wire pitch = " << GetEPitch() << " " << GetWirePitch() << endl;
  cout << "stereo angle = " << GetEStereoAngle() << endl;
  cout << "nr of mono layers = " << GetNMonoLayers() << endl;
  cout << " cos = " << GetGEDE()->cos << " sin = " << GetGEDE()->sin << endl;
  /*
  cout << "index       : " << GetGedeIndex() << endl;
  cout << "cmp         : " << fGEDE->cmp << endl;
  cout << "shape       : " << fGEDE->shp  << endl;
  cout << "xmin        : " << fGEDE->xmin << " xmax        : " << fGEDE->xmax << endl;
  cout << "ymin        : " << fGEDE->ymin << " ymax        : " << fGEDE->ymax << endl;
  cout << "zmin        : " << fGEDE->zmin << " zmax        : " << fGEDE->zmax << endl;
  cout << "cos, sin = " << fGEDE->cos << " " << fGEDE->sin << endl;
  for ( int i = 0; i < 9; i++ ) {
    cout << fGEDE->rot[i] << " ";
  }
  cout << endl;
  cout << "acc, eff         = " << fGEDE->acc << " " << fGEDE->eff << endl;
  cout << "nsl              = " << fGEDE->nsl << endl;
  cout << "fSuperLayerId = " << fSuperLayerId << " gmat = "<< fGEDE->gmat << endl;
  cout << "xmad, ymad, zmad = " << fGEDE->xmad << " " 
       << fGEDE->ymad << " " << fGEDE->zmad << endl;
  cout << "y1w = " << fGEDE->y1w[0] << " " << fGEDE->y1w[1] << " "  
       << fGEDE->y1w[2] << " " << fGEDE->y1w[3] << endl;
  cout << "y2w = " << fGEDE->y2w[0] << " " << fGEDE->y2w[1] << " "  
       << fGEDE->y2w[2] << " " << fGEDE->y2w[3] << endl;
   cout << "dedx        : " << fGEDE->dedx << endl;
  cout << "gcop        : " << fGEDE->gcop << endl;
  cout << "sec         : " << fGEDE->sec << endl;
  cout << "name        : " << fSuperLayerName << endl;
  getchar();
  */
}

ostream & operator << (ostream& os, const OTrGede& g)
{ 
  os << " gede = " << g.GetGEDEIndex() 
     << " " << g.superLayerId() 
     << " " << g.layerId() 
     << " " << g.determineSection() << endl;
  //  os << " pmin = " << g.GetPMin() << " pmax = " << g.GetPMax();
  return os;
}

/***** obsolete **********/
// int OTrGede::determineSection() const
// {
//   // some ARTE GEDE sections are "unphysical", they are used in GEANT
//   // they can easily recognized, since they are very short: 
//   // ... and they have no monolayers !! (wouter)
//   if ( GetNMonoLayers()<1 || GetPMax().y() - GetPMin().y()< 10 ) { return -2; }

//   double xtmp = GetMean().x() ;

//   // October 1998
//   // if(superLayerId()==14) xtmp += 30.6 ; //cm

//   // 17 november 1998
//   //if(superLayerId()==14) xtmp += 11.6 ; //cm
//   //if(superLayerId()==15) xtmp += 26.6 ; //cm

//   if(superLayerId()==31) xtmp += 11.6 ; //cm    used to be SL11
//   if(superLayerId()==32) xtmp += 26.6 ; //cm    used to be SL12

//   /* -- obsolete --
//   if ( xtmp < -40 ) {
//     if ( y() < -10 ) {
//       s = 100;
//     } else {
//       s = 90;
//     }
//   } 
//   if ( xtmp > -40 && xtmp < -10 ) {
//     if ( y() > 0 ) { 
//       if ( y() >  15 ) { 
// 	s = 10; 
//       } else {
// 	s = 50;
//       }
//     }
//     if ( y() < 0 ) {
//       if ( y() < -15 ) {
// 	s = 20; 
//       } else {
// 	s = 60;
//       }
//     }
//   } 
//   if ( xtmp > -5 ) {
//     if ( y() > 0 ) {
//       s = 15;
//     } else {
//       s = 25;
//     }
//   }
//   */
//   int s = GetSectorNumber()*10;
//   if ( xtmp > -5  ) {
//     if ( GetMean().y() > 0 ) {
//       s = 15;
//     } else {
//       s = 25;
//     }
//   } 
//   if ( (GetSectorNumber()==11 || GetSectorNumber()==12 ) && xtmp < -40 ) {
//     if ( GetMean().y() < -10 ) {
//       s = 100;
//     } else {
//       s = 90;
//     }
//   } 

//   /*  -- obsolete --
//   if ( s <= 0 ) {
//     cout << "*** ERROR in ATGEDE::determineSection = " << s << endl;
//     cout << "x,y,z = " << x() << " " << y() << " " << z() << endl;
//   }
//   */
//   return s;
// }
