#ifdef USE_ARTE
#include <algorithm>
#include <CLHEP/Matrix/Matrix.h>
#include "arte/RSEG_cmp.hh"
#include "wax/OTrHit.hh"
#include "wax/OTrWire.hh"
#endif
#include "wax/OTrReconstructedTrackInterface.hh"
#include "wax/OTrReconstructedHit.hh"

float 
OTrReconstructedTrackInterface::fgMinChi2Probability=1 ;

OTrReconstructedTrackInterface::OTrReconstructedTrackInterface() 
  : fHits(), fChi2(0), fError(0), fRSEG(0)
{
  return ;
} ;

OTrReconstructedTrackInterface::OTrReconstructedTrackInterface(const OTrReconstructedTrackInterface& rhs) 
  : fHits(), fChi2(0), fError(0), fRSEG(0)
{
  OTrReconstructedTrackInterface::operator=(rhs) ;
  return ;
};

OTrReconstructedTrackInterface::~OTrReconstructedTrackInterface()
{
  EraseHits() ;
};

OTrReconstructedTrackInterface& 
OTrReconstructedTrackInterface::operator=(const OTrReconstructedTrackInterface& rhs) {
  OTrTrackDescription::operator=(rhs) ;
  fChi2  = rhs.GetChi2() ;
  fError = rhs.GetError() ;
  fRSEG  = rhs.GetRSEG() ;
  // delete old hits
  EraseHits() ;
  // create copies
  for(HitContainer::const_iterator rhsHitIt=rhs.fHits.begin() ;
      rhsHitIt!=rhs.fHits.end() ; rhsHitIt++)
    fHits.push_back(new OTrReconstructedHit(**rhsHitIt)) ;
  return *this ;
};

void
OTrReconstructedTrackInterface::EraseHits() 
{
  for(HitContainer::const_iterator hitIt=fHits.begin() ;
      hitIt!=fHits.end() ; hitIt++) delete *hitIt ;
  fHits.erase(fHits.begin(),fHits.end()) ;
}
const OTrReconstructedHit* 
OTrReconstructedTrackInterface::FindHit(const OTrHit* hit) const 
{
  const OTrReconstructedHit* rv(0) ;
  for(HitContainer::const_iterator hitIt=fHits.begin() ;
      hitIt!=fHits.end() ; hitIt++)
    if((*hitIt)->GetMeasuredHit()==hit) {
      rv = *hitIt ;
      break ;
    };
  return rv ;
} ;

#ifdef USE_ARTE

inline bool 
CompareReconstructedHits(const OTrReconstructedHit* lhs,
			 const OTrReconstructedHit* rhs)
{
  return lhs->GetMeasuredHit()->GetWire()->GetRawZ() >
    rhs->GetMeasuredHit()->GetWire()->GetRawZ() ;
} ;

ArtePointerRSEG
OTrReconstructedTrackInterface::AppendRSEG()
{
 // Append this reconstructed track to the RSEG bank.
  if(fRSEG.index()==0) {
    RSEG newrseg ;
    fRSEG = ArteTable<RSEG>::push_back(newrseg) ;
    AppendRSEG(fRSEG) ;
  }
  return fRSEG ;
}

void
OTrReconstructedTrackInterface::AppendRSEG(ArtePointerRSEG rseg)
{
  // Append this reconstructed track to given RSEG entry rseg.
  //cout << "OTrReconstructedTrackInterface::AppendRSEG()" << endl ; 
  //cout << "This track: " << GetX() << " " << GetY() << " "
  //     << GetTx() << " " << GetTy() << " " << GetDim() << endl ;

  sort(fHits.begin(), fHits.end(), CompareReconstructedHits) ;

  float dzf = fHits.back()->GetMeasuredHit()->GetWire()->GetRawZ()-GetZ() ;
  rseg->xf = GetX() + dzf*GetTx() ;
  rseg->yf = GetY() + dzf*GetTy() ;
  rseg->zf = GetZ() + dzf ;
  float dze = fHits.front()->GetMeasuredHit()->GetWire()->GetRawZ()-GetZ() ;
  rseg->xf = GetX() + dze* GetTx() ;
  rseg->yf = GetY() + dze* GetTy() ;
  rseg->zf = GetZ() + dze ;
  rseg->txe = rseg->txf = GetTx();
  rseg->tye = rseg->tyf = GetTy() ;

  // THE COVARIANCE MATRICES
  HepMatrix H(5,GetDim(),0) ; // transformation to hb state vector
  switch(GetDim())             // only non-zero elements:
    {
    case 2:
      H[0][0] = dzf ;
      H[1][1] = dzf ;
      H[2][0] = 1;
      H[3][1] = 1;
      break ;
    case 4:
      H[0][0] = 1; H[0][2] = dzf ;
      H[1][1] = 1; H[1][3] = dzf ;
      H[2][2] = 1;
      H[3][3] = 1;
      break;
    default:
      cout << "Error in OTrReconstructedTrackInterface::AppendRSEG(): no transf.matrix !" 
	   << endl ; 
    }
  HepSymMatrix hbCov = GetCov().similarity(H) ;int i=0;
  for(int row=0; row<hbCov.num_row(); row++)
    for(int col=0; col<=row; col++, i++)
      rseg->cvf[i] = hbCov[row][col] ;
  
  switch(GetDim())             // only changes
    {
    case 2:
      H[0][0] = dze ;
      H[1][1] = dze ;
    default: ;
    }
  hbCov = GetCov().similarity(H) ; i=0;
  for(int row=0; row<hbCov.num_row(); row++)
    for(int col=0; col<=row; col++, i++)
      rseg->cve[i] = hbCov[row][col] ;

  // Other elements
  //rseg->cmp = 2 ;   // that's us
  rseg->cmp = Rsegc::wch ; // I think that that is us ...
  rseg->fit = 0 ; if(GetDim()==2) rseg->fit = 5 ;
  rseg->chi = GetChi2() ;
  rseg->pe  = rseg->pf  = 1; //1E-6 ;

  // create the rhits and set relations
  for(HitContainer::iterator hitIt=fHits.begin();
      hitIt!=fHits.end(); hitIt++) {
    ArtePointer<RHIT> rhit = (*hitIt)->AppendRHIT(this) ;
    //cout << rhit << " " << rhit->hitb << endl ;
    createRelation(rseg,rhit,&RSEG::rhit) ;       // rseg<->rhit
    createRelation(rseg,rhit->hitb,&RSEG::hitb) ; // rseg<->hitb
  }
}
#endif
