#include <iomanip.h>
#include "OTrLayer.hh"
#include "OTrHit.hh"
#include "OTrGede.hh"
#include "OTrEvent.hh"
#include "OTrSection.hh"
#include "OTrAlignmentProjection.hh"

#ifdef USE_ROOT
 #include <TH1.h>
 #include <TPad.h>
#endif

//__________________________________________________________________
// OTrSection
//
// Container for:
// * OTrWires
// Functionality:
// * OTrSection is derived from OTrGede ... :-)
//
// Author: Wouter Hulsbergen && Ivar Siccama
//__________________________________________________________________

#ifdef __OTR_CLASSDEF__
ClassImp(OTrSection) ;
//void OTrSection::Streamer(TBuffer& b) {} ;
//void OTrSection::ShowMembers(TMemberInspector &, char *) {} ;
#endif

bool OTrSection::fgAlignmentOn=true ;

OTrSection::OTrSection( OTrGede* gede, OTrLayer* lay, 
			const ComponentId_t id ) :
  OTrGede(*gede), OTrDetectorComponent(id),  
  fWires(), fLayer(lay), fAlignmentProjection(0)
{
  //CreateWires(); create the wires later, when the pointer to section is fixed
  //cout << "OTrSection, creating a section with id " << GetId() << endl ;
  //  fHitList = list<OTrHit*>();
} 

OTrSection::~OTrSection() 
{ 
} ;

OTrSection::OTrSection() :
  OTrGede(), OTrDetectorComponent(), 
  fWires(), fLayer(0), fAlignmentProjection(0)
{
  cout << "** OTrSection::OTrSection(): do not call !!" << endl ;
  //fHitList = list<OTrHit*>();
}

//OTrSection::OTrSection(OTrLayer* layer, ComponentId_t id) :
//OTrDetectorComponent(id), OTrGede(), fWires(), fLayer(layer),  
//fSectorPlot(0)
//{
//CreateWires();
  //cout << "1: creating a section with id " << GetId() << endl ;
  //fHitList = list<OTrHit*>();
//}

void OTrSection::CreateWires()
{
  for ( int w = 0; w < GetNWires(); w++ ) {
    fWires.push_back( OTrWire(w, this, SetWireId(GetId(),w)) ) ;
  }
  if(fgAlignmentOn) 
    fAlignmentProjection = new OTrAlignmentProjection(this) ;
}

// const OTrWire*
// OTrSection::FindWire( const int wirenumber ) const 
// {
//   const OTrWire* w = 0;
//   for ( vector<OTrWire>::const_iterator wireIt = GetWires().begin() ;
//         wireIt != GetWires().end() ; wireIt++) {
//     //cout << wireIt->GetWireNumber() << " ";
//     if ( wireIt->GetWireNumber() == wirenumber ) { 
//       w = wireIt;
//       break;
//     }
//   } 
//   return w;
// }

#ifdef USE_ROOT

TString OTrSection::GetName() const
{
  char name[16];
  sprintf(name,"Sec_%.3d%.3d%.3d",GetSuperLayerId(),GetLayerId(),GetSectionId()) ;
  return TString(name) ;
}

void 
OTrSection::DumpBadChannels( ostream& os ) 
{
  os << "superlayer: " << GetSuperLayerId() << " "
     << "layer: " << GetLayerId() << " " 
     << "module: " << GetModuleId() << " "
     << "sector: " << GetSectorId() << " " 
     << "section: " << GetSectionId() << " " 
     << endl;
  for ( WireContainer::const_iterator wire = GetWires().begin();
	wire != GetWires().end(); wire++ ) {
    OTrChannel* chan = (*wire).GetChannel();
    if ( chan ) {
      float occ = chan->GetOccupancy();
      if ( occ < 0.01 || occ > 0.25 ) {
	os << (*wire).GetId() << " " << setprecision(3) 
	   << (*wire).GetChannel()->GetOccupancy() << " "; 
      }
    }
  }
  os << endl;
}
#endif

ostream & operator << (ostream & os, const OTrSection& s )
{
  return os << "section: " << s.GetId() << " " << s.GetSectionId();
}
