#include <iostream.h>
#ifdef USE_ROOT
#include "TH1.h"
#endif
#include "wax/OTrTdcBoard.hh"
#include "wax/OTrFedBuffer.hh"

//__________________________________________________________________
// OTrTdcboard
//
// Container for:
// * OTrAsdBoard
// Functionality:
// * 
//
// Author: Wouter Hulsbergen && Ivar Siccama
//__________________________________________________________________

#ifdef __OTR_CLASSDEF__
ClassImp(OTrTdcBoard) ;
#endif //USE_ROOT

const int OTrTdcBoard::fgDefaultNAsdBoards = 8 ;

OTrTdcBoard::OTrTdcBoard() 
  : OTrDaqComponent(), fAsds(), fNProcessedEvents(0) 
{
  cout << "OTrTdcBoard::OTrTdcBoard() : I do not think you want to use this." << endl ;
}

OTrTdcBoard::OTrTdcBoard(ComponentId_t id) 
  : OTrDaqComponent(id), fAsds(fgDefaultNAsdBoards), fNProcessedEvents(0) 
{
  for(int asdId=0; asdId<fgDefaultNAsdBoards; asdId++)
    fAsds[asdId] = OTrAsdBoard(SetAsdId(GetId(),asdId)) ;
}

OTrTdcBoard::~OTrTdcBoard() {
  //cout << "OTrTdcBoard::~OTrTdcBoard()" << endl ;
}

void
OTrTdcBoard::ProcessFedBuffer(const OTrFedBuffer& fb, OTrEvent *event)
{
  fNProcessedEvents++;
  for(int asdId=0; asdId<fgDefaultNAsdBoards; asdId++)
    fAsds[asdId].
      ProcessAsdBuffer(fb.GetData() + OTrAsdBoard::GetNChannels()*asdId,
		       event) ;
  fNProcessedEvents++ ;
}

void
OTrTdcBoard::Reset()
{
  for(int asdId=0; asdId<fgDefaultNAsdBoards; asdId++) fAsds[asdId].Reset() ;
  fNProcessedEvents=0 ;
}

#ifdef USE_ROOT
TString OTrTdcBoard::GetName() const {
  char name[8];
  sprintf(name,"Tdc_%d",GetTdcId()) ;
  return TString(name) ;
}

TH1*
OTrTdcBoard::GetTimeH1() const
{
  TString timeH1Name(GetName());
  timeH1Name.Append("::TimeH1") ;
  AsdConstIterator asdIt(*this) ;
  TH1* timeH1 = (*asdIt).GetTimeH1() ;
  timeH1->SetName(timeH1Name) ; 
  for(asdIt.next(); asdIt.valid(); asdIt.next()) {
    TH1* histo = (*asdIt).GetTimeH1() ;
    timeH1->Add(histo,1) ;
    delete histo ;
  }
  return timeH1;
}
#endif
