// ROOT event display
#ifdef USE_ROOT

#include <iostream.h>
#include <iomanip.h>
#include <CLHEP/Geometry/Point3D.h>
typedef HepPoint3D Point3D ;
#include <TControlBar.h>
#include <TButton.h>
#include <TView.h>
#include <TPaveText.h>
#include <TPaveLabel.h>
#include <TNode.h>
#include <TTUBE.h>
#include <TBRIK.h>

#include "wax/OTrView.hh"
#include "wax/OTrEvent.hh"
#include "wax/OTrDetector.hh"

#define DEFAULT_ZOOM 70
#define ZBLOWUP 1
#define XCENTER  -20
#define YCENTER    0
#define ZCENTER  270

OTrView*  gWaxView=0 ;

OTrView::OTrView(OTrDetector* detector) 
  : fWaxNode(0), fWaxPad(0),
    fViewMode(END), fDrawAllViews(false),
    fCanvas(0), fDisplayPad(0),
    fButtonsPad(0), fEvent(0),
    fLongitude(-90), fLatitude(0), fPsi(0), fZoomMode(false),
    fZoomFactor(1.2)
{
  if(gWaxView) {
    cout << "There is already an WaxView object !!" << endl ;
    return ;
  }
  gWaxView = this;
  CreateNode(detector) ;

  fCanvas = new TCanvas("waxcanvas","Wax display",200,40,800,740);
  fCanvas->SetEditable(kIsNotEditable);
  //fCanvas->SetFillColor(50);

  fDisplayPad = new TPad("viewpad", "Wax display",0.15,0,1,1,-1);
  fDisplayPad->Draw();
  fDisplayPad->Modified();
  //fDisplayPad->SetFillColor(1);
  //fDisplayPad->SetPhi(-90-m_Phi);
  //fDisplayPad->SetTheta(90-m_Theta);
  //fDisplayPad->Update();
  //gWaxPad = fDisplayPad;

  DisplayButtons();

  fDisplayPad->cd();
  //fView = new TView(1);
  //fRangeMin = new Point3D(-55,-35,235);
  //fRangeMax = new Point3D(15,35,305);

  //  fView->Draw(); 
  // Initialize display default parameters 
  SetGeometry(DEFAULT_ZOOM);
  
  //Setup();
  //fView->SetRange( fRangeMin->x(), fRangeMin->y(), fRangeMin->z(),
  //		   fRangeMax->x(), fRangeMax->y(), fRangeMax->z() );
  //int irep;
  //waxview->SetView(90,-90,0,irep); // view from above (or down)
  DrawView( fLongitude, fLatitude, fPsi );
  //  fView->SetView( fLongitude, fLatitude, fPsi, irep); 

  //CreateMenuBar();

  DrawBeamPipe();

  fDisplayPad->cd();
}

void
OTrView::CreateNode(OTrDetector* detector)
{  
  float hallsize = 750;
  TShape* hall = new TBRIK("HALL","HALL","void",hallsize,hallsize,hallsize);
  fWaxNode = new TNode("waxnode","waxnode",hall);
  fWaxNode->SetPosition( 0, 0, 0 );
  fWaxNode->cd(); 
  //fWaxNode->Draw("same"); 
  fWaxNode->cd();
  // now create TNodes for all sections
  for ( vector<OTrSuperLayer>::iterator slIt =  detector->GetSuperLayers().begin();
	slIt != detector->GetSuperLayers().end(); slIt++ )
    for(OTrSuperLayer::LayerIterator lIt(*slIt) ; lIt.valid(); lIt.next())
      for(OTrLayer::SectionIterator sIt(*lIt) ; sIt.valid() ; sIt.next())
	(*sIt).CreateNode();
}

void
OTrView::Display(OTrEvent* event)
{
  if(event) {
    fEvent=event ;
    fEventNumber = fEvent->GetId() ;
    //TPad* pad = fWaxView->GetDisplayPad();
    //pad->cd();
    fWaxNode->cd();
    fEvent->DrawHits() ;
    fEvent->DrawClusters() ;
    fWaxNode->Modified();
    Draw();
    //fEvent->DrawTrackCandidates( pad );
    //fWaxNode->Draw("same");
    Refresh();  
  }
}

OTrView::~OTrView()
{ 
  if ( fButtonsPad ) {
    fButtonsPad->Delete();
  }
  if ( fCanvas ) { 
    TCollection::StartGarbageCollection();
    TCollection::GarbageCollect(fCanvas);
    fCanvas->Delete(); 
    TCollection::EmptyGarbageCollection();
  }
}

void OTrView::Refresh() const
{
  fCanvas->Modified();
  fCanvas->Update();
  fDisplayPad->cd();
}

int OTrView::Zoom( const float z )
{
  //cout << "zooming factor " << z << endl;
  fXin *= z;
  fYin *= z;
  fZmin *= z;
  fZmax *= z;
  TView *view = fDisplayPad->GetView();
  if (view) {
    view->SetRange(fXcenter-fXin,fYcenter-fYin,fZcenter-fZmin,
		   fXcenter+fXin,fYcenter+fYin,fZcenter+fZmax);
  }
  fDisplayPad->Modified();
  return 0;
}

int OTrView::SetRange( const float x1, const float y1, const float z1, 
		       const float x2, const float y2, const float z2 )
{
  //cout << "zooming factor " << z << endl;
  fXcenter = (x1 + x2)/2;
  fYcenter = (y1 + y2)/2;
  fZcenter = (z1 + z2)/2;
  fXin =  (x2 - x1 )/2;
  fYin =  (y2 - y1 )/2; 
  fZmin = (z2 - z1 )/2;
  fZmax = fZmin;

  Draw();
  fDisplayPad->Modified();
  return 0;
}

void OTrView::SetAngles( const float longitude, const float latitude,
			const float psi )
{
  fLongitude = longitude;
  fLatitude = latitude;
  fPsi = psi;
}

void OTrView::SetStandardView(char mode)
{
  //  change viewing angles for current event

  fDisplayPad->cd();
  fDrawAllViews = kFALSE;
  
  SetGeometry(DEFAULT_ZOOM);
  
  switch(mode) {
  case 't' : 
    SetAngles(90,90,90);
    fViewMode = TOP;
    break;
  case 'b' :
    SetAngles(270,90,90); 
    fViewMode = BOT;
    break;
  case 'r' : 
    SetAngles(180,90,90);
    fViewMode = RSIDE;
    break;
  case 'l' : 
    SetAngles(180,270,90);
    fViewMode = LSIDE;
    break;
  case 'e' : 
    SetAngles(180,0,90);
    fViewMode = END;
    break;
  case 'a' :
    fDrawAllViews = kTRUE;
    fViewMode = ALL;
    break;
  } 

  TView *view = fDisplayPad->GetView();
  if (view) { 
    int iret;
    view->SetView( fLongitude, fLatitude, fPsi, iret);
    SetGeometry(DEFAULT_ZOOM);
    Draw();
  }
  else      Draw();
  
  fDisplayPad->SetPhi(-90-fLongitude);
  fDisplayPad->SetTheta(90-fLatitude);
  
  gPad->Modified();
  fDisplayPad->Modified();
}

int
OTrView::DrawBeamPipe()
{
  float radius = 2;
  float thickness = 0.1;
  float length = 500;
  TTUBE* BeamPipeTube = new TTUBE("beampipe_tube","beampipe_tube","void",
				 (radius-thickness/2),
				 (radius+thickness/2),
				  length*0.5);    
  double theta = 0*180/M_PI;
  double phi   = 0*180/M_PI;
  TRotMatrix* BeamPipeMatrix = new TRotMatrix("beampipe_rot",
					     "beampipe_rot",
					     90+theta,phi,
					     90,90+phi,
					     theta,phi);

  float zstart = 200;
  TNode* BeamPipeNode = new TNode("beampipe_node","beampipe_node",
				 BeamPipeTube, 0, 0 ,(zstart+length)/2,
		 	         BeamPipeMatrix ); 
  BeamPipeNode->SetLineColor(1);
  return 0;
}

 
/*
int OTrView::CreateMenuBar()
{
  fMenuBar = new TControlBar("vertical");
  fMenuBar->AddButton("large range",
		      "gWaxView->SetRange(-100,-100,250,100,100,900);",
		      " ");
  fMenuBar->AddButton("MC2",
		     "gWaxView->SetRange(-55,-35,235,15,35,315);",
		     " ");
  fMenuBar->AddButton("MSL1",
		     "gWaxView->SetRange(-85,-35,755,-15,35,825);",
		      " ");
  fMenuBar->AddButton("30 degree view",
		      "gWaxView->SetView(0,30,0);",
		      " ");
  fMenuBar->AddButton("Bot",
		      "gWaxView->SetStandardView('b');",
		      " ");
  fMenuBar->AddButton("Top",
		      "gWaxView->SetStandardView('t');",
		      " ");
  fMenuBar->AddButton("Left",
		      "gWaxView->SetStandardView('l');",
		      " ");
  fMenuBar->AddButton("Right",
		      "gWaxView->SetStandardView('r');",
		      " ");
  fMenuBar->AddButton("End",
		      "gWaxView->SetStandardView('e');",
		      " ");
  
		      //fMenuBar->AddButton("xy view",
		      //"gWaxView->SetView(-90,0,0);",
		      //" ");
		      //fMenuBar->AddButton("top view",
		      //"gWaxView->SetView(90,-90,0);",
		      //" ");
		      
  fMenuBar->AddButton("zoom in",
		     "gWaxView->Zoom(0.8);",
		      " ");
  fMenuBar->AddButton("zoom out",
		     "gWaxView->Zoom(1.25);",
		      " ");
  fMenuBar->Show();
  return 0;
} 
*/

void OTrView::ToggleZoomMode() 
{ 
  fZoomMode = ! fZoomMode; 
}

void OTrView::DisplayButtons()
{
//    Create the user interface buttons

   fButtonsPad = new TPad("buttons", "buttons",0,0.05,0.15,1);
   fButtonsPad->Draw();
   fButtonsPad->SetFillColor(38);
   fButtonsPad->SetBorderSize(2);
   fButtonsPad->cd();

   Int_t butcolor = 33;
   Float_t dbutton = 0.04;
   Float_t y  = .99;
   Float_t dy = 0.005;
   Float_t x0 = 0.05;
   Float_t x1 = 0.95;

   TButton *button;
   //   char *but1 = "theApp->Terminate(0)";
   char *but1 = "otr.next()";
   button = new TButton("Next",but1,x0,y-dbutton,x1,y);
   button->SetFillColor(38);
   button->Draw();
   y -= dbutton +dy;
   /*
   char *but2 = "gWaxView->ShowNextEvent(-1)";
   button = new TButton("Previous",but2,x0,y-dbutton,x1,y);
   button->SetFillColor(38);
   button->Draw();
   */
   char *but2 = "gWaxView->SetRange(-100,-100,250,100,100,900)";
   button = new TButton("large range",but2,x0,y-dbutton,x1,y);
   button->SetFillColor(38);
   button->Draw();

   y -= dbutton +dy;
   char *but3 = "gWaxView->SetStandardView('b')";
   button = new TButton("Bot",but3,x0,y-dbutton,x1/2,y);
   button->SetFillColor(butcolor);
   button->Draw();
   char *but3a = "gWaxView->SetStandardView('t')";
   button = new TButton("Top",but3a,x1/2,y-dbutton,x1,y);
   button->SetFillColor(butcolor);
   button->Draw();

   y -= dbutton +dy;
   char *but4 = "gWaxView->SetStandardView('l')";
   button = new TButton("Left",but4,x0,y-dbutton,x1/2,y);
   button->SetFillColor(butcolor);
   button->Draw();
   char *but4a = "gWaxView->SetStandardView('r')";
   button = new TButton("Right",but4a,x1/2,y-dbutton,x1,y);
   button->SetFillColor(butcolor);
   button->Draw();

   y -= dbutton +dy;
   char *but5 = "gWaxView->SetStandardView('e')";
   button = new TButton("End View",but5,x0,y-dbutton,x1,y);
   button->SetFillColor(butcolor);
   button->Draw();

   y -= dbutton +dy;
   char *but6 = "gWaxView->DrawAllViews()";
   button = new TButton("All Views",but6,x0,y-dbutton,x1,y);
   button->SetFillColor(butcolor);
   button->Draw();

   y -= dbutton +dy;
   //   char *but7 = "gWaxView->DrawViewGL()";
   //button = new TButton("OpenGL",but7,x0,y-dbutton,x1,y);
   char *but7 = "gWaxView->SetRange(-55,-35,235,15,35,315)";
   button = new TButton("MC2",but7,x0,y-dbutton,x1,y);
   button->SetFillColor(38);
   button->Draw();

   y -= dbutton +dy;
   char *but8 = "gWaxView->SetRange(-85,-35,755,-15,35,825)";
   button = new TButton("MSL1",but8,x0,y-dbutton,x1,y);
   //char *but8 = "gWaxView->DrawViewX3D()";
   //button = new TButton("X3D",but8,x0,y-dbutton,x1,y);
   button->SetFillColor(38);
   button->Draw();

   y -= dbutton +dy;
   char *but9 = "gWaxView->Delete()";
   button = new TButton("QUIT",but9,x0,y-dbutton,x1,y);
   button->SetFillColor(38);
   button->Draw();

   y -= dbutton +dy;
   y -= dbutton +dy;
   char *but10 = "gWaxView->ToggleDisplayAttribute('a')";
   button = new TButton("Axis",but10,x0,y-dbutton,x1,y);
   button->SetFillColor(38);
   button->Draw();

   y -= dbutton +dy;
   char *but11 = "gWaxView->ToggleDisplayAttribute('t')";
   button = new TButton("Tracks",but11,x0,y-dbutton,x1,y);
   button->SetFillColor(38);
   button->Draw();

   y -= dbutton +dy;
   char *but12 = "gWaxView->ToggleDisplayAttribute('c')";
   button = new TButton("Chambers",but12,x0,y-dbutton,x1,y);
   button->SetFillColor(38);
   button->Draw();

   y -= dbutton +dy;
   char *but13 = "gWaxView->ToggleDisplayAttribute('w')";
   button = new TButton("Wires",but13,x0,y-dbutton,x1,y);
   button->SetFillColor(38);
   button->Draw();

   y -= dbutton +dy;
   char *but14 = "gWaxView->ToggleDisplayAttribute('h')";
   button = new TButton("Hits",but14,x0,y-dbutton,x1,y);
   button->SetFillColor(38);
   button->Draw();

   y -= dbutton +dy;
   char *but15 = "gWaxView->ToggleDisplayAttribute('+')";
   button = new TButton("+",but15,x0,y-dbutton,x1/2,y);
   button->SetFillColor(38);
   button->Draw();
   char *but15a = "gWaxView->ToggleDisplayAttribute('-')";
   button = new TButton("-",but15a,x1/2,y-dbutton,x1,y);
   button->SetFillColor(38);
   button->Draw();

   y -= dbutton +dy;
   char *but16 = "gWaxView->ToggleDisplayAttribute('^')";
   button = new TButton("^",but16,x1/4,y-dbutton,3*x1/4,y);
   button->SetFillColor(38);
   button->Draw();

   y -= dbutton +dy;
   char *but17 = "gWaxView->ToggleDisplayAttribute('<')";
   button = new TButton("<",but17,x0,y-dbutton,x1/2,y);
   button->SetFillColor(38);
   button->Draw();
   char *but17a = "gWaxView->ToggleDisplayAttribute('>')";
   button = new TButton(">",but17a,x1/2,y-dbutton,x1,y);
   button->SetFillColor(38);
   button->Draw();

   y -= dbutton +dy;
   char *but18 = "gWaxView->ToggleDisplayAttribute('v')";
   button = new TButton("v",but18,x1/4,y-dbutton,3*x1/4,y);
   button->SetFillColor(38);
   button->Draw();

   y -= dbutton +dy;
   char *but19 = "gWaxView->ToggleZoomMode()";
   button = new TButton("ZOOM",but19,x0,y-dbutton,x1,y);
   button->SetFillColor(38);
   button->Draw();

   y -= dbutton +dy;
   char *but20 = "gWaxView->ToggleDisplayAttribute('U')";
   button = new TButton("UP",but20,x0,y-dbutton,x1/2,y);
   button->SetFillColor(38);
   button->Draw();
   char *but20a = "gWaxView->ToggleDisplayAttribute('D')";
   button = new TButton("DN",but20a,x1/2,y-dbutton,x1,y);
   button->SetFillColor(38);
   button->Draw();

   y -= dbutton +dy;
   char *but21 = "gWaxView->ToggleDisplayAttribute('A')";
   button = new TButton("all",but21,x0,y-dbutton,x1,y);
   button->SetFillColor(38);
   button->Draw();

}

void OTrView::DrawViewGL()
{
  // Draw current view using OPENGL

  fDisplayPad->cd();
  TView *view = fDisplayPad->GetView();
  if (!view) return;
  fDisplayPad->x3d("OPENGL");
}

void OTrView::DrawViewX3D()
{
  // Draw current view using X3D

  fDisplayPad->cd();
  TView *view = fDisplayPad->GetView();
  if (!view) return;
  fDisplayPad->x3d();
}

void OTrView::DrawAllViews()
{
//    Draw end,top,side and 30 deg views

   fDrawAllViews = kTRUE;
   fViewMode = ALL;
   fDisplayPad->cd();
   //fDisplayPad->SetFillColor(15);
   fDisplayPad->Clear();
   fDisplayPad->Divide(2,2);

   // draw 30 deg view
   fDisplayPad->cd(1);
   DrawView(180, 30, 90);
   char title[256];
   int eventnr = 123;
   sprintf(title,"Event %d",eventnr);
   DrawTitle(title);

   // draw end view
   fDisplayPad->cd(2);
   DrawView(180,  0,90);
   DrawTitle("End");

   // draw top view
   fDisplayPad->cd(3);
   DrawView( 90, 90, 90);
   DrawTitle("Top");

   // draw side view
   fDisplayPad->cd(4);
   DrawView(180, 90, 90);
   DrawTitle("Side");

   fDisplayPad->cd(2);
}

/*
void OTrView::DrawEvent()
{
  if ( !fDrawAllViews ) {
    fDisplayPad->cd();  Draw();
  } else {
    fDisplayPad->cd(1); Draw(); 
    fDisplayPad->cd(2); Draw(); 
    fDisplayPad->cd(3); Draw(); 
    fDisplayPad->cd(4); Draw();      
  }  
}
*/

void OTrView::DrawTitle(Option_t *option)
{
  //    Draw the event title

  Float_t xmin = gPad->GetX1();
  Float_t xmax = gPad->GetX2();
  Float_t ymin = gPad->GetY1();
  Float_t ymax = gPad->GetY2();
  Float_t dx   = xmax-xmin;
  Float_t dy   = ymax-ymin;
  if (strlen(option) == 0) {
    TPaveText *title = new TPaveText(xmin +0.01*dx, ymax-0.09*dy, xmin +0.5*dx, ymax-0.01*dy);
    title->SetBit(kCanDelete);
    title->SetFillColor(42);
    title->Draw();
    char ptitle[100];
    title->AddText(ptitle);
    title->AddText(ptitle);
  } else {
    TPaveLabel *label = new TPaveLabel(xmin +0.01*dx, ymax-0.07*dy, xmin +0.2*dx, ymax-0.01*dy,option);
    label->SetBit(kCanDelete);
    label->SetFillColor(42);
    label->Draw();
  }
}

void OTrView::DrawView(float theta, float phi, float psi)
{
  //    Draw a view 

  //gPad->SetFillColor(1);
  // Display outline
  gPad->Clear();
  
  TView *view = new TView(1);
  /*
  view->SetRange(fXcenter-fXin,fYcenter-fYin,fZcenter-fZmin,
		 fXcenter+fXin,fYcenter+fYin,fZcenter+fZmax);
		 */
  // add itself to the list
  AppendPad();
  
  /*
    if (m_DrawChambers) {
    TIter nextc(m_Chambers);
    DriftCham *chamber;
    while((chamber = (DriftCham*)nextc())) {
           if (IsInView(chamber->GetSlide()))
               chamber->AppendPad();
       }
   }

   if (m_DrawWireHits) {
       TIter nextw(m_WireHits);
       Wire *wire;
       while((wire = (Wire*)nextw())) {
           if (IsInView(wire->GetSlide()))
           wire->AppendPad();
       }
   }
   
   if (m_DrawTracks) {
       TIter nextt(m_Tracks);
       Track *track;
       while((track = (Track*)nextt())) {
           track->AppendPad();
       }
   }
   
   if (m_DrawAxis) {
       TIter nexta(m_Axis);
       TPolyLine3D *axis;
       while((axis = (TPolyLine3D*)nexta())) {
           axis->AppendPad();
       }
    }
   
   if (m_DrawHitPoints) {
       TIter nexth(m_HitPoints);
       DtHit *hits;
       while((hits = (DtHit*)nexth())) {
           if (IsInView(hits->GetSlide()))
           hits->AppendPad();
       }
    }
   

   TIter nextf(m_Fruits);
   Fruit *fruit;
   while((fruit = (Fruit*)nextf())) {
       fruit->Draw("");
   }
   */
  if ( fWaxNode ) fWaxNode->Draw("same");
  if ( fEvent ) fEvent->DrawTrackCandidates( GetDisplayPad() );

  int iret;
  view->SetRange(fXcenter-fXin,fYcenter-fYin,fZcenter-fZmin,
		 fXcenter+fXin,fYcenter+fYin,fZcenter+fZmax);
  view->SetView(theta, phi, psi, iret);
}

void OTrView::SetGeometry(float rin)
{
   if (rin <= 0) {
       rin = DEFAULT_ZOOM;
   } 

   fXin  = rin;
   fYin  = rin;
   /*
   fZmin  = (rin/DEFAULT_ZOOM)*ZBLOWUP;
   fZmax  = (rin/DEFAULT_ZOOM)*ZBLOWUP;
   */
   fZmin  = rin;
   fZmax  = rin;

   fXcenter = XCENTER;
   fYcenter = YCENTER;
   fZcenter = ZCENTER;
   //fXcenter = fYcenter = fZcenter = 0;
}

void OTrView::ZoomIn(){ 

  fDisplayPad->cd();

  switch(fViewMode) {
    case END :
      fXin = fXin/fZoomFactor - .2;
      fYin = fYin/fZoomFactor - .2;
      fZmin = fZmin;
      fZmax = fZmax;
      break;
    case RSIDE :
    case LSIDE :
      fXin = fXin;
      fYin = fYin/fZoomFactor - .2;
      fZmin = fZmin/fZoomFactor - .2;
      fZmax = fZmax/fZoomFactor - .2;
      break;
    case TOP :
    case BOT :
      fXin = fXin/fZoomFactor - .2;
      fYin = fYin;
      fZmin = fZmin/fZoomFactor - .2;
      fZmax = fZmax/fZoomFactor - .2;
      break;
    case ALL :
      fXin = fXin/fZoomFactor - .2;
      fYin = fYin/fZoomFactor - .2;
      fZmin = fZmin/fZoomFactor - .2;
      fZmax = fZmax/fZoomFactor - .2;
      break;
    default :
      printf("Can't Zoom, Unknown view mode %d\n", fViewMode);
   }

  if (fXin < 0 || fYin < 0 || fZmin < 0)
      SetGeometry(DEFAULT_ZOOM);

  Draw();
}


void OTrView::ZoomOut(){ 

  fDisplayPad->cd();

  switch(fViewMode) {
    case END :
      fXin = (fXin+.2)*fZoomFactor;
      fYin = (fYin+.2)*fZoomFactor;
      fZmin = fZmin;
      fZmax = fZmax;
      break;
    case RSIDE :
    case LSIDE :
      fXin = fXin;
      fYin = (fYin+.2)*fZoomFactor;
      fZmin = (fZmin+.2)*fZoomFactor;
      fZmax = (fZmax+.2)*fZoomFactor;
      break;
    case TOP :
    case BOT :
      fXin = (fXin+.2)*fZoomFactor;
      fYin = fYin;
      fZmin = (fZmin+.2)*fZoomFactor;
      fZmax = (fZmax+.2)*fZoomFactor;
      break;
    case ALL :
      fXin = (fXin+.2)*fZoomFactor;
      fYin = (fYin+.2)*fZoomFactor;
      fZmin = (fZmin+.2)*fZoomFactor;
      fZmax = (fZmax+.2)*fZoomFactor;
      break;
    default :
      printf("Can't Zoom, Unknown view mode %d\n", fViewMode);
   }

  //  if (fXin > 50 || fYin > 50 || fZmin > 90)
  //  SetGeometry(DEFAULT_ZOOM);

  Draw();
}

void OTrView::Pan(char direction){ 

  fDisplayPad->cd();

  switch(fViewMode) {
    case END:
      switch(direction) {
        case '<' :
          fXcenter = fXcenter+fXin/5;
          break;
        case '>' :
          fXcenter = fXcenter-fXin/5;
          break;
        case '^' :
          fYcenter = fYcenter-fYin/5;
          break;
        case 'v' :
          fYcenter = fYcenter+fYin/5;
          break;
        case 'U' :
          if (fZmax == 0) {
              fZmax = ZBLOWUP;
          } else {
              fZmax = 0;
              fZmin = ZBLOWUP;
          }
          break;
        case 'D' :
          if (fZmin == 0) {
              fZmin = ZBLOWUP;
          } else {
              fZmin = 0;
              fZmax = ZBLOWUP;
          }
          break;
        default :
          printf("Can't Pan %c\n", direction);
      }
      break;
    case TOP:
      switch(direction) {
        case '<' :
          fZcenter = fZcenter+fZmin/5;
          break;
        case '>' :
          fZcenter = fZcenter-fZmin/5;
          break;
        case '^' :
          fXcenter = fXcenter-fXin/5;
          break;
        case 'v' :
          fXcenter = fXcenter+fXin/5;
          break;
        case 'U' :
          if (fZmax == 0) {
              fZmax = ZBLOWUP;
          } else {
              fZmax = 0;
              fZmin = ZBLOWUP;
          }
          break;
        case 'D' :
          if (fZmin == 0) {
              fZmin = ZBLOWUP;
          } else {
              fZmin = 0;
              fZmax = ZBLOWUP;
          }
          break;
        default :
          printf("Can't Pan %c\n", direction);
      }
      break;
    case BOT:
      switch(direction) {
        case '<' :
          fZcenter = fZcenter+fZmin/5;
          break;
        case '>' :
          fZcenter = fZcenter-fZmin/5;
          break;
        case '^' :
          fXcenter = fXcenter+fXin/5;
          break;
        case 'v' :
          fXcenter = fXcenter-fXin/5;
          break;
        case 'U' :
          if (fZmax == 0) {
              fZmax = ZBLOWUP;
          } else {
              fZmax = 0;
              fZmin = ZBLOWUP;
          }
          break;
        case 'D' :
          if (fZmin == 0) {
              fZmin = ZBLOWUP;
          } else {
              fZmin = 0;
              fZmax = ZBLOWUP;
          }
          break;
        default :
          printf("Can't Pan %c\n", direction);
      }
      break;
    case RSIDE:
      switch(direction) {
        case '<' :
          fZcenter = fZcenter+fZmin/5;
          break;
        case '>' :
          fZcenter = fZcenter-fZmin/5;
          break;
        case '^' :
          fYcenter = fYcenter-fYin/5;
          break;
        case 'v' :
          fYcenter = fYcenter+fYin/5;
          break;
        case 'U' :
          if (fZmax == 0) {
              fZmax = ZBLOWUP;
          } else {
              fZmax = 0;
              fZmin = ZBLOWUP;
          }
          break;
        case 'D' :
          if (fZmin == 0) {
              fZmin = ZBLOWUP;
          } else {
              fZmin = 0;
              fZmax = ZBLOWUP;
          }
          break;
        default :
          printf("Can't Pan %c\n", direction);
      }
      break;
    case LSIDE:
      switch(direction) {
        case '<' :
          fZcenter = fZcenter-fZmin/5;
          break;
        case '>' :
          fZcenter = fZcenter+fZmin/5;
          break;
        case '^' :
          fYcenter = fYcenter-fYin/5;
          break;
        case 'v' :
          fYcenter = fYcenter+fYin/5;
          break;
        case 'U' :
          if (fZmax == 0) {
              fZmax = ZBLOWUP;
          } else {
              fZmax = 0;
              fZmin = ZBLOWUP;
          }
          break;
        case 'D' :
          if (fZmin == 0) {
              fZmin = ZBLOWUP;
          } else {
              fZmin = 0;
              fZmax = ZBLOWUP;
          }
          break;
        default :
          printf("Can't Pan %c\n", direction);
      }
      break;
    case ALL:
      break;
        case 'U' :
          if (fZmax == 0) {
              fZmax = ZBLOWUP;
          } else {
              fZmax = 0;
              fZmin = ZBLOWUP;
          }
          break;
        case 'D' :
          if (fZmin == 0) {
              fZmin = ZBLOWUP;
          } else {
              fZmin = 0;
              fZmax = ZBLOWUP;
          }
          break;
    default:
      printf("Can't Pan, ViewMode %d undefined\n", fViewMode);
  }

  Draw();
}

void OTrView::ExecuteEvent(Int_t event, Int_t px, Int_t py)
{
  cout << "in my private execute event: event = " << event << endl;
  // Execute action corresponding to one event
  static Float_t x,y;
  static Float_t xmin,ymin;
  static Float_t xrange,yrange;
  static Float_t pxold, pyold;

  if (fZoomMode) {
    cout << "in zoom mode " << endl;
    if (event == kMouseMotion) {
      gPad->SetCursor(kCross);
    }
    
    if (event == kButton1Down) {
      cout << "wow !" << endl;
      fDisplayPad->cd();
      xmin   = gPad->GetX1();
      ymin   = gPad->GetY1();
      xrange = gPad->GetX2() - xmin;
      yrange = gPad->GetY2() - ymin;
      x = px;
      y = py;
      pxold = px;
      pyold = py;
    }
    
    if (event == kButton1Motion) {
      
      
      gGXW->DrawBox((int)pxold, (int)y, (int)x,(int)pyold, TGXW::kHollow);
      
      if (TMath::Abs(x-px)>TMath::Abs(y-py)) { 
	pxold = px;
	if (py > y) {
	  pyold = y+TMath::Abs(x-px);
	} else {
	  pyold = y-TMath::Abs(x-px);
	}
      } else {
	if (px > x) {
	  pxold = x+TMath::Abs(y-py);
	} else {
	  pxold = x-TMath::Abs(y-py);
	}
	pyold = py;
      }
      gGXW->DrawBox((int)pxold, (int)y, (int)x, (int)pyold, TGXW::kHollow);
      
    }
    
    if (event == kButton1Up) {
      
      fDisplayPad->cd();
      
      float NewScale;
      float Xoffset;
      float Yoffset;
      int X1, Y1, X2, Y2;
      X1 = gPad->XtoAbsPixel(gPad->GetX1());
      Y1 = gPad->YtoAbsPixel(gPad->GetY1());
      X2 = gPad->XtoAbsPixel(gPad->GetX2());
      Y2 = gPad->YtoAbsPixel(gPad->GetY2());
      
      
      NewScale = TMath::Abs(x-pxold)/500.0;
      if (NewScale < 0.01)
	NewScale = .5;
      
      Xoffset = (x+pxold-X1-X2)/((X2-X1)/2)*.85;
      Yoffset = (y+pyold-Y1-Y2)/((Y1-Y2)/2)*.85;
      
      switch( fViewMode) {
      case END :
	fXcenter = fXcenter+Xoffset*fXin;
	fYcenter = fYcenter-Yoffset*fYin;
	fZcenter = fZcenter;
	fXin = fXin*NewScale;
	fYin = fYin*NewScale;
	fZmin = fZmin;
	break;
      case RSIDE :
	fXcenter = fXcenter; 
	fYcenter = fYcenter-Yoffset*fYin;
	fZcenter = fZcenter+Xoffset*fZmin;
	fXin = fXin; 
	fYin = fYin*NewScale;
	fZmin = fZmin*NewScale;
	fZmax = fZmax*NewScale;
	break;
      case LSIDE :
	fXcenter = fXcenter; 
	fYcenter = fYcenter-Yoffset*fYin;
	fZcenter = fZcenter-Xoffset*fZmin;
	fXin = fXin; 
	fYin = fYin*NewScale;
	fZmin = fZmin*NewScale;
	fZmax = fZmax*NewScale;
	break;
      case TOP :
	fXcenter = fXcenter-Yoffset*fXin;
	fYcenter = fYcenter;  
	fZcenter = fZcenter+Xoffset*fZmin;
	fXin = fXin*NewScale;
	fYin = fYin; 
	fZmin = fZmin*NewScale;
	fZmax = fZmax*NewScale;
	break;
      case BOT :
	fXcenter = fXcenter+Yoffset*fXin;
	fYcenter = fYcenter;  
	fZcenter = fZcenter+Xoffset*fZmin;
	fXin = fXin*NewScale;
	fYin = fYin;  
	fZmin = fZmin*NewScale;
	fZmax = fZmax*NewScale;
	break;
      case ALL :
	break;
      default :
	printf("Can't Zoom, Unknown view mode %d\n", fViewMode);
      }
      
      if (fXin < .1 || fYin < .1 || fZmin < .1)
	SetGeometry(DEFAULT_ZOOM);
      Draw();
    }
    
  } 
  
  /*else { 		
    if (gPad->GetView()) {
    gPad->GetView()->ExecuteRotateView(event, py, -px);
    }
    }*/
}

Int_t OTrView::DistancetoPrimitive(Int_t px, Int_t py)
{
// Compute distance from point px,py to objects in event

  /*
   const Int_t big = 9999;
   Int_t dist = big;
   Float_t xmin = gPad->GetX1();
   Float_t xmax = gPad->GetX2();
   Float_t dx   = 0.025*(xmax - xmin);
   Float_t x    = gPad->AbsPixeltoX(px);
   //if (x < xmin+dx || x > xmax-dx) return dist;

   // scan list of fruits
   TIter nextf(m_Fruits);
   Fruit *fruit;
   while((fruit=(Fruit*)nex1tf())) {
        dist = fruit->DistancetoPrimitive(px, py);
      if (dist < 5) {
         gPad->SetSelected(fruit->gFruit());
         gPad->SetCursor(kPointer);
         return 0;
      }
   }
   */
   if ( fWaxNode ) gPad->SetSelected(fWaxNode);
   return 0;
}

void OTrView::Draw(Option_t *)
{
//    Insert current event in graphics pad list

   if (fDrawAllViews) {
      DrawAllViews();
      return;
   }

   fDisplayPad->cd();

   if (fDisplayPad->GetView()) {
       fLongitude = fDisplayPad->GetView()->GetLongitude();
       fLatitude = fDisplayPad->GetView()->GetLatitude();
       fPsi = fDisplayPad->GetView()->GetPsi();
   }

   DrawView(fLongitude, fLatitude, fPsi);

   // Display the event number and title
   fDisplayPad->cd();
   char title[256];
   sprintf (title,"Event %d",fEventNumber);
   DrawTitle(title);
}

void OTrView::ToggleDisplayAttribute(char attribute)
{
  switch(attribute) {

  case 'a' : m_DrawAxis = ! m_DrawAxis; break;
  case 't' : m_DrawTracks = ! m_DrawTracks; break;
  case 'c' : m_DrawChambers = ! m_DrawChambers; break;
  case 'w' : m_DrawWireHits = ! m_DrawWireHits; break;
  case 'h' : m_DrawHitPoints = ! m_DrawHitPoints; break;
  case '+' : ZoomIn(); break;
  case '-' : ZoomOut(); break;
  case '>' : Pan('>'); break;
  case '<' : Pan('<'); break;
  case '^' : Pan('^'); break;
  case 'v' : Pan('v'); break;
  case 'U' : 
    m_CurrentSlides[0] = kTRUE;
    m_CurrentSlides[1] = kFALSE;
    break;
    
  case 'D' : 
    m_CurrentSlides[0] = kFALSE;
    m_CurrentSlides[1] = kTRUE;
    break;
    
  case 'A' : 
    m_CurrentSlides[0] = kTRUE;
    m_CurrentSlides[1] = kTRUE;
    break;
    
    
  default  : printf("Bad display attribute\n");
  }
  
  Draw();
  
}

#endif








