#include <iostream.h>
#include <iomanip.h>
#include <string>
#include <list>
#include <algorithm>

#ifdef USE_ROOT
 #include <TProfile.h>
 #include <TH2.h>
 #include <TSystem.h>
 #include <TPaveLabel.h>
 #include <TCanvas.h>
 #include <TH1.h>
 #include <TControlBar.h>
 #include <TGraphErrors.h>
 #include <TText.h>
//#include <TFile.h>
//#include <TTree.h>
#endif

#ifdef USE_ARTE
 #include <arte/ArteFlag.h>
 #include <rang/Track.hh>
 #include <rang/Node.hh>
 #include <arte/arte.hh>
#endif

#include "wax/controller.hh"
#include "wax/OTrSector.hh"
#include "wax/OTrDetector.hh"
#include "wax/OTrDaq.hh"
#include "wax/OTrEventBuilder.hh"
#include "wax/OTrEventBuffer.hh"
#include "wax/OTrEvent.hh"
#include "wax/OTrView.hh"
#include "wax/OTrRTResult.hh"
#include "wax/OTrHit.hh"
#include "wax/OTrTZeroCalib.hh"
#include "wax/OTrTrack.hh"
#include "wax/OTrTrackCandidate.hh"
#include "wax/OTrSuperCluster.hh"
#include "wax/OTrRTCalib.hh"
#include "wax/OTrRunSummary.hh"
#include "wax/OTrDirectory.hh"
#include "wax/OTrAlignment.hh"
#include "wax/OTrReconstructedHit.hh"
#include "wax/OTrChannelHistogrammer.hh"
#include "wax/OTrSplineRTRelation.hh"

//__________________________________________________________________
// controller
//
// Provides the main interface to the wax package.
//
// Container for:
// * an OTrAlignment object
// * an OTrDaq object
// * an OTrDetector object
// * an OTrTZeroCalib object
// * an OTrRTCalib
// Functionality:
// * next()
//
// Author: Wouter Hulsbergen && Ivar Siccama
//__________________________________________________________________

#ifdef __OTR_CLASSDEF_CONTROLLER__
ClassImp(controller) ;
#endif

controller* gOTr=0;
char controller::fgConfigDirName[256] = "/afs/desy.de/group/hera-b/OTR/db/caldata" ;
//char controller::fgConfigDirName[256] = "caldata" ;

controller::controller() :
  fDaq(0), fDetector(0), fEventBuilder(0), fEvent(0),
  fRunSummary(0), fTZeroCalib(0), fRTCalib(0), fAlignment(0), fView(0)
{
  if(gOTr) {
    cerr << "controller::controller(): Already one controller object created !"
	 << endl ;
    return ;
  } else {
    gOTr = this ;
  }
  
  // The directory where all calibration and geometry is ...
  string dir(fgConfigDirName) ;
  cout << "Your Outer Tracker configuration directory is: \"" << fgConfigDirName << "\"" << endl ;

  //fEventBuffer = new OTrEventBuffer() ; 
  fDaq = new OTrDaq() ;
  OTrDaqComponent::SetDaq(fDaq) ;
#ifdef USE_ARTE
  fDetector = new OTrDetector( fDaq );
#else 
  fDetector = 
    new OTrDetector(const_cast<char *>((dir + "/geometry/geometry.txt").c_str()),fDaq);
#endif
  fEventBuilder = new OTrEventBuilder(fDetector) ;

  // read in the RT relations, masks, t0's
#ifdef USE_ARTE
  if(ArteTable<EVHD>::begin()->exp <= 0) { // MC run
    cout << "Initializing rtrelations for MC." << endl ;
    OTrChannel::SetRTRelation05mm(new OTrSplineRTRelation(const_cast<char *>((dir + "/rtrelation/mcrtr05mm.dat").c_str()))) ;
    OTrChannel::SetRTRelation10mm(new OTrSplineRTRelation(const_cast<char *>((dir + "/rtrelation/mcrtr10mm.dat").c_str()))) ;
    // These are used for digitization, so you only need them when reading mc data
    //ifstream mcrt05mmifs(const_cast<char *>((dir + "/rtrelation/mcrtr05mm.dat").c_str()));
    //OTrChannel::SetMCRTRelation05mm(new OTrSplineRTRelation(mcrt05mmifs)) ;
    //ifstream mcrt10mmifs(const_cast<char *>((dir + "/rtrelation/mcrtr10mm.dat").c_str()));
    //OTrChannel::SetMCRTRelation10mm(new OTrSplineRTRelation(mcrt10mmifs)) ;
    OTrChannel::SetMCRTRelation05mm(OTrChannel::GetRTRelation05mm()) ;
    OTrChannel::SetMCRTRelation10mm(OTrChannel::GetRTRelation10mm()) ;
  } else {
#endif //USE_ARTE
    // read in Sergey Solunin's quality mask
    fDaq->ReadTable( const_cast<char *>((dir + "/masks/big_mask.dat").c_str() )) ;
    fDaq->ReadTable( const_cast<char *>((dir + "/masks/mask.dat").c_str())) ;
    // read in the tzero values
    fDaq->ReadTable(const_cast<char *>((dir + "/tzero/tzero.dat").c_str()));
    OTrChannel::SetRTRelation05mm(new OTrSplineRTRelation(const_cast<char *>((dir + "/rtrelation/rtr05mm.dat").c_str()))) ;
    OTrChannel::SetRTRelation10mm(new OTrSplineRTRelation(const_cast<char *>((dir + "/rtrelation/rtr10mm.dat").c_str()))) ;
    // Alignment
    //     cout << "Replacing MSL's." << endl ;
    //     ThreeVector vecMSL1(-1.019,0,0) ; //+-0.007 //837*-1.24E-3
    //     fDetector->FindSuperLayer(31)->Translate(vecMSL1) ;
    //     ThreeVector vecMSL2(-1.416,0,0) ; //+-0.015 //1209*-1.23E-3
    //     fDetector->FindSuperLayer(32)->Translate(vecMSL2) ;
#ifdef USE_ARTE
  }
#endif
#ifdef USE_ROOT
  gOTrDirectory = new TDirectory("OTrDirectory","OTrDirectory") ;
  gOTrDirectory->cd() ;
  //fSparcifiedOutputFile = new ofstream("asciotrrun.dat") ;
  //fSparcifiedOutputRootFile = 0 ;//new TFile("otrrun.root","RECREATE") ;
  //fTree = 0 ; //new TTree("OTrTree","OTrTree") ;
  //fEventBranch = 0;//fTree->Branch("Events","OTrEvent",&fEvent,64000,0) ;
#endif
  cout << "controller object created." << endl;
} 

controller::~controller()
{
  if(fDaq) delete fDaq ;
  if(fDetector) delete fDetector ;
  if(fRunSummary) delete fRunSummary ;
  if(fTZeroCalib) delete fTZeroCalib ;
  if(fRTCalib) delete fRTCalib ;
  if(fAlignment) delete fAlignment ;
#ifdef USE_ROOT
  if(fView) delete fView ;
#endif
  if(gOTr==this) gOTr=0;
}

OTrTZeroCalib* 
controller::CreateTZeroCalib() {
  if(!fTZeroCalib) fTZeroCalib=new OTrTZeroCalib( fDaq, fDetector );
  return fTZeroCalib;
}

OTrRTCalib* 
controller::CreateRTCalib() {
  if(!fRTCalib) {
    CreateTZeroCalib() ;
    fRTCalib = new OTrRTCalib(fTZeroCalib) ;
  }
  return fRTCalib;
}

OTrAlignment*
controller::CreateAlignment() {
  if(!fAlignment) fAlignment = new OTrAlignment() ;
  return fAlignment ;
}

void
controller::WriteOccupancyMap(char* filename)
{
  if(strlen(filename)==0) filename = "caldata/masks/occup_last.dat" ;
  fDaq->WriteTable(filename,OTrDaq::kMaskTable) ;
}

void controller::ProcessOneEvent()
{
  // Process one event. Calls
  // OTrEventBuilder::CreateNextEvent(). Passes the event to
  // PerformRawogramming() and PerformRecogramming() for
  // analysis. Calls gSystem->ProcessEvents().
  fEvent = GetEventBuilder()->CreateNextEvent() ;
#ifdef USE_ROOT   
  if(fEvent) {
    PerformRawogramming() ;
    PerformRecogramming() ;
  }
  gSystem->ProcessEvents() ;
#endif //USE_ROOT
}

extern "C" int nextarte_();

int controller::Run( const int n )
{
  // Runs over n events. Calls 
  cout << "controller::Run(" << n << ")"<< endl ;
  int nClusters=0;
  int nTrackCandidates=0;
  int nValidEvents=0 ;

  for ( int ev = 0; ev < n; ev++ ) {
#ifdef USE_ARTE
    nextarte_() ;
#endif
    ProcessOneEvent() ;
    if(fEvent) {
      nValidEvents++ ;
      nClusters+=fEvent->GetNClusters() ;
      nTrackCandidates+=fEvent->GetTrackCandidates().size() ;
    } else {
      break ;
    }
    if (!(ev%100)) cout << ev/100 << "|" << flush ;
  }
#ifdef USE_ROOT 
  if(fView && fEvent) Display();
#endif //USE_ROOT
  cout << "number of events processed:  " << GetEventBuilder()->GetNProcessedEvents() << endl
       << "number of clusters found:    " << nClusters << endl 
       << "number of track candidates: " << nTrackCandidates << endl ;
  return nValidEvents;
}

void controller::Reset() { fDaq->Reset() ; }

int controller::open(char* filename, int farm) {
  return GetEventBuilder()->GetEventBuffer()->OpenFile(filename,farm) ; } 

void controller::SetSearchClusters(bool on) { 
  OTrEventBuilder::SetPerformReconstruction(on) ; } 

#ifdef USE_ARTE
int 
controller::EventRowSummary( ostream& os ) const
{
  // get first 4-byte word of table
  ArtePointer<EVHD> evhd( 1 );
  ArtePointer<DIHD> dihd( 1 );
  
  ArtePointer<DDAQ> ddaq( 1 );
  ArtePointer<DSVD> dsvd( 1 );
  //ArtePointer<DITR> ditr( 1 );
  ArtePointer<DPTC> dptc( 1 );
  ArtePointer<DOTR> dotr( 1 );
  ArtePointer<DRIC> dric( 1 );
  //ArtePointer<DTRD> dtrd( 1 );
  ArtePointer<DCAL> dcal( 1 );
  ArtePointer<DMUT> dmut( 1 );
  //ArtePointer<DMUX> dmux( 1 );
  ArtePointer<DMUP> dmup( 1 );
  //ArtePointer<DFLT> dflt( 1 );
  //ArtePointer<DSLT> dslt( 1 );
  //ArtePointer<DTLT> dtlt( 1 );
  //ArtePointer<DTAR> dtar( 1 );
  //ArtePointer<DFCS> dfcs( 1 );
  
  // rows
  os << "event="      << arteflag_.numevt << " "
     << "run="        << arteflag_.numrun << " "
     << "experiment=" << arteflag_.numexp << endl;
  os << "nrow(): "
     << "EVHD=" << evhd.nrow() << " "
     << "DIHD=" << dihd.nrow() << " "
     << "DDAQ=" << ddaq.nrow() << " "
     << "DSVD=" << dsvd.nrow() << " "
    //<< "DITR=" << ditr.nrow() << " "
     << "DPTC=" << dptc.nrow() << " "
     << "DOTR=" << dotr.nrow() << " "
     << "DRIC=" << dric.nrow() << " "
    //<< "DTRD=" << dtrd.nrow() << " "
     << "DCAL=" << dcal.nrow() << " "
     << "DMUT=" << dmut.nrow() << " "
    //<< "DMUX=" << dmux.nrow() << " "
     << "DMUP=" << dmup.nrow() << " "
    //<< "DFLT=" << dflt.nrow() << " "
    //<< "DSLT=" << dslt.nrow() << " "
    //<< "DTLT=" << dtlt.nrow() << " "
    //<< "DTAR=" << dtar.nrow() << " "
    //<< "DFCS=" << dfcs.nrow() 
     << endl; 
  return 0;
}

#endif //USE_ARTE

#ifdef USE_ROOT

void controller::Display() {
  if (!fView) fView = new OTrView(fDetector) ;
  fView->Display(fEvent) ;
}


void controller::PlotAllTdcSpectra() const
{
  TCanvas* mycanvas = new TCanvas("mycanvas","mycanvas") ;
  mycanvas->Divide(4,2) ;
  mycanvas->Show() ;
  vector<TH1*> histos ;
  for(OTrDaq::TdcIterator tdcIt(*fDaq); tdcIt.valid(); tdcIt.next()) {
    int j=0;
    for(OTrTdcBoard::AsdIterator asdIt(*tdcIt); asdIt.valid(); asdIt.next()) {
      mycanvas->cd(++j) ;
      histos.push_back((*asdIt).GetTimeH1()) ;
      histos.back()->Draw() ;
    };    
    mycanvas->Update() ;
    TString name = (*tdcIt).GetName() ;
    name.Append(".ps") ;
    mycanvas->Print(name.Data()) ;
    getchar() ;
    for(vector<TH1*>::iterator histoIt=histos.begin() ; histoIt!=histos.end(); histoIt++)
      delete *histoIt ;
    histos.erase(histos.begin(), histos.end()) ;
  };
  delete mycanvas ;
  //cout << "You have been watching " << n << " tdcspectra." << endl ;
};

void 
controller::PlotChannelTdcSpectra(int tdcId) const
{
  TCanvas* mycanvas = new TCanvas("mycanvas","") ;
  mycanvas->Divide(4,4) ;
  OTrTdcBoard* tdcIt = fDaq->FindTdcBoard(tdcId) ;
  if ( !tdcIt ) {
    cout << "tdc identifier " << tdcId << " not found." << endl;
    return;
  }
  for(OTrTdcBoard::AsdIterator asdIt(*tdcIt); asdIt.valid(); asdIt.next()) {
    int j=0;  
    for(OTrAsdBoard::ChannelConstIterator chanIt(*asdIt) ; chanIt.valid() ;
	chanIt.next() )
      {
	mycanvas->cd(++j) ;
	TH1* histo = (*chanIt).GetTimeH1() ;
	cout << "wire = " << (*chanIt).GetWire()->GetId() << endl;
	if ( (*chanIt).GetWire()->GetSuperLayerId() == 10 ) {
	  cout << "tzero = " << (*chanIt).GetTZero() << endl;
	  cout << "average time = " << (*chanIt).GetAverageTime() << endl;
	  cout << "mask = " << (*chanIt).GetMask() << endl;
	  histo->Draw() ;
	}
	mycanvas->Update();
	getchar();
      };
    mycanvas->Update() ;
    TString name = (*asdIt).GetName() ;
    name.Append(".ps") ;
    mycanvas->Print(name.Data()) ;
    getchar() ;
  };
  delete mycanvas ;
};

void controller::DrawRTRelation() const
{
  static TCanvas* rtcanvas = 0 ;
  if(!rtcanvas) {
    rtcanvas=new TCanvas("drawrtcanvas","") ;
    rtcanvas->Divide(2,2) ;
  };

  rtcanvas->cd(1) ; gPad->DrawFrame(0,0,0.3,256) ;
  TGraph* tgraph05mm = OTrChannel::GetRTRelation05mm()->GetTGraph(50) ; 
  tgraph05mm->Draw("lp") ;
  rtcanvas->cd(2) ; gPad->DrawFrame(0,0,0.6,256) ;
  TGraph* tgraph10mm = OTrChannel::GetRTRelation10mm()->GetTGraph(50) ; 
  tgraph10mm->Draw("lp") ;
  rtcanvas->cd(3) ; gPad->DrawFrame(0,0,0.3,0.2) ;
  TGraph* sgraph05mm = OTrChannel::GetRTRelation05mm()->GetSigmaGraph(50) ; 
  sgraph05mm->Draw("lp") ;
  rtcanvas->cd(4) ; gPad->DrawFrame(0,0,0.6,0.2) ;
  TGraph* sgraph10mm = OTrChannel::GetRTRelation10mm()->GetSigmaGraph(50) ;
  sgraph10mm->Draw("lp") ;

  rtcanvas->Update() ;
}

#include "TGraphErrors.h"
void controller::CreateRTRelation(bool drawFlag) const
{
  // ** FILL REFERENCE SPECTRA ** //
  TH1F* ref5mmH1 = new TH1F("ref5mm","",OTrChannel::GetNTimeBins(),
			    OTrChannel::GetMinimumTimeAxis(),
			    OTrChannel::GetMaximumTimeAxis()) ; 
  ref5mmH1->SetDirectory(0) ;
  TH1F* ref10mmH1 = new TH1F("ref10mm","",OTrChannel::GetNTimeBins(),
			     OTrChannel::GetMinimumTimeAxis(),
			     OTrChannel::GetMaximumTimeAxis()) ;    
  ref10mmH1->SetDirectory(0) ;

  TH1F* refH1 ;
  for ( vector<OTrSuperLayer>::iterator sulIt =  
	  fDetector->GetSuperLayers().begin();
	sulIt != fDetector->GetSuperLayers().end(); sulIt++ ) { 
    for ( OTrSuperLayer::LayerIterator layIt(*sulIt) ; layIt.valid(); 
	  layIt.next() ) {
      for ( OTrLayer::SectionIterator secIt(*layIt) ; secIt.valid();
	    secIt.next() ) { 
	if ( (*secIt).GetEPitch() == OTrGede::kFiveMM )
	  refH1 = ref5mmH1 ;
	else
	  refH1 = ref10mmH1 ;
	for ( vector<OTrWire>::const_iterator wireIt = 
		(*secIt).GetWires().begin();
	      wireIt != (*secIt).GetWires().end(); wireIt++ ) {
	  OTrChannel* chanIt = wireIt->GetChannel();
	  int trbin = -(*chanIt).GetTZeroBin() ;
	  if(trbin<0) trbin+=OTrChannel::GetNTimeBins() ;
	  for(int bin=0; bin<OTrChannel::GetNTimeBins(); bin++,trbin++) {
	    refH1->AddBinContent(bin+1,
	      (*chanIt).GetTimeHistogram()[trbin%OTrChannel::GetNTimeBins()]) ;
	  }
	}
      }
    }
  }

  /** Create RTRelations and set pointers **/
  OTrSplineRTRelation* rt05mm = new OTrSplineRTRelation(10,6,ref5mmH1,0.00125,0.29,250,130,20) ;
  OTrSplineRTRelation* rt10mm = new OTrSplineRTRelation(10,6,ref10mmH1,0.00125,0.58,250,30,20) ;
  OTrChannel::SetRTRelation05mm(rt05mm) ;
  OTrChannel::SetRTRelation10mm(rt10mm) ;
  /** Draw **/
  if(drawFlag) {
    TCanvas rtcanvas("rtcanvas","") ;
    rtcanvas.Divide(2,2) ;
    rtcanvas.cd(1) ; ref5mmH1->Draw() ;
    rtcanvas.cd(3) ; ref10mmH1->Draw() ;
    rtcanvas.cd(2) ; gPad->DrawFrame(0,0,0.3,256) ;
    TGraph* tgraph05mm = rt05mm->GetTGraph(50) ; tgraph05mm->Draw("lp") ;
    rtcanvas.cd(4) ; gPad->DrawFrame(0,0,0.6,256) ;
    TGraph* tgraph10mm = rt10mm->GetTGraph(50) ; tgraph10mm->Draw("lp") ;
    rtcanvas.Update() ;
    getchar() ;
    delete tgraph05mm;
    delete tgraph10mm;
  };
  delete ref5mmH1;
  delete ref10mmH1;
};

int 
controller::plot( const int sl, const int choice, const bool use_mask )
{
  if ( sl == -1 ) {
    cout << "syntax: plot( int superlayer_id, int choice, bool use_mask ) "
	 << endl;
    cout << "choice = 1 : occupancy" << endl; 
    cout << "choice = 2 : TDC count for each hit channel" << endl;
    cout << "choice = 3 : average TDC counts" << endl;
    cout << "choice = 4 : TDC spectra (summed per section)" << endl;
    cout << "choice = 5 : RMS of TDC counts" << endl;
    cout << "choice = 6 : t0 values" << endl;
    cout << "choice = 7 : occupancy differences with next channel" << endl;
    return 0;
  }
 
  static TCanvas* canvas = 0;
  if (!canvas) {
    canvas = new TCanvas("occcanvas","OTR superlayer",200,40,700,500);
    canvas->SetFillColor(50);
  }

  canvas->cd();
  OTrSuperLayer* sul = fDetector->GetSuperLayer(sl) ;
  if(sul) {
    sul->Plot(canvas, choice, fEvent, use_mask );
    canvas->cd();
    char title[256] ;
    sprintf(title,"Run: %d -- SuperLayer: %s", 
	    GetEventBuilder()->GetRunNumber(), sul->GetName().Data()) ; 
    TPaveLabel *label = new TPaveLabel(0.05, 0.95, 0.95, 0.99, title );
    label->SetBit(kCanDelete);
    label->SetFillColor(42);
    label->Draw();
    canvas->Modified();
    canvas->Update();
  } else cout << "Sorry, no superlayer " << sl << "." << endl ;
  return 0;
}

int 
controller::plot_combined( const int sl ) 
{
  static TCanvas* canvas = 0;
  static TPad* pads[3] ;
  if(canvas==0) {
    canvas = new TCanvas("combinedcanvas","5 and 10 mm combined",
			 250,60,600,750);
    canvas->SetFillColor(36);
    pads[0] = new TPad("comb0","comb0",0.02,0.02,0.98,0.93,-1);
    pads[0]->Draw();
  }
  if(fCombinedLayer) 
    fCombinedLayer->PlotCombinedOccupancy( canvas,pads[0], "" );
  return 0;
}

void
controller::SetCombinedLayer( const int slId, const int layId ) 
{ 
  OTrSuperLayer* sl = fDetector->FindSuperLayer(slId); 
  if ( sl ) {
    OTrLayer* l = sl->FindLayer( layId );
    if ( l ) {
      fCombinedLayer = l;
      fCombinedLayer->BookCombinedOccupancy();
    } else {
      cout << "layer " << layId << "not found." << endl;
      exit(1);
    }
  } else {
    cout << "superlayer " << slId << "not found." << endl;
    exit(1);
  }
}

void
controller::PerformRawogramming()
{
  //cout << "controller::PerformRawogramming()" << endl ;
  if(fRunSummary) fRunSummary->Add(fEvent) ;
  // occupancy plot of one layer in which 5mm cells are OR-ed
  if ( fCombinedLayer ) {
    int sulId = fCombinedLayer->GetSuperLayerId() ;
    const OTrSuperLayerEvent* slEvent = fEvent->FindSuperLayerEvent(sulId) ;
    if ( slEvent ) {
      fCombinedLayer->FillCombinedOccupancy( slEvent );
    } 
  }

//   static TH1F* wireDifMC2H1 = new TH1F("wireDifMC2H1","",128,-0.5,127.5) ;
//   static TH1F* wireDifMSL1H1 = new TH1F("wireDifMSL1H1","",128,-0.5,127.5) ;
//   static TH1F* wireDifMSL2H1 = new TH1F("wireDifMSL2H1","",128,-0.5,127.5) ;
//   static TH1F* wireDifMTC1H1 = new TH1F("wireDifMTC1H1","",128,-0.5,127.5) ;
//   TH1F* wireDifH1 ;
  
//   for(OTrEvent::SuperLayerEventContainer::const_iterator 
//  	sleIt=fEvent->GetSuperLayerEvents().begin();
//       sleIt!=fEvent->GetSuperLayerEvents().end(); sleIt++) {
//     int slId = (*sleIt).first ;
//     switch(slId) 
//       {
//       case 2:
// 	wireDifH1 = wireDifMC2H1 ;
// 	break ;
//       case 31:
// 	wireDifH1 = wireDifMSL1H1 ;
// 	break ;
//       case 32:
// 	wireDifH1 = wireDifMSL2H1 ;
// 	break ;
//       case 33:
// 	wireDifH1 = wireDifMTC1H1 ;
// 	break ;
//       default:
// 	wireDifH1 = wireDifMTC1H1 ;
// 	break ;
//       };
//     for(OTrSuperLayerEvent::HitContainer::const_iterator 
// 	  hitIt=(*sleIt).second.GetHits().begin() ;
// 	hitIt!=(*sleIt).second.GetHits().end() ; hitIt++)
//       for(OTrSuperLayerEvent::HitContainer::const_iterator 
// 	    hitIt2=hitIt ;
// 	  hitIt2!=(*sleIt).second.GetHits().end() ; hitIt2++)
// 	if((*hitIt)->GetWire()->GetGede()==
// 	   (*hitIt2)->GetWire()->GetGede())
// 	  wireDifH1->Fill(fabs((*hitIt)->GetWire()->GetPRWireNumber()
// 			       -(*hitIt2)->GetWire()->GetPRWireNumber())) ;
//   } ;
};

extern "C" float prob_(const float&, const int&) ;

void
controller::PerformRecogramming()
{
  //CombineClusters(); //old
  static TDirectory* recdir  = new TDirectory("OTrReco","OTrReco") ;
  static TH1F* trackMultH1   = new TH1F("trackMultiplicity","",15,-0.5,14.5) ;
  static TH2F* XvsXH2        = new TH2F("XvsX","XvsX",100,-80,30,100,-110,-30) ;
  static TH2F* YvsYH2        = new TH2F("YvsY","YvsY",60,-60,60,60,-60,60) ;
  static TH2F* targetXYH2    = new TH2F("targetXYH2","xy at z=0",100,-30,30,100,-30,30) ;
  static TH1F* trackSizeH1   = new TH1F("trackSizeH1","size of tracks",15,-0.5,14.5) ;
  static TH1F* chi2H1        = new TH1F("chi2","chi2/#dof",100,0,10) ;
  static TH1F* chi2probH1    = new TH1F("chi2probH1","chi2prob",100,0,1) ;

  static TH2F* TxVsTyMC01H2  = new TH2F("TxVsTyMC01H2","",1000,-0.3,0.3,1000,-0.3,0.3) ;
  static TH2F* TxVsTyMC02H2  = new TH2F("TxVsTyMC02H2","",1000,-0.3,0.3,1000,-0.3,0.3) ;
  static TH2F* TxVsTyMSL1H2 = new TH2F("TxVsTyMSL1H2","",1000,-0.3,0.3,1000,-0.3,0.3) ;
  static TH2F* TxVsTyMSL2H2 = new TH2F("TxVsTyMSL2H2","",1000,-0.3,0.3,1000,-0.3,0.3) ;
  static TH2F* TxVsTyTC02H2 = new TH2F("TxVsTyTC02H2","",1000,-0.3,0.3,1000,-0.3,0.3) ;

  static TH1F* clusterSizeMC01H1  = new TH1F("clusterSizeMC01H1","",10,0.5,10.5) ;
  static TH1F* clusterSizeMC02H1  = new TH1F("clusterSizeMC02H1","",10,0.5,10.) ;
  static TH1F* clusterSizeMSL1H1  = new TH1F("clusterSizeMSL1H1","",10,0.5,10.) ;
  static TH1F* clusterSizeMSL2H1  = new TH1F("clusterSizeMSL2H1","",10,0.5,10.) ;
  static TH1F* clusterSizeTC02H1  = new TH1F("clusterSizeTC02H1","",10,0.5,10.) ;

  static TH1F* clusterMultMC01H1  = new TH1F("clusterMultMC01H1","",21,-0.5,20.5) ;
  static TH1F* clusterMultMC02H1  = new TH1F("clusterMultMC02H1","",21,-0.5,20.5) ;
  static TH1F* clusterMultMSL1H1  = new TH1F("clusterMultMSL1H1","",21,-0.5,20.5) ;
  static TH1F* clusterMultMSL2H1  = new TH1F("clusterMultMSL2H1","",21,-0.5,20.5) ;
  static TH1F* clusterMultTC02H1  = new TH1F("clusterMultTC02H1","",21,-0.5,20.5) ;

  static TH1F* hitMultMC01H1  = new TH1F("hitMultMC01H1","",21,-0.5,20.5) ;
  static TH1F* hitMultMC02H1  = new TH1F("hitMultMC02H1","",21,-0.5,20.5) ;
  static TH1F* hitMultMSL1H1  = new TH1F("hitMultMSL1H1","",21,-0.5,20.5) ;
  static TH1F* hitMultMSL2H1  = new TH1F("hitMultMSL2H1","",21,-0.5,20.5) ;
  static TH1F* hitMultTC02H1  = new TH1F("hitMultTC02H1","",21,-0.5,20.5) ;

  TH2F *TxVsTyH2 ; TH1F *clusterSizeH1, *clusterMultH1, *hitMultH1 ; 
  for(OTrEvent::SuperLayerEventContainer::const_iterator 
  	sleIt=fEvent->GetSuperLayerEvents().begin();
      sleIt!=fEvent->GetSuperLayerEvents().end(); sleIt++) {
    switch((*sleIt).first)
      {
      case 1: 
	TxVsTyH2      = TxVsTyMC01H2 ;
	clusterSizeH1 = clusterSizeMC01H1 ;
	clusterMultH1 = clusterMultMC01H1 ;
	hitMultH1     = hitMultMC01H1 ;
	break ;
      case 2:
	TxVsTyH2      = TxVsTyMC02H2 ;
	clusterSizeH1 = clusterSizeMC02H1 ;
	clusterMultH1 = clusterMultMC02H1 ;
	hitMultH1     = hitMultMC02H1 ;
	break ;
      case 31:
	TxVsTyH2      = TxVsTyMSL1H2 ;
	clusterSizeH1 = clusterSizeMSL1H1 ;
	clusterMultH1 = clusterMultMSL1H1 ;
	hitMultH1     = hitMultMSL1H1 ;
	break ;
      case 32:
	TxVsTyH2      = TxVsTyMSL2H2 ;
	clusterSizeH1 = clusterSizeMSL2H1 ;
	clusterMultH1 = clusterMultMSL2H1 ;
	hitMultH1     = hitMultMSL2H1 ;
	break ;
      default:
	TxVsTyH2      = TxVsTyTC02H2 ;
	clusterSizeH1 = clusterSizeTC02H1 ;
	clusterMultH1 = clusterMultTC02H1 ;
	hitMultH1     = hitMultTC02H1 ;
	break;
      }
    clusterMultH1->Fill(float((*sleIt).second.GetClusters().size())) ;
    hitMultH1->Fill(float((*sleIt).second.GetHits().size())) ;
    for(OTrSuperLayerEvent::ClusterContainer::const_iterator 
	  clIt=(*sleIt).second.GetClusters().begin() ;
	clIt!=(*sleIt).second.GetClusters().end() ; clIt++) {
      TxVsTyH2->Fill((float)(*clIt)->GetTx(),(float)(*clIt)->GetTy()) ;
      clusterSizeH1->Fill((float)(*clIt)->GetHits().size()) ;
    }  
  }
  trackMultH1->Fill(fEvent->GetTrackCandidates().size()) ;

  if(fRTCalib) fRTCalib->Add(fEvent) ;
  if(fAlignment) fAlignment->Add(fEvent) ;

  for(OTrEvent::TrackContainer::const_iterator trackIt = 
	fEvent->GetTrackCandidates().begin() ;
      trackIt!=fEvent->GetTrackCandidates().end() ; trackIt++) {
    OTrReconstructedTrackInterface* track = (*trackIt)->GetTrack() ;
    if(track) {
      Float_t xco = track->GetX() ;
      Float_t yco = track->GetY() ;
      trackSizeH1->Fill(track->GetHits().size()) ;
      targetXYH2->Fill(xco,yco) ;
      chi2H1->Fill(float(track->GetChi2()/track->GetNDof() )) ;
      chi2probH1->Fill(track->GetChi2Prob()) ;
    }
#ifdef USE_ARTE
    rangerTrack* rtrack = (*trackIt)->GetRangerTrack() ;
    if(rtrack) {
      rangerTrack::rangerNodeIterator firstNode = (*rtrack).last() ;
      const rangerState& state = (*firstNode)->getState() ;
      targetXYH2->Fill(float(state.x()-(*firstNode)->zNode()*state.tx()),
		       float(state.y()-(*firstNode)->zNode()*state.ty())) ;
      int ndof = rtrack->getNodes().size()-state.vec().num_row() ;
      float chi2 = rtrack->totalChiSq() ;
      chi2H1->Fill(float(chi2/ndof )) ;
      chi2probH1->Fill(prob_(chi2,ndof)) ;
      trackSizeH1->Fill(rtrack->getNodes().size()) ;
    }
#endif //USE_ARTE
  }
}

void
controller::UpdateRT(bool fitT, bool fitSigma2, bool fit05mm, bool fit10mm)
{
  if(fRTCalib) {
    fRTCalib->Fit(fitT,fitSigma2,fit05mm,fit10mm) ; 
    fRTCalib->Reset() ;
  } ;
  if(fTZeroCalib) fTZeroCalib->CalibFromResiduals() ;
}  


void
controller::DrawTxTy() const
{
  typedef vector<TGraph*> GraphContainer ;
  static GraphContainer clusterGraphs ;
  static GraphContainer trackGraphs ;
  static TCanvas* txtyCanvas = new TCanvas("txtyCanvas","",2) ;

  /*** cleanup ***/
  for(GraphContainer::iterator grIt=clusterGraphs.begin() ;
      grIt!=clusterGraphs.end() ; grIt++)
    delete *grIt ;
  clusterGraphs.erase(clusterGraphs.begin(),clusterGraphs.end()) ;
  for(GraphContainer::iterator grIt=trackGraphs.begin() ;
      grIt!=trackGraphs.end() ; grIt++)
    delete *grIt ;
  trackGraphs.erase(trackGraphs.begin(),trackGraphs.end()) ;
  txtyCanvas->Clear() ;
  txtyCanvas->DrawFrame(-0.3,-0.3,0.3,0.3) ;

  /*** draw clusters ***/
  int color = 2;
  const int maxnclusters=100;
  float* tx = new float[maxnclusters] ;
  float* txe = new float[maxnclusters] ;
  float* ty = new float[maxnclusters] ;
  float* tye = new float[maxnclusters] ;
  for(OTrEvent::SuperLayerEventContainer::const_iterator 
	sleIt=fEvent->GetSuperLayerEvents().begin() ;
      sleIt!=fEvent->GetSuperLayerEvents().end(); sleIt++,color++) {
    int n=0;
    for(OTrSuperLayerEvent::ClusterContainer::const_iterator 
	  clIt=(*sleIt).second.GetClusters().begin() ;
	(clIt!=(*sleIt).second.GetClusters().end()) && (n<maxnclusters); clIt++,n++) {
      tx[n] = (*clIt)->GetTx() ;
      ty[n] = (*clIt)->GetTy() ;
      txe[n] = sqrt(((*clIt)->GetCov())[0][0]) ;
      tye[n] = sqrt(((*clIt)->GetCov())[1][1]) ;
    };
    if(n>0) {
      TGraphErrors* graph = new TGraphErrors(n,tx,ty,txe,tye) ;
      graph->SetMarkerColor(color) ;
      graph->SetLineColor(color) ;
      graph->Draw("p") ;
      clusterGraphs.push_back(graph) ;
    } ;
  };
  /*** draw track candidates ***/
  color = 1;
  for(OTrEvent::TrackContainer::const_iterator trackIt=
	fEvent->GetTrackCandidates().begin() ;
      trackIt!=fEvent->GetTrackCandidates().end() ; trackIt++) {
     int n=0;
     for(OTrTrackCandidate::ClusterContainer::const_iterator 
	   clIt=(*trackIt)->GetClusters().begin() ;
	 (clIt!=(*trackIt)->GetClusters().end()) && (n<maxnclusters); clIt++,n++) {
       tx[n] = (*clIt)->GetTx() ;
       ty[n] = (*clIt)->GetTy() ;
     };
     TGraph* graph = new TGraph(n,tx,ty) ;
     graph->SetMarkerColor(color) ;
     graph->SetMarkerSize(0) ;
     graph->SetLineColor(color) ;
     graph->Draw("lp") ;
     trackGraphs.push_back(graph) ;
  };
  txtyCanvas->Update() ;
}

class TRunHVContent
{
public:
  int fRunId ;
  float fPercentage ;
  float fHV05mm ;
  float fHV10mm ;
  OTrObservable<double> fSigma05mm ;
  OTrObservable<double> fTZero05mm ;
  OTrObservable<double> fTEdge05mm ;
  OTrObservable<double> fSigma10mm ;
  OTrObservable<double> fTZero10mm ;
  OTrObservable<double> fTEdge10mm ;
} ;

void controller::PrintSummary() const { 
  GetDaq()->PrintSummary();
}

void controller::ReadRTRelations(const char* name)
{
  char* fullname = new char[50+strlen(name)] ;

  delete OTrChannel::GetRTRelation05mm() ;
  sprintf(fullname,"caldata/rtrelation/rtr05mm_%s.dat",name) ;
  OTrChannel::SetRTRelation05mm(new OTrSplineRTRelation(fullname)) ;

  delete OTrChannel::GetRTRelation10mm() ;
  sprintf(fullname,"caldata/rtrelation/rtr10mm_%s.dat",name) ;
  OTrChannel::SetRTRelation10mm(new OTrSplineRTRelation(fullname)) ;
}

void controller::WriteRTRelations(const char* name)
{
  char* fullname = new char[50+strlen(name)] ;
  sprintf(fullname,"caldata/rtrelation/rtr05mm_%s.dat",name) ;
  OTrChannel::GetRTRelation05mm()->Write(fullname) ;
  sprintf(fullname,"caldata/rtrelation/rtr10mm_%s.dat",name) ;
  OTrChannel::GetRTRelation10mm()->Write(fullname) ;
  delete fullname ;
}
 
inline
bool CompareHV(const TRunHVContent& lhs, const TRunHVContent& rhs)
{
  return lhs.fPercentage < rhs.fPercentage ;
}


void controller::temp(const char *name)
{
  vector<TRunHVContent> runs ;
  ifstream runVsHVFile("macros/RunVsHV.txt") ;
  TRunHVContent thisRun ;
  runVsHVFile >> thisRun.fRunId ;
  while(runVsHVFile)
    {
      runVsHVFile >> thisRun.fPercentage 
		  >> thisRun.fHV05mm >> thisRun.fHV10mm ;
      cout << thisRun.fRunId << " "
	   << thisRun.fPercentage << " "
	   << thisRun.fHV05mm << " "
	   << thisRun.fHV10mm << endl ;
      if(thisRun.fRunId!=3396 && thisRun.fRunId!=3397 && thisRun.fRunId!=3395)
	runs.push_back(thisRun) ;
      thisRun.fRunId=-1; 
      runVsHVFile >> thisRun.fRunId ;
    }
  cout << "There are " << runs.size() << " runs in the file :)" << endl ;
  sort(runs.begin(),runs.end(),CompareHV) ;
  OTrSplineRTRelation* rtr ;
  ifstream* rtfile ;
  vector<TRunHVContent*> goodruns ;
  TRunHVContent* referencerun ;
  const float rmax = 0.5; 
  float HVmax05mm(0), HVmin05mm(3000) ; 
  float HVmax(0), HVmin(3000) ; 

  for(vector<TRunHVContent>::iterator runIt=runs.begin() ;
      runIt!=runs.end(); runIt++) {
    cout << runIt->fRunId << " " << runIt->fPercentage << " "
	 << runIt->fHV05mm << " " << runIt->fHV10mm << endl ;
    if(runIt->fRunId==3388) referencerun=runIt ;
    char rtfilename[256] ;
    sprintf(rtfilename,"caldata/rtrelation/rtr05mm_%s0%4d.dat",
	    name,runIt->fRunId) ;
    rtfile = new ifstream(rtfilename) ;
    cout << rtfilename << " " << rtfile->rdbuf()->is_open() << endl ;
    bool found05mmfile = (*rtfile).rdbuf()->is_open()>0 ;
    if(found05mmfile) {
      rtr = new OTrSplineRTRelation(*rtfile) ;
      OTrRTResult rtresult = rtr->CalcRTResult(rtr->GetRMin()) ;
      runIt->fSigma05mm = rtresult.GetSigma() ;
      runIt->fTZero05mm = rtresult.GetT() ;
      runIt->fTEdge05mm = rtr->CalcRTResult(rmax).GetT() ;
      delete rtr ;
    }
    //delete rtfile ;
    sprintf(rtfilename,"caldata/rtrelation/rtr10mm_%s0%4d.dat",
	    name,runIt->fRunId) ;
    rtfile = new ifstream(rtfilename) ;
    cout << rtfilename << " " << rtfile->rdbuf()->is_open() << endl ;
    bool found10mmfile = (*rtfile).rdbuf()->is_open()>0  ;
    if(found10mmfile) {
      rtr = new OTrSplineRTRelation(*rtfile) ;
      OTrRTResult rtresult = rtr->CalcRTResult(rtr->GetRMin()) ;
      runIt->fSigma10mm = rtresult.GetSigma() ;
      runIt->fTZero10mm = rtresult.GetT() ;
      runIt->fTEdge10mm = rtr->CalcRTResult(rmax).GetT() ;
      delete rtr ;
    }
    if(found05mmfile&&found10mmfile) {
      goodruns.push_back(runIt) ;
      HVmin = min(HVmin,runIt->fHV05mm) ;
      HVmax = max(HVmax,runIt->fHV10mm) ;
      HVmin05mm = min(HVmin05mm,runIt->fHV05mm) ;
      HVmax05mm = max(HVmax05mm,runIt->fHV05mm) ;
    }  
    //delete rtfile ;
  }

  TGraphErrors* sigmaVsHv05mm = new TGraphErrors(goodruns.size()) ;
  TGraphErrors* sigmaVsHv10mm = new TGraphErrors(goodruns.size()) ;
  TGraphErrors* tzeroVsHv05mm = new TGraphErrors(goodruns.size()) ;
  TGraphErrors* tzeroVsHv10mm = new TGraphErrors(goodruns.size()) ;
  TGraphErrors* tedgeVsHv05mm = new TGraphErrors(goodruns.size()) ;
  TGraphErrors* tedgeVsHv10mm = new TGraphErrors(goodruns.size()) ;

  int i=0;
  for(vector<TRunHVContent*>::const_iterator runIt=goodruns.begin() ;
      runIt!=goodruns.end(); runIt++,i++) {
    sigmaVsHv05mm->SetPoint(i,(*runIt)->fHV05mm/1000.,(*runIt)->fSigma05mm.GetValue()) ;
    sigmaVsHv05mm->SetPointError(i,10/1000,5*(*runIt)->fSigma05mm.GetError()) ;
    sigmaVsHv10mm->SetPoint(i,(*runIt)->fHV10mm/1000.,(*runIt)->fSigma10mm.GetValue()) ;
    sigmaVsHv10mm->SetPointError(i,10/1000,5*(*runIt)->fSigma10mm.GetError()) ;
    tzeroVsHv05mm->SetPoint(i,(*runIt)->fHV05mm/1000.,(*runIt)->fTZero05mm.GetValue()) ;
    tzeroVsHv05mm->SetPointError(i,10/1000,2*(*runIt)->fTZero05mm.GetError()) ;       
    tzeroVsHv10mm->SetPoint(i,(*runIt)->fHV10mm/1000.,(*runIt)->fTZero10mm.GetValue()) ;
    tzeroVsHv10mm->SetPointError(i,10/1000,2*(*runIt)->fTZero10mm.GetError()) ;
    tedgeVsHv05mm->SetPoint(i,(*runIt)->fHV05mm/1000.,(*runIt)->fTEdge05mm.GetValue()-referencerun->fTEdge05mm.GetValue()) ;
    tedgeVsHv05mm->SetPointError(i,10/1000,2*(*runIt)->fTEdge05mm.GetError()) ;
    tedgeVsHv10mm->SetPoint(i,(*runIt)->fHV10mm/1000.,(*runIt)->fTEdge10mm.GetValue()-referencerun->fTEdge10mm.GetValue()) ;
    tedgeVsHv10mm->SetPointError(i,10/1000.,2*(*runIt)->fTEdge10mm.GetError()) ;
  }

  static TCanvas* canvas = new TCanvas("RtVsHV") ;
  canvas->Clear() ;
  canvas->Divide(2,1) ;
  canvas->cd(1) ;
  gPad->DrawFrame((HVmin-50)/1000.,0.01,(HVmax+50)/1000.,0.08) ;
  sigmaVsHv05mm->Draw("lp") ;
  sigmaVsHv10mm->Draw("lp") ;
  sigmaVsHv10mm->GetHistogram()->GetXaxis()->SetTitle("HV") ;
  sigmaVsHv10mm->GetHistogram()->GetXaxis()->SetTitleSize(0.05) ;
  sigmaVsHv10mm->GetHistogram()->GetYaxis()->SetTitle("`s#  [cm]") ; 
  sigmaVsHv10mm->GetHistogram()->GetYaxis()->SetTitleSize(0.05) ;
  sigmaVsHv10mm->GetHistogram()->GetYaxis()->SetTitleOffset(1.7) ;
  sigmaVsHv10mm->SetLineColor(2) ;

  canvas->cd(2) ;
  gPad->DrawFrame((HVmin-50)/1000,220,(HVmax+50)/1000,250) ;
  tzeroVsHv05mm->Draw("lp") ;
  // tedgeVsHv05mm->Draw("lp") ;
  //   tedgeVsHv05mm->SetMarkerColor(2) ;
  //   tedgeVsHv05mm->SetLineColor(2) ;
  //  canvas->cd(3) ;
  //   gPad->DrawFrame(1700,-15,2050,15) ;
  tzeroVsHv10mm->Draw("lp") ;
  //   tedgeVsHv10mm->Draw("lp") ;
  //   tedgeVsHv10mm->SetMarkerColor(2) ;
  //   tedgeVsHv10mm->SetLineColor(2) ;
  tzeroVsHv10mm->GetHistogram()->GetXaxis()->SetTitle("HV [kV]") ;
  tzeroVsHv10mm->GetHistogram()->GetXaxis()->SetTitleSize(0.05) ;
  tzeroVsHv10mm->GetHistogram()->GetYaxis()->SetTitle("t0 [tdc]") ;
  tzeroVsHv10mm->GetHistogram()->GetYaxis()->SetTitleSize(0.05) ;
  tzeroVsHv10mm->GetHistogram()->GetYaxis()->SetTitleOffset(1.5) ;
  tzeroVsHv10mm->SetLineColor(2) ;
  TLine* line = new TLine(1850/1000.,225,1900/1000.,225) ;
  line->Draw() ;
  TText* text = new TText(1925/1000.,224.5," 5 mm") ;
  text->SetTextFont(132) ;
  text->SetTextSize(0.07) ;
  text->Draw() ;
  line = new TLine(1850/1000.,223,1900/1000.,223) ;
  line->SetLineColor(2) ;
  line->Draw() ;
  text = new TText(1925/1000.,222.5,"10 mm") ;
  text->SetTextFont(132) ;
  text->SetTextSize(0.07) ;
  text->SetTextColor(2) ;
  text->Draw() ;
  canvas->cd() ;
  // gPad->DrawFrame((HVmin05mm-50)/1000,0.02,(HVmax05mm+50)/1000,0.05) ;
  //   sigmaVsHv05mm->Draw("lp") ;
  //   sigmaVsHv05mm->GetHistogram()->GetXaxis()->SetTitle("HV") ;
  //   sigmaVsHv05mm->GetHistogram()->GetXaxis()->SetTitleSize(0.06) ;
  //   sigmaVsHv05mm->GetHistogram()->GetYaxis()->SetTitle("`s#  [cm]") ; 
  //   sigmaVsHv05mm->GetHistogram()->GetYaxis()->SetTitleSize(0.06) ;

//   gPad->DrawFrame((HVmin05mm-50)/1000.,220,(HVmax05mm+50)/1000.,250) ;
//   tzeroVsHv05mm->Draw("lp") ;
//   tzeroVsHv05mm->GetHistogram()->GetXaxis()->SetTitle("HV") ;
//   tzeroVsHv05mm->GetHistogram()->GetXaxis()->SetTitleSize(0.06) ;
//   tzeroVsHv05mm->GetHistogram()->GetYaxis()->SetTitle("t0  [tdc]") ;
//   tzeroVsHv05mm->GetHistogram()->GetYaxis()->SetTitleSize(0.06) ;
  canvas->Update() ;
}

#endif //USE_ROOT
