#ifndef __OTR_DAQ_HH__
#define __OTR_DAQ_HH__

#ifdef __OTR_CLASSDEF__
#ifndef ROOT_Rtypes
#include <Rtypes.h>
#endif //ROOT_Rtypes
#endif //__OTR_CLASSDEF__
#ifndef __OTR_TDCBOARD_HH__
#include "OTrTdcBoard.hh"
#endif
#ifndef __OTR_COMPONENTMOTHER_HH__
#include "OTrComponentMother.hh"
#endif

class OTrTdcBoard ;

class OTrDaq : public OTrDaqComponent, public OTrComponentMother<OTrTdcBoard>
{
public:
  // typedefs, enums
  enum ETableType { kMaskTable=0, kTZeroTable=1, kQMaskTable=2 } ;
  typedef OTrDaqComponent::ItemId_t ItemId_t ;
  typedef ChildContainer TdcContainer ;
  typedef Iterator TdcIterator ;
  typedef ConstIterator TdcConstIterator ;

  // constructors, destructor, assignment
  OTrDaq() ;
  OTrDaq(int nTdcs) ; // Constructor of nTdcs tdcs
  virtual ~OTrDaq() ; // The Tdc's are destructed !!!

  // setters and getters
  int GetNProcessedEvents() const { return fNProcessedEvents ; } 

  // a OTrDaq is container for OTrTdcBoards:
  TdcContainer& GetTdcBoards() { return GetChildren() ; } ;
  const  TdcContainer& GetTdcBoards() const { return GetChildren() ; } ;
  OTrTdcBoard* GetTdcBoard(ItemId_t tdcId) { return GetChild(tdcId) ; } ;
  const OTrTdcBoard* GetTdcBoard(ItemId_t tdcId) const { 
    return GetConstChild(tdcId) ; } ;
  OTrTdcBoard* FindTdcBoard(ItemId_t tdcId) { return FindChild(tdcId) ; } ;
  OTrTdcBoard* AddTdcBoard(ItemId_t tdcId) ;

  // other inlines
  void WriteAsciMask(char * filename) const { 
    WriteTable(filename,kMaskTable) ; }
  void ReadAsciMask(char * filename) { ReadTable(filename) ; }
  void IncreaseNProcessedEvents() { fNProcessedEvents++ ; } 
  
  // other methods
  void ReadTable(char *) ;                   // Read a db file
  void WriteTable(char *,ETableType) const ; // write a db file
  virtual void Reset(bool=true) ;            // Call Tdc reset method
  void PrintSummary() const ;

private:  
  int fNProcessedEvents ;     // number of processed events
  int fHBComponentId ;        // component id of outer tracker in HB daq system
  static int fgDefaultNTdcs ; // Default number of tdcs
#ifdef USE_ROOT
#ifdef __OTR_CLASSDEF__
  ClassDef(OTrDaq,0)            // The Outer Tracker daq class
#endif
  virtual TString GetName() const { return TString("Staring ... DAQ") ; }
#endif


} ;

#endif
