#ifndef __OTR_EVENT_HH__
#define __OTR_EVENT_HH__

// #ifndef __SGI_STL_LIST
// #include <list>
// #endif
#include "event.h"

#ifndef __SGI_STL_VECTOR
#include <vector>
#endif
#ifndef __SGI_STL_MAP
#include <map>
#endif
#ifndef __OTR_SUPERLAYEREVENT_HH__
#include "OTrSuperLayerEvent.hh"
#endif

#ifdef __OTR_CLASSDEF__
 #ifndef ROOT_Rtypes
 #include <Rtypes.h>
 #endif
#endif

#ifdef USE_ROOT
 class TPad ;
 class TBuffer ;
#endif

class OTrEventBuffer ;
class OTrTrackCandidate ;

class OTrEvent
{
public:
  // typedefs, enumerates
  typedef vector<OTrTrackCandidate*> TrackContainer ;
  typedef map<int, OTrSuperLayerEvent , less<int> > SuperLayerEventContainer ;

  // constructors, destructor, assignment
  //OTrEvent() ;
  //OTrEvent(const OTrEventBuffer*) ;
  OTrEvent(int id=0) ;
  virtual ~OTrEvent() ;

  // setters and getters
  const TrackContainer& GetTrackCandidates() const { return fTrackCandidates ; } ;
  const SuperLayerEventContainer& GetSuperLayerEvents() const { return fSuperLayerEvents ; }
  SuperLayerEventContainer& GetSuperLayerEvents() { return fSuperLayerEvents ; }
  int GetId() const { return fId ; } 

  // other methods
  OTrHit* Add( OTrHit* hit ) ;  // Add a hit o the event
  void PerformHistogramming() ;
  int SearchClusters() ;        // search clusters
  int SearchTrackCandidates() ; // serahc tracks
  int FitTracks() ;             // fit those tracks
  void DumpHits(ostream& os=cout) const ;
  void DumpClusters(ostream& os=cout) const ;
  void SortHits(); // sort the hits per superlayer according to GEDE index and x
  void Write(ostream& ofs) const ;
  void AppendRSEG(); // append all track candidates to the RSEG bank
  void AppendClustersToRSEG() ;
  void AppendHITB(); // append all hits to the HITB bank
  static void SetAppendClustersToRSEG(bool on=true) { fgAppendClustersToRSEG=on ; } ;
#ifndef CINTDICT
  const OTrSuperLayerEvent* FindSuperLayerEvent(int) const ;
#endif
  int GetNHits() const ;
  int GetNClusters() const ;

#ifdef USE_ROOT
 #ifdef __OTR_CLASSDEF__
  ClassDef(OTrEvent,1) // The Outer Tracker event class
 #else
  void Streamer(TBuffer&) ;
 #endif
  void DrawHits() ; // draw the hits in the event display
  void DrawClusters() ;// draw the clusters in the event display
  void DrawTrackCandidates( TPad* pad ) ;// draw the track candidates in the event display
#endif

private:
  int fId ; // Event id 
  TrackContainer fTrackCandidates ; // Tracks
  SuperLayerEventContainer fSuperLayerEvents ; // Superlayer events
  static bool fgAppendClustersToRSEG ;
} ;

/******************* inlines **********************/
#ifndef CINTDICT
inline const OTrSuperLayerEvent*
OTrEvent::FindSuperLayerEvent(int id) const {
  const SuperLayerEventContainer::const_iterator superLayerEvent = 
    fSuperLayerEvents.find(id) ;
  if(superLayerEvent==fSuperLayerEvents.end()) return 0;
  return &((*superLayerEvent).second) ;
}
#endif
#endif
