#ifndef __OTR_EVENTBUFFER_HH__
#define __OTR_EVENTBUFFER_HH__

/*** This is the the event buffer class for the dmon program. On
*construction it reserves space for the ETB fed buffer, with a length
*of fgMaxNFedBuffers. GetEvent is the interface to the ETB_get_feds
*routine.
**/
#ifdef __OTR_CLASSDEF__
 #ifndef ROOT_Rtypes
 #include <Rtypes.h>
 #endif
#endif
#ifndef __OTR_ITERATORBASE_HH__
#include "OTrIteratorBase.hh"
#endif
#ifndef __OTR_FEDBUFFER_HH__
#include "OTrFedBuffer.hh"
#endif

struct ETB_fed;
struct EVENT_HDR;
struct LogHdr;
class OTrEvent ;

class OTrEventBuffer
{
public:
  // constructors, destructor, assignment
  OTrEventBuffer() ; 
  OTrEventBuffer(int datastream) ;
  // OTrEventBuffer(FILE *inputfile) ;
  virtual ~OTrEventBuffer() ;

  // setters and getters
  //void SetFile( FILE* inputFile ) {  fInputFile = inputFile; }
  int GetNFedBuffers() const { return fNFedBuffers ; } 
  const ETB_fed* GetFedBuffers() const { return fFedBuffers ; } 
  const EVENT_HDR* GetEventHdr() const { return fEvtHdr ; } 
  const LogHdr* GetLogHdr() const { return fLogHdr ; } 
  int GetDataStream() const { return fDataStream ; }
  static int GetDaqComponentId() { return fgDaqComponentId ;} 
  static int GetMaxNFedBuffers() {  return fgMaxNFedBuffers ; }

  // definition of the iterator over FED buffers
  class FedConstIterator : public OTrConstIteratorBase<ETB_fed>
  {
  public:
    FedConstIterator(const OTrEventBuffer&) ;
    inline const OTrFedBuffer operator*() ;
    inline const OTrFedBuffer operator->() ;
  };

  int OpenFile(char* filename, int fileformat) ; // Open a new file
  void DumpFedBuffers() ;                        // Dump all info in the FedBuffers
  int NextFEDBuffers() ;                         // Get next FED buffers from data stream

#ifdef __OTR_CLASSDEF__
  ClassDef(OTrEventBuffer,0)
#endif

private:
  // private methods
  int ETBNextEventHdr() ;             // Get next event header,
  int ETBUpdateFedBuffers() ;         // Update fed buffers using the ETB
  int DOTRUpdateFedBuffers() ;        // Update fed buffers from current DOTR bank
  // data members
  static const int fgMaxNFedBuffers ; // Maximum number of Fed buffers 
  static const int fgMaxEventLength ; // Length (in bytes) of the event header
  static const int fgDaqComponentId ; // Component mask for FED buffers
  int fDataStream ;                   // Data stream (DOTR,FARM,STANDALONE)
  FILE* fInputFile ;                  // Input file
  int fNFedBuffers ;                  // The current number of valid FED buffers
  ETB_fed* fFedBuffers ;              // Pointer to FED buffers (owner)
  char* fEvtBuffer ;                  // Buffer for the event header 
  EVENT_HDR *fEvtHdr ;                // Pointer to current event header
  LogHdr* fLogHdr ;                   // Log header (runnr, eventnr, etc)
} ;

// Definition of inline functions

inline
OTrEventBuffer::FedConstIterator::FedConstIterator(const OTrEventBuffer& parent) :
  OTrConstIteratorBase(parent.GetFedBuffers(),
		       parent.GetFedBuffers()+parent.GetNFedBuffers())  {} ;

inline const OTrFedBuffer 
OTrEventBuffer::FedConstIterator::operator*(){ 
  return (const OTrFedBuffer)(current()) ; };

inline const OTrFedBuffer
OTrEventBuffer::FedConstIterator::operator->(){
  return (const OTrFedBuffer)(current()) ; };

#endif
