#ifndef __OTR_HIT_HH__
#define __OTR_HIT_HH__

#ifndef __OTR_CHANNEL_HH__
#include "OTrChannel.hh"
#endif
#ifdef __OTR_CLASSDEF__
 #ifndef ROOT_Rtypes
 #include "Rtypes.h"
 #endif
#endif

class OTrWire ;
class TTUBE ;
class TRotMatrix ;
class TNode ;
class TBuffer ;

#if defined(USE_ARTE) && !defined(CINTDICT)
 #ifndef __ARTE_HH__
 #include "arte/arte.hh"
 #endif
 typedef ArtePointer<HITB> ArtePointerHITB ;
#else
 typedef int ArtePointerHITB ;
#endif

/** OTrHit class */
class OTrHit
{
public:
  // typedefs, enumerates
  typedef OTrChannel::TdcCount_t TdcCount_t ;

  // constructors, destructors, assignment
  OTrHit() ;
  OTrHit(OTrChannel* chan, TdcCount_t tdc) ;
  OTrHit( const OTrHit& h );
  virtual ~OTrHit() ;
  OTrHit& operator=( const OTrHit& h );

  // setters and getters
  OTrChannel* GetChannel() const { return fChannel ;}
  TdcCount_t GetTdcCounts() const { return fTdcCounts ;}
  ArtePointerHITB GetHITB() const { return fHITB; } ;

  // other inlines
  float GetDriftTime() const { return fChannel->GetDriftTime(fTdcCounts) ; };
  const OTrWire* GetWire() const { return fChannel->GetWire() ; }

  // other methods
  OTrDObservable GetDriftDistance() const ;
  float GetZ() const ;
  
#ifndef CINTDICT
  void SetHITB(ArtePointerHITB hitb) { fHITB = hitb ; } ;
#endif

#ifdef USE_ROOT
  void Draw(int color=1) ; // draw the hit as a tube in the event display
  void PerformHistogramming() const {GetChannel()->PerformHistogramming(GetTdcCounts());} 
 #ifdef __OTR_CLASSDEF__
  ClassDef(OTrHit,0) // The Outer Tracker hit class
 #else 
  void Streamer(TBuffer&) ;
 #endif
#endif

#ifdef USE_ARTE
  ArtePointerHITB AppendHITB();// append this hit to the HITB bank
  void DumpHITB( ostream& os = cout ) const;// dump the HITB contents
#endif

private:
  OTrChannel* fChannel ;  // pointer to channel
  TdcCount_t fTdcCounts ; // this hit's tdc value
  ArtePointerHITB fHITB ; // pointer to arte hitb bank
#ifdef USE_ROOT
  TNode *fNode ;          // pointer to node
#endif
};

#endif
