#ifndef __OTR_OBSERVABLE_HH__
#define __OTR_OBSERVABLE_HH__

#ifndef _CLHEP_H_
#include <CLHEP/config/CLHEP.h>
#endif
#ifdef __OTR_CLASSDEF__
 #ifndef ROOT_Rtypes
 #include <Rtypes.h>
 #endif
 #ifndef __RTYPEST_HH_
 #include <Util/RtypesT.h>
 #endif
#endif

inline double sqr(const double x) { return x*x; } ;
inline float sqr(const float x) { return x*x; } ;
/** Stores a measurement with a value and an error. */

template <class T>
class OTrObservable
{
public:
  ///
  OTrObservable():fValue(0),fError(0){} ;
  ///
  OTrObservable(const T& val, const T& err=0):fValue(val),fError(err) {} ;
  /// copy constructor
  OTrObservable(const OTrObservable& o) : fValue(o.fValue),fError(o.fError) {}
  virtual ~OTrObservable() {} ;
  ///
  const T& GetValue() const { return fValue ; } ;
  ///
  const T& GetError() const { return fError ; } ;
  ///
  void SetError(const T& err) { fError=err; } ;
  void SetValue(const T& val) { fValue=val; } ;
  void SetValue(const T& val, const T& err) { fValue=val; fError=err; } ;
#ifdef __OTR_CLASSDEF__
  ClassDefT(OTrObservable<T>,0) // Represents a measured value
#endif
private:
  T fValue ;
  T fError ;
};

#ifndef _IOSTREAM_H
#include <iostream.h>
#endif
class ostream ;
template <class T>
ostream& operator<<(ostream &s, const OTrObservable<T> &q) 
{
  float error = q.GetError() ;
  float value = q.GetValue() ;
  return s << value << "(" << error << ")";
};

typedef OTrObservable<float> OTrFObservable ; 
typedef OTrObservable<double> OTrDObservable ; 

#endif

