#ifndef __OTR_RECONSTRUCTEDHIT_HH__
#define __OTR_RECONSTRUCTEDHIT_HH__

#ifndef __OTR_OBSERVABLE_HH__
#include "OTrObservable.hh"
#endif

#if defined(USE_ARTE) && !defined(CINTDICT)
 #ifndef __ARTE_HH__
 #include "arte/arte.hh"
 #endif
 typedef ArtePointer<RHIT> ArtePointerRHIT ;
#else
 typedef unsigned char ArtePointerRHIT ;
#endif //USE_ARTE

#ifdef __OTR_CLASSDEF__
 #ifndef ROOT_Rtypes
 #include <Rtypes.h>
 #endif
#endif

class OTrHit ;
class OTrTrackDescription ;
class OTrNodeProjection ;

class OTrReconstructedHit
{
public:
  // typedefs, enumerates
  typedef signed char LRSign_t ; // The left-right sign

  // constructors, destructor, assignment
  OTrReconstructedHit(const OTrReconstructedHit& rhs) ;
  OTrReconstructedHit() ;
  OTrReconstructedHit(const OTrHit* hit, LRSign_t lrSign=0) ;
  virtual ~OTrReconstructedHit() ;

  // setters and getters
  LRSign_t GetLRSign() const { return fLRSign ; }
  bool GetLRSignIsFixed() const { return fLRSignIsFixed; }
  const OTrHit* GetMeasuredHit() const { return fHit ; }

  // other inlines
  void FixLRSign() { fLRSignIsFixed=true ; }
  void ReleaseLRSign() { fLRSignIsFixed=false; }
  void FlipLRSign() { fLRSign *= -1 ; } ;
  inline void SetLRSign(double s) ;
  inline void SetLRSign(LRSign_t s) ;
  inline OTrNodeProjection* GetNodeProjection() ;
  inline void DeleteNodeProjection() ;
  inline double GetChi2() const ;

  // other methods
  bool UpdateInternal() ;
#ifdef USE_ARTE
  ArtePointerRHIT AppendRHIT(const OTrTrackDescription*) ;
  ArtePointerRHIT GetRHIT() const { return fRHIT ; } ;
#endif
  //#ifdef __OTR_CLASSDEF__
  //ClassDef(OTrReconstructedHit,0)
  //#endif
private:
  const OTrHit* fHit ;  // Pointer to corresponding OTrHit
  LRSign_t fLRSign ;    // Left-right sign (ambiguity)
  bool fLRSignIsFixed ; // Status flag for left-right sign
  OTrNodeProjection* fProjection ; //! Corresponding projection to track
#ifdef USE_ARTE
  ArtePointerRHIT fRHIT ; // ArtePointer to RHIT bank
#endif
} ;

inline void OTrReconstructedHit::SetLRSign(double s) { 
  if(!GetLRSignIsFixed()) fLRSign  = (s>0) ? 1 : -1; 
} ;

inline void OTrReconstructedHit::SetLRSign(LRSign_t s) { 
  if(!GetLRSignIsFixed()) {
    if(s<0) fLRSign = -1 ;
    else if(s>0) fLRSign = 1;
    else fLRSign =0 ;
    //fLRSign  = (s>0) ? 1 :-1;
    if(abs(s)>=2) FixLRSign() ;
  }
} 

#ifndef __OTR_NODEPROJECTION_HH__
#include "OTrNodeProjection.hh"
#endif

inline OTrNodeProjection* 
OTrReconstructedHit::GetNodeProjection()
{
  if(!fProjection) {
    fProjection = new OTrNodeProjection(this) ;
  };
  return fProjection ;
}

inline void 
OTrReconstructedHit::DeleteNodeProjection()
{
  if(fProjection) {
    delete fProjection ;
    fProjection=0;
  };
}

inline double
OTrReconstructedHit::GetChi2() const { return fProjection->GetChi2() ; } ;
#endif
