#ifndef __OTR_SECTION_HH__
#define __OTR_SECTION_HH__

#ifndef __OTR_GEDE_HH__
#include "OTrGede.hh"
#endif
#ifndef __OTR_WIRE_HH__
#include "OTrWire.hh"
#endif
#ifndef __OTR_COMPONENT_HH__
#include "OTrComponent.hh"
#endif
#ifndef __SGI_STL_VECTOR
#include <vector>
#endif
#ifdef __OTR_CLASSDEF__
 #ifndef ROOT_Rtypes
 #include <Rtypes.h>
 #endif
#endif
#ifdef USE_ROOT
 #ifndef ROOT_TString
 #include <TString.h>
 #endif
#endif

class TH1 ;
class TH1F ;
class TH1D ;
class TPad ;
class OTrLayer ;
class OTrHit ;
class OTrSuperLayerEvent ;
class OTrAlignmentProjection ;
class OTrHit ;
class OTrSector ;

class OTrSection : public OTrGede, public OTrDetectorComponent
{
public:
  // typedefs, enumerates
  typedef vector<OTrWire> WireContainer ;

  // constructors, destructors, assigment
  OTrSection();
  OTrSection( OTrGede* gede, OTrLayer* lay, const ComponentId_t id ) ;
  virtual ~OTrSection() ;

  // setters and getters
  //OTrSector* GetSector() const { return fSector ; }
  const WireContainer& GetWires() const { return fWires ; }
  WireContainer& GetWires() { return fWires ; }
  virtual ItemId_t GetItemId() const { return GetSectionId() ; }
  OTrGede* GetGede() { return (OTrGede*)(this); }
  const OTrWire* GetWire( const int wirenumber ) const { 
    return &(GetWires()[wirenumber]) ; }  // find a certain wire with a give wire number
  OTrAlignmentProjection* GetAlignmentProjection() { return fAlignmentProjection ; }
  static bool GetAlignmentOn() { return fgAlignmentOn ; }

  // other methods
  void CreateWires(); // create the array of wires 
  
#ifdef USE_ROOT
  TString GetName() const ;
  void DumpBadChannels( ostream& os = cout ) ;
#endif

#ifdef __OTR_CLASSDEF__
  ClassDef(OTrSection,0) // The Outer Track section class
#endif

private:
  WireContainer fWires ; // array of wires
  OTrLayer* fLayer ;     // pointer to parent sector
  OTrAlignmentProjection* fAlignmentProjection ; // keeps track of alignment stuff
  static bool fgAlignmentOn ; 
} ;

#endif


