#ifndef __OTR_SECTIONCLUSTER_HH__
#define __OTR_SECTIONCLUSTER_HH__

//#include "OTrEvent.hh"

#ifndef __OTR_HIT_HH__
#include "OTrHit.hh"
#endif
#ifndef __OTR_WIRE_HH__
#include "OTrWire.hh"
#endif
#ifndef __OTR_GEDE_HH__
#include "OTrGede.hh"
#endif
#ifndef __SGI_STL_VECTOR
#include <vector>
#endif
// #ifdef __OTR_CLASSDEF__
// #ifndef ROOT_Rtypes
// #include "Rtypes.h"
// #endif
// #endif

// Cluster in one section. 
class OTrSuperCluster ;
class OTrHit ;

class OTrSectionCluster
{
public:
  //typedef OTrSuperLayerEvent::HitContainer::const_iterator HitIterator ;
  // There is a bug in rootcint (repaired in new version)
  typedef vector<const OTrHit*> HitContainer ;
  typedef HitContainer::const_iterator HitIterator ;
  OTrSectionCluster() ;
  virtual ~OTrSectionCluster() {} ;
  bool WithinYAcceptance(float tx, float ty) const ;
  float GetXDif(float tx, float ty=0) const ;
  float GetRawX() const ;
  float GetRawZ() const ;
  float GetRawTx() const ;
  //float GetStereoAngle() const ;
  float GetCosStereoAngle() const ;
  float GetSinStereoAngle() const ;
  OTrGede* GetGede() const ;
  OTrSection* GetSection() const ;
  int GetSize() const { return size() ; } ;
  float GetAverageWireNumber() const ;
  HitIterator begin() const { return fHits.begin() ; } ;
  HitIterator  end() const { return fHits.end() ; } ;
  const OTrHit* front() const { return fHits.front() ; } ;
  const OTrHit* back() const { return fHits.back() ; } ;
  int size() const { return fHits.size() ; } ;
  void push_back(const OTrHit* hit) ;
  void SetSuperCluster(const OTrSuperCluster* cl) { fSuperCluster=cl; } ;
  const OTrSuperCluster* GetSuperCluster() const { return fSuperCluster; } ;
  signed char GetLRSign(const OTrHit* hit) const ;
#ifdef USE_ROOT
  //ClassDef(OTrSectionCluster,0)
  void Draw(int color) ;
  void PerformHistogramming() const ;
#endif
  void Dump() const ;
  bool ValidNHits() const ;
  static bool SameSectionCluster(const OTrWire* lhsWire, const OTrWire* rhsWire) ;
private:
  const OTrSuperCluster* fSuperCluster ;
  HitContainer fHits ;
  float fSumX ;
  float fSumZ ;
} ;

inline
OTrSectionCluster::OTrSectionCluster() : fSuperCluster(0), fSumX(0), fSumZ(0) {} ; 

inline void
OTrSectionCluster::push_back(const OTrHit* hit) { 
  fHits.push_back(hit) ; 
  fSumX += back()->GetWire()->GetRawX() ;
  fSumZ += back()->GetWire()->GetRawZ() ;
} ;

inline OTrGede*
OTrSectionCluster::GetGede() const {
  return front()->GetWire()->GetGede() ;
};

inline OTrSection*
OTrSectionCluster::GetSection() const {
  return front()->GetWire()->GetSection() ;
};

inline float
OTrSectionCluster::GetAverageWireNumber() const {
  return 0.5*(front()->GetWire()->GetWireNumber()+ back()->GetWire()->GetWireNumber()) ;
};

inline float
OTrSectionCluster::GetRawX() const {
  //return 0.5*(front()->GetWire()->GetRawX() + back()->GetWire()->GetRawX()) ;
  return fSumX/size() ;
};

inline float
OTrSectionCluster::GetRawZ() const {
  //return 0.5*(front()->GetWire()->GetRawZ() + back()->GetWire()->GetRawZ()) ;
  return fSumZ/size() ;
};

inline float 
OTrSectionCluster::GetRawTx() const {
  return GetRawX()/GetRawZ() ;
};

// inline float
// OTrSectionCluster::GetStereoAngle() const {
//   return GetGede()->GetStereoAngle() ;
// };

inline float
OTrSectionCluster::GetCosStereoAngle() const {
  return GetGede()->GetCosStereoAngle() ;
};

inline float
OTrSectionCluster::GetSinStereoAngle() const {
  return GetGede()->GetSinStereoAngle() ;
};

inline bool
OTrSectionCluster::WithinYAcceptance(float tx, float ty) const {
  return GetGede()->WithinYAcceptance(tx,ty) ;
};

inline float
OTrSectionCluster::GetXDif(float tx, float ty) const {
  //for zero degree only 
  return GetRawX()- GetRawZ() * 
    (tx*GetCosStereoAngle() - ty*GetSinStereoAngle()) ;
};

#ifdef USE_ROOT
inline void
OTrSectionCluster::PerformHistogramming() const {
  for(HitIterator hitIt = fHits.begin() ; hitIt!= fHits.end() ; hitIt++)
    (*hitIt)->PerformHistogramming() ;
};
#endif

inline bool
OTrSectionCluster::SameSectionCluster(const OTrWire* lhsWire, const OTrWire* rhsWire)
{ 
  // For this routine it is essential that rhsPrWireNumber > lhsPRWireNumber !!
  int numdif ;
  return (lhsWire->GetGede()==rhsWire->GetGede()) &&
    (((numdif = rhsWire->GetPRWireNumber()- lhsWire->GetPRWireNumber()) < 3) ||
     ((numdif==3) && rhsWire->GetMonoLayer() == lhsWire->GetMonoLayer()+3)) ;
  // This was for single layers :
  //(*hitIt)->GetWire()->GetWireNumber()!=(*nextHitIt)->GetWire()->GetWireNumber()-1;
} ;

#endif
