#ifndef __OTR_TZEROCALIB_HH__
#define __OTR_TZEROCALIB_HH__

#ifndef __SGI_STL_VECTOR
#include <vector>
#endif
#ifdef __OTR_CLASSDEF__
#ifndef ROOT_Rtypes
#include <Rtypes.h>
#endif
#endif

class OTrDaq;
class OTrDetector;
class TH1F;

class OTrTZeroCalib
{
public:
  // constructors, destructor, assignment
  OTrTZeroCalib();
  OTrTZeroCalib( OTrDaq* daq, OTrDetector* det );
  virtual ~OTrTZeroCalib() ;
 
  // setters and getters
  unsigned int GetSuperLayerId() const { return fSuperLayerId ; }
  void SetSuperLayerId( const int id ) { fSuperLayerId = id; };

  // other methods
#ifdef USE_ROOT
  int Calibration( const int method = 0, bool inter = false ); // calibration by different methods
  void Compare(); // compare different methods of tzero calibration
  void CompareCalibrationFiles(char* file1, char* file2 );  // compare two calibration files
  void CalibFromResiduals() ; // calib from residuals in ChannelHistogrammer
  void SubtractAverageMC02() ;
  void SubtractAverage( const double av5mm, const double av10mm );
  void Monitor() ; // create monitor histograms
#endif

#ifdef __OTR_CLASSDEF__
  ClassDef(OTrTZeroCalib,0)
#endif

private:
  void FillReferenceSpectra( const int nSteps );// fill reference spectra
  TH1F* CreateReferenceHistogram( vector<double>& ref, char* name,
				  const int color = 4);// create a ROOT reference histogram
 
  OTrDaq* fDaq; // pointer to DAQ object
  OTrDetector* fDetector;// pointer to detector object
  unsigned int fSuperLayerId; // superlayer ID, of the superlayer to calibrate
  vector<double> fReferenceHistogram5mm; // 5mm reference histogram used for the reference histogram method
  vector<double> fReferenceHistogram10mm; // 10mm reference histogram used for the reference histogram method 
  TH1F* fRef5mmH1;
  TH1F* fRef10mmH1;
} ;

#endif
