#ifndef __OTR_TDCBOARD_HH__
#define __OTR_TDCBOARD_HH__

#ifndef __OTR_ASDBOARD_HH__
#include "OTrAsdBoard.hh"
#endif
#ifdef __OTR_CLASSDEF__
 #ifndef ROOT_Rtypes
 #include "Rtypes.h"
 #endif
#endif

class OTrFedBuffer ;
class OTrEvent ;

class OTrTdcBoard : public OTrDaqComponent
{
public:
  // typedefs, enums
   typedef vector<OTrAsdBoard> AsdContainer ;
 
  // constructors, destrctor, assignment
  OTrTdcBoard() ; // Default constructor: creates NO asd boards
  OTrTdcBoard(ComponentId_t) ;// This is the constructor you are supposed to use
  virtual ~OTrTdcBoard() ;
  
  //setters and getters
  int GetNProcessedEvents() const { return fNProcessedEvents ; }
  static int GetNAsdBoards() { return fgDefaultNAsdBoards ; } ;
  AsdContainer& GetAsdBoards() { return fAsds ; } ;
  virtual ItemId_t GetItemId() const { return GetTdcId() ; } ;
  const AsdContainer& GetAsdBoards() const { return fAsds ; } ;

   // iterators 
  class AsdIterator : public OTrIteratorBase<OTrAsdBoard> 
  { public: AsdIterator(OTrTdcBoard&) ; };
  friend class AsdIterator ;
  ///
  class AsdConstIterator : public OTrConstIteratorBase<OTrAsdBoard>
  { public: AsdConstIterator(const OTrTdcBoard&) ; };
  friend class AsdConstIterator ;
   
  // other methods
  inline OTrAsdBoard* FindAsdBoard(AsdId_t id) ;
  inline OTrAsdBoard& GetAsdBoard(AsdId_t id) ;
  void ProcessFedBuffer(const OTrFedBuffer&, OTrEvent*) ; // Process an eventbuffer
  void Reset() ;

#ifdef USE_ROOT
  TH1* GetTimeH1() const ;
  TString GetName() const ;
#endif

#ifdef __OTR_CLASSDEF__
  ClassDef(OTrTdcBoard,0) // The Outer Tracker TDC Board class
#endif

private:
  static const int fgDefaultNAsdBoards ; // The number of asd boards per tdc board
  AsdContainer fAsds ; // Container of asd boards connected to this tdc
  int fNProcessedEvents ; // Number of events in which this FED buffer was present.
};

/// Implement the constructor for the iterators

inline
OTrTdcBoard::AsdIterator::AsdIterator(OTrTdcBoard& tdc) : 
  OTrIteratorBase(tdc.fAsds.begin(),tdc.fAsds.end()) {} ;
inline
OTrTdcBoard::AsdConstIterator::AsdConstIterator(const OTrTdcBoard& tdc) : 
  OTrConstIteratorBase(tdc.fAsds.begin(),tdc.fAsds.end()) {} ;

inline OTrAsdBoard&
OTrTdcBoard::GetAsdBoard(AsdId_t id) {
  if ( id >= 0 && id < fAsds.size() ) {
    return fAsds[id] ; 
  } else {
    cout << "** ERROR in OTrTdcBoard::GetAsdBoard, id = " << id << endl; 
    exit(1);
  }
}

inline OTrAsdBoard*
OTrTdcBoard::FindAsdBoard(AsdId_t id) {  
  if ( id >= 0 && id < fAsds.size() ) {
    return &(fAsds[id]) ; 
  } else {
    exit(1);
  }
}

#endif
