#ifndef __OTR_TRACKCANDIDATE_HH__
#define __OTR_TRACKCANDIDATE_HH__

#ifndef __SGI_STL_VECTOR
#include <vector>
#endif
#ifndef __OTR_TRACK_HH__
#include "OTrTrack.hh"
#endif

class OTrSuperCluster ;
class rangerTrack ;
class OTrTrack ;
class OTrHit ;
class TPolyLine3D ; 
class TPad ;

#ifdef USE_ARTE
 #ifndef __ARTE_HH__
 #include "arte/arte.hh"
 #endif
typedef ArtePointer<RSEG> ArtePointerRSEG ;
#else
 typedef int ArtePointerRSEG ;
#endif

class OTrTrackCandidate 
{
public:
  typedef vector<const OTrSuperCluster*> ClusterContainer ;
  /// default constructor
  OTrTrackCandidate();
  /// constructor
  OTrTrackCandidate(const OTrSuperCluster*, const OTrSuperCluster*) ;
  OTrTrackCandidate(const OTrSuperCluster* cl) ;
  /// copy constructor
  OTrTrackCandidate( const OTrTrackCandidate& );
  /// destructor
  ~OTrTrackCandidate();

  bool Valid() const ;
  bool CleanUp(const OTrTrackCandidate&) ;
  bool AddIfValid(const OTrSuperCluster*) ;
  void Add(const OTrSuperCluster*) ;
  const ClusterContainer& GetClusters() const { return fClusters ; } ;
  double GetTargetChi2() const { return fTargetChi2 ; } ;
  rangerTrack* GetRangerTrack() const { return fRangerTrack ; } ; 
  OTrTrack* GetTrack() const { return fTrack ; } ;
#ifdef USE_ROOT
  /// draw as a PolyLine3D in the event display
  int Draw( TPad* pad, const int color = 1 );
#endif
  
#ifdef USE_ARTE
  ArtePointerRSEG GetRSEG() const { return fRSEG ; } ;
  ArtePointerRSEG AppendRSEG();
  bool RangerFit() ;
#endif
  bool Fit(bool=true) ;
private:
  //
  ClusterContainer fClusters ;
  float fTargetChi2 ;
  ArtePointerRSEG fRSEG ;
  rangerTrack* fRangerTrack ;
  OTrTrack* fTrack ;
  /// polyline for the ROOT event display 
  TPolyLine3D *fPolyLine3D;
  HepVector fQ ;
  HepSymMatrix fCov ;
  float fChi2 ;
} ;


#endif
