#ifndef _OTrView__
#define _OTrView__

#ifdef USE_ROOT

#include <TView.h>
#include <TCanvas.h>

class TRint ;
class TControlBar ;
class OTrEvent ;
class OTrDetector ;
class TNode ;
class TPad ;

enum ViewMode {END, RSIDE, LSIDE,  TOP,  BOT,  ALL};

class OTrView : public TObject
{ 
public:
  // constructors, destructor, assignment
  OTrView(OTrDetector*);
  ~OTrView();

  // setters and getters
  //void SetEvent( OTrEvent* event ) { fEvent = event; }
  //void SetEventNumber( const int eventnr ) { fEventNumber = eventnr; }
  bool GetZoomMode() { return fZoomMode; }
  virtual bool AllViews() { return fDrawAllViews; }
  TPad* GetDisplayPad() { return fDisplayPad; }

  // other methods
  void CreateNode(OTrDetector*) ;
  void Display(OTrEvent*) ;

  int Zoom( const float z ); // zoom function (still being tested)
  void Refresh() const;  // refresh the canvas
  void DisplayButtons(); // display the buttons
  int SetRange( const float x1, const float y1, const float z1, 
		const float x2, const float y2, const float z2 ); // set the range
  void SetAngles( const float longitude, const float latitude,
		  const float psi ); // set the view orientation
  void SetStandardView(char mode); // set to one of the standard views
  void ToggleZoomMode(); // toggle zoom mode
  void DrawViewGL(); // draw current view using OPENGL
  void DrawViewX3D(); // draw current view using X3D
  void SetGeometry(float rin); // set the view ranges
  void DrawView( float theta, float phi, float psi );// draw a view
  void DrawTitle(Option_t *option); // draw a title in a pad
  void DrawAllViews(); // draw all views
  void ZoomIn();// zoom in 
  void ZoomOut();// zoom out
  void Pan(char direction);
  virtual void ExecuteEvent(Int_t event, Int_t px, Int_t py);// execute an event
  Int_t DistancetoPrimitive(Int_t px, Int_t py);// Distance to primitive
  void Draw(Option_t *option=""); // Draw
  virtual void ToggleDisplayAttribute(char attribute); 

  /// draw the beampipe
  int DrawBeamPipe();  

private:
  // Hi Ivar. I moved these here from controller ...
  TNode* fWaxNode; // main ROOT node for the WAX detector display
  TPad*  fWaxPad ;

  // create the menubar of buttons, used in the constructor
  //CreateMenuBar();

  ViewMode fViewMode; // viewing mode
  bool fDrawAllViews; // flag true if AllViews selected
  //TView* fView;
  TCanvas* fCanvas; // canvas
  TPad* fDisplayPad; // display pad
  TPad* fButtonsPad;// buttons pad
  OTrEvent* fEvent; // pointer to the current event
  int fEventNumber;
   //Point3D* fRangeMin; // minimum point of the range of the detector display view
  //Point3D* fRangeMax;// maximum point of the range of the detector display view 
  float fLongitude; // longitudinal angle of the detector display view
  float fLatitude; // latitudinal angle of the detector display view
  float fPsi;// psi angle of the detector display view
  float           fXcenter;// X center
  float           fYcenter;// Y center
  float           fZcenter;// Z Center
  float           fXin;// X range
  float           fYin;// Y range
  float           fRout; // Outer radius
  float           fZmin;// Inner length along Z
  float           fZmax;// Inner length along Z
  float           fZout; // Outer length along Z
  bool fZoomMode;// flag true in zoom mode
  float fZoomFactor;// zoom factor
  //TControlBar* fMenuBar; // menubar of buttons

  bool m_DrawAxis;
  bool m_DrawTracks;
  bool m_DrawChambers;
  bool m_DrawWireHits;
  bool m_DrawHitPoints;
  bool m_CurrentSlides[2];
} ;

extern OTrView* gWaxView;
extern TRint* myApp;

#endif

#endif








