#ifndef __WDH_POSITIVENORMALDISTRIBUTION_HH__
#define __WDH_POSITIVENORMALDISTRIBUTION_HH__

#ifndef _MATH_H
#include <math.h>
#endif

class PositiveNormalDistribution
{
public:
  static inline double FirstMoment(double x, double sigma=1/M_SQRT2) ; 
  static inline double SecondMoment(double x, double sigma=1/M_SQRT2) ; 
  static inline double Variance(double x, double sigma=1/M_SQRT2) ;
  static inline float FirstMoment(float x, float sigma=1/M_SQRT2) ; 
  static inline float SecondMoment(float x, float sigma=1/M_SQRT2) ; 
  static inline float Variance(float x, float sigma=1/M_SQRT2) ;

 
} ;

// double
// PositiveNormalDistribution::StandardNorm() { return 1/sqrt(M_PI) ; }

// double
// PositiveNormalDistribution::StandardFirstMoment(double xi) 
// { 
//    return exp(-sqr(xi*xi))/sqrt(M_PI) + xi * (erfc_(-xi)-erfc_(xi))/2 ;
// }

// double
// PositiveNormalDistribution::StandardSecondMoment(double xi) 
// { 
//   return sqr(xi) + 1/2 ;
// }

// double 
// PositiveNormalDistribution::StandardVariance(double xi)
// {
//   return StandardSecondMoment(xi) - sqr(StandardFirstMoment(xi)) ;
// }

//inline double sqr(double x) { return x*x ; }

extern "C" 
{
  double derfc_(const double&) ;
  float erfc_(const float&) ;
}

#define M_1_SQRT_PI (M_2_SQRTPI/2)

inline double
PositiveNormalDistribution::FirstMoment(double x, double sigma) 
{ 
  double xi = x/(sigma*M_SQRT2) ;
  //return sigma*M_SQRT2*(exp(-sqr(xi))*M_1_SQRT_PI + xi * (derfc_(-xi)-derfc_(xi))/2.0) ;
  return sigma*M_SQRT2*(xi + (exp(-sqr(xi)) -2*xi*derfc_(xi)/M_2_SQRTPI)*M_1_SQRT_PI) ;
}

inline double
PositiveNormalDistribution::SecondMoment(double x, double sigma) 
{ 
  double xi = x/(sigma*M_SQRT2) ;
  return sqr(sigma*M_SQRT2)*(sqr(xi) + 1/2.0) ;
}

inline double 
PositiveNormalDistribution::Variance(double x, double sigma)
{
  double xi = x/(sigma*M_SQRT2) ;
  double tmp = (exp(-sqr(xi)) -2*xi*derfc_(xi)/M_2_SQRTPI)*M_1_SQRT_PI ;
  return sqr(sigma*M_SQRT2) * (1/2.0 - 2*xi*tmp - tmp*tmp) ;
  //  return sqr(sigma*M_SQRT2) * (1/2.0 - sqr((exp(-sqr(xi)) -2*xi*derfc_(xi)/M_2_SQRTPI)*M_1_SQRT_PI) -
  //			       2*xi*((exp(-sqr(xi)) -2*xi*derfc_(xi)/M_2_SQRTPI)*M_1_SQRT_PI )) ;
  //   return sqr(sigma*M_SQRT2) * (1/2.0 + sqr(xi)*(1 - sqr((derfc_(-xi)-derfc_(xi))/2.0)) -
  // 			       (xi * (exp(-sqr(xi))*M_1_SQRT_PI) * (derfc_(-xi)-derfc_(xi)) +
  // 				sqr(exp(-sqr(xi))*M_1_SQRT_PI))) ;
}

// inline double 
// PositiveNormalDistribution::Variance(double x, double sigma)
// {
//   return SecondMoment(x,sigma)-sqr(FirstMoment(x,sigma)) ;
// }


inline float
PositiveNormalDistribution::FirstMoment(float x, float sigma) 
{ 
  float xi = x/(sigma*M_SQRT2) ;
  return sigma*M_SQRT2*(xi + (exp(-sqr(xi)) -2*xi*erfc_(xi)/M_2_SQRTPI)*M_1_SQRT_PI) ;
}

inline float
PositiveNormalDistribution::SecondMoment(float x, float sigma) 
{ 
  float xi = x/(sigma*M_SQRT2) ;
  return sqr(sigma*M_SQRT2)*(sqr(xi) + 1/2.0) ;
}

inline float 
PositiveNormalDistribution::Variance(float x, float sigma)
{
  float xi = x/(sigma*M_SQRT2) ;
  float tmp = (exp(-sqr(xi)) -2*xi*erfc_(xi)/M_2_SQRTPI)*M_1_SQRT_PI ;
  return sqr(sigma*M_SQRT2) * (1/2.0 - 2*xi*tmp - tmp*tmp) ;
}

#endif
